/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.bhie.common;

import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public final class AppConfig {
    private static final Logger LOGGER = Logger.getLogger(AppConfig.class);

    private static final String BHIE_ENDPOINT = "BHIE_ENDPOINT";
    private static final String VLER_ENDPOINT = "VLER_ENDPOINT";
    private static final String VLER_TEST_PAT = "VLER_TEST_PAT";
    private static final String PDF_SERVICE_ENDPOINT = "PDF_SERVICE_ENDPOINT";
    private static final String BHIE_VERSION = "BHIE_VERSION";
    private static final String DOWNLOAD_TIMEOUT_MS = "DOWNLOAD_TIMEOUT_MS";
    private static final String REST_CONNECTION_TIMEOUT_MS = "REST_CONNECTION_TIMEOUT_MS";
    private static final String REST_READ_TIMEOUT_MS = "REST_READ_TIMEOUT_MS";
    private static final String REST_POLLER_TIMEOUT_MS = "REST_POLLER_TIMEOUT_MS";
    private static final String REST_ASYNC_QUERY_PATH = "REST_ASYNC_QUERY_PATH";
    private static final String REST_ASYNC_COMET_PATH = "REST_ASYNC_COMET_PATH";
    private static final String REST_ASYNC_QUERY_PATH3 = "REST_ASYNC_QUERY_PATH3";
    private static final String REST_ASYNC_COMET_PATH3 = "REST_ASYNC_COMET_PATH3";
    private static final String REST_ASYNC_QUERY_PATH4 = "REST_ASYNC_QUERY_PATH4";
    private static final String REST_ASYNC_COMET_PATH4 = "REST_ASYNC_COMET_PATH4";
    private static final String REST_ASYNC_QUERY_PATHVLER = "REST_ASYNC_QUERY_PATHVLER";
    private static final String REST_ASYNC_COMET_PATHVLER = "REST_ASYNC_COMET_PATHVLER";
    private static final String COMPLEX_NOTE_PATH = "COMPLEX_NOTE_PATH";
    private static final String COMPLEX_NOTE_HOST_PLACEHOLDER = "COMPLEX_NOTE_HOST_PLACEHOLDER";
    private static final String DATASOURCE = "DATASOURCE";
    private static final String RELAYENDPOINTS = "RELAYENDPOINTS";
    private static final String RELAYENDPOINTAUTHS = "RELAYENDPOINTAUTHS";
    private static final String NOTE_CONVERT_FORMAT = "NOTE_CONVERT_FORMAT";

    //app name is passed to dod adaptor as user facility id
    private static final String USER_FACILITY_ID = "USER_FACILITY_ID";
    private static final String DEBUG_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS = "DEBUG_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS";
    private static final String DEBUG_ENTITY = "DEBUG_ENTITY";
    private static final String DEBUG_ENTITY_TOKEN = "DEBUG_ENTITY_TOKEN";

    private static final AppConfig INSTANCE = new AppConfig();

    private Properties appProperties;

    private AppConfig() {
        String propertiesFile = "bhierelay.properties";

        InputStream inputStream = null;

        appProperties = new Properties();

        ClassLoader classLoader = AppConfig.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        if (inputStream != null) {
            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load application properties: " + e.getMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        try {

            inputStream = new FileInputStream(propertiesFile);

            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load application properties: " + e.getLocalizedMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }

        } catch (FileNotFoundException e) {
            LOGGER.info("Failed to load app properties from file (bhierelay.properties) -"
                + " utilizing default app properties.");

        }
    }

    public static AppConfig getInstance() {
        return INSTANCE;
    }

    public String getBhieEndpoint() {
        return appProperties.getProperty(BHIE_ENDPOINT);
    }

    public String getVlerEndpoint() {
        return appProperties.getProperty(VLER_ENDPOINT);
    }

    public String getPdfServiceEndpoint() {
        return appProperties.getProperty(PDF_SERVICE_ENDPOINT);
    }

    public String getVlerTestPat() {
        return appProperties.getProperty(VLER_TEST_PAT);
    }

    public int getRestConnectionTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REST_CONNECTION_TIMEOUT_MS));
    }

    public int getRestReadTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REST_READ_TIMEOUT_MS));
    }

    public int getRestPollerTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REST_POLLER_TIMEOUT_MS));
    }

    public int getDownloadTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(DOWNLOAD_TIMEOUT_MS));
    }

    public boolean getDebugTrustAllSslHostnamesAndCerts() {
        return Boolean.parseBoolean(appProperties.getProperty(DEBUG_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS));
    }

    public String getRestAsyncQueryPath() {
        return appProperties.getProperty(REST_ASYNC_QUERY_PATH);
    }

    public String getRestAsyncCometPath() {
        return appProperties.getProperty(REST_ASYNC_COMET_PATH);
    }

    public String getRestAsyncQueryPath3() {
        return appProperties.getProperty(REST_ASYNC_QUERY_PATH3);
    }

    public String getRestAsyncCometPath3() {
        return appProperties.getProperty(REST_ASYNC_COMET_PATH3);
    }

    public String getRestAsyncQueryPath4() {
        return appProperties.getProperty(REST_ASYNC_QUERY_PATH4);
    }

    public String getRestAsyncCometPath4() {
        return appProperties.getProperty(REST_ASYNC_COMET_PATH4);
    }

    public String getRestAsyncQueryPathVLER() {
        return appProperties.getProperty(REST_ASYNC_QUERY_PATHVLER);
    }

    public String getRestAsyncCometPathVLER() {
        return appProperties.getProperty(REST_ASYNC_COMET_PATHVLER);
    }

    public String getComplexNotePath() {
        return appProperties.getProperty(COMPLEX_NOTE_PATH);
    }

    public String getComplexNoteHostPlaceholder() {
        return appProperties.getProperty(COMPLEX_NOTE_HOST_PLACEHOLDER);
    }

    public String getUserFacilityId() {
        return appProperties.getProperty(USER_FACILITY_ID);
    }
    
    public String getDataSource() {
        return appProperties.getProperty(DATASOURCE);
    }

    public String getBhieVersion() {
        return appProperties.getProperty(BHIE_VERSION, "2");
    }
    
    public String getRelayEndpoints() {
        return appProperties.getProperty(RELAYENDPOINTS, "");
    }
    
    public int getRelayEndpointCount() {
        int rtc = 1;
        String endp = getRelayEndpoints();
        if (endp != null && endp.length() > 0) {
            String[] points = endp.split("\\|");
            if (points.length > 1) {
                rtc = points.length;
            }
        }
        return rtc;
    }
    
    public String getRelayEndpointAuths() {
        return appProperties.getProperty(RELAYENDPOINTAUTHS, "");
    }
    
    public String getNoteConvertFormat() {
        return appProperties.getProperty(NOTE_CONVERT_FORMAT, "pdf");
    }
    
    public Properties getProperties() {
        return appProperties;
    }

    public String getDebugEntity() {return appProperties.getProperty(DEBUG_ENTITY);}

    public String getDebugEntityToken() {return appProperties.getProperty(DEBUG_ENTITY_TOKEN);}
}
