/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.bhie.patient;

import gov.va.med.bhie.BaseTest;
import gov.va.med.bhie.common.ClinicalDomainLoincCode;
import gov.va.med.bhie.dao.DaoException;
import gov.va.med.bhie.dao.patient.V4Dao;
import gov.va.med.bhie.dao.patient.VLERDao;

import gov.va.med.bhie.resultcollection.V4ResultCollection;
import org.junit.Test;

import static org.junit.Assert.assertTrue;

public class VlerDaoTest extends BaseTest
{

    @Test
    public void testGetVLERData() throws DaoException {

        VLERDao vlerDao = new VLERDao();
        V4ResultCollection vlerResults = vlerDao.getVLERData(queryBean);
        assertTrue(vlerResults.getDataRecordList().getAllergy().size() > 0);
    }

    @Test
    public void testGetPatientNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getNotes().size() > 0);
    }

    @Test
    public void testGetPatientEncounterNotesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ENCOUNTER_NOTES, queryBean);
        assertTrue(v4Results.getDataRecordList().getEncounterNotes().size() > 0);
    }

    @Test
    public void testGetPatientEncountersV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ENCOUNTERS, queryBean);
        assertTrue(v4Results.getDataRecordList().getEncounters().size() > 0);
    }

    @Test
    public void testGetPatientProceduresV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PROCEDURES, queryBean);
        assertTrue(v4Results.getDataRecordList().getProcedures().size() > 0);
    }

    @Test
    public void testGetPatientAppointmentsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.APPOINTMENTS, queryBean);
        assertTrue(v4Results.getDataRecordList().getAppointments().size() > 0);
    }

    @Test
    public void testGetPatientAdmissionsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ADMISSIONSV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getAdmissions().size() > 0);
    }

    @Test
    public void testGetPatientImmunizationsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.IMMUNIZATIONS, queryBean);
        assertTrue(v4Results.getDataRecordList().getImmunizations().size() > 0);
    }

    @Test
    public void testGetPatientMedicationsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.MEDICATIONS, queryBean);
        assertTrue(v4Results.getDataRecordList().getMedications().size() > 0);
    }

    @Test
    public void testGetPatientOrdersV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ALL_ORDERS, queryBean);
        assertTrue(v4Results.getDataRecordList().getMedicationOrders().size() > 0
            && v4Results.getDataRecordList().getLabOrders().size() > 0
            && v4Results.getDataRecordList().getRadiologyOrders().size() > 0);
    }

    @Test
    public void testGetPatientDemographicsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PATIENT_DEMOGRAPHICSV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getDemographics().size() > 0);
    }

    @Test
    public void testGetPatientProblemsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.PROBLEMS, queryBean);
        assertTrue(v4Results.getDataRecordList().getProblems().size() > 0);
    }

    @Test
    public void testGetPatientQuestionnairesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.QUESTIONNAIRES, queryBean);
        assertTrue(v4Results.getDataRecordList().getQuestionnaires().size() > 0);
    }

    @Test
    public void testGetPatientDeploymentFormsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.DEPLOYMENT_FORMS, queryBean);
        assertTrue(v4Results.getDataRecordList().getNotes().size() > 0);
    }

    @Test
    public void testGetPatientVitalsV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.VITALS, queryBean);
        assertTrue(v4Results.getDataRecordList().getVitals().size() > 0);
    }

    @Test
    public void testGetPatientInsuranceV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.INSURANCES, queryBean);
        assertTrue(v4Results.getDataRecordList().getVitals().size() > 0);
    }

    @Test
    public void testGetPatientAllergiesV4() throws DaoException {
        V4Dao v4Dao = new V4Dao();
        V4ResultCollection v4Results = v4Dao.getV4Data(ClinicalDomainLoincCode.ALLERGIESV4, queryBean);
        assertTrue(v4Results.getDataRecordList().getAllergy().size() > 0);
    }
}
