/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for Pacific Telehealth & Technology Hui/JITC
 */

package gov.va.med.foundations.utilities;

import org.yaml.snakeyaml.Yaml;

import java.io.*;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AppConfigBase {

    private static AppConfigBase thiz = null;
    private static LinkedHashMap<String, Object> config = null;

    public static final String ACTIVE_CONFIG = "ActiveConfig";

    protected AppConfigBase()
    {
        if(null == config)
        {
            parseAppConfig();
        }
    }

    @SuppressWarnings("unchecked")
    private void parseAppConfig()
    {
        InputStream is = getInputStream();
        String activeConfigName = null;
        Yaml yaml = new Yaml();
        for(Object o : yaml.loadAll(is))
        {
            if(!(o instanceof LinkedHashMap))
            {
                continue;
            }
            LinkedHashMap map = (LinkedHashMap) o;
            if(map.containsKey(ACTIVE_CONFIG))
            {
                activeConfigName = (String) map.get(ACTIVE_CONFIG);
                continue;
            }
            if(null != activeConfigName && map.containsKey(activeConfigName))
            {
                Object oo = map.get(activeConfigName);
                if(oo instanceof ArrayList)
                {
                    ArrayList arrayList = (ArrayList) oo;
                    if(arrayList.size() > 0)
                    {
                        if(arrayList.get(0) instanceof LinkedHashMap)
                        {
                            config = (LinkedHashMap<String, Object>) arrayList.get(0);
                            config.put(ACTIVE_CONFIG, activeConfigName);
                        }
                    }
                }
            }
        }

        try {
            is.close();
        } catch (IOException e) {
            Logger.getLogger(AppConfigBase.class.getName()).log(Level.SEVERE, e.getMessage(), "There was a problem closing appconfig input stream: ");
        }
    }

    private InputStream getInputStream()
    {
        InputStream is = null;
        FileInputStream fis = null;

        ClassLoader classLoader = this.getClass().getClassLoader();

        if (classLoader != null)
        {
            is = classLoader.getResourceAsStream("AppConfig.yaml");
            if(null != is)
            {
                return is;
            }

            try
            {
                fis = new FileInputStream("src"+File.separator+"AppConfig.yaml");
                return fis;
            }
            catch (FileNotFoundException e)
            {
                try
                {
                    //for use with unit tests
                    return new FileInputStream(String.format("src"+File.separator+"AppConfig-test.yaml"));
                }
                catch(FileNotFoundException e2) {return null;}
            }
        }

        throw new RuntimeException("Could not retrieve input stream: ClassLoader is null.");
    }

    public static String get(String key)
    {
        if(null == config)
        {
            thiz = new AppConfigBase();
        }

        if (config != null)
        {

            Object oValue = config.get(key);

            if (oValue != null)
            {
                if(oValue instanceof Integer)
                {
                    return Integer.toString((Integer) oValue);
                }
                else if(oValue instanceof Boolean)
                {
                    return Boolean.toString((Boolean) oValue);
                }
                else
                {
                    return (String) oValue;
                }
            }
            else Logger.getLogger(AppConfigBase.class.getName()).log(Level.SEVERE, null, "AppConfigBase: " + key + " is null");
        }

        return null;

    }
}
