/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.foundations.utilities;

import org.apache.log4j.Logger;

import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;


/**
* Simple encryption utility.
*
 * @author <a href="mailto:karl.a.weller@saic.com">karl.a.weller</a>
* @version 1.0
*
*/
public final class EncryptUtil {

    /**
     * Logger.
     */
    private static final Logger LOGGER =
        Logger.getLogger(EncryptUtil.class.getName());

    private EncryptUtil() {
    }

    /**
     * Encrypt.
     *
     * @param message input
     *
     * @return output
     */
    public static String encrypt(String message) {
        String rtc;
        final MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("SHA-512");
        } catch (NoSuchAlgorithmException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] digestOfPassword;

        try {
            digestOfPassword = digest.digest("HG58YZ3CR9".getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] keyBytes = Arrays.copyOf(digestOfPassword, 16);
        int index = 0;
        int kindex = 8;
        while (index < 8) {
            keyBytes[kindex++] = keyBytes[index++];
        }

        final SecretKey key = new SecretKeySpec(keyBytes, "AES");
        final IvParameterSpec ivspec = new IvParameterSpec(new byte[16]);
        final Cipher cipher;

        try {
            cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
        } catch (NoSuchAlgorithmException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (NoSuchPaddingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            cipher.init(Cipher.ENCRYPT_MODE, key, ivspec);
        } catch (InvalidKeyException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (InvalidAlgorithmParameterException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] plainTextBytes;

        try {
            plainTextBytes = message.getBytes("utf-8");
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] cipherText;

        try {
            cipherText = cipher.doFinal(plainTextBytes);
        } catch (IllegalBlockSizeException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (BadPaddingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        rtc = new sun.misc.BASE64Encoder().encode(cipherText);

        return rtc;
    }

    /**
     * Decrypt.
     *
     * @param enc input
     *
     * @return output
     */
    public static String decrypt(String enc) {
        String rtc;

        byte[] message;

        try {
            message = new sun.misc.BASE64Decoder().decodeBuffer(enc);
        } catch (IOException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("SHA-512");
        } catch (NoSuchAlgorithmException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] digestOfPassword;

        try {
            digestOfPassword = digest.digest("HG58YZ3CR9".getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] keyBytes = Arrays.copyOf(digestOfPassword, 16);
        int index = 8;
        int jindex = 0;
        while (jindex < 8) {
            keyBytes[index++] = keyBytes[jindex++];
        }

        final SecretKey key = new SecretKeySpec(keyBytes, "AES");
        final IvParameterSpec ivspec = new IvParameterSpec(new byte[16]);
        final Cipher decipher;

        try {
            decipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
        } catch (NoSuchAlgorithmException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (NoSuchPaddingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            decipher.init(Cipher.DECRYPT_MODE, key, ivspec);
        } catch (InvalidKeyException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (InvalidAlgorithmParameterException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        // final byte[] encData = new
        // sun.misc.BASE64Decoder().decodeBuffer(message);
        final byte[] plainText;

        try {
            plainText = decipher.doFinal(message);
        } catch (IllegalBlockSizeException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        } catch (BadPaddingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            rtc = new String(plainText, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getMessage(), e);
            return null;
        }

        return rtc;
    }

}
