/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.foundations.utilities;

import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class Messages
{
    private static final Logger logger = Logger.getLogger(Messages.class);

    /** Message properties **/
    private static Properties messages = new Properties();

    /** Gets Message **/
    public static String get(String msgProp)
    {
        if (messages.isEmpty())
        {
            init();
        }

        return messages.getProperty(msgProp);

    }

    /**
     * Initialize Messages object.
     */
    private static void init()
    {
        InputStream resStream = null;

        try {
            ClassLoader msgClassLoader = Messages.class.getClassLoader();
            if (msgClassLoader != null)
            {
                resStream = msgClassLoader.getResourceAsStream("messages.properties");
                messages.load(resStream);
                resStream.close();
            }
            else throw new RuntimeException("Error initializing Messages class: null classLoader");

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        finally {
            if (resStream != null)
            {
                try {
                    resStream.close();
                } catch (IOException e) {
                    logger.error("Could not close input stream.");
                }
            }
        }
    }
}
