//= require ./button.js
//= require ./watermark.js

//editable textfield with drop down menu


$.widget("ui.combobox", {

    options: {
        radio: false,
        placeholder: null,
        enabled: true
    },
    _create: function () {

        this.element.addClass("ui-combobox")
            .data('ui.combobox', true);

        this._selectedValue = null;
        this._selected = -1;

        this._createInput();
        this._createMenu();
        this._createMenuButton();
    },

    _createInput: function () {

        this.input = $("<input type='text'>")
            .appendTo(this.element)
            .attr({
                title: '',
                'aria-label': this.element.attr('aria-label'),
                'role': 'combobox',
                'aria-haspopup': 'true'
            })
            .addClass("ui-combobox-input ui-widget ui-widget-content ui-state-default ui-corner-left");

        var self = this;

        this.input.on('keypress', function (evt) {

            var letter = String.fromCharCode(evt.which);
            if (letter.match(/[a-z]|[0-9]/i)) {
                if (self._menu) {

                    var idx = self._menu.menu("itemThatBeginsWithString", letter);
                    if (idx > -1) {
                        self.openMenu();
                        self._menu.menu('highlight', idx);

                    }
                }
            }
        });


        this.input.on('keydown', function (evt) {

            if (!self.options.enabled)
                return;

            if (evt.keyCode === $.ui.keyCode.DOWN) {
                evt.preventDefault();
                evt.stopPropagation();

                if (self._menu.is(":visible")) {
                    self._menu.focus();
                    self._menu.menu("highlight", 0);
                }
                else {
                    self.openMenu();
                }
            }
            else if (evt.keyCode === $.ui.keyCode.UP) {
                evt.preventDefault();
                evt.stopPropagation();

                if (self._menu.is(":visible")) {
                    self._menu.focus();
                    self._menu.menu("highlight", self._menu.children('.ui-menu-item').length - 1);
                }
                else {
                    self.openMenu();
                }
            }
        });
    },
    setMenu: function (aMenu) {

        if (this._menu) {
            this._menu.remove();
        }

        this._menu = null;

        if (aMenu) {
            this._menu = aMenu;
            this._menu.addClass("ui-combobox-menu");
            var self = this;
            this._menu.on("selection", function (event, menuItem) {
                self.selected($(this).children('.ui-menu-item').index(menuItem.element), false);
            });
        }

    },
    _createMenu: function () {

        var m = this.element.children(".appkit-menu");
        if (m.length == 0) {
            m = this.element.data("menu");
        }
        else {
            m = m.first();
        }
        if (m) {

            this.setMenu(m);
        }
    },
    selected: function (idx, silent) {
        if (idx !== undefined && this._menu) {
            var menuItems = this._menu.children(".ui-menu-item");

            if (idx < 0) {
                this._selected = idx;
                this._selectedValue = null;
                this.input.val("");
                return this._selected;
            }

            if (idx > -1 && idx < menuItems.length) {
                var menuItem = menuItems.get(idx);
                if (menuItem) {
                    $(menuItems).menuitem("checked", false);
                    if (this.options.radio) {
                        $(menuItem).menuitem("checked", true);
                    }
                    this._selected = idx;
                    this._selectedValue = $(menuItem).data("value");
                    if (!this._selectedValue) {
                        this._selectedValue = $(menuItem).text();
                    }

                    var label = $(menuItem).data("label");
                    if (!label) {
                        label = this._selectedValue;
                    }

                    this.input.val(label);

                    //fix strange IE 11 bug
                    if (this._selectedValue && this._selectedValue.replace) {
                        this._selectedValue = this._selectedValue.replace(String.fromCharCode(62391), "");
                    }

                    if (silent !== true) {
                        this.element.trigger("selection", [this._selected, this._selectedValue]);
                    }
                }

            }
        }

        return this._selected;
    },
    selectedValue: function (aString, silent) {

        if (aString !== undefined) {

            if (aString == null) {
                this.selected(-1, false);
                return null;
            }

            var menuItems = this._menu.children(".ui-menu-item"),
                count = menuItems.length,
                i = 0, found = false;

            for (; i < count; i++) {
                var menuItem = menuItems.get(i);

                var menuValue = $(menuItem).data('value');
                if (!menuValue) {
                    menuValue = $(menuItem).data('label');
                }

                if (!menuValue) {
                    menuValue = $(menuItem).text();
                }

                if (menuValue === aString) {
                    found = true;
                    this.selected(i, silent);
                    break;
                }
            }

            if (found == false) {
                this.selected(-1, silent);
                return null;
            }
        }

        return this._selectedValue;
    },
    _createMenuButton: function () {

        var self = this,
            wasOpen = false;

        $("<div>")
            .html("<div class='icon-combo-toggle'></div>")
            .attr("tabIndex", -1)
            .attr("title", "Show Menu")
            .appendTo(this.element)
            .button({
                text: false
            })
            .removeClass("ui-corner-all")
            .addClass("ui-combobox-toggle")
            .mousedown(function (evt) {
                if (self._menu) {
                    wasOpen = self._menu.is(":visible");

                    self._menu.width(self.element.width());
                    var offset = self.element.offset();

                    self._menu.css({
                        left: offset.left,
                        top: offset.top + self.element.height() - 2
                    });

                    var ymax = offset.top + self.element.height() - 2 + self._menu.height();
                    if (ymax >= $(window).height()) {
                        self._menu.css({
                            top: offset.top - self._menu.outerHeight() - 3
                        });
                    }
                }

            })
            .click(function () {

                if (!self.options.enabled)
                    return;

                self.input.focus();
                if (self._menu) {
                    if (wasOpen)
                        self._menu.menu("close");
                    else
                        self._menu.menu("open");
                }


            });
    },
    openMenu: function () {
        if (this._menu) {
            this._menu.outerWidth(this.element.width());
            var offset = this.element.offset();
            this._menu.css({
                left: offset.left,
                top: offset.top + this.element.height() - 2
            });

            var ymax = offset.top + this.element.height() - 2 + this._menu.height();
            if (ymax >= $(window).height()) {
                this._menu.css({
                    top: offset.top - this._menu.outerHeight() - 3
                });
            }

            this._menu.menu("open");
        }
    },
    _setOption: function (key, value) {
        if (key === "placeholder") {
            if (value) {
                this.input.watermark(value);
            }
            else
                $.watermark.hide(this.input);
        }

        if (key === "enabled") {
            if (value) {
                this.element.removeClass("disabled");
                this.input.removeClass("disabled");
                this.input.removeAttr("readonly");
                this.input.attr('tabindex', 0);
            }
            else {
                this.element.addClass("disabled");
                this.input.addClass("disabled");
                this.input.attr("readonly", "true");
                this.input.attr('tabindex', -1);
            }
        }

        this._super(key, value);
    }

});