//= require ./Dialog.js
//= require ./Button.js

AppKit.AlertDialog = AppKit.Dialog.extend({
    classID: 'appkit-alert',
    options: {
        width: 500,
        height: 200,
        resizable: false,
        confirmButton: true,
        cancelButton: true,
        thirdButton: false,
        confirmLabel: 'OK',
        cancelLabel: 'Cancel',
        thirdLabel: '',
        modal: true,
        message: "",
        destroyOnClose: true,
        icon: undefined,
        callback: null
    },
    initialize: function (options) {

        var self = this;
        this._ok = false;
        this._confirmBtn = new AppKit.Button({
            type: 'primary',
            className: 'alert-confirm',
            action: function () {
                self._ok = true;
                self._third = false;
                self.set('visible', false);

            }
        });

        this._cancelBtn = new AppKit.Button({
            className: 'alert-cancel',
            action: function () {
                self._ok = false;
                self._third = false;
                self.set('visible', false);
            }
        });


        this._thirdBtn = new AppKit.Button({
            className: 'alert-third',
            type: 'primary',
            action: function () {
                self._ok = false;
                self._third = true;
                self.set('visible', false);
            }
        });

        var bbar = $('<div>').addClass('button-bar').appendTo(this.$el);

        this._cancelBtn.$el.prependTo(bbar);
        this._confirmBtn.$el.prependTo(bbar);
        this._thirdBtn.$el.prependTo(bbar);


        this._msg = $('<div>').addClass('ui-alert-msg').appendTo(this.$el);
        this._icon = $('<div>').addClass('ui-alert-icon').appendTo(this.$el);

        this.$el.on('dialogclose', function () {
            if (_.isFunction(self.callback)) {
                self.callback.call(self, self._ok, self._third);
            }
            if (self.destroyOnClose) {
                $(this).remove();
            }
        });

        this._super(options);

        this._cancelBtn.$el.attr({
            'aria-label': this.get('cancelLabel'),
            title: this.get('cancelLabel')

        });

        this._confirmBtn.$el.attr({
            'aria-label': this.get('confirmLabel'),
            title: this.get('confirmLabel')

        });

        this.$el.addClass('ui-alert');
        this.$el.parent().attr({
            'role': 'alertdialog'
        });
    },
    confirmButton: function (aBOOL) {
        if (aBOOL === true) {
            this._confirmBtn.set('hidden', false);
        }
        else if (aBOOL === false) {
            this._confirmBtn.set('hidden', true);
        }

        return !this._confirmBtn.get('hidden');

    },
    confirmLabel: function (aString) {
        if (aString !== undefined) {
            this._confirmBtn.set('label', aString);
        }

        return this._confirmBtn.get('label');
    },
    cancelButton: function (aBOOL) {
        if (aBOOL === true) {
            this._cancelBtn.set('hidden', false);
        }
        else if (aBOOL === false) {
            this._cancelBtn.set('hidden', true);
        }

        return !this._cancelBtn.get('hidden');
    },
    cancelLabel: function (aString) {
        if (aString !== undefined) {
            this._cancelBtn.set('label', aString);
        }

        return this._cancelBtn.get('label');

    },
    thirdButton: function (aBOOL) {
        if (aBOOL === true) {
            this._thirdBtn.set('hidden', false);
        }
        else if (aBOOL === false) {
            this._thirdBtn.set('hidden', true);
        }

        return !this._thirdBtn.get('hidden');
    },
    thirdLabel: function (aString) {
        if (aString !== undefined) {
            this._thirdBtn.set('label', aString);
        }

        return this._thirdBtn.get('label');

    },
    icon: function (aString) {
        if (aString !== undefined) {
            this._icon.addClass(aString);
            this._iconClass = aString;
        }

        return this._iconClass;
    },
    message: function (aString) {
        if (aString !== undefined) {
            this._msg.html(aString);
        }

        return this._msg.text();
    }
});


AppKit.AlertDialog.alert = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-warning',
        cancelButton: false,
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();

};

AppKit.AlertDialog.confirm = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-confirm',
        cancelButton: true,
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();

};

AppKit.AlertDialog.yesNo = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-confirm',
        cancelButton: true,
        cancelLabel: 'NO',
        confirmLabel: 'YES',
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();
};