//= require ./Control.js
//= require ../jquery/multiselect.js


AppKit.MultiSelect = AppKit.Control.extend({
    classID: 'appkit-multiselect',
    options: {
        title: '',
        tabindex: 0,
        items: [],
        action: undefined,
    },
    initialize: function (options) {

        this.$el.multiselect({
            height: 225,
            header: true,
            selectedText: _.bind(function (numChecked, numTotal, checkedItems) {
                return this.get('title') + ' (' + numChecked + ' selected)';
            }, this),
            click: _.bind(function () {
                var action = this.get('action');
                if (_.isFunction(action))
                    action.call(this);
            }, this),
            clearAll: _.bind(function () {
                var action = this.get('action');
                if (_.isFunction(action))
                    action.call(this);

                this.$el.multiselect('close');
            }, this),
            beforeopen: _.bind(function () {

                var $menu = this.$el.multiselect('widget');
                $menu.width(250);

            }, this)


        });

        this._super(options);
    },

    width: function (aNumber) {
        if (aNumber !== undefined) {
            this.$el.multiselect('option', 'minWidth', aNumber);
        }

        return this.$el.multiselect('option', 'minWidth');
    },

    title: function (aString) {
        if (aString !== undefined) {
            this.$el.multiselect('option', 'noneSelectedText', aString);
        }

        return this.$el.multiselect('option', 'noneSelectedText');
    },

    items: function (anArray) {
        if (anArray !== undefined) {

            this._items = anArray;

            this.$el.empty();
            var count = anArray.length,
                i = 0;

            for (; i < count; i++) {
                $('<option>').data(this._items[i]).text(this._items[i]['data-label'])
                    .appendTo(this.$el);
            }

            this.$el.multiselect('refresh');
            this.$el.multiselect('uncheckAll');

        }

        return this._items;


    },

    selected: function (value) {

        if (_.isString(value)) {
            this.uncheckAll();
            var checkbox = this.$el.multiselect('widget').find(':checkbox[value="' + value + '"]');
            if (checkbox) {
                checkbox.click();
            }

        }

        return this.$el.multiselect("getChecked").map(function () {
            return this.value;
        }).get();
    },

    selectedIds: function (value) {

        if (_.isString(value)) {
            this.uncheckAll();
            var checkbox = this.$el.multiselect('widget').find(':checkbox[value="' + value + '"]');
            if (checkbox) {
                checkbox.click();
            }

        }

        return this.$el.multiselect("getChecked").map(function () {
            if (this.value) {
                return this.value.hashCode();
            }
        }).get();
    },


    uncheckAll: function () {
        this.$el.multiselect('uncheckAll');
    },

    checkAll: function () {
        this.$el.multiselect('checkAll');
    }


});