//= require ./Control.js
//= require ../jquery/tooltip.js

AppKit.TextField = AppKit.Control.extend({
    classID: 'appkit-textfield',
    initialize: function (options) {
        var self = this;
        this.$el.bind('focus change keydown paste', function () {
            AppKit.Form.clearValidationWarnings();
        }).on('cut change keydown paste', function () {
            setTimeout(function () {
                self.$el.trigger("valueChanged", [self.get('value')]);
                if (self._delegate) {
                    if (_.isFunction(self._delegate.textDidChange)) {
                        self._delegate.textDidChange.call(self._delegate, self);
                    }
                }
            }, 0);

        }).on('mousedown', function (evt) {
            if (!self.get('enabled')) {
                evt.preventDefault();
                evt.stopPropagation();
            }
        });

        this._super(options);
    },
    enabled: function (aBool) {
        if (aBool == true) {
            this.$el.removeClass('disabled');
            this.$el.attr('tabindex', 0);
            this.$el.removeAttr('readonly');
        }
        else if (aBool == false) {
            this.$el.addClass('disabled');
            this.$el.attr('tabindex', -1);
            this.$el.attr('readonly', 'true');
        }
        return !this.$el.hasClass('disabled');
    },
    clearValidationWarning: function () {
        if (this.$el.hasClass('validation-error')) {
            this.$el.removeClass('validation-error');
            this.$el.tooltip('destroy');
            this.$el.removeAttr('title');
        }
    },
    validate: function (aValidator) {
        if (!_.isFunction(aValidator))
            return true;

        var message = aValidator(this.value());

        if (message) {
            if (this.$el.hasClass('validation-error'))
                this.$el.tooltip('destroy');

            this.$el.tooltip({
                content: message,
                items: 'input',
                show: {
                    duration: 0,
                    easing: false,
                    effect: null
                },
                position: {
                    my: "left center",
                    at: "right+10 center",
                    collision: "flipfit",
                    using: function (position) {
                        $(this).css(position);
                        $('<div>')
                            .addClass("arrow").appendTo(this);
                    }
                },
                tooltipClass: "right"
            });


            this.$el.addClass('validation-error');
            this.$el.tooltip('open');


            return false;
        }

        return true;
    },
    placeholder: function (aString) {
        if (aString !== undefined) {
            this.$el.watermark(aString);
        }

        return this.$el.watermark();
    },
    value: function (aValue) {
        if (aValue !== undefined) {
            this.$el.val(aValue);
            this.clearValidationWarning();
        }

        return this.$el.val();
    }
});


