//= require lib/AppKit/AppKit.js


var JLV = {
    common: {
        /** constants **/
        DODADAPTER_STATUS_REPORT_FLAG: "DODADAPTER_SOURCE_STATUS_REPORT",
        DODADAPTER_STATUS_NO_REPORT_MSG: "",
        DIAGNOSIS_UNAVAILABLE_TXT: "No diagnosis available.",
        SENSITIVE_NOTE_MSG: "This is a Sensitive note and can not be displayed in JLV.<br/><br/>Please contact the local facility to obtain this note information.",
        SENSITIVE_ENCOUNTER_MSG: "This is a Sensitive encounter and can not be displayed in JLV.<br/><br/>Please contact the local facility to obtain this encounter information.",
        NOTE_TITLE_UNAVAILABLE_TXT: "Click to view note",
        CODING_SYSTEM_CPT: 'CPT',
        CODING_SYSTEM_CPT_DISPLAY: '<b>Current Procedural Terminology (CPT)</b>',
        CODING_SYSTEM_UMLS: 'UMLS',
        CODING_SYSTEM_UMLS_DISPLAY: 'Unified Medical<br/>Language System (UMLS)',
        CODING_SYSTEM_CVX: 'CVX',
        CODING_SYSTEM_CVX_DISPLAY: '<b>National Vaccine<br/>Administered Code<br/> (CVX)</b>',
        CODING_SYSTEM_RXNORM: 'RXNORM',
        CODING_SYSTEM_RXNORM_DISPLAY: '<b>National Library of Medicine<br/>RxNorm Concept Unique Identifiers</b>',
        CODING_SYSTEM_SNOMED: 'SNOMED',
        CODING_SYSTEM_SNOMED_DISPLAY: '<b>Systematized Nomenclature<br/>of Medicine - Clinical Terms<br/>(SNOMED CT)</b>',
        CODING_SYSTEM_LOINC: 'LOINC',
        CODING_SYSTEM_LOINC_DISPLAY: '<b>Logical Observation Identifiers<br/>Names and Codes<br/>(LOINC)</b>',
        CODING_SYSTEM_LABS: "LABS",
        CODING_SYSTEM_NOTES: "NOTES",
        CODING_SYSTEM_VITALS: "VITALS",
        CONNECTION_UNAVAILABLE: 'Connection unavailable',
        DOD_VLER_LOADING_MSG: "Community partner data retrieval in progress...",
        DOD_VLER_AVAILABLE_MSG: "Community partner data may be available for this widget.",
        DOD_VLER_DATA_IN_WIDGET_MSG: "Additional data is available in Community Health Summaries",
        DOD_VLER_NOT_AVAILABLE_MSG: "Connection unavailable. Community partner data is not available to display.",
        DOD_VLER_STANDARD_MSG: "Additional external partner data is available by accessing the narrative view in this widget. The content and format of this information is displayed as provided by the external partner(s).",
        BROWSER_WINDOWS: [],
        VISTA_IMAGING_ERROR_MSG: "An error occurred while attempting to retrieve Vista Imaging Viewer URL",
        /** Hold browser tab/window objects for closing on patient switch, inactivity timeout, and user logout **/
        closeAuxBrowserWindows: function () {
            if (JLV.common.BROWSER_WINDOWS) {
                for (var i in JLV.common.BROWSER_WINDOWS) {
                    if (JLV.common.BROWSER_WINDOWS[i]) {
                        JLV.common.BROWSER_WINDOWS[i].close();
                    }
                }
                JLV.common.BROWSER_WINDOWS.length = 0;
            }
        },
        /** common utility functions **/
        openAuxBrowserWindow: function (URL, windowTitle) {
            //always open a unique window
            var windowObj = window.open(URL, '_blank');

            // add tab/window object to array
            if (windowObj) {
                JLV.common.BROWSER_WINDOWS.push(windowObj);
            }
        },
        openRadVixViewer: function (URL, recToken) {
            $.post('patientRadiologyExams/auditVixViewer',
                {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: recToken
                }
            );
            JLV.common.openAuxBrowserWindow(URL, 'JLV Vista Imaging');
        },
        getUrlAndOpenRadVixViewer: function (siteCode, hasImage, caseNumber, examId, facilityName, recToken, imageId, spinnerId) {
            $('#' + imageId).hide();
            $('#' + spinnerId).show();

            $.post('patientRadiologyExams/getRadiologyVixStudyViewerUrl', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                siteCode: siteCode,
                hasImage: hasImage,
                caseNumber: caseNumber,
                examId: examId,
                facilityName: facilityName
            }, function (response) {
                $.post('patientRadiologyExams/auditVixViewer',
                    {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: recToken
                    }
                );

                $('#' + imageId).show();
                $('#' + spinnerId).hide();

                if (response.vixStudyViewerUrl) {
                    JLV.common.openAuxBrowserWindow(response.vixStudyViewerUrl, 'JLV Vista Imaging');
                } else {
                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            }).fail(function (failed) {
                    $('#' + imageId).show();
                    $('#' + spinnerId).hide();

                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            );
        },
        openDocVixViewer: function (URL, recToken) {
            $.post('patientDocuments/auditVixViewer',
                {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: recToken
                }
            );
            JLV.common.openAuxBrowserWindow(URL, 'JLV Vista Imaging');
        },
        getUrlAndOpenDocVixViewer: function (siteCode, noteId, images, recToken, imageId, spinnerId) {
            $('#' + imageId).hide();
            $('#' + spinnerId).show();
            $.post('patientDocuments/getDocumentVixStudyViewerUrl', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                siteCode: siteCode,
                noteId: noteId,
                images: images
            }, function (response) {
                $.post('patientDocuments/auditVixViewer',
                    {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: recToken
                    }
                );

                $('#' + imageId).show();
                $('#' + spinnerId).hide();

                if (response.vixStudyViewerUrl) {
                    JLV.common.openAuxBrowserWindow(response.vixStudyViewerUrl, 'JLV Vista Imaging');
                } else {
                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            }).fail(function (failed) {
                    $('#' + imageId).show();
                    $('#' + spinnerId).hide();

                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            );
        },
        getUrlAndOpenProgressNoteVixViewer: function (siteCode, noteId, images, recToken, imageId, spinnerId, controllerName) {
            $('#' + imageId).hide();
            $('#' + spinnerId).show();
            $.post('patientDocuments/getDocumentVixStudyViewerUrl', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                siteCode: siteCode,
                noteId: noteId,
                images: images
            }, function (response) {
                $.post('patientProgressNotes/auditVixViewer',
                    {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: recToken
                    }
                );

                $('#' + imageId).show();
                $('#' + spinnerId).hide();

                if (response.vixStudyViewerUrl) {
                    JLV.common.openAuxBrowserWindow(response.vixStudyViewerUrl, 'JLV Vista Imaging');
                } else {
                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            }).fail(function (failed) {
                    $('#' + imageId).show();
                    $('#' + spinnerId).hide();

                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            );
        },
        openEncounterVixViewer: function (URL, recToken) {
            $.post('patientEncounters/auditVixViewer',
                {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: recToken
                }
            );
            JLV.common.openAuxBrowserWindow(URL, 'JLV Vista Imaging');
        },
        getUrlAndOpenEncounterVixViewer: function (documentId, documentSiteId, recToken, imageId, spinnerId) {
            $('#' + imageId).hide();
            $('#' + spinnerId).show();

            $.post('patientEncounters/getEncounterVixStudyViewerUrl', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                documentId: documentId,
                documentSiteId: documentSiteId
            }, function (response) {
                $.post('patientEncounters/auditVixViewer',
                    {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: recToken
                    }
                );

                $('#' + imageId).show();
                $('#' + spinnerId).hide();

                if (response.vixStudyViewerUrl) {
                    JLV.common.openAuxBrowserWindow(response.vixStudyViewerUrl, 'JLV Vista Imaging');
                } else {
                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            }).fail(function (failed) {
                    $('#' + imageId).show();
                    $('#' + spinnerId).hide();

                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            );
        },
        openProcedureVixViewer: function (URL, recToken) {
            $.post('patientProcedures/auditVixViewer',
                {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: recToken
                }
            );
            JLV.common.openAuxBrowserWindow(URL, 'JLV Vista Imaging');
        },
        getUrlAndOpenProcedureVixViewer: function (documentId, documentSiteId, recToken, imageId, spinnerId) {
            $('#' + imageId).hide();
            $('#' + spinnerId).show();

            $.post('patientProcedures/getProcedureVixStudyViewerUrl', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                documentId: documentId,
                documentSiteId: documentSiteId
            }, function (response) {
                $.post('patientProcedures/auditVixViewer',
                    {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: recToken
                    }
                );

                $('#' + imageId).show();
                $('#' + spinnerId).hide();

                if (response.vixStudyViewerUrl) {
                    JLV.common.openAuxBrowserWindow(response.vixStudyViewerUrl, 'JLV Vista Imaging');
                } else {
                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            }).fail(function (failed) {
                    $('#' + imageId).show();
                    $('#' + spinnerId).hide();

                    AppKit.AlertDialog.alert('Warning', JLV.common.VISTA_IMAGING_ERROR_MSG);
                }
            );
        },
        copyWindowHTML: function (theHTML) {
            if (window.clipboardData && window.clipboardData.setData) {

                theHTML = theHTML.replace(/\r\n/gi, "");


                var dataHTML = "";
                dataHTML += "For Official Use Only\r\n<br/>";
                dataHTML += theHTML;
                //remove linebreaks

                //remove <div>
                dataHTML = dataHTML.replace(/<div(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/div\>/gi, "");
                dataHTML = dataHTML.replace(/<content(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/content\>/gi, "");
                dataHTML = dataHTML.replace(/<pre(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/pre\>/gi, "");
                dataHTML = dataHTML.replace(/\<span(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/span\>/gi, "");
                //strip table tags
                dataHTML = dataHTML.replace(/\<table(.|[\r\n])*?\>/gi, "\r\n");
                dataHTML = dataHTML.replace(/\<\/table\>/gi, "");
                dataHTML = dataHTML.replace(/\<thead(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/thead\>/gi, "");
                dataHTML = dataHTML.replace(/\<th(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/th\>/gi, "\t");
                dataHTML = dataHTML.replace(/\<tbody(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/tbody\>/gi, "");
                dataHTML = dataHTML.replace(/\<tr(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/tr\>/gi, "\r\n");
                dataHTML = dataHTML.replace(/\<td(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/td\>/gi, "\t");
                //strip anchors <a>
                dataHTML = dataHTML.replace(/\<a(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/a\>/gi, "");
                //strip images <img>
                dataHTML = dataHTML.replace(/\<img(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/img\>/gi, "");
                //strip &nbsp;
                dataHTML = dataHTML.replace(/\&nbsp\;/gi, " ");
                //strip <br>
                dataHTML = dataHTML.replace(/\<br\>/gi, "\r\n");
                dataHTML = dataHTML.replace(/\<br\/\>/gi, "\r\n");
                //strip <b>
                dataHTML = dataHTML.replace(/\<b\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/b\>/gi, "");
                //strip <strong>
                dataHTML = dataHTML.replace(/\<strong\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/strong\>/gi, "");
                //strip </n>
                dataHTML = dataHTML.replace(/\<\/n\>/gi, "");

                clipboardData.setData('text', _.unescape(dataHTML));
            }
            else {
                console.info("Clipboard only supported in IE8+.");
            }
        },

        copyGridPanelHTML: function (theHTML) {
            if (window.clipboardData && window.clipboardData.setData) {
                var dataHTML = "";

                //remove linebreaks
                theHTML = theHTML.replace(/\r\n/gi, "");

                dataHTML += "For Official Use Only\r\n<br/>";

                dataHTML += theHTML;

                dataHTML = dataHTML.replace(/\<\/div><th(.|[\r\n])+?\>/gi, "\t");

                //remove <div>
                dataHTML = dataHTML.replace(/\<div(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/div\>/gi, "");
                dataHTML = dataHTML.replace(/<content(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/content\>/gi, "");
                dataHTML = dataHTML.replace(/\<pre(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/pre\>/gi, "");
                dataHTML = dataHTML.replace(/\<span(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/span\>/gi, "");
                //strip table tags
                dataHTML = dataHTML.replace(/\<table(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/table\>/gi, "");
                dataHTML = dataHTML.replace(/\<thead(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/thead\>/gi, "");
                dataHTML = dataHTML.replace(/\<tbody(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/tbody\>/gi, "");
                dataHTML = dataHTML.replace(/\<tr(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/tr\>/gi, "\r\n");
                dataHTML = dataHTML.replace(/\<td(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/td\>/gi, "\t");
                dataHTML = dataHTML.replace(/\<th(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/th\>/gi, "");

                //strip anchors <a>
                dataHTML = dataHTML.replace(/\<a(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/a\>/gi, "");
                //strip images <img>
                dataHTML = dataHTML.replace(/\<img(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/img\>/gi, "");
                //strip <ABBR>
                dataHTML = dataHTML.replace(/\<ABBR(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/ABBR\>/gi, "");
                //strip <CAPTION>
                dataHTML = dataHTML.replace(/\<CAPTION(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/CAPTION\>/gi, "\r\n");
                //strip <BUTTON>
                dataHTML = dataHTML.replace(/\<BUTTON(.|[\r\n])+?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/BUTTON\>/gi, "");
                //strip <LABEL>xxx</LABEL>
                dataHTML = dataHTML.replace(/\<LABEL(.|[\r\n])+?(<\/LABEL>)/gi, "");
                //strip &nbsp;
                dataHTML = dataHTML.replace(/\&nbsp\;/gi, " ");
                //strip <br>
                dataHTML = dataHTML.replace(/\<br\>/gi, "");
                dataHTML = dataHTML.replace(/\<br\/\>/gi, "");
                //strip <b>
                dataHTML = dataHTML.replace(/\<b(.|[\r\n])*?\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/b\>/gi, "");
                //strip <strong>
                dataHTML = dataHTML.replace(/\<strong\>/gi, "");
                dataHTML = dataHTML.replace(/\<\/strong\>/gi, "");
                //strip </n>
                dataHTML = dataHTML.replace(/\<\/n\>/gi, "");

                window.clipboardData.setData('text', _.unescape(dataHTML));
            }
            else {
                console.info("Clipboard only supported in IE8+.");
            }

        },
        dateFormatter: function (aDate, model, dateFormat) {

            if (!_.isString(dateFormat)) {
                dateFormat = 'MMM DD, YYYY';
            }

            if (!aDate)
                return '';

            if (_.isString(aDate)) {
                aDate = new Date(aDate);
            }

            var str = moment(aDate).format(dateFormat);
            if (str === "Invalid date") {
                return "Unknown";
            }
            return str;
        },
        siteSorter: function (model1, model2) {

            var site1 = model1.get('site'),
                site2 = model2.get('site');


            var name1 = site1.name,
                name2 = site2.name;

            if (site1.agency == JLV.common.AGENCY_DOD) {
                name1 = 'DoD';
            }

            if (site2.agency == JLV.common.AGENCY_DOD) {
                name2 = 'DoD';
            }

            return name1 > name2 ? 1 : -1;
        },
        linkTextSorter: function (model1, model2, sortInfo) {

            var mId = sortInfo.column.id;

            var val1 = model1.get(mId),
                val2 = model2.get(mId);

            if (_.isFunction(sortInfo.column.formatter)) {
                val1 = _.text(sortInfo.column.formatter(val1, model1));
                val2 = _.text(sortInfo.column.formatter(val2, model2));
            }

            if (val1 === null || val1 === undefined) {
                val1 = "";
            }

            if (val2 === null || val2 === undefined) {
                val2 = "";
            }

            val1 = new String(val1).toUpperCase().trim();
            val2 = new String(val2).toUpperCase().trim();

            if (val1 == val2) {
                return 0;
            }

            return val1 > val2 ? 1 : -1;

        },
        contentSorter: function (model1, model2, sortInfo) {

            var mId = sortInfo.column.id;

            var val1 = model1.get(mId),
                val2 = model2.get(mId);

            if (_.isFunction(sortInfo.column.formatter)) {
                val1 = sortInfo.column.formatter(val1, model1)
                val2 = sortInfo.column.formatter(val2, model2)
            }

            if (val1 === null || val1 === undefined) {
                val1 = "";
            }

            if (val2 === null || val2 === undefined) {
                val2 = "";
            }

            return val1.length > val2.length ? 1 : -1;

        },
        dateSortWithBlanksFirst: function (model1, model2, sortInfo) {
            try {
                var mId = sortInfo.column.id;

                var val1 = model1.get(mId),
                    val2 = model2.get(mId);

                if (val1 === null || val1 === undefined) {
                    return 1;
                }
                else if (val2 === null || val2 === undefined) {
                    return -1;
                }

                if (val1 == val2) {
                    return 0;
                }

                var val1d = new Date(val1),
                    val2d = new Date(val2);

                return val1d > val2d ? 1 : -1;
            }
            catch (e) {
                return 1;
            }
        },
        formatSiteDisplay: function (site) {

            if (!site || !site.agency) return "No site agency";

            var imgTag = "No site agency marker";

            if (site.agency == JLV.common.AGENCY_VA) {
                imgTag = String.format("VA: {0}", site.name);
            }
            else if (site.agency == JLV.common.AGENCY_DOD_VLER) {
                imgTag = String.format("DoD: {0}", site.moniker);
            }
            else if (site.agency == JLV.common.AGENCY_DAS_VLER) {
                imgTag = String.format("VA DAS: {0}", site.moniker);
            }
            else if (site.agency == JLV.common.AGENCY_DOD) {
                imgTag = String.format("DoD: {0}", site.moniker);
            }

            return imgTag;
        },
        formatSiteIconDisplay: function (site) {

            if (!site || !site.agency) return 'No site agency';

            var divTag = 'No site agency marker';

            if (site.agency === JLV.common.AGENCY_VA) {
                divTag = '<div class="vaMarker"  alt="VA Marker"></div>';
            }
            else if (site.agency === JLV.common.AGENCY_DOD_VLER || site.name === JLV.common.AGENCY_DOD_VLER) {
                divTag = '<div class="dodVlerMarker"  alt="DoD VLER Marker"></div>';
            }
            else if (site.agency === JLV.common.AGENCY_DAS_VLER || site.name === JLV.common.AGENCY_DAS_VLER) {
                divTag = '<div class="dasVlerMarker"  alt="VA VLER Marker"></div>';
            }
            else if (site.agency === JLV.common.AGENCY_DOD) {
                divTag = '<div class="dodMarker"  alt="DoD Marker"></div>';
            }


            return divTag;
        },
        formatSiteMonikerDisplay: function (site) {

            if (!site || !site.agency) return "No site agency";

            var imgTag = "NA";

            if (site.agency == JLV.common.AGENCY_VA) {
                imgTag = String.format('<div class="vaMarker"  alt="VA Marker" title="VA: {0}"></div>&nbsp;<span title="VA: {0}">{1}</span>', site.name, site.moniker);
            }
            else if (site.agency === JLV.common.AGENCY_DOD_VLER) {
                imgTag = String.format('<div class="dodVlerMarker" alt="DoD VLER Marker" title="DoD VLER: {1}"></div>&nbsp;<span title="DoD VLER: {1}">{0}</span>', site.name, site.moniker);
            }
            else if (site.agency === JLV.common.AGENCY_DAS_VLER) {
                imgTag = String.format('<div class="dasVlerMarker" alt="VA VLER Marker" title="VA VLER: {1}"></div>&nbsp;<span title="VA VLER: {1}">{0}</span>', site.name, site.moniker);
            }
            else if (site.agency == JLV.common.AGENCY_DOD) {
                imgTag = String.format('<div class="dodMarker" alt="DoD Marker" title="{0}: {1}"></div>&nbsp;<span title="{0}: {1}">DoD</span>', site.name, site.moniker);
            }

            return imgTag;

        },
        formatMonikerDisplay: function (site) {
            if (!site) return "No site agency";
            return JLV.common.formatSiteIconDisplay(site) + "&nbsp;" + site.moniker;
        },
        formatVLERMonikerDisplay: function (siteName) {
            return String.format('<div style="padding:2px;"><div class="vlerMarker" alt="Community Health Partner Marker" title="Community Health Partner: {0}"></div>&nbsp;<span title="Community Health Partner: {0}">{0}</span></div>', siteName);
        },
        linkFormatter: function (value, model, tooltip, route) {
            if (!route) {
                route = "";
            }

            var cid = _.uniqueId();
            if (model) {
                cid = model.cid;
            }

            if (value && value.indexOf(JLV.common.CONNECTION_UNAVAILABLE) > -1) {
                return value;
            }

            if (tooltip) {
                return value ? String.format('<a href="#" title="{0}" data-model-id="{1}" data-route="{2}" class="jlv-clickable-link">{0}</a>', value, cid, route) : '';
            }
            else {
                return value ? String.format('<a href="#" data-model-id="{1}" data-route="{2}" class="jlv-clickable-link">{0}</a>', value, cid, route) : '';
            }
        },
        linkDateFormatter: function (value, model, format) {
            return JLV.common.linkFormatter(JLV.common.dateFormatter(value, model, format), model);
        },
        createUTCDate: function (dateStr) {
            var resultDate = new Date(dateStr);
            var UTCResultDate = new Date();
            UTCResultDate.setUTCFullYear(resultDate.getFullYear());
            UTCResultDate.setUTCMonth(resultDate.getMonth());
            UTCResultDate.setUTCDate(resultDate.getDate());
            UTCResultDate.setUTCHours(resultDate.getHours());
            UTCResultDate.setUTCMinutes(resultDate.getMinutes());
            UTCResultDate.setUTCSeconds(resultDate.getSeconds(), 0);

            return UTCResultDate;
        },
        isDoDProvider: function (user) {
            return user.agency !== "VA";
        },
        canAddTypeToReportBuilder: function (mimeType) {
            return mimeType === 'text/xml' || mimeType === 'application/rtf'
                || mimeType === 'application/pdf' || mimeType === 'application/octet-stream';
        },
        getColumnIndex: function (columns, title) {
            for (var i = 0; i < columns.length; i++) {
                if (columns[i].title === title) {
                    return i;
                }
            }

            return -1;
        },
        labLabels: {
            CHEM: "Chemistry",
            MICRO: "Microbiology",
            AP: "Anatomic Pathology",
            EM: "Electron Microscopy",
            SP: "Surgical Pathology",
            ST: "Skin Test",
            CY: "Cytology",
            AU: "Autopsy",
            BB: "Blood Bank",
            "H.I.V": "HIV"
        }
    },
    alerts: []
};


JLV._registeredJLVWidgets = {};

/**
 * JLV widget factory
 * @param aWidgetID string
 * @param options
 * @returns JLV widget instance of type aWidgetID
 */
JLV.createJLVWidget = function (aWidgetID, options, autoLoad) {
    if (JLV._registeredJLVWidgets[aWidgetID]) {

        var widget = new JLV._registeredJLVWidgets[aWidgetID](options);

        if (_.isFunction(widget.loadData) && autoLoad) {
            widget.loadData(_.bind(function () {
                JLV.VLERDataAvailable();
            }, this));
        }

        return widget;
    }

    return undefined;
};

/**
 * register the widget of type widgetID
 * @param aWidgetID string
 * @param widgetClass the widget prototype object
 */

JLV.registerJLVWidget = function (aWidgetID, widgetClass) {
    JLV._registeredJLVWidgets[aWidgetID] = widgetClass;
};

JLV.isRegisteredWidget = function (aWidgetID) {
    return JLV._registeredJLVWidgets[aWidgetID] !== undefined;
};

JLV.VlerDataFetchedNotification = "JLV.VlerFetchedNotification";
JLV.VlerDataReadyNotification = "JLV.VlerDataReady";
JLV.VlerDataFailNotification = "JLV.VlerDataFail";
JLV.VlerDataRefreshNotification = "JLV.VlerDataRefresh";
JLV.VlerDataRefreshConfirmNotification = "JLV.VlerDataRefreshConfirm";

JLV.DasVlerDataFetchedNotification = "JLV.DasVlerFetchedNotification";
JLV.DasVlerDataReadyNotification = "JLV.DasVlerDataReady";
JLV.DasVlerDataFailNotification = "JLV.DasVlerDataFail";
JLV.DasVlerDataRefreshNotification = "JLV.DasVlerDataRefresh";
JLV.DasVlerDataRefreshConfirmNotification = "JLV.DasVlerDataRefreshConfirm";


JLV.VlerConfirmController = {
    show: function () {
        this.dialog.center();
        this.dialog.open();

    },
    hide: function () {
        this.dialog.close();

        $('.jlv-widget').each(function (index, element) {
            if ($(element).widget().options.vlerDomain) {
                $(element).widget().hideVlerLoadMessage();
            }
            if ($(element).widget()._moreWindow) {
                $(element).widget()._moreWindow.hideVlerLoadMessage();
            }
        });
        $('.jlv-insurance-widget').each(function (index, element) {
            $(element).widget().hideVlerLoadMessage();
        });

    },
    confirm: function () {
        this.refresh();
        JLV.NotificationCenter.postNotificationName(JLV.VlerDataRefreshConfirmNotification);
    },
    refresh: function () {
        JLV.NotificationCenter.postNotificationName(JLV.VlerDataRefreshNotification);
        //JLV.VLERDataAvailable();
        setTimeout(function () {
            JLV.VLERDataAvailable();
        }, 500);
        this.hide();

    }

};

JLV.DasVlerConfirmController = {
    show: function () {
        this.dialog.center();
        this.dialog.open();

    },
    hide: function () {
        this.dialog.close();

        $('.jlv-widget').each(function (index, element) {
            if ($(element).widget().options.dasvlerDomain) {
                $(element).widget().hideVlerLoadMessage();
            }
            if ($(element).widget()._moreWindow) {
                $(element).widget()._moreWindow.hideVlerLoadMessage();
            }
        });
        $('.jlv-insurance-widget').each(function (index, element) {
            $(element).widget().hideVlerLoadMessage();
        });
    },
    confirm: function () {
        this.refresh();
        JLV.NotificationCenter.postNotificationName(JLV.DasVlerDataRefreshConfirmNotification);
    },
    refresh: function () {
        JLV.NotificationCenter.postNotificationName(JLV.DasVlerDataRefreshNotification);
        //JLV.VLERDataAvailable();
        setTimeout(function () {
            JLV.VLERDataAvailable();
        }, 500);
        this.hide();

    }

};

JLV.LoadVlerData = function () {

    JLV.Session.dodVLERReady = false;
    JLV.Session.dodVLERFail = false;

    JLV.Session.dasVLERReady = false;
    JLV.Session.dasVLERFail = false;

    if (JLV.Session.dodVLER == true) {

        JLV.NotificationCenter.postNotificationName(JLV.VlerDataFetchedNotification);

        //query for DOD VLER Data. if success, alert user that VLER data is ready (at BHIERelay)
        $.post('patientDODVLER/list.json', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                forceRefresh: JLV.Session.forceRefresh
            },
            function (response) {
                //alert('success!');
                console.info("DOD VLER data is ready");
                JLV.Session.dodVLERReady = true;
                JLV.Session.forceRefresh = false;
                JLV.Session.vlerData = [];

                //JLV.Session.vlerData contains domains of which we will notify users that
                //related data exists in EHDV widget
                if (response && response.length > 0) {
                    for (var i = 0; i < response.length; i++) {
                        JLV.Session.vlerData.push(response[i]);
                    }
                }

                JLV.NotificationCenter.postNotificationName(JLV.VlerDataReadyNotification);

                JLV.VlerConfirmController.show();

            }).fail(function (response) {
            //alert('failed to retrieve DoD VLER Data: ' + response.responseText);
            console.info("Error retrieving VLER Data: " + response.responseText);
            JLV.Session.dodVLERReady = false;
            JLV.Session.forceRefresh = false;
            JLV.Session.vlerData = [];
            JLV.Session.dodVLERFail = true;


            JLV.NotificationCenter.postNotificationName(JLV.VlerDataFailNotification);

            $('.jlv-widget').each(function (index, element) {

                var widget = $(element).widget();

                widget.hideVlerLoadMessage();

                if (_.isFunction(widget.checkConnectionUnavailable)) {
                    widget.checkConnectionUnavailable();
                }
                });

        });

    }

    if (JLV.Session.dasVLER) {

        JLV.NotificationCenter.postNotificationName(JLV.DasVlerDataFetchedNotification);

        //query for DAS VLER Data. if success, alert user that VLER data is ready (at VistaDataService)
        $.post('patientDASVLER/list.json', {
                patToken: JLV.ActivePatientModel.get('patientToken'),
                forceRefresh: JLV.Session.forceRefresh
            },
            function (response) {
                //alert('success!');
                console.info("VA VLER data is ready");
                JLV.Session.dasVLERReady = true;
                JLV.Session.forceRefresh = false;
                JLV.Session.dasvlerData = [];

                //JLV.Session.dasvlerData contains domains of which we will notify users that
                //related data exists in EHDV widget
                if (response && response.length > 0) {
                    for (var i = 0; i < response.length; i++) {
                        JLV.Session.dasvlerData.push(response[i]);
                    }
                }

                JLV.NotificationCenter.postNotificationName(JLV.DasVlerDataReadyNotification);

                var displayedConfirm = false;

                function checkWidgetLoading(){
                    var isDataLoading = false;
                    $('.jlv-widget').each(function (index, element) {
                        var widget = $(element).widget();
                        if(widget._dataIsLoading){
                            isDataLoading = true;
                        }
                    })
                    if(!isDataLoading){
                        JLV.DasVlerConfirmController.show();
                        displayedConfirm = true;
                        clearInterval(timerId);
                    }
                }

                function clearAndShow(){
                    clearInterval(timerId);
                    if(!displayedConfirm){
                        JLV.DasVlerConfirmController.show();
                        displayedConfirm = true;
                    }
                }

                //while widgets are loading... check widgets every 2 secs
                var timerId = setInterval(checkWidgetLoading, 2000);

                // after 60 seconds of waiting, cancel 2sec check.
                setTimeout(clearAndShow, 60000);

            }).fail(function (response) {
            //alert('failed to retrieve DoD VLER Data: ' + response.responseText);
            console.info("Error retrieving VA VLER Data: " + response.responseText);
            JLV.Session.dasVLERReady = false;
            JLV.Session.forceRefresh = false;
            JLV.Session.dasvlerData = [];
            JLV.Session.dasVLERFail = true;


            JLV.NotificationCenter.postNotificationName(JLV.DasVlerDataFailNotification);

            $('.jlv-widget').each(function (index, element) {

                var widget = $(element).widget();

                widget.hideVlerLoadMessage();

                if (_.isFunction(widget.checkConnectionUnavailable)) {
                    widget.checkConnectionUnavailable();
                }
            });

        });

    }
};



