AppKit.SidePane = AppKit.View.extend({
    classID: 'appkit-sidepane',
    show: function () {

        this.$el.show()
    },
    hide: function () {
        this.$el.hide();
    }
});


JLV.ReportBuilderController = {

    init: function () {

        this._selectedIndex = -1;

        this.table.on('afterRender', _.bind(function () {

            var $errorLink = this.table.$el.find('.link-status-error-report-builder');
            $errorLink.on('click', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                $(this).tooltip({
                    items: ".link-status-error-report-builder",
                    content: "This record cannot be printed within the report. Please print the record directly from the widget.",
                    position: { my: "left+10 center", at: "right center" },
                    close: function () {
                        $(this).tooltip("destroy");
                    }
                }).tooltip("open");
            });

            var $rows = this.table.$el.find('tr');
            var self = this;
            $rows.on('click', function () {
                var index = $rows.index($(this)) - 2;
                self.selectRow(index);
            });

            $rows.on('keydown', function (evt) {

                var index = $rows.index($(this)) - 2;
                if (evt.which === $.ui.keyCode.DOWN) {
                    index++;
                    self.selectRow(index);

                }
                else if (evt.which === $.ui.keyCode.UP) {
                    index--;
                    self.selectRow(index);
                }
            });

            this.selectRow(this._selectedIndex);

        }, this));

        this.vtitle.on('click', _.bind(function () {
            this.maximize();
        }, this));

        this.vtitle.$el.tooltip({
            items: ".v-title",
            content: "To build a report, right-click an entry in the patient's record and select \"Add Record to Report\". <br/><br/> To see selected records click the > at the top of the Report Builder sidebar.",
            position: {my: "left+5 center", at: "right center"}
        });


        this.pane.$el.find('.max-btn').on('keydown', function (evt) {

            if (evt.keyCode === $.ui.keyCode.TAB) {

                evt.preventDefault();

                var next = $('.jlv-patient-portal').nextInDOM('.jlv-patient-demographics .ui-tab');
                if (next.length) {
                    next.focus()
                }
                else {
                    next = $('.jlv-patient-portal').nextInDOM('.jlv-widget');
                    if (next.length) {
                        next.focus();
                    }
                    else {
                        $('#AddWidgetButton').focus();
                    }
                }
            }
        });

        $('#buildLoadingIcon').css('visibility', 'hidden');
    },

    selectRow: function (rowIndex) {

        if (rowIndex > -1 && rowIndex < JLV.ReportBuilderController.Records.length) {

            var rows = this.table.$el.find('tr');
            rows.removeClass('highlighted');
            var selectedRow = rows.eq(rowIndex + 2);
            if (selectedRow) {
                selectedRow.addClass('highlighted');
                selectedRow.focus();
            }

            this._selectedIndex = rowIndex;

            var model = JLV.ReportBuilderController.Records.at(rowIndex);
            var details = model.get('rbDetails');

            if (details) {

                if (model.get('hasAttachment')) {
                    var idx = this._getAttachemntIndex(model);

                    if (idx > -1)
                        details += '<div style="padding:8px 0;font-style: italic;">See attachment ' + idx + '</div>';
                }

                $('#ReportBuilderDetailsView .details').html(details);
            }

            this.disableButtons(false);
        }
    },

    _getAttachemntIndex: function (model) {

        var idx = 0,
            count = this.Records.length,
            i = 0;

        for (; i < count; i++) {
            var r = this.Records.at(i);
            if (r.get('hasAttachment')) {
                idx++
            }

            if (r.cid == model.cid) {
                return idx;
            }
        }

        return -1;

    },

    show: function () {

        this.pane.show();
        this.maximize();

        if (this.table.get('tableData').length == 0) {
            this.table.showMessage('No records added to report.')
        }
        else {
            this.selectRow(0);
        }

    },

    doClose: function () {

        setTimeout(function () {
            $('#ReportBuilderButton').focus();
        }, 100);

        this.close();

    },

    close: function () {
        this.pane.hide();
    },


    minimize: function () {
        this.title.$el.hide();
        this.content.$el.hide();
        this.pane.$el.width(30);
        this.vtitle.$el.show();
        setTimeout(_.bind(function () {
            this.pane.$el.find('.max-btn').focus();
        }, this), 10);

    },


    maximize: function () {

        this.vtitle.$el.hide();
        this.title.$el.show();
        this.content.$el.show();
        this.pane.$el.width(460);
        setTimeout(_.bind(function () {
            this.pane.$el.find('.min-btn').focus();
        }, this), 10);

    },

    addDemographic: function (record, siteName, isDetail) {

        if (record) {

            $.post('auditUserAction/auditDetails', {
                recordType: "addToReportBuilder",
                patToken: JLV.ActivePatientModel.get('patientToken'),
                info: siteName,
                widgetName: 'Demographic'
            });

            if(isDetail){
                var html = '<span>' + 'Patient Demographic Detail ';
            }else {
                var html = '<span>' + 'Patient Demographic ';
            }

            record.set('reportStatus', "Added");
            var data = record.toJSON();
            var rbDetails = '';

            if (siteName.indexOf('DoD') > -1) {
                record.set('name', 'Site: DoD');
                html = html + '<div class="dodMarker" alt="DoD Marker" title="PDWS: PDWS"></div> DoD</span>';

                rbDetails = this.dodDemographicsTemplate(_.extend(_.extend({}, data), {
                    siteName: JLV.common.formatSiteDisplay(data.site),
                    edipi: this.maskEdipi(data.edipi),
                    hieSharingOptInLastUpdateDate: data.hieSharingOptInLastUpdateDate || '',
                    fmp: this.maskFMP(data.fmp)
                }));

                if (isDetail) {
                    rbDetails += '<div><div>Details:</div><pre> ' + this.maskDemographicsReport(_.text(data.report), data.fmpssn, data.edipi) + '</pre></div>';
                }

            } else if (siteName.indexOf('VLER') > -1) {
                record.set('name', 'Site: VLER');
                html = html + '<div class="dodVlerMarker" alt="DoD VLER Marker" title="DoD VLER: DoD Community Partners"></div> VLER</span>';
                rbDetails = this.vlerDemographicsTemplate(_.extend(_.extend({}, data), {
                    siteName: JLV.common.formatSiteDisplay(data.site) + ' (VLER)',
                    edipi: this.maskEdipi(data.edipi)
                }));

            } else {
                siteName = siteName.replace(/\W/g, '');
                record.set('name', 'Site: ' + siteName);
                html = html + '<div class="vaMarker" alt="VA Marker" ></div> ' + siteName + '</span>';
                rbDetails = this.vaDemographicsTemplate(_.extend(_.extend({}, data), {
                    siteName: JLV.common.formatSiteDisplay(data.site),
                    fmpssn: this.maskSSN(data.fmpssn),
                    hieSharingOptInLastUpdateDate: data.hieSharingOptInLastUpdateDate || ''
                }));

                if (isDetail) {
                    rbDetails += '<br/><br/><div>Details:</div><pre>' + this.maskDemographicsReport(_.text(data.report), data.fmpssn, data.edipi) + '</pre>';
                }

            }

            record.set('record', html);
            record.set('demographics', true);
            record.set('rbDetails', rbDetails);

        }

        // this.Records.push(record);
        JLV.ReportBuilderController.Records.add(record, {at: 0});
        this.updateStatusString(0, this.table.get('tableData').length, 'ready');
        this.buildBtn.set('enabled', true);
        this.table.render();

    },

    addRecord: function (record, minColumns, expColumns) {
        if (record && minColumns && expColumns) {

            var html = '<span>' + record.get('_name') + ' Record:&nbsp; </span>';
            var recordInformation = '';


            var count = minColumns.length,
                i = 0;

            for (; i < count; i++) {
                var c = minColumns[i],
                    v = record.get(c.id);

                if (!_.isEmpty(v)) {
                    if (!v) {
                        v = "";
                    }

                    if (_.isFunction(c.formatter)) {
                        html += ('<span>' + c.formatter(v, record, true) + '</span>');
                    }
                    else {
                        html += ('<span>' + v + '</span>');
                    }

                    html += (' &nbsp;&nbsp;|&nbsp;&nbsp; ');

                }
                if ( i < 2) {
                    recordInformation += v;
                    if(i < 1){
                        recordInformation += '^';
                    }
                }
            }

            record.set('reportStatus', "added");
            record.set('record', html);

            var cols = [];
            count = expColumns.length;
            i = 0;

            var rbDetals = '<table class="rbDetails" cellspacing="0">';

            rbDetals += '<tr>';
            for (; i < count; i++) {
                rbDetals += ('<th>' + expColumns[i].title + '</th>');
                cols.push({
                    title: _.text(expColumns[i].title),
                    id: expColumns[i].id
                });
            }

            rbDetals += '</tr>';

            rbDetals += '<tr>';

            i = 0;
            for (; i < count; i++) {
                var column = expColumns[i],
                    data = record.get(column.id),
                    value = data;

                if (_.isFunction(column.formatter)) {
                    value = _.text(column.formatter(data, record, true));
                }

                if (!value) {
                    value = "";
                }

                record.set(column.id, value);
                rbDetals += ('<td>' + value + '</td>');
            }

            rbDetals += '</table>';


            if (record.get('details') && record.get('detailsTitle')) {
                if(record.get('detailsTitle').indexOf('Allergy Details') > -1){
                    record.set('details', this.removeTagsInPre(record.get('details')));
                }
                rbDetals += '<div>' + record.get('detailsTitle') + '<div class="record-details">' + record.get('details') + '</div></div>';
            }


            record.set('rbDetails', rbDetals);
            record.set('columns', cols);


            console.log(record)

        }

        this.Records.push(record);

        $.post('auditUserAction/auditDetails', {
            recordType: "addToReportBuilder",
            patToken: JLV.ActivePatientModel.get('patientToken'),
            recordSiteMoniker: record.get('site') == null ? '' : record.get('site').moniker,
            recordSiteAgency: record.get('site') == null ? '' : record.get('site').agency,
            info: recordInformation,
            widgetName: record.get('_name')
        });

        this.updateStatusString(0, this.Records.length, 'ready');
        this.table.render();

        this.setBuildStatus(_.fill(Array(this.Records.length), {status: 'added'}));
        this.buildBtn.set('enabled', true);
        this.printBtn.set('enabled', false);
        this.selectRow(this._selectedIndex);

    },

    disableButtons: function (aBOOL) {

        this.buildBtn.set('enabled', !aBOOL);
        this.upBtn.set('enabled', !aBOOL);
        this.downBtn.set('enabled', !aBOOL);
        this.upAllBtn.set('enabled', !aBOOL);
        this.downAllBtn.set('enabled', !aBOOL);

    },

    clearRecords: function () {

        this.Records.reset();
        this.updateStatusString(0, 0, 'ready');
        this.table.render();

        if (this.table.get('tableData').length == 0) {
            this.table.showMessage('No records added to report.')
        }

        $('#ReportBuilderDetailsView .details').empty();
        this.disableButtons(true);


        this._selectedIndex = -1;
        this.disableButtons(true);

    },

    clearSelected: function () {

        if (this._selectedIndex > -1) {
            this.removeRecord(this._selectedIndex);

            if (this.table.get('tableData').length > 0) {
                if (this._selectedIndex - 1 > -1) {
                    this.selectRow(this._selectedIndex - 1);
                }
                else {
                    this.selectRow(0);
                }
            }

        }
        this.updateStatusString(0, this.table.get('tableData').length, 'ready');
        this.buildBtn.set('enabled', this.table.get('tableData').length > 0);
        this.printBtn.set('enabled', false);

    },

    moveRecordUp: function () {

        if (this._selectedIndex > 0) {
            var model = JLV.ReportBuilderController.Records.at(this._selectedIndex);
            JLV.ReportBuilderController.Records.remove(model)
            JLV.ReportBuilderController.Records.add(model, {at: this._selectedIndex - 1});
            this.table.render();
            this.selectRow(this._selectedIndex - 1);
        }


    },

    moveRecordAllUp: function () {

        if (this._selectedIndex > 0) {
            var model = JLV.ReportBuilderController.Records.at(this._selectedIndex);
            JLV.ReportBuilderController.Records.remove(model)
            JLV.ReportBuilderController.Records.add(model, {at: 0});
            this.table.render();
            this.selectRow(0);
        }

    },

    moveRecordDown: function () {

        if (this._selectedIndex > -1 && this._selectedIndex + 1 < JLV.ReportBuilderController.Records.length) {
            var model = JLV.ReportBuilderController.Records.at(this._selectedIndex);
            JLV.ReportBuilderController.Records.remove(model)
            JLV.ReportBuilderController.Records.add(model, {at: this._selectedIndex + 1});
            this.table.render();
            this.selectRow(this._selectedIndex + 1);
        }

    },

    moveRecordAllDown: function () {

        var l = JLV.ReportBuilderController.Records.length - 1;
        if (this._selectedIndex > -1 && this._selectedIndex < l) {
            var model = JLV.ReportBuilderController.Records.at(this._selectedIndex);
            JLV.ReportBuilderController.Records.remove(model)
            JLV.ReportBuilderController.Records.add(model, {at: l});
            this.table.render();
            this.selectRow(l);
        }

    },

    removeRecord: function (anIndex) {

        if (anIndex > -1 && anIndex < this.Records.length) {
            this.Records.remove(this.Records.at(anIndex));
            this.table.render();

            if (this.table.get('tableData').length == 0) {
                this.table.showMessage('No records added to report.')
                $('#ReportBuilderDetailsView .details').empty();
                this.disableButtons(true);
            }
        }


    },

    recordFormatter: function (value) {
        return '<div style="margin:5px;width:100%;">' + value + '</div>';
    },


    build: function () {

        var count = this.Records.length,
            i = 0;

        $('#buildLoadingIcon').css('visibility', 'visible');
        this.buildBtn.set('enabled', false);

        this.setBuildStatus(_.fill(Array(count), {status: 'processing'}));

        if (count > 0) {
            var patient = JLV.ActivePatientModel,
                toPrint = [];

            i = 0;
            for (; i < count; i++) {
                toPrint.push(this.Records.at(i).toJSON());
            }

            $.post('print/setupReportBuilder', {
                patToken: patient.get('patientToken'),
                data: JSON.stringify({
                    data: toPrint,
                    patient: patient.toJSON(),
                    datetime: moment().format('MMMM DD, YYYY h:mm a')
                })
                },
                _.bind(function (response) {
                    JLV.ReportBuilderController.jobId = response.jobId;
                    var reportStatus = response.reportStatus;

                    var readyCount = this.getStatusCount(reportStatus, 'ready');

                    this.updateStatusString(readyCount, reportStatus.length, 'ready');
                    this.setBuildStatus(reportStatus);
                    this.buildBtn.set('enabled', readyCount == 0);
                    this.printBtn.set('enabled', readyCount > 0);
                }, this))
                .error(function () {
                    this.setBuildStatus(_.fill(Array(count), {status: 'error'}));
                }.bind(this))
                .complete(function () {
                    $('#buildLoadingIcon').css('visibility', 'hidden');
                }.bind(this));
        }
    },

    setBuildStatus: function(statuses) {
        var reportStatuses = {
            'added': _.template('Added'),
            'processing': _.template('<span class="column-loading-icon" alt="Processing"></span>'),
            'ready': _.template('<span class="icon-status-ok-report-builder" alt="Ready"/>&nbsp;Ready'),
            'error': _.template('<span class="icon-status-error-report-builder" alt="Error"/>&nbsp;<a class="link-status-error-report-builder" data-ec="{{errorCode}}" href="#" style="text-decoration: underline;">Error</a>')
        };

        if (statuses.length === JLV.ReportBuilderController.Records.models.length) {
            _.each(statuses, function(status, index) {
                if (!status.errorCode) {
                    status.errorCode = '';
                }
                JLV.ReportBuilderController.Records.models[index].set('reportStatus', reportStatuses[status.status](status) || 'Processing..');
            });
        }
        this.table.render();
    },

    updateStatusString: function(count, total, state){
        var string = "( " + count + " of " + total + " " + state + " )";
        $("#reportStatusString").text(string);
    },

    getStatusCount: function(reportStatus, state) {
        return _.reduce(reportStatus, function(p,c){
            if (c.status === state)
                p++;
            return p;
        },0);
    },

    print: function(){
        var patient = JLV.ActivePatientModel;
        var jobId = JLV.ReportBuilderController.jobId;

        var printUrl = 'print/report';
                printUrl += ('?jobId=' + jobId);
                printUrl += ('&patToken=' + patient.get('patientToken'));
        printUrl += ('&patEdipi=' + patient.get('edipi'));
                window.open(printUrl, '_blank');

        JLV.ReportBuilderController.buildBtn.set('enabled', true);
        JLV.ReportBuilderController.printBtn.set('enabled', false);

        this.setBuildStatus(_.fill(Array(JLV.ReportBuilderController.Records.length), {status: 'added'}));
        this.updateStatusString(0, JLV.ReportBuilderController.Records.length, 'ready')
    },

    removeTagsInPre: function (record) {
        var preText = $(record).find('pre').html();
        if(preText){
            var preLength = preText.length;
            var replacedPreText = preText.replace(/<\/\S+>$/, '');
            replacedPreText = replacedPreText.replace('<', '&lt;');
            replacedPreText = replacedPreText.replace('>', '&gt;');
            var preIndex = record.indexOf(preText);
            if (preIndex > -1) {
                record = record.substring(0, preIndex) + replacedPreText + record.substring(preIndex + preLength, record.length);
            }
        }

        return record;
    },

    maskSSN: function (ssn) {
        return "***-**-" + ssn.substring(ssn.length - 4, ssn.length);
    },

    dashedSSN: function (ssn) {
        return ssn.substring(0, 3) + "-" + ssn.substring(3, 5) + "-" + ssn.substring(5, ssn.length);
    },

    maskEdipi: function (edipi) {
        return "******" + edipi.substring(edipi.length - 4, edipi.length);
    },

    maskFMP: function (fmp) {
        return fmp.substring(0, 2) + " / *****" + fmp.substring(8, fmp.length);
    },

    maskDemographicsReport: function (report, ssn, edipi) {

        if (ssn.length > 0) {
            var ssnIdx = report.indexOf(ssn);

            while (ssnIdx > -1) {
                var maskedSSN = this.maskSSN(ssn);
                report = report.substring(0, ssnIdx) + maskedSSN + report.substring(ssnIdx + ssn.length, report.length);
                ssnIdx = report.indexOf(ssn);
            }

            var dashedSSN = this.dashedSSN(ssn);

            ssnIdx = report.indexOf(dashedSSN);
            while (ssnIdx > -1) {
                var maskedSSN = this.maskSSN(ssn);
                report = report.substring(0, ssnIdx) + maskedSSN + report.substring(ssnIdx + dashedSSN.length, report.length);
                ssnIdx = report.indexOf(dashedSSN);
            }

            var edipiIdx = report.indexOf(edipi);

            while (edipiIdx > -1) {
                var maskedEdipi = this.maskEdipi(edipi);
                report = report.substring(0, edipiIdx) + maskedEdipi + report.substring(edipiIdx + edipi.length, report.length);
                edipiIdx = report.indexOf(edipi);
            }
        }

        return report;

    },

    /* use inline styles for iText printing */
    dodDemographicsTemplate: _.template(
        '<div>' +
        '<div>' +
        '<label style="font-size:14px;font-weight:bold;float:left;">{{-name}}</label>' +
        '<span>&nbsp;&nbsp;{{siteName}}</span>' +
        '</div>' +
        '<br/>' +
        '<table style="width:250px;">' +
        '<tr>' +
        '<td style="vertical-align:top;font-size:12px;border-width:0;padding:2px;">' +
        '<div>{{-address1}}</div>' +
        '<div>{{-address2}}</div>' +
        '<div>{{-city}}, {{state}}</div>' +
        '<div>{{-zipCode}}</div>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '<table>' +
        '<tr>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;margin-top:12px;">' +
        '<tr><td style="border-width:0;padding:2px;" >Rank:</td><td style="border-width:0;padding:2px;">{{-rank}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;" >FMP/SSN:</td><td style="border-width:0;padding:2px;">{{-fmp}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;" >Sponsor Name:</td><td style="border-width:0;padding:2px;">{{-sponsorName}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Name:</td><td style="border-width:0;padding:2px;">{{-pcm}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Date:</td><td style="border-width:0;padding:2px;">{{-pcmDate}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Team:</td><td style="border-width:0;padding:2px;">{{-pcTeam}}</td></tr>' +
        '{[ if(JLV.ActivePatientModel.get("hieSharingOptIn")===false && JLV.ActivePatientModel.get("hieSharingOptInLastUpdateDate")) { ]}' +
        '<tr><td style="border-width:0;padding:2px;">DoD Sharing Status:</td><td style="border-width:0;padding:2px;"><b> Opted Out <br/> ({{hieSharingOptInLastUpdateDate}})</b></td></tr>' +
        '{[ } ]}' +
        '</table>' +
        '</td>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;">' +
        '<tr><td style="border-width:0;padding:2px;">Dod ID:</td><td style="border-width:0;padding:2px;">{{-edipi}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Age:</td><td style="border-width:0;padding:2px;">{{-age}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">DOB:</td><td style="border-width:0;padding:2px;">{{-dob}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Race:</td><td style="border-width:0;padding:2px;">{{-race}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(h):</td><td style="border-width:0;padding:2px;">{{-phone1}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(w):</td><td style="border-width:0;padding:2px;">{{-phone2}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Gender:</td><td style="border-width:0;padding:2px;">{{-gender}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Organ Donor: </td><td style="border-width:0;padding:2px;"><b>{{-organDonorIndDateDisplay}}</b></td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Blood Type: </td><td style="border-width:0;padding:2px;"><b>  {{-bloodType}}</b></td></tr>' +
        '</table>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '</div>'
    ),
    vaDemographicsTemplate: _.template(
        '<div>' +
        '<div>' +
        '<label style="font-size:14px;font-weight:bold;padding-right:8px;float:left;">{{-name}}</label>' +
        '<span>&nbsp;&nbsp;{{siteName}}</span>' +
        '</div>' +
        '<br/>' +
        '<table style="width:250px;">' +
        '<tr>' +
        '<td style="vertical-align:top;font-size:12px;border-width:0;padding:2px;">' +
        '<div>{{-address1}}</div>' +
        '<div>{{-address2}}</div>' +
        '<div>{{-city}}, {{state}}</div>' +
        '<div>{{-zipCode}}</div>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '<table>' +
        '<tr>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;margin-top:12px;">' +
        '<tr><td style="border-width:0;padding:2px;" >SC Percent:</td><td style="border-width:0;padding:2px;">{{-percentServiceConnected}}%</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Name:</td><td style="border-width:0;padding:2px;">{{-pcm}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Date:</td><td style="border-width:0;padding:2px;">{{-pcmDate}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">PCM Team:</td><td style="border-width:0;padding:2px;">{{-pcTeam}}</td></tr>' +
        '{[ if(JLV.ActivePatientModel.get("hieSharingOptIn")===false && JLV.ActivePatientModel.get("hieSharingOptInLastUpdateDate")) { ]}' +
        '<tr><td style="border-width:0;padding:2px;">DoD Sharing Status:</td><td style="border-width:0;padding:2px;"><b> Opted Out <br/> ({{hieSharingOptInLastUpdateDate}})</b></td></tr>' +
        '{[ } ]}' +
        '</table>' +
        '</td>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;">' +
        '<tr><td style="border-width:0;padding:2px;">SSN:</td><td style="border-width:0;padding:2px;">{{-fmpssn}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Age:</td><td style="border-width:0;padding:2px;">{{-age}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">DOB:</td><td style="border-width:0;padding:2px;">{{-dob}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Race:</td><td style="border-width:0;padding:2px;">{{-race}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(h):</td><td style="border-width:0;padding:2px;">{{-phone1}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(w):</td><td style="border-width:0;padding:2px;">{{-phone2}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Gender:</td><td style="border-width:0;padding:2px;">{{-gender}}</td></tr>' +
        '</table>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '</div>'
    ),
    vlerDemographicsTemplate: _.template(
        '<div>' +
        '<div>' +
        '<label style="font-size:14px;font-weight:bold;padding-right:8px;float:left;">{{-name}}</label>' +
        '<span>&nbsp;&nbsp;{{siteName}}</span>' +
        '</div>' +
        '<br/>' +
        '<table style="width:250px;">' +
        '<tr>' +
        '<td style="vertical-align:top;font-size:12px;border-width:0;padding:2px;">' +
        '<div>{{-address1}}</div>' +
        '<div>{{-address2}}</div>' +
        '<div>{{-city}}, {{state}}</div>' +
        '<div>{{-zipCode}}</div>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '<table>' +
        '<tr>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;margin-top:12px;">' +
        '<tr><td style="border-width:0;padding:2px;white-space:nowrap;" >Language:</td><td style="border-width:0;padding:2px;">{{-languages}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;white-space:nowrap;">Religious Affiliation:</td><td style="border-width:0;padding:2px;">{{-religion}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Next of Kin:</td><td style="border-width:0;padding:2px;">' +
        '{[ _.each(nextOfKin, function(kin) { ]}' +
        '{{-kin.name}} <br/>' +
        'Relationship: {{-kin.relationship}} <br/>' +
        '{{-kin.street1}} {{-kin.street2}} {{-kin.street3}}<br/>' +
        '{{-kin.city}}, {{-kin.stateProvince}} {{-kin.postalCode}}<br/>' +
        'Phone: ' +
        '{[ _.each(kin.telecoms, function(telecom) { ]}' +
        '{{-telecom.usageType}} {{-telecom.value}} <br/><br/>' +
        '{[ }) ]}' +
        '{[ }) ]}' +
        '</td></tr>' +
        '</table>' +
        '</td>' +
        '<td style="border-width:0;padding:2px;">' +
        '<table style="font-size:12px;white-space:normal;word-wrap:normal;">' +
        '<tr><td style="border-width:0;padding:2px;">Patient ID:</td><td style="border-width:0;padding:2px;">{{-edipi}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Age:</td><td style="border-width:0;padding:2px;">{{-age}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">DOB:</td><td style="border-width:0;padding:2px;">{{-dob}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Race:</td><td style="border-width:0;padding:2px;">{{-race}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(h):</td><td style="border-width:0;padding:2px;">{{-phone1}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">(w):</td><td style="border-width:0;padding:2px;">{{-phone2}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Gender:</td><td style="border-width:0;padding:2px;">{{-gender}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Marital Status:</td><td style="border-width:0;padding:2px;">{{-maritalStatus}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Source:</td><td style="border-width:0;padding:2px;">{{-sourceProtocol}}</td></tr>' +
        '<tr><td style="border-width:0;padding:2px;">Emergency Contact:</td>' +
        '<td style="border-width:0;padding:2px;">' +
        '{[ _.each(emergencyContact, function(contact) { ]}' +
        '{{-contact.name}} <br/>' +
        'Relationship: {{-contact.relationship}} <br/>' +
        '{{-contact.street1}} {{-contact.street2}} {{-contact.street3}}<br/>' +
        '{{-contact.city}}, {{-contact.stateProvince}} {{-contact.postalCode}}<br/>' +
        'Phone: ' +
        '{[ _.each(contact.telecoms, function(telecom) { ]}' +
        '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
        '{[ }) ]}' +
        '{[ }) ]}' +
        '</td></tr>' +
        '</table>' +
        '</td>' +
        '</tr>' +
        '</table>' +
        '</div>'
    )




};


JLV.ReportBuilderController.Records = new Backbone.Collection();


AppKit.launch(function () {
    JLV.ReportBuilderController.init();
});
