//= require ./desktop.js


JLV.Desktop.WindowBar = AppKit.View.extend({
    classID: 'jlv-window-bar',
    initialize: function (options) {
        this._windows = [];

        this._super(options);
    },
    addWindow: function (aWindow) {
        if (!aWindow.get('id')) {
            aWindow.set('id', _.uniqueId('jlv-window-'));
        }

        var windowID = aWindow.get('id');

        if (!_.findWhere(this._windows, {id: windowID})) {
            $("<div tabindex='0'>").addClass('jlv-window-button')
                .attr('data-id', windowID)
                .attr('tabindex', 0)
                .html(aWindow.get('title'))
                .button()
                .on('click', function () {
                    aWindow.set('minimized', false);

                    setTimeout(function () {
                        aWindow.focus();
                    }, 0)
                })
                .on('keydown', function (evt) {

                    if (evt.which === $.ui.keyCode.TAB) {

                        evt.preventDefault();
                        evt.stopPropagation();

                        if (!evt.shiftKey) {

                            var windowItem = $(this).nextInDOM('.jlv-window-button');
                            if (windowItem.length) {
                                setTimeout(function () {
                                    windowItem.focus();
                                }, 0);
                                return;
                            }

                            windowItem = $('.ui-dialog.open');
                            if (windowItem.length) {
                                setTimeout(function () {
                                    windowItem.find('.ui-dialog-titlebar-close').focus();
                                }, 0);
                                return;
                            }


                            $('#PatientSearchToolbarButton').focus();
                        }
                    }


                })
                .appendTo(this.$el);

            aWindow.on('restored', function () {
                JLV.Desktop.RemoveWindowFromTaskBar(aWindow);
            });

            this._windows.push({
                windowID: windowID
            });
        }
    },
    removeWindow: function (aWindow) {
        var windowID = aWindow.get('id');

        this.$el.find('[data-id="' + windowID + '"]').remove();

        var w = _.findWhere(this._windows, {windowID: windowID});

        if (w) {
            this._windows.splice(_.indexOf(this._windows, w), 1);
        }
    },
    clear: function () {
        this.$el.find('.jlv-window-button').remove();
        this._windows = [];
    }
});

JLV.Desktop.AddWindowToTaskBar = function (aWindow) {
    AppKit.get('WindowDock').addWindow(aWindow);
    $('.jlv-tabview, .patient-select-screen').css('bottom', $('#WindowDock').height());
};

JLV.Desktop.RemoveWindowFromTaskBar = function (aWindow) {
    AppKit.get('WindowDock').removeWindow(aWindow);
    $('.jlv-tabview, .patient-select-screen').css('bottom', $('#WindowDock').height());
};

JLV.Desktop.ClearOpenWindows = function () {
    AppKit.get('WindowDock').clear();
    JLV.ReportBuilderController.close();
    JLV.NotificationCenter.postNotificationName('closeNotification');
};


JLV.Desktop.registerWindow = function (aWindow) {
    if (aWindow) {
        aWindow.on('minimized', function () {
            JLV.Desktop.AddWindowToTaskBar(this);
        });
    }
};
