//= require lib/AppKit/AppKit.js
//= require ../JLV.js
//= require ./desktop.js


JLV.Desktop.AddWidgetButton = AppKit.Button.extend({
    classID: 'jlv-add-widget',
    options: {
        tabindex: 0
    },
    initialize: function (options) {

        this.$el.click(function (evt) {
            evt.stopPropagation();

        });

        //508 nav
        this.$el.on('keydown', function (evt) {

            if (evt.which === $.ui.keyCode.TAB) {
                evt.preventDefault();
                evt.stopPropagation();

                if (!evt.shiftKey) {
                    var widgetMenu = JLV.Desktop.WidgetMenu.getActive();

                    var firstWidget = widgetMenu.$el.nextInDOM('.jlv-widget-menu-item');
                    if (firstWidget.length) {
                        firstWidget.focus();
                        return;
                    }

                    var windowItem = $('#WindowDock').nextInDOM('.jlv-window-button');
                    if (windowItem.length) {
                        windowItem.focus();
                        return;

                    }

                    windowItem = $('.ui-dialog.open');
                    if (windowItem.length) {
                        windowItem.find('.ui-dialog-titlebar-close').focus();
                        return;
                    }

                    $('#PatientSearchToolbarButton').focus();

                }
                else {

                    var prev = $('#JLV-Desktop').find('.jlv-widget:visible');
                    if (prev.length) {
                        prev.focus();
                    }
                    else {
                        $('#JLV-Desktop').find('.jlv-tabview-add').focus();
                    }
                }

            }
        });


        this._super(options);
    },
    showMode: function (aBool) {

        if (aBool === false) {

            this.$el.find('label').html('Close Widget Tray');
            this.$el.find('.icon')
                .removeClass('icon-chevron-up')
                .addClass('icon-chevron-down');


        }
        else {
            this.$el.find('label').html('Open Widget Tray');
            this.$el.find('.icon')
                .removeClass('icon-chevron-down')
                .addClass('icon-chevron-up')
        }
    }
});


JLV.Desktop.WidgetMenu = AppKit.View.extend({
    classID: 'jlv-widget-menu',
    options: {
        orientation: 'bottom' //bottom or right
    },
    initialize: function () {

        this.set('lastScrollPos', 0);

        this.$el.find('.jlv-widget-menu-body').on('scroll', _.bind(function (evt) {
            this.checkScroll();
        }, this));


        this.$el.find('.jlv-widget-menu-right').on('click', _.bind(function (evt) {
            evt.preventDefault();
            this.set('scrollPosition', this.get('scrollPosition') + 100);

        }, this));

        this.$el.find('.jlv-widget-menu-left').on('click', _.bind(function (evt) {

            evt.preventDefault();
            this.set('scrollPosition', Math.max(0, this.get('scrollPosition') - 100));


        }, this));


    },
    events: {
        click: function (evt) {
            evt.stopPropagation();
        },
        mousedown: function (evt) {
            _mouseDownInMenu = true;
            evt.stopPropagation();
        }
    },
    orientation: function (theOrientation) {

        if (theOrientation !== undefined) {
            this._orientation = theOrientation;

            if (this._orientation === 'right') {
                this.$el.addClass('sidebar');
                this.$el.css({
                    height: 'auto',
                    width: 0
                });
            }
            else {
                this.$el.removeClass('sidebar');
                this.$el.css({
                    'height': 0,
                    'width': '100%'
                });
            }
        }

        return this._orientation;
    },
    scrollPosition: function (pos) {
        if (pos !== undefined) {
            this.$el.children('.jlv-widget-menu-body').scrollLeft(pos);
        }
        return this.$el.children('.jlv-widget-menu-body').scrollLeft();
    },
    checkScroll: function () {
        var scrollPos = this.get('scrollPosition'),
            w = this.$el.find('.jlv-widget-menu-body')[0].scrollWidth,
            elW = this.$el.width();
        this.$el.find('.jlv-widget-menu-left').widget().set('hidden', scrollPos <= 0);
        this.$el.find('.jlv-widget-menu-right').widget().set('hidden',
            (w <= elW ? true : scrollPos >= (w - elW))
        );
    }
});

JLV.Desktop.WidgetMenuItem = AppKit.View.extend({
    classID: 'jlv-widget-menu-item',
    initialize: function (options) {

        this.$el.draggable({
            revert: false,
            appendTo: 'body',
            helper: 'clone',

            stop: function () {
                var portal = $(this).data('portal');

                if (portal) {
                    portal.portal('removePlaceholder');
                }
            },

            drag: function (event, ui) {

                var portal = $(this).data('portal');

                if (portal) {
                    portal.portal('computeDrop', event, ui, true);
                }
            }
        });

        this.$el.attr('tabindex', 0);

        this.$el.on('mousedown', function (evt) {
            $(this).focus();
        });
        //508 nav
        this.$el.on('keydown', _.bind(function (evt) {
            if (evt.which === $.ui.keyCode.TAB) {
                evt.preventDefault();
                evt.stopPropagation();
                if (!evt.shiftKey) {
                    var nextWidget = this.$el.nextInDOM('.jlv-widget-menu-item');
                    if (nextWidget.length) {
                        nextWidget.focus();
                        return;
                    }

                    var windowItem = $('#WindowDock').nextInDOM('.jlv-window-button');
                    if (windowItem.length) {
                        setTimeout(function () {
                            windowItem.focus();
                        }, 0);
                        return;
                    }

                    $('#PatientSearchToolbarButton').focus();
                }
                else {

                    var prevWidget = this.$el.prevInDOM('.jlv-widget-menu-item');
                    if (prevWidget.length) {
                        prevWidget.focus();
                        return;
                    }

                    $('#AddWidgetButton').focus();
                }
            }
            else if (evt.which === 49) //press 1
            {
                var portal = JLV.Desktop.WidgetPortal.getActive();
                if (portal) {

                    var widgetPortal = portal.getActiveWidgetPortal();
                    if (widgetPortal) {
                        var widgetId = this.$el.data('widget-id');
                        if (widgetId) {
                            //drop the represented widget
                            var theWidget = JLV.createJLVWidget(widgetId, {
                                patient: JLV.ActivePatientModel,
                                portal: widgetPortal
                            }, true);

                            if (theWidget) {
                                widgetPortal.insertWidgetInColumnAtIndex(theWidget, 0, 1);
                            }
                        }
                    }

                }

            }
            else if (evt.which === 50) //press 2
            {

                var portal = JLV.Desktop.WidgetPortal.getActive();
                if (portal) {

                    var widgetPortal = portal.getActiveWidgetPortal();
                    if (widgetPortal) {
                        var widgetId = this.$el.data('widget-id');
                        if (widgetId) {
                            //drop the represented widget
                            var theWidget = JLV.createJLVWidget(widgetId, {
                                patient: JLV.ActivePatientModel,
                                portal: widgetPortal
                            }, true);

                            if (theWidget) {
                                widgetPortal.insertWidgetInColumnAtIndex(theWidget, 1, 0);
                            }
                        }
                    }

                }

            }
            else if (evt.which === 51) //press 3
            {
                var portal = JLV.Desktop.WidgetPortal.getActive();
                if (portal) {

                    var widgetPortal = portal.getActiveWidgetPortal();
                    if (widgetPortal) {
                        var widgetId = this.$el.data('widget-id');
                        if (widgetId) {
                            //drop the represented widget
                            var theWidget = JLV.createJLVWidget(widgetId, {
                                patient: JLV.ActivePatientModel,
                                portal: widgetPortal
                            }, true);

                            if (theWidget) {
                                widgetPortal.insertWidgetInColumnAtIndex(theWidget, 2, 0);
                            }
                        }
                    }
                }

            }
        }, this));


        this._super(options);
    },
    widgetId: function (aString) {
        if (aString !== undefined)
            this._widgetId = aString;

        return this._widgetId;
    }
});


//gets the current active widget menu
JLV.Desktop.WidgetMenu.getActive = function () {
    var activePortal = JLV.Desktop.WidgetPortal.getActive();
    if (activePortal) {
        var widgetMenu = null;
        if (activePortal.id === "ProviderPortal") {
            widgetMenu = AppKit.get("ProviderWidgetMenu");
        }
        else if (activePortal.id.indexOf("PatientPortal") > -1) {
            widgetMenu = AppKit.get("PatientWidgetMenu");
        }
    }

    return widgetMenu;
};

JLV.Desktop.WidgetMenu.hideActive = function () {

    var activePortal = JLV.Desktop.WidgetPortal.getActive();
    if (activePortal) {
        var widgetMenu = JLV.Desktop.WidgetMenu.getActive();
        if (widgetMenu) {

            widgetMenu.set('lastScrollPos', widgetMenu.get('scrollPosition'));

            if (widgetMenu.get('orientation') === 'bottom') {
                widgetMenu.$el.animate({
                    "height": 0
                }, 200, function () {
                    widgetMenu.set('hidden', true);
                });

                $('#AddWidgetButton').widget().set('showMode', true);
            }
            else {

                widgetMenu.$el.animate({
                    "width": 0
                }, 200, function () {
                    widgetMenu.set('hidden', true);
                });

            }

        }
    }
};

JLV.Desktop.WidgetMenu.showActive = function () {

    var activePortal = JLV.Desktop.WidgetPortal.getActive();
    if (activePortal) {
        var widgetMenu = JLV.Desktop.WidgetMenu.getActive();
        if (widgetMenu) {

            widgetMenu.set('orientation', JLV.Session.ui_conf.options[0].menu || 'bottom');
            widgetMenu.set('hidden', false);
            widgetMenu.$el.css('bottom', $('#WindowDock').height());


            if (widgetMenu.get('orientation') === 'bottom') {

                widgetMenu.set('scrollPosition', widgetMenu.get('lastScrollPos'));
                widgetMenu.$el.animate({
                    height: 140
                }, 200);

                $('#AddWidgetButton').widget().set('showMode', false);
                widgetMenu.checkScroll();
            }
            else {
                widgetMenu.set('scrollPosition', widgetMenu.get('lastScrollPos'));
                widgetMenu.$el.animate({
                    width: 240
                }, 200);

            }

        }

    }
};

var _mouseDownInMenu = false;

$(function () {

    $(document).on('click', function () {
        if (!_mouseDownInMenu) {
            JLV.Desktop.WidgetMenu.hideActive();
        }
    }).on('mousedown', function () {
        _mouseDownInMenu = false;
    });


});

