JLV.DetailsWindow = AppKit.Dialog.extend({
    className: "jlv-detail-dialog",
    options: {
        closable: true,
        maximizable: true,
        minimizable: true,
        width: 600,
        height: 400,
        url: null,
        modal: false,
        queryParams: null,
        print: true,
        clipboard: true,
        vlerDomain: false,
        dasvlerDomain: false,
        reportBuilder: false
    },
    initialize: function (options) {

        var self = this;
        var tools = $('<div>').addClass('jlv-window-tools');

        this._printBtn = $('<div title="Print" tabindex="0" class="jlv-window-toolbutton">')
            .append('<div class="icon-print">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doPrint)) {
                    self.doPrint();
                }
            }).appendTo(tools);

        this._clipboardBtn = $('<div title="Copy to clipboard" tabindex="0" class="jlv-window-toolbutton">')
            .append('<div class="icon-clipboard2">')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doClipboard)) {
                    self.doClipboard();
                }
            }).appendTo(tools);


        this._reportBuilderBtn = $('<div title="Add to Report Builder" tabindex="0" class="jlv-window-toolbutton" style="padding-top:5px;">')
            .append('<div class="icon-plus">')
            .button()
            .on('click', function () {
                self.addToReportBuilder(self.model);
            }).appendTo(tools);


        if (JLV.Session.dodVLER || JLV.Session.dasVLER) {

            this._vlerLoadBtn = $('<div title="Force VLER Data Refresh - This can take a while" tabindex="0" class="jlv-window-toolbutton vler">')
                .append('<div class="icon-refresh-alt">')
                .button()
                .on('click', function () {
                    //self.vlerLoadMessage("Community partner data retrieval in progress...");

                    AppKit.AlertDialog.yesNo('Confirm VLER Refresh',
                        'A forced refresh of community partner data may take a few minutes. Click Yes to continue. Click No to cancel.',
                        function (yes) {
                            if (yes) {
                                JLV.Session.forceRefresh = true;
                                if (_.isFunction(JLV.LoadVlerData)) {
                                    JLV.LoadVlerData();
                                }
                            }
                        }
                    );
                }).appendTo(tools);
        }

        this._super(options);

        this.$el.on('dialogclose', function () {
            $(this).remove(); //destroy on close
        });

        tools.appendTo(this.$el.parent().find('.ui-dialog-titlebar-tools'));

        JLV.Desktop.registerWindow(this);

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });


        this.loadDetails();
    },
    _disablePrintButtons: function (aFlag) {
        if (aFlag) {
            this._reportBuilderBtn.button('option', 'disabled', true);
            this._reportBuilderBtn.attr('title', 'Adding to Report Builder is disabled until details load');

            this._printBtn.button('option', 'disabled', true);
            this._printBtn.attr('title', 'Printing is disabled until details load');

            this._clipboardBtn.button('option', 'disabled', true);
            this._clipboardBtn.attr('title', 'Copying is disabled until details load');
        }
        else {
            this._reportBuilderBtn.button('option', 'disabled', false);
            this._reportBuilderBtn.attr('title', 'Add to Report Builder');

            this._printBtn.button('option', 'disabled', false);
            this._printBtn.attr('title', 'Print');

            this._clipboardBtn.button('option', 'disabled', false);
            this._clipboardBtn.attr('title', 'Copy to clipboard');
        }
    },

    loadDetails: function () {
        var url = this.get('url'),
            query = this.get('queryParams');

        this._disablePrintButtons(false);

        if (url && query) {
            $('<div>')
                .addClass('jlv-loader').html(
                '<div class="jlv-loader-image">' +
                '<div class="jlv-table-data-loading" alt="loading indicator"></div></div>')
                .appendTo(this.$el);

            var self = this;

            this._disablePrintButtons(true);
            $.post(url, query, function (response) {

                self.response = response;
                if (_.isFunction(self.renderer)) {
                    self.$el.html('<div style="padding:10px;">' + self.renderer(self.response, false) + '</div>');
                }

                self._loaded = true;
                self._disablePrintButtons(false);

            }).fail(function (response) {
                self.response = response;
                if (_.isFunction(self.fail)) {
                    self.$el.html('<div style="padding:10px;">' + self.fail(response) + '</div>');
                }
                else {
                    self.$el.html('<div style="padding:10px;">An error has occurred.</div>');
                }
            });
        }
        else {
            if (_.isFunction(this.renderer))
                this.$el.html('<div style="padding:10px;">' + this.renderer({}, false) + '</div>');
        }
    },
    _printableHtml: function () {

        var html = ''

        if (_.isFunction(this.renderer)) {
            if (this.response) {
                html = '<div><div style="padding:10px;">' + this.renderer(this.response, true) + '</div></div>';
            }
            else {
                html = '<div><div style="padding:10px;">' + this.renderer({}, true) + '</div></div>';
            }
        }
        else {
            html = '<div>' + this.$el.html() + '</div>';
        }

        return html;

    },

    doPrint: function () {

        var patient = JLV.ActivePatientModel;

        var title = _.text(this.get('title')),
            html = this._printableHtml();

        $.post('print/setup', {
            patToken: patient.get('patientToken'),
            data: JSON.stringify({
                title: title,
                data: html,
                patient: patient.toJSON(),
                datetime: moment().format('MMMM DD, YYYY h:mm a')
            })
        }, function (response) {

            var jobID = response;

            JLV.Widget.PrintConfirm(function (ok) {

                if (ok) {
                    var printUrl = 'print/details';
                    printUrl += ('?jobId=' + jobID);
                    printUrl += ('&patToken=' + patient.get('patientToken'));
                    window.open(printUrl, '_blank');
                }
            });

        });
    },
    doClipboard: function () {
        JLV.common.copyWindowHTML(this.$el.html());
    },
    print: function (aBool) {

        if (aBool === true) {
            this._printBtn.show();
        }
        else if (aBool === false) {
            this._printBtn.hide();
        }

        return this._printBtn.is(':visible');
    },
    clipboard: function (aBool) {
        if (aBool === true) {
            this._clipboardBtn.show();
        }
        else if (aBool === false) {
            this._clipboardBtn.hide();
        }

        return this._clipboardBtn.is(':visible');
    },
    vlerDomain: function (aBool) {

        if (this._vlerLoadBtn) {
            if (aBool === true) {
                this._vlerLoadBtn.show();
            }
            else if (aBool === false) {
                this._vlerLoadBtn.hide();
            }

            return this._vlerLoadBtn.is(':visible');
        }

        return false;

    },

    reportBuilder: function (aBool) {

        if (aBool === true) {
            this._reportBuilderBtn.show();

        }
        else if (aBool === false) {
            this._reportBuilderBtn.hide();
        }

        return this._reportBuilderBtn.is(':visible');
    },


    addToReportBuilder: function (model) {

        if (model) {

            model.set('details', this._printableHtml());
            model.set('detailsTitle', this.get('title'));

            if (this.widget) {

                if (_.isFunction(this.widget._addToReportBuilder)) {
                    this.widget._addToReportBuilder(model);
                }
                else {

                    model.set('name', this.widget.get('name'));
                    var columns = this.widget.options.columns;
                    JLV.ReportBuilderController.addRecord(model, columns, columns);

                }
            }

        }

    }


});



