//= require ./WidgetWindow.js

jQuery(function () {


    JLV.WidgetWindow.FilterPanel = AppKit.View.extend({
        className: 'jlv-more-filter-panel',
        template: $('#WidgetMoreFilterTemplate').template(),
        window: null,
        filters: [],
        initialize: function (options) {

            this._super(options);

            var self = this;
            this.subview('.jlv-more-filter-close').on('click', function () {
                if (self.window) {
                    self.window.toggleFilterPanel();
                }
            }).on('keydown', function (evt) {
                if (evt.which === $.ui.keyCode.SPACE) {
                    if (self.window) {
                        self.window.toggleFilterPanel();
                        setTimeout(function () {
                            self.window.$el.find('.filter').focus();
                        }, 0);
                    }
                }
            });

            this.updateFilters();
        },
        clearFilters: function (skip) {

            var combo = null;
            if (this.filters.length > 0 && skip != 0) {
                combo = this.subview('.first-combo');
                combo.uncheckAll();
            }

            if (this.filters.length > 1 && skip != 1) {
                combo = this.subview('.second-combo');
                combo.uncheckAll();

            }

            if (this.filters.length > 2 && skip != 2) {
                combo = this.subview('.third-combo');
                combo.uncheckAll();

            }

            if (this.filters.length > 3 && skip != 3) {
                combo = this.subview('.fourth-combo');
                combo.uncheckAll();
            }


        },
        updateFilters: function () {

            if (this.filters.length > 0) {

                this.$el.find('.first-filter').css('display', 'inline-block');
                this.$el.find('.first-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.first-combo'), 0);
            }
            else {
                this.$el.find('.first-filter').hide();
            }

            if (this.filters.length > 1) {
                this.$el.find('.second-filter').css('display', 'inline-block');
                this.$el.find('.second-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.second-combo'), 1);
            }
            else {
                this.$el.find('.second-filter').hide();
            }

            if (this.filters.length > 2) {
                this.$el.find('.third-filter').css('display', 'inline-block');
                this.$el.find('.third-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.third-combo'), 2);
            }
            else {
                this.$el.find('.third-filter').hide();
            }

            if (this.filters.length > 3) {
                this.$el.find('.fourth-filter').css('display', 'inline-block');
                this.$el.find('.fourth-filter').removeAttr('aria-hidden');
                this._updateFilterAtIndex(this.subview('.fourth-combo'), 3);
            }
            else {
                this.$el.find('.fourth-filter').hide();
            }
        },
        _updateFilterAtIndex: function (combo, idx) {

            if (combo && idx < this.filters.length) {

                combo.set('title', 'Filter by ' + _.text(this.filters[idx].title));
                combo.$el.attr('data-column', this.filters[idx].id);

                var values = _.uniq(this.data.pluck(this.filters[idx].id)).sort();

                var count = values.length,
                    i = 0,
                    filterItems = [];

                for (; i < count; i++) {
                    var val = values[i],
                        formatter = this.filters[idx].formatter;
                    if (val) {

                        val = _.text(_.isFunction(formatter) ? formatter(val) : val);
                        filterItems.push(val);

                    }
                }

                filterItems = _.uniq(filterItems).sort();

                var menuItems = [];

                i = 0;
                count = filterItems.length;

                for (; i < count; i++) {
                    var item = filterItems[i];
                    if (item && item.trim().length > 0) {
                        menuItems.push({
                            'data-label': item
                        });
                    }
                }


                combo.set('items', menuItems);

                var self = this;
                combo.set('action', function () {
                    self.clearFilters(idx);
                    var val = this.get('selected');
                    self.window.$el.trigger('beforeFilter', [val]);
                    self.applyFilters();
                    self.window.$el.trigger('afterFilter', [val]);
                });
            }
        },
        filterBy: function (filterInfo) {

            var $combo = this.$el.find('[data-column="' + filterInfo.id + '"]');
            if ($combo) {
                var combo = $combo.widget();
                if (combo) {
                    this.clearFilters();
                    combo.set('selected', filterInfo.value);
                    this.window.$el.trigger('beforeFilter', [[filterInfo.value]]);
                    this.applyFilters();
                    this.window.$el.trigger('afterFilter', [[filterInfo.value]]);
                }
            }
        },
        applyFilters: function () {

            if (this.window) {

                var datagrid = this.window.get('datagrid');
                var self = this;

                datagrid.set('filter', function (model) {
                    var keep = true;
                    var val = [];
                    if (self.filters.length > 0) {
                        var firstCombo = self.subview('.first-combo');
                        if(firstCombo) {
                            val = firstCombo.get('selectedIds');
                        }

                        if (val.length === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[0].formatter,
                                testVal = model.get(self.filters[0].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (_.indexOf(val, testVal.hashCode()) > -1);

                        }
                    }
                    if (self.filters.length > 1) {
                        var secondCombo = self.subview('.second-combo')
                        if(secondCombo) {
                            val = secondCombo.get('selectedIds');
                        }

                        if (val.length === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[1].formatter,
                                testVal = model.get(self.filters[1].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (_.indexOf(val, testVal.hashCode()) > -1);
                        }
                    }
                    if (self.filters.length > 2) {
                        var thirdCombo = self.subview('.third-combo')
                        if(thirdCombo) {
                            val = thirdCombo.get('selectedIds');
                        }
                        if (val.length === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[2].formatter,
                                testVal = model.get(self.filters[2].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (_.indexOf(val, testVal.hashCode()) > -1);
                        }
                    }

                    if (self.filters.length > 3) {
                        var fourthCombo = self.subview('.fourth-combo')
                        if(fourthCombo) {
                            val = fourthCombo.get('selectedIds');
                        }

                        if (val.length === 0) {
                            keep = keep && true;
                        }
                        else {
                            var formatter = self.filters[3].formatter,
                                testVal = model.get(self.filters[3].id);
                            testVal = _.text(_.isFunction(formatter) ? formatter(testVal) : testVal);

                            keep = keep && (_.indexOf(val, testVal.hashCode()) > -1);
                        }
                    }

                    return keep;
                });

                this.window.renderData();
            }
        }
    });


    JLV.WidgetWindow.DateFilterPanel = JLV.WidgetWindow.FilterPanel.extend({
        className: 'jlv-more-date-filter-panel',
        template: $('#WidgetMoreDateFilterTemplate').template(),
        window: null,
        startDate: null,
        endDate: null,
        filters: [],
        initialize: function (options) {

            this._super(options);

            var self = this;
            this.subview('.jlv-more-date-filter-apply').on('click', function () {
                if (self.window) {
                    var data = self.window.get('data');
                    if (data) {

                        data.startDate = moment(self.startDate);
                        data.endDate = moment(self.endDate);

                        var startDay = data.startDate.diff(moment(), 'days');
                        if (startDay > 0) {//fix diff rounding error
                            startDay++;
                        }

                        var endDay = data.endDate.diff(moment(), 'days');
                        if (endDay > 0) {
                            endDay++;
                        }

                        self.window.widget.set('startDay', startDay);
                        self.window.widget.set('endDay', endDay);
                        self.window.loadData();

                        if (self.window.widget.savePortalUI) {
                            self.window.widget.savePortalUI();
                        }
                    }
                }
            });


            this.$el.find('.preset')
                .on('click', function () {
                    self.applyDatePreset($(this).data('value'));

                })
                .on('keydown', function (evt) {
                    console.log(evt.which)
                    if (evt.which === $.ui.keyCode.SPACE || evt.which === $.ui.keyCode.ENTER) {
                        self.applyDatePreset($(this).data('value'));
                    }
                });

            this._startDateInput = this.subview('.jlv-start-date-input');
            this._endDateInput = this.subview('.jlv-end-date-input');

            this._startDateInput.set('date', this.startDate);
            this._endDateInput.set('date', this.endDate);

            this._startDateInput.on('dateSet', function () {
                self.startDate = this.get('date');
                self._endDateInput.set('minDate', self.startDate);
            });

            this._endDateInput.on('dateSet', function () {
                self.endDate = this.get('date');
                self._startDateInput.set('maxDate', self.endDate);
            });

        },
        disabled: function (aFlag) {

            if (aFlag !== undefined) {
                this.subview('.jlv-more-date-filter-apply').set('enabled', !aFlag);
                this._startDateInput.set('enabled', !aFlag);
                this._endDateInput.set('enabled', !aFlag);
            }

        },
        applyDatePreset: function (value) {

            if (this.window && !this.window.widget.isLoading()) {
                var data = this.window.get('data');
                if (data) {
                    data.endDate = moment();
                    data.startDate = moment();
                    if (value === '1w') {
                        data.startDate.subtract(7, 'days');
                    }
                    else if (value === '2w') {
                        data.startDate.subtract(14, 'days');
                    }
                    else if (value === '1m') {
                        data.startDate.subtract(1, 'months');
                    }
                    else if (value === '3m') {
                        data.startDate.subtract(3, 'months');
                    }
                    else if (value === '6m') {
                        data.startDate.subtract(6, 'months');
                    }
                    else if (value === '1y') {
                        data.startDate.subtract(1, 'years');
                    }
                    else if (value === '2y') {
                        data.startDate.subtract(2, 'years');
                    }
                    else if (value === '3y') {
                        data.startDate.subtract(3, 'years');
                    }

                    this.window.widget.set('endDay', data.endDate.diff(moment(), 'days'));
                    this.window.widget.set('startDay', data.startDate.diff(moment(), 'days'));

                    //end date must be set before start date
                    this._endDateInput.set('date', data.endDate.toDate());
                    this._startDateInput.set('date', data.startDate.toDate());


                    this.window.loadData();
                }
            }
        },
        applyFilters: function () {

            if (this.window) {
                var data = this.window.widget.get('data');

                this._startDateInput.set('date', data.startDate.toDate());
                this._endDateInput.set('date', data.endDate.toDate());
            }

            this._super();

        }
    });
});