//= require lib/AppKit.js
//= require ../JLV.js
//= require ../desktop/WindowManager.js

JLV.WidgetWindow = AppKit.Dialog.extend({
    classID: 'jlv-widget-window',
    normCodeSystems: [],
    widget: null,
    options: {
        name: "",
        closable: true,
        dateFilter: false,
        vlerDomain: false,
        dasvlerDomain: false,
        filter: false,
        print: false,
        fitColumns: true,
        refresh: false,
        status: false,
        settings: true,
        clipboard: false,
        width: 800,
        height: 560,
        minWidth: 800,
        maximizable: true,
        minimizable: true,
        columns: undefined,
        filters: [],
        data: null,
        defaultSort: null,
        defaultFilter: null,
        gridHeaderHeight: 24,
        groupBy: undefined,
        groupFormatter: undefined,
        groupSorter: undefined,
        displayMode: 'paged'
    },
    initialize: function (options) {

        this._super(_.extend({dialogClass: 'jlv-widget-window'}, options));

        this.$el.html('<table style="width:100%;height:100%;"><td style="text-align:center;">Rendering expanded view...</td></table>');

        JLV.Desktop.registerWindow(this);

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });

        this.set('title', this.get('name'));

        this._isSetup = false;

        this.on('dialogopen', _.bind(function () {
            if (!this._isSetup) {
                setTimeout(_.bind(function () {
                    this._setup();
                    this.onBecomesVisible();
                }, this), 5);
            }
            else {
                this.onBecomesVisible();
            }

        }, this));
    },


    _setup: function () {

        this._isSetup = true;

        this.$el.empty();

        var self = this;
        this._tools = $('<div>').addClass('jlv-window-tools');

        this._refreshBtn = $('<div title="Refresh" tabindex="0" class="jlv-window-toolbutton refresh">')
            .append('<div class="icon-refresh">')
            .attr('aria-label', 'Refresh button, press space bar to activate.')
            .button()
            .prependTo(this._tools)
            .on('click', function (evt) {
                self.loadData();
            });

        this.set('refresh', this._refresh);


        this._settingsBtn = $('<div title="Column Settings" tabindex="0" class="jlv-window-toolbutton settings">')
            .append('<div class="icon-columns">')
            .attr('aria-label', 'Settings button, press space bar to activate.')
            .button()
            .prependTo(this._tools);

        this.set('settings', this._settings);

        this._setupColumnSettings();

        this._printBtn = $('<div title="Print" tabindex="0" class="jlv-window-toolbutton print">')
            .append('<div class="icon-print">')
            .attr('aria-label', 'Print button, press space bar to activate.')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doPrint)) {
                    self.doPrint();
                }
            }).prependTo(this._tools);

        this.set('print', this._print);

        this._clipboardBtn = $('<div title="Copy to clipboard" tabindex="0" class="jlv-window-toolbutton clipboard">')
            .append('<div class="icon-clipboard2">')
            .attr('aria-label', 'Copy button, press space bar to activate.')
            .button()
            .on('click', function () {
                if (_.isFunction(self.doClipboard)) {
                    self.doClipboard();
                }
            }).prependTo(this._tools);

        this.set('clipboard', this._clipboard);


        if (this.options && (JLV.Session.dodVLER && this.options.vlerDomain) || (JLV.Session.dasVLER && this.options.dasvlerDomain)) {
            this._vlerLoadBtn = $('<div title="Force VLER Data Refresh - This can take a while" tabindex="0" class="jlv-window-toolbutton vler">')
                .append('<div class="icon-refresh-alt">')
                .attr('aria-label', 'VLER refresh button, press space bar to activate.')
                .button()
                .on('click', function () {
                    //self.vlerLoadMessage("Community partner data retrieval in progress...");

                    AppKit.AlertDialog.yesNo('Confirm VLER Refresh',
                        'A forced refresh of community partner data may take a few minutes. Click Yes to continue. Click No to cancel.',
                        function (yes) {
                            if (yes) {
                                JLV.Session.forceRefresh = true;
                                if (_.isFunction(JLV.LoadVlerData)) {
                                    JLV.LoadVlerData();
                                }
                            }
                        }
                    );
                }).prependTo(this._tools);
        }

        this._filterBtn = $('<div title="Filter" tabindex="0" class="jlv-window-toolbutton filter">')
            .append('<div class="icon-filter">')
            .attr('aria-label', 'Filter toggle button, press space bar to activate.')
            .button()
            .on('click', function () {
                self.toggleFilterPanel();
            }).prependTo(this._tools);

        //this.set('filter', this._filter);

        this._addBtn = $('<div title="Add" tabindex="0" class="jlv-window-toolbutton add">')
            .append('<div class="icon-plus">')
            .attr('aria-label', 'Add button, press space bar to activate.')
            .button()
            .prependTo(this._tools).hide();

        this.set('plus', this._plus);

        this._interfaceStatusBtn = $('<div title="Interface Status" tabindex="0" class="jlv-window-toolbutton status">')
            .append('<div class="icon-info">')
            .attr('aria-label', 'Status button, press space bar to activate.')
            .button()
            .on('click', function () {
                if (self.widget) {
                    self.widget.showInterfaceStatus();
                }
                else if (_.isFunction(self.showInterfaceStatus)) {
                    self.showInterfaceStatus();
                }
            }).prependTo(this._tools);

        this.set('status', this._status);


        this._body = $('<div>').addClass('jlv-window-body').appendTo(this.$el);
        this._footer = $('<div>').addClass('jlv-window-footer').appendTo(this.$el);

        if (this.options && this.options.columns) {
            var tablePager = new AppKit.Pager();

            this._datagrid = new AppKit.PagedTableView({
                headerHeight: this.options.gridHeaderHeight,
                controller: this,
                pager: tablePager,
                fitColumns: this.options.fitColumns
            });

            this._datagrid.$el.addClass('jlv-window-table').appendTo(this._body);

            tablePager.$el.appendTo(this._footer);

            tablePager.on('selection', function () {
                self._updatePageInfoDisplay();
            });

            this._showAllBtn = new AppKit.Button({
                className: 'jlv-show-all-btn',
                label: 'Show All',
                action: function () {

                    if (self.get('displayMode') === 'all') {
                        self.set('displayMode', 'paged');
                    }
                    else {
                        self.set('displayMode', 'all');
                    }
                }
            });

            this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {

                if (this.widget) {
                    if (_.findIndex(this.widget.options.columns, {
                            id: sortInfo.column.id
                        }) > -1) {

                        this.widget._defaultSort = {
                            asc: sortInfo.asc,
                            id: sortInfo.column.id
                        };

                        if (_.isFunction(this.widget.savePortalUI)) {
                            this.widget.savePortalUI();
                        }
                    }

                }

                this._defaultSort = {
                    asc: sortInfo.asc,
                    id: sortInfo.column.id
                };

            }, this));

            this._footer.append(this._showAllBtn.$el);

            this._tablePageInfo = $('<span>')
                .addClass('jlv-table-page-info')
                .appendTo(this._footer)
                .hide();

        }


        this._dateLabel = $('<div tabindex="0">').addClass('jlv-widget-window-dateLabel jlv-clickable-link')
            .attr("unselectable", "on")
            .button()
            .on('click', _.bind(function () {
                this.toggleFilterPanel();
            }, this));


        this.set('filter', this._filter);

        var header = this.$el.parent().find('.ui-widget-header');

        header.append($('<br/>'));

        this._dateLabel.appendTo(header).hide();
        this._tools.appendTo(this.$el.parent().find('.ui-dialog-titlebar-tools'));

        if (this._datagrid) {
            this._datagrid.sizer();
            this.$el.on('finishresize maximized restored', function () {
                self._datagrid.sizer();
            });
        }

        this.set('columns', this._columns);
        this.set('hiddenColumns', this._hiddenColumns);
        this.set('groupFormatter', this._groupFormatter);

        if (this.widget) {
            this.set('data', this.widget.get('data'));

            if (_.isFunction(this.widget.itemClicked)) {
                this._datagrid.on('afterRender.click', function () {

                    this.$el.find('.jlv-clickable-link').on('click', function () {
                        var modelId = $(this).data('model-id'),
                            model = self._datagrid.get('source').get(modelId);

                        self.widget.itemClicked(model, $(this));
                        $(this).addClass('visited');
                    });
                });
            }

            this.$el.addClass(this.widget.get('widgetId'));
        }


        if (this._filter)
            this.toggleFilterPanel();


        this.adjustHeight();


        if (_.isUndefined(this.get('data'))) {
            this.set('data', this._data);
        }
    },
    /**
     * Update the pages displayed information in the widget footer
     */
    _updatePageInfoDisplay: function () {

        if (this._datagrid) {
            var range = this._datagrid.displayedRange(),
                count = this._datagrid.get('rowCount');
            if (range && count) {
                this._tablePageInfo.html(_.template('Displaying {{start}}-{{end}} of {{total}}')({
                    start: range.start + 1,
                    end: range.end,
                    total: count
                })).show();
            }
            else {
                this._tablePageInfo.hide();
            }
        }

        this._body.css({
            bottom: this._footer.outerHeight()
        });

        setTimeout(_.bind(function () {
            this.adjustHeight();
        }, this), 0);
    },

    updateDateLabel: function () {

        var data = this.get('data');
        if (data && data.startDate && data.endDate) {

            var beginDate = data.startDate.format("M/D/YY"),
                endDate = data.endDate.format("M/D/YY");

            this._dateLabel.text('Filtered date range: ' + beginDate + " - " + endDate);
            this.adjustHeight();
        }
    },

    _setupColumnSettings: function () {

        this._settingsBtn.on('click', _.bind(function (evt) {

            evt.stopPropagation();

            if (this._settingsPopover) {
                this._settingsPopover.remove();
                this._settingsPopover = null;
                $(document).off('click.columnSettings, keydown.columnSettings');
                return;
            }

            this._settingsPopover = $('<div>')
                .addClass('ui-tooltip bottom arrow').hide().appendTo('body');


            var content = $('<div>').addClass('content').on('click', function (evt) {
                evt.stopPropagation();
            });

            var columns = this._datagrid.get('columns'),
                count = columns.length,
                i = 0;

            var table = $('<table>'),
                tr = null;
            for (; i < count; i++) {

                if (i % 2 == 0) {

                    if (tr !== null) {
                        table.append(tr);
                    }

                    tr = $('<tr>')
                }

                var col = columns[i];

                var td = $('<td>');

                td.append(
                    $(String.format('<input type="checkbox" value="{0}" {1} />',
                        col.id, (col.isHidden ? '' : 'checked')))
                );

                td.append($(String.format('<span>&nbsp;{0}</span>', col.title)));

                tr.append(td);
            }

            if (tr !== null) {
                table.append(tr);
            }


            content.append(table);

            content.append(
                $('<div class="primary" style="margin-top:14px;;height:25px;width:80px;">Apply</div>')
                    .button().on('click', _.bind(function () {

                        this._settingsPopover.remove();
                        this._settingsPopover = null;

                        var notChecked = content.find('input:checkbox:not(:checked)');

                        var toHide = [];
                        $.each(notChecked, function (idx, el) {

                            var id = $(el).val();
                            toHide.push(id);

                        });

                        this.set('hiddenColumns', toHide);
                        this.$el.trigger('columnsHidden', [toHide]);

                        if (this.widget)
                            this.widget.savePortalUI();

                        $(document).off('click.columnSettings, keydown.columnSettings');
                    }, this))
            );

            this._settingsPopover.append(content);

            this._settingsPopover.css({
                left: (this._settingsBtn.offset().left -
                this._settingsPopover.outerWidth() / 2 + 12),
                top: this._settingsBtn.offset().top + 24
            });

            this._settingsPopover.show();

            var self = this;
            $(document).on('click.columnSettings, keydown.columnSettings', function (evt) {
                if (self._settingsPopover) {
                    self._settingsPopover.remove();
                }
                self._settingsPopover = null;
                $(document).off('click.columnSettings, keydown.columnSettings');
            });

        }, this));


    },



    doClipboard: function () {

        if (this._datagrid && this.get('displayMode') !== 'all') {
            var clone = this._datagrid.$el.clone();
            clone.find('.appkit-table-column, .ui-datatable-loading-rows, .dummy-row-1, .dummy-column, .hidden').remove();
            JLV.common.copyGridPanelHTML(clone.html());
        }

    },
    loadData: function () {


        if (this.widget && this._datagrid) {

            this.widget.loadData(_.bind(function () {
                if (this._displayMode === 'all') {
                    this._datagrid.set('recordsPerPage', this.get('data').size());
                }

                this.renderData();
                JLV.VLERDataAvailable();
            }, this));

            this._datagrid.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');


            this._updatePageInfoDisplay();
        }
    },
    vlerLoadMessage: function (mesg){

        if (!this._vlerLoadPanel) {

            var data = this.get('data');

            if (data) {

                this._vlerLoadPanel = new JLV.WidgetWindow.VlerLoadPanel({
                    window: this
                });

                if (this._vlerLoadPanel) {
                    this._vlerLoadPanel.$el.insertBefore(this._body);
                    this._vlerLoadPanel.$el.hide();
                }
            }
        }

        if (this._vlerLoadPanel) {
            var $loading = this.$el.find('.jlv-vler-loading');

            $loading.html(mesg);
            $loading.removeClass('jlv-vler-loading-img');

            if (this._vlerLoadPanel.get('hidden')) {
                this._vlerLoadPanel.set('hidden', false);
            }

            this.adjustHeight();
        }
    },
    displayMode: function (mode) {

        if (mode !== undefined && this._datagrid) {

            this._displayMode = mode;

            if (this._displayMode === 'all') {


                this._datagrid.set('groupToggle', false);
                this._datagrid.set('recordsPerPage', this.get('data').size());
                this.renderData(true);
                this._updatePageInfoDisplay();
                this._showAllBtn.set('label', 'Show Paged');

                this._clipboardBtn.tooltip({
                    content: 'Clipboard is disabled when displaying the table in "Show All" mode. To enable the clipboard, please set the table to "Show Paged" mode.'
                });

                this._clipboardBtn.css({opacity: 0.5});

                this._printBtn.tooltip({
                    content: 'Printing is disabled when displaying the table in "Show All" mode. To enable printing, please set the table to "Show Paged" mode.'
                });

                this._printBtn.css({opacity: 0.5});

            }
            else if (this._datagrid && this._displayMode === 'paged') {

                this._datagrid.set('groupToggle', true);
                this._datagrid.set('recordsPerPage', 25);

                this.renderData(true);
                this._updatePageInfoDisplay();
                this._showAllBtn.set('label', 'Show All');


                this._clipboardBtn.css({opacity: 1.0});
                if (this._clipboardBtn.tooltip('instance'))
                    this._clipboardBtn.tooltip('destroy');

                this._printBtn.css({opacity: 1.0});
                if (this._printBtn.tooltip('instance'))
                    this._printBtn.tooltip('destroy');

            }
        }


        return this._displayMode;
    },
    hideVlerLoadMessage: function (){
        if (this._vlerLoadPanel) {
            this._vlerLoadPanel.set('hidden', true);
            this.adjustHeight();
        }
    },
    renderData: function () {

        if (this._datagrid) {
            if (this._datagrid.get('rowCount') === 0) {
                this._datagrid.showMessage("No Data");
            }
            else if (this._defaultSort !== null) {
                var column = _.findWhere(this.options.columns, {id: this._defaultSort.id});
                if (column) {
                    column.index = _.indexOf(this.options.columns, column);

                    var sortInfo = {
                        asc: this._defaultSort.asc,
                        column: column
                    };

                    this._datagrid.sortByColumn(sortInfo);
                    this._datagrid.render();

                    this._updatePageInfoDisplay();

                }
                else {
                    this._datagrid.render();
                }

            }
            else {
                this._datagrid.render();
            }

            this._updatePageInfoDisplay();

            //update the window title to reflect the number of records
            this.set('title', this.get('name') + ' (' + this._datagrid.get('rowCount') + ')');


            this._dataRendered = true;

        }
    },
    /** data setter & getter */
    data: function (aBBCollection) {

        if (this._datagrid) {

            if (aBBCollection !== undefined) {

                this._dataRendered = false;
                if (typeof(aBBCollection) === 'function') {
                    this._datagrid.set('source', aBBCollection ? new aBBCollection() : null);
                }
                else {
                    this._datagrid.set('source', aBBCollection);

                    if (this.get('visible')) {
                        this.renderData();
                    }

                    if (this._filterPanel) {
                        this._filterPanel.updateFilters();
                        this._filterBtn.applyFilters();
                    }

                    this.updateDateLabel();
                }

                var data = this._datagrid.get('source');
                data.bind('reset', _.bind(function () {
                    if (this._filterPanel) {
                        this._filterPanel.updateFilters();
                    }
                }, this));
            }

            return this._datagrid.get('source');
        }
        else {
            if (aBBCollection !== undefined) {
                this._data = aBBCollection;
            }
        }
    },
    datagrid: function () {
        return this._datagrid;
    },


    hiddenColumns: function (anArrayOfColumnIds) {

        this._hiddenColumns = anArrayOfColumnIds || [];

        if (this._datagrid) {
            if (anArrayOfColumnIds !== undefined) {
                this._datagrid.set('hiddenColumns', anArrayOfColumnIds);
                if (this.widget)
                    this.widget.set('hiddenColumnsExpanded', anArrayOfColumnIds)
            }


        }

        return this._hiddenColumns;

    },

    /**
     * Columns setter/getter
     * @param anArray - an Array of column config objects
     * @returns anArray of column configurations
     */
    columns: function (anArray) {

        if (anArray !== undefined) {

            this._columns = anArray;


            if (this._datagrid) {
                var colcount = this._columns.length,
                    i = 0;

                this._datagrid.removeAllColumns();

                for (; i < colcount; i++) {
                    var colInfo = this._columns[i];
                    this._datagrid.addColumn(colInfo);
                }
            }
        }

        return this._columns;
    },
    /**
     * widget window clipboard tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    clipboard: function (aBOOL) {

        this._clipboard = aBOOL === undefined ? this._clipboard : aBOOL;

        if (this._clipboardBtn) {
            if (aBOOL === true) {
                this._clipboardBtn.show();
            }
            else if (aBOOL === false) {
                this._clipboardBtn.hide();
            }
        }


        return this._clipboard;
    },
    /**
     * widget window filter tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    filter: function (aBOOL) {

        this._filter = aBOOL === undefined ? this._filter : aBOOL;

        if (this._filterBtn) {
            if (aBOOL === true) {
                this._filterBtn.show();

            }
            else if (aBOOL === false) {
                this._filterBtn.hide();

            }
        }


        return this._filter;

    },

    dateFilter: function (aBOOL) {

        this._datefilter = aBOOL === undefined ? this._datefilter : aBOOL;

        if (this._dateLabel) {
            if (aBOOL === true) {
                this._dateLabel.show();

            }
            else if (aBOOL === false) {
                this._dateLabel.hide();

            }
        }


        return this._datefilter;

    },
    /**
     * widget date filter tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    vlerDomain: function (aBOOL) {

        this._vlerDomain = aBOOL === undefined ? this._vlerDomain : aBOOL;

        if (this._vlerLoadBtn) {
            if (aBOOL === true) {
                this._vlerLoadBtn.show();
            }
            else if (aBOOL === false) {
                this._vlerLoadBtn.hide();
            }
        }

        return this._vlerDomain;

    },
    /**
     * DAS VLER switch
     * @param aBOOL
     * @returns {*}
     */
    dasvlerDomain: function (aBOOL) {

        this._dasvlerDomain = aBOOL === undefined ? this._dasvlerDomain : aBOOL;

        if (this._vlerLoadBtn) {
            if (aBOOL === true) {
                this._vlerLoadBtn.show();
            }
            else if (aBOOL === false) {
                this._vlerLoadBtn.hide();
            }
        }

        return this._dasvlerDomain;

    },

    /**
     * does widget have column settings getter/setter
     * @param aBOOL
     * @returns {*}
     */
    settings: function (aBOOL) {

        this._settings = aBOOL === undefined ? this._settings : aBOOL;

        if (this._settingsBtn) {
            if (aBOOL === true) {
                this._settingsBtn.show();
            }
            else if (aBOOL === false) {
                this._settingsBtn.hide();
            }
        }

        return this._settings;

    },


    /**
     * widget window print tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    print: function (aBOOL) {

        this._print = aBOOL === undefined ? this._print : aBOOL;

        if (this._printBtn) {
            if (aBOOL === true) {
                this._printBtn.show();
            }
            else if (aBOOL === false) {
                this._printBtn.hide();
            }
        }


        return this._print;

    },
    /**
     * widget window refresh tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    refresh: function (aBOOL) {

        this._refresh = aBOOL === undefined ? this._refresh : aBOOL;

        if (this._refreshBtn) {
            if (aBOOL === true) {
                this._refreshBtn.show();
            }
            else if (aBOOL === false) {
                this._refreshBtn.hide();
            }
        }

        return this._refresh;
    },
    /**
     * widget window status tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    status: function (aBOOL) {


        this._status = aBOOL === undefined ? this._status : aBOOL;

        if (this._interfaceStatusBtn) {
            if (aBOOL === true) {
                this._interfaceStatusBtn.show();
            }
            else if (aBOOL === false) {
                this._interfaceStatusBtn.hide();
            }
        }

        return this._status
    },

    /* converts status icon to yellow alert icon */
    statusAlert: function (aBOOL) {

        if (aBOOL === true) {
            this._interfaceStatusBtn.find('.icon-info').addClass('yellow');
        }
        else if (aBOOL === false) {
            this._interfaceStatusBtn.find('.icon-info').removeClass('yellow');

        }

        return this._interfaceStatusBtn.find('.icon-info').hasClass('yellow');

    },

    /**
     * widget window plus/add tool available setter/getter
     * @param aBOOL
     * @returns {*}
     */
    plus: function (aBOOL) {

        this._plus = aBOOL === undefined ? this._plus : aBOOL;

        if (this._addBtn) {
            if (aBOOL === true) {
                this._addBtn.show();
            }
            else if (aBOOL === false) {
                this._addBtn.hide();
            }
        }


        return this._plus;
    },
    fitColumns: function (aBOOL) {

        if (this._datagrid) {
            if (aBOOL !== undefined) {
                this._datagrid.set('fitColumns', aBOOL);
            }

            return this._datagrid.get('fitColumns');
        }

        return false;

    },
    groupBy: function (aString) {
        if (this._datagrid)
            return this._datagrid.set('groupBy', aString);
    },
    groupFormatter: function (aFunction) {

        if (_.isFunction(aFunction)) {
            this._groupFormatter = aFunction;
            if (this._datagrid)
                this._datagrid.set('groupFormatter', this._groupFormatter);
        }
        return this._groupFormatter;

    },
    groupSorter: function (aFunction) {
        if (this._datagrid)
            return this._datagrid.set('groupSorter', aFunction);
    },
    gridHeaderHeight: function (aNumber) {
        if (this._datagrid) {
            if (_.isNumber(aNumber)) {
                this._datagrid.set('headerHeight', aNumber);
            }

            return this._datagrid.get('headerHeight');
        }
    },
    defaultSort: function (sort) {

        if (sort !== undefined) {
            if (sort && this._defaultSort) {
                if (sort.id !== this._defaultSort.id) {
                    this._dataRendered = false
                }
            }
            this._defaultSort = sort;
        }

        return this._defaultSort;
    },


    onBecomesVisible: function () {

        if (this._datagrid)
            this._datagrid.showMessage("Loading...");

        setTimeout(_.bind(function () {

            if (!this._dataRendered) {
                this.renderData();
            }

            if (this._filterPanel) {

                if (this.get('defaultFilter')) {
                    this._filterPanel.filterBy(this.get('defaultFilter'));
                }
                else {
                    this._filterPanel.applyFilters();
                }
            }
            else {
                this.renderData();
            }

            if (this._datagrid)
                this._datagrid.sizer();

            this._body.css({
                bottom: this._footer.outerHeight()
            });

        }, this), 0);

        if (this._filterPanel) {
            this._filterPanel.clearFilters();
        }

    },

    disableFilterPanel: function (aFlag) {

        if (this._filterPanel) {
            this._filterPanel.set('disabled', aFlag);
        }

    },

    adjustHeight: function(){
        var newHeight = 0; //this._header.outerHeight();
        if(this._vlerLoadPanel)
            newHeight += this._vlerLoadPanel.el.offsetHeight;

        if(this.$el.find(".jlv-widget-warning-banner").length > 0)
            newHeight += this.$el.find(".jlv-widget-warning-banner")[0].offsetHeight;

        this._body.css({
            top: newHeight,
            bottom: this._footer.outerHeight()
        });
    },

    //cleans up the widget
    destroy: function () {

        JLV.NotificationCenter.removeObserverForNotificationName(this, 'closeNotification');

        this.close();

        $('#WindowDock').widget().removeWindow(this);

        setTimeout(_.bind(function () {
            this.$el.remove();
        }, this), 0);
    },


    toggleFilterPanel: function () {

        if (this.get('filter')) {

            if (!this._filterPanel) {

                var filterIds = this.get('filters');
                var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));


                var data = this.get('data');

                if (data) {

                    if (this.get('dateFilter')) {
                        this._dateLabel.show();

                        if (data.startDate !== undefined && data.endDate !== undefined) {

                            this._filterPanel = new JLV.WidgetWindow.DateFilterPanel({
                                window: this,
                                startDate: data.startDate.toDate(),
                                endDate: data.endDate.toDate(),
                                filters: filters,
                                data: data
                            });
                        }
                    }
                    else {

                        this._filterPanel = new JLV.WidgetWindow.FilterPanel({
                            window: this,
                            filters: filters,
                            data: data
                        });
                    }

                    if (this._filterPanel) {
                        this._filterPanel.$el.prependTo(this._body);
                        this._filterPanel.$el.hide();
                    }
                }
            }

            if (this._filterPanel) {

                var filterHeight = this._filterPanel.$el.outerHeight(),
                    top = this._datagrid.el.offsetTop;


                if (this._filterPanel.get('hidden')) {
                    this._filterPanel.set('hidden', false);
                    this._filterPanel.$el.prependTo(this._body);

                    if (this.get('filter')) {
                        if (this._datagrid)
                            this._datagrid.$el.css('top', top + filterHeight);
                    }
                    else {
                        if (this._datagrid){
                            this._datagrid.$el.css('top',
                                Math.max(0, top - filterHeight));

                        }

                    }
                }
                else {
                    this._filterPanel.set('hidden', true);
                    if (this._datagrid) {

                        this._datagrid.$el.css('top', Math.max(0, top - filterHeight));
                    }

                }
            }

        }

    },


    addBanner: function (config) {

        if (this._body) {
            if (!this._bannerHeight) {
                this._bannerHeight = 0;
            }

            this.clearBanner(config.id);

            var b = $('<div>')
                .attr('id', config.id || _.uniqueId())
                .attr('tabindex', 0)
                .css({
                    height: config.height || 32,
                    background: config.bgcolor || '#d0d0d0',
                    color: config.fgcolor || '#111'
                }).addClass('jlv-widget-banner')
                .html(config.text || '');

            if (config.closable) {

                $('<div>').
                    attr('title', 'Close Banner').
                    addClass('appkit-button jlv-widget-banner-close')
                    .awaken('appkit-button')
                    .html('x')
                    .on('click', _.bind(function () {
                        this.clearBanner(config.id)
                    }, this))
                    .appendTo(b)


            }

            var filter = this._body.find('.jlv-more-date-filter-panel:visible, .jlv-more-filter-panel:visible');

            var extra = 0;
            if (filter.length) {
                b.insertAfter(filter);
                extra += filter.height();
            }
            else {
                b.prependTo(this._body);
            }

            this._bannerHeight += b.outerHeight();


            if (this._datagrid) {
                this._datagrid.$el.css('top', this._bannerHeight + extra);
            }

        }

    },

    clearBanner: function (id) {

        if (id) {

            if (this._body) {

                var b = this._body.find('#' + id);

                if (b) {

                    var h = b.outerHeight();

                    b.remove();

                    if (this._bannerHeight) {
                        this._bannerHeight -= h;
                    }

                    var extra = 0;

                    var filter = this._body.find('.jlv-more-date-filter-panel:visible, .jlv-more-filter-panel:visible');
                    if (filter.length) {
                        extra = filter.height();
                    }

                    if (this._datagrid) {
                        this._datagrid.$el.css('top', this._bannerHeight + extra);
                    }
                }

            }
        }
    },

    clearBanners: function () {

        if (this._body) {
            this._body.find('.jlv-widget-banner').remove();

            var filter = this._body.find('.jlv-more-date-filter-panel:visible, .jlv-more-filter-panel:visible');
            var extra = 32;
            if (filter.length) {
                extra = filter.height() + 1;
            }

            if (this._datagrid) {
                this._datagrid.$el.css('top', extra);
            }
        }
    }


});

$(function () {
    JLV.WidgetWindow.VlerLoadPanel = AppKit.View.extend({
        className: 'jlv-vler-loading-panel',
        template: $('#WidgetVlerLoadingTemplate').template(),
        window: null,
        initialize: function (options) {
            this._super(options);

            var self = this;
            this.subview('.jlv-vler-loading').on('click', function () {
                if (self.window) {
                    //self.window.hideVlerLoadMessage();
                }
            });

        }
    });
});
