//= require ./PatientWidget.js
//= require ./windows/PatientConsultsWidgetWindow.js

JLV.PatientConsultsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'service',
    url: 'patientConsults/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['requestDate'],
        parse: function (data) {

            if (data.outCode) {
                if (data.outCode.indexOf("<") === 0) {
                    data.outCode = data.outCode.replace("<", "").replace(">", "");
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientConsults = JLV.PatientWidget.extend({
    options: {
        widgetId: 'consults_widget',
        name: 'Consult Encounters',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientConsultsWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'requestDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm')
                },
                sortable: true,
                width: 80
            },
            {
                title: 'Consult Order',
                id: 'outCode',
                width: 210,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Status',
                id: 'outStatus',
                width: 100,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'requestDate',
            asc: false
        },
        data: JLV.PatientConsultsCollection
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {

            if (model.get('noteAsBinaryStream')) {

                var consultURL = String.format("patientConsults/getNoteDataStream?patToken={0}&recToken={1}&widgetName={2}",
                    JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), this.name);

                if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                    JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                        if (download) {
                            JLV.common.openAuxBrowserWindow(consultURL, "JLV_CONSULT_REPORT");
                        }
                        else if (rb) {
                            this._addToReportBuilder(model, true, 'GetNoteDataStream');
                        }

                    }, this));
                }
                else {
                    JLV.common.openAuxBrowserWindow(consultURL, "JLV_CONSULT_REPORT");
                }



            }
            else {
                var dialog = new JLV.DetailsWindow({
                    title: String.format("Consult Encounters Detail ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('outCode')),
                    url: 'patientConsults/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken'),
                        title: model.get('service'),
                        recordDate: model.get('requestDate')
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response) {
                        return '<pre>' + response.report + '</pre>';
                    }
                });

                dialog.center();
                dialog.set('visible', true);
            }


        }

    }

});


JLV.registerJLVWidget('consults_widget', JLV.PatientConsults);