//= require ./PatientWidget.js
//= require ./windows/patientdemographicsWidgetWindow.js
//= require ./windows/PatientInsuranceWidgetWindow.js
//= require ./windows/PatientFlagsWidgetWindow.js


JLV.PatientDemographics = AppKit.View.extend({
    className: 'jlv-patient-demographics',
    siteData: [],
    vlerSiteData: [],
    dasvlerSiteData: [],
    vlerTabPanel: null,
    dasvlerTabPanel: null,
    initialize: function (options) {
        this._tabview = new AppKit.TabView();
        this.$el.append(this._tabview.$el);

        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFetchedNotification,
            action: _.bind(function () {
                this.vlerTabPanel.empty();
                $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataReadyNotification,
            action: _.bind(function () {
                if (this.vlerTabPanel)
                    this.vlerTabPanel.find('.jlv-vler-content .tr').remove();
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {
                this.vlerTabPanel.empty();
                $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
                //vler data is ready, neet to refetch demographics data
                JLV.PatientDemographics.Fetch(JLV.ActivePatientModel, _.bind(function (data) {
                    JLV.ActivePatientModel.set('demographics', data);
                    this.render(true);
                }, this));

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                if (this.vlerTabPanel) {
                    this.vlerTabPanel.empty();
                    $(this._generateVlerErrorContent()).appendTo(this.vlerTabPanel);
                    var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" aria-label="interface status button">')
                        .awaken('appkit-button')
                        .attr('data-type', 'yellow').addClass('yellow')
                        .empty()
                        .append($('<i class="icon-info yellow">'))
                        .appendTo(this.vlerTabPanel);

                    interfaceBtn.widget().set('action', _.bind(function () {
                        this.showInterfaceStatus();

                    }, this));
                }
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.DasVlerDataFetchedNotification,
            action: _.bind(function () {
                this.dasvlerTabPanel.empty();
                $(this._generateDasVlerLoadingContent()).appendTo(this.dasvlerTabPanel);
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.DasVlerDataReadyNotification,
            action: _.bind(function () {
                if (this.dasvlerTabPanel)
                    this.dasvlerTabPanel.find('.jlv-vler-content .tr').remove();
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.DasVlerDataRefreshConfirmNotification,
            action: _.bind(function () {
                this.dasvlerTabPanel.empty();
                $(this._generateDasVlerLoadingContent()).appendTo(this.dasvlerTabPanel);
                //vler data is ready, neet to refetch demographics data
                JLV.PatientDemographics.Fetch(JLV.ActivePatientModel, _.bind(function (data) {
                    JLV.ActivePatientModel.set('demographics', data);
                    this.render(true);
                }, this));

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.DasVlerDataFailNotification,
            action: _.bind(function () {
                if (this.dasvlerTabPanel) {
                    this.dasvlerTabPanel.empty();
                    $(this._generateDasVlerErrorContent()).appendTo(this.dasvlerTabPanel);

                    var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" aria-label="interface status button">')
                        .awaken('appkit-button')
                        .attr('data-type', 'yellow').addClass('yellow')
                        .empty()
                        .append($('<i class="icon-info yellow">'))
                        .appendTo(this.dasvlerTabPanel);

                    interfaceBtn.widget().set('action', _.bind(function () {
                        this.showInterfaceStatus();

                    }, this));
                }
            }, this)
        });

    },
    _renderDemographicsDataToElement: function ($el, data, isWindow) {

        var self = this;
        //remove any old content
        $el.find('.jlv-patient-demographics-content').remove();
        //$('.jlv-demographicsAlert').hide();

        $(this._generatePatientDemographicsContent(data)).appendTo($el);

        data.standardizedRaceCode = 'None';
        data.standardizedRaceDisplay = 'None';

        if (data.codes) {
            for (var j in data.codes) {
                if (data.codes[j].system) {
                    if (data.codes[j].system === '2.16.840.1.113883.6.238' ||
                        data.codes[j].system.toUpperCase().indexOf("CDC") >= 0) {
                        data.standardizedRaceCode = data.codes[j].code;
                        data.standardizedRaceDisplay = data.codes[j].display;
                        break;
                    }
                }
            }
        }

        if (data.codeGray) {
            //this.portal.tabView.$el.find('.alert-btn').show();
            if (this.portal.tabView.$el.find('.jlv-demographicsAlert').length < 1) {
                var warnBtn = new AppKit.Button({
                    className: 'jlv-demographicsAlert',
                    label: 'Flags',
                    action: function () {
                        var ddata = $(this).data('demographics');
                        try {
                            $.post('patientDemographics/auditAlerts',
                                {
                                    patToken: JLV.ActivePatientModel.get('patientToken'),
                                    recordSiteMoniker: ddata.site.moniker,
                                    recordSiteAgency: ddata.site.agency
                                });
                        }
                        catch (err) {
                        }

                        self._showAlerts();
                    }
                });

                warnBtn.$el.on('keydown', function (evt) {
                    if (evt.which === $.ui.keyCode.SPACE) {
                        $(this).trigger('click');
                    }
                });

                warnBtn.$el.attr('title', 'Patient Flags and Postings')
                    .data('demographics', data)
                    .appendTo(this.portal.tabView.$el.find('.alert-btn'))



            }

            for (var i = 0, len = data.flags.length; i < len; i++) {
                var site = data.flags[i].site;
                if (site && site.moniker === JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {
                    //do not add DES status record
                    continue;
                } else {
                    this.patFlags.push(data.flags[i]);
                }
            }

            if (this.patFlags.length === 0 && warnBtn) {
                warnBtn.$el.hide();
            }
        }

        $el.find('.jlv-patient-race-display').tooltip({
            content: _.template('<b>National Standard CDC Race </b>' +
                '<div>Standardized Name: {{standardizedRaceDisplay}}<br/>' +
                'Standardized Code: {{standardizedRaceCode}}</div>')(data),
            show: {
                delay: 600,
                duration: 0,
                easing: false,
                effect: null
            },
            hide: false,
            items: 'div',
            position: {my: "center top", at: "center bottom+5"}
        });

        if (!isWindow) {
            var showInsurance = _.bind(function () {
                if (!this.insuranceWindow) {
                    this.insuranceWindow = new JLV.PatientInsuranceWidgetWindow({
                        patient: JLV.ActivePatientModel,
                        defaultSort: JLV.PatientDemographics.InsuraceSortInfo,
                        hiddenColumns: JLV.PatientDemographics.HiddenColumns
                    });

                    this.insuranceWindow.on('sortChange', function (evt, sortInfo) {
                        JLV.PatientDemographics.InsuraceSortInfo = sortInfo;
                        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SavePatientPortalUINotification);
                    });

                    this.insuranceWindow.on('columnsHidden', function (evt, hiddenColumns) {
                        JLV.PatientDemographics.HiddenColumns = hiddenColumns;
                        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SavePatientPortalUINotification);

                    })
                }

                this.insuranceWindow.center();
                this.insuranceWindow.open();


            }, this);

            $el.find('.jlv-insurance-link')
                .click(showInsurance)
                .keydown(function (evt) {
                    if (evt.keyCode === $.ui.keyCode.TAB) {
                        evt.stopPropagation();
                        if (!evt.shiftKey) {
                            var next = $el.nextInDOM('.jlv-widget');
                            evt.preventDefault();
                            if (next.length) {
                                if (!next.is(":visible")) {
                                    $('#AddWidgetButton').focus();
                                } else {
                                    next.focus();
                                }
                            }
                            else {
                                $('#AddWidgetButton').focus();
                            }
                        }
                    }
                });


            var addToRb = _.bind(function () {

                var tabIndex = this._tabview.get('activeTab');
                var siteName = this._tabview.tabLabelAtIndex(tabIndex).toString();
                var siteData = this.siteData[tabIndex];
                if (siteName.indexOf('VLER') === -1) {
                    var model = new Backbone.Model(siteData);
                    JLV.ReportBuilderController.addDemographic(model, siteName);
                }

            }, this);


            /*$el.find('.add-to-rb')
                .click(addToRb)
                .keydown(function (evt) {
                    if (evt.keyCode === $.ui.keyCode.TAB) {
                        evt.stopPropagation();
                        if (!evt.shiftKey) {
                            var next = $el.nextInDOM('.jlv-widget');
                            evt.preventDefault();
                            if (next.length) {
                                if (!next.is(":visible")) {
                                    $('#AddWidgetButton').focus();
                                } else {
                                    next.focus();
                                }
                            }
                            else {
                                $('#AddWidgetButton').focus();
                            }
                        }
                    }
             }); */

            /* add back in right-click option to add to RB */
            $el.on('contextmenu', _.bind(function (evt) {

                evt.preventDefault();

                var menu = new AppKit.Menu({
                    menuItems: [
                        new AppKit.MenuItem({
                            label: 'Add Record to Report Builder',
                            action: addToRb
                        })
                    ]
                });

                menu.open({
                    left: evt.clientX - 30,
                    top: evt.clientY
                });

                menu.on('close', function () {
                    setTimeout(_.bind(function () {
                        this.$el.remove();
                    }, this), 0);
                });
            }, this));
        } else {
            $el.find('.jlv-insurance-link').hide();
        }
    },
    render: function (vlerLoading) {

        this._tabview.removeAllTabs(true);

        if (this.get('patient')) {

            var demographics = this.get('patient').get('demographics'),
                count = demographics.length,
                i = 0;


            this.dodAdapterSourceReport = [];
            this.dodAdapterStatusRec = [];
            this.vlerSiteData = [];
            this.dasvlerSiteData = [];
            this.siteData = [];
            this.vistaAlerts = [];
            this.patFlags = []

            if (this.demographicsDetailsWindow) {
                this.demographicsDetailsWindow.$el.remove();
                this.demographicsDetailsWindow = null;
            }

            for (; i < count; i++) {

                var data = demographics[i],
                    moniker = data.site.moniker,
                    name = data.site.name,
                    agency = data.site.agency;

                if (moniker === JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {
                    this.dodAdapterSourceReport.push(data.name);
                    this.dodAdapterStatusRec.push(data);
                }
                else if (agency === JLV.common.AGENCY_DOD_VLER) {
                    data.guiId = _.uniqueId('gui-id-');
                    this.vlerSiteData.push(data);
                }
                else if (agency === JLV.common.AGENCY_DAS_VLER) {
                    data.guiId = _.uniqueId('gui-id-');
                    this.dasvlerSiteData.push(data);
                }
                else if (name !== 'VLER') {

                    this.siteData.push(data);

                    var title = JLV.common.formatSiteMonikerDisplay(data.site);

                    if (data.name) {
                        data.name = data.name.replace(/,/g, ', ');
                    }

                    var tabID = _.uniqueId('tab');

                    var $el = $('<div>').attr('id', tabID).appendTo(this._tabview.$el);

                    var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" title="Interface Status" aria-label="interface status button">')
                        .html("i")
                        .appendTo($el);

                    //if connection unavailable, interface button shows yellow icon
                    if (data.name.toLowerCase().indexOf('connection unavailable') > -1) {
                        interfaceBtn.attr('data-type', 'yellow').addClass('yellow');
                        interfaceBtn.empty();
                        interfaceBtn.append($('<i class="icon-info yellow">'));
                    }

                    interfaceBtn.awaken('appkit-button')
                        .widget().set('action', _.bind(function () {
                        this.showInterfaceStatus();

                    }, this));

                    this._renderDemographicsDataToElement($el, data, false);

                    this._tabview.addTab(title, tabID, false);

                    var showDetails = _.bind(function () {
                        if (!this.demographicsDetailsWindow) {
                            this.demographicsDetailsWindow = new JLV.PatientDemographicsWidgetWindow({
                                data: this.siteData,
                                demographicsWidget: this
                            });
                        }

                        this.demographicsDetailsWindow.center();
                        this.demographicsDetailsWindow.open();

                    }, this);

                    $el.find('.jlv-sponsor-link, .jlv-patient-name')
                        .click(showDetails)
                        .mousedown(function (evt) {
                            evt.preventDefault();
                        });

                    $el.find('.jlv-sponsor-link, .jlv-patient-name')
                        .parent()
                        .keypress(function (evt) {
                            if (evt.which === $.ui.keyCode.ENTER)
                                showDetails();
                        });
 
                    if (data.site.agency === 'DOD' && data.sponsors.length < 2) {
                        $el.find('.jlv-sponsor-link').remove();
                    }

                }
            }

            if (!vlerLoading) {
                this._displayErrorAlerts(this.siteData);
            }

            if (JLV.Session.dodVLER == true) {
                var vlerSite = {
                    agency: "VLER",
                    name: "VLER",
                    moniker: "DoD Community Partners"
                };

                var tabID = _.uniqueId('tab-vler-');

                if (this.vlerTabPanel) {
                    this.vlerTabPanel.remove();
                }

                this.vlerTabPanel = $('<div>').attr('id', tabID).appendTo(this._tabview.$el);

                var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" aria-label="interface status button">')
                    .html("i")
                    .appendTo(this.vlerTabPanel)
                    .awaken('appkit-button');

                interfaceBtn.widget().set('action', _.bind(function () {
                    this.showInterfaceStatus();

                }, this));

                this._tabview.addTab(JLV.common.formatSiteMonikerDisplay(vlerSite)
                    , tabID, false);

                if (JLV.Session.dodVLERReady == true) {

                    //remove any old content
                    this.vlerTabPanel.find('.jlv-patient-demographics-content').remove();
                    $(this._generateVlerContent(this.vlerSiteData)).appendTo(this.vlerTabPanel);

                    var self = this;
                    $('.jlv-vler-demographics-narrative a').on('click', function (evt) {

                        var guiId = $(this).data('gui-id');
                        var dataIndex = _.findIndex(self.vlerSiteData, {
                            guiId: guiId
                        });

                        var vlerData = null;

                        if (dataIndex > -1)
                            vlerData = self.vlerSiteData[dataIndex];


                        if (vlerData) {

                            var vlerDemographicDetail = new JLV.PatientDemographics.VlerDemographicsDetail({
                                title: "Demographics - " + vlerData.site.name,
                                data: vlerData
                            });

                            self._renderDemographicsDataToElement(vlerDemographicDetail.$el, vlerData, true);
                            vlerDemographicDetail.center();
                            vlerDemographicDetail.open();

                        }

                    }).mousedown(function (evt) {
                        evt.preventDefault();
                    });

                }
                else {
                    $(this._generateVlerLoadingContent()).appendTo(this.vlerTabPanel);
                }
            }

            if (JLV.Session.dasVLER == true) {
                var dasvlerSite = {
                    agency: "DASVLER",
                    name: "VLER",
                    moniker: "VA Community Partners"
                };

                var tabID = _.uniqueId('tab-dasvler-');

                if (this.dasvlerTabPanel) {
                    this.dasvlerTabPanel.remove();
                }

                this.dasvlerTabPanel = $('<div>').attr('id', tabID).appendTo(this._tabview.$el);

                var interfaceBtn = $('<div class="appkit-button jlv-demographics-interfaceStatus-button" aria-label="interface status button">')
                    .html("i")
                    .appendTo(this.dasvlerTabPanel)
                    .awaken('appkit-button');

                interfaceBtn.widget().set('action', _.bind(function () {
                    this.showInterfaceStatus();

                }, this));

                // interfaceBtn.widget().set('action', _.bind(function () {
                //     this.showInterfaceStatus();
                //
                // }, this));

                this._tabview.addTab(JLV.common.formatSiteMonikerDisplay(dasvlerSite)
                    , tabID, false);

                if (JLV.Session.dasVLERReady == true) {

                    //remove any old content
                    this.dasvlerTabPanel.find('.jlv-patient-demographics-content').remove();
                    $(this._generateDasVlerContent(this.dasvlerSiteData)).appendTo(this.dasvlerTabPanel);

                    var self = this;
                    $('.jlv-vler-demographics-narrative a').on('click', function (evt) {

                        var guiId = $(this).data('gui-id');
                        var dataIndex = _.findIndex(self.dasvlerSiteData, {
                            guiId: guiId
                        });

                        var dasvlerData = null;

                        if (dataIndex > -1)
                            dasvlerData = self.dasvlerSiteData[dataIndex];


                        if (dasvlerData) {

                            var dasvlerDemographicDetail = new JLV.PatientDemographics.VlerDemographicsDetail({
                                title: "Demographics - " + dasvlerData.site.name,
                                data: dasvlerData
                            });

                            self._renderDemographicsDataToElement(dasvlerDemographicDetail.$el, dasvlerData, true);
                            dasvlerDemographicDetail.center();
                            dasvlerDemographicDetail.open();

                        }

                    }).mousedown(function (evt) {
                        evt.preventDefault();
                    });

                }
                else {
                    $(this._generateVlerLoadingContent()).appendTo(this.dasvlerTabPanel);
                }
            }

            if (count > 0) {
                this._tabview.selectTabAtIndex(0);
            }

            if (this.patFlags.length > 0 && JLV.Session.viewedFlags === false) {
                this._showAlerts();
            }

        }


    },

    _showAlerts: function () {


        if (this.patFlags.length > 0) {

            var showFlags = _.bind(function () {
                if (!this.flagsWindow) {
                    this.flagsWindow = new JLV.PatientFlagsWidgetWindow({
                        patient: JLV.ActivePatientModel,
                        data: new Backbone.Collection(this.get('patFlags'))
                    });

                }

                this.flagsWindow.center();
                this.flagsWindow.open();

            }, this);

            setTimeout(function () {
                showFlags();
            }, 5);

        }

    },

    destroy: function () {

        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFetchedNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataReadyNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataRefreshConfirmNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.VlerDataFailNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataFetchedNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataReadyNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataRefreshConfirmNotification);
        JLV.NotificationCenter.removeObserverForNotificationName(this, JLV.DasVlerDataFailNotification);

        if (this.insuranceWindow) {
            this.insuranceWindow.destroy();
        }

        if (this._interfaceStatusWindow)
            this._interfaceStatusWindow.close();

        setTimeout(_.bind(function () {
            this.$el.remove();
        }, this), 0);

    },

    sizer: function () {
        this._tabview.refresh();
    },
    _generateVlerContent: function (vlerDataArray) {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        var templateString = '<div class="jlv-vler-content" tabindex="0" style="padding:15px;"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div>' +
            '';

        var count = vlerDataArray.length,
            i = 0;

        templateString += '<div class="jlv-vler-demographics-narratives">';
        for (; i < count; i++) {

            var vlerData = vlerDataArray[i];
            templateString += _.template('<div class="jlv-vler-demographics-narrative"><a data-gui-id="{{guiId}}" title="DoD VLER: {{site.moniker}}" href="#">{{site.name}} </a></div>')(vlerData)

        }

        templateString += "</div>";

        return _.template(templateString)(data);

    },
    _generateVlerLoadingContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-vler-content"> <div tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div><table class="table jlv-vler-demographics-loading-label"><div class="tr"><div class="td jlv-vler-demographics-label">Community partner data retrieval in progress...</div><div class="td"><div style="width:22px;" class="jlv-table-data-loading"></div></div></div></table></div>')(data);
    },
    _generateVlerErrorContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DOD',
                name: 'VLER',
                moniker: 'DoD Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-vler-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div><div class="jlv-vler-demographics-label">Connection unavailable. Community partner data is not available to display.</div>')(data);
    },
    _generateDasVlerContent: function (vlerDataArray) {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DASVLER',
                name: 'VLER',
                moniker: 'VA Community Partners'
            },
            name: name
        };

        var templateString = '<div class="jlv-vler-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div>' +
            '';

        var count = vlerDataArray.length,
            i = 0;

        templateString += '<div class="jlv-vler-demographics-narratives">';
        for (; i < count; i++) {

            var vlerData = vlerDataArray[i];
            templateString += _.template('<div class="jlv-vler-demographics-narrative"><a data-gui-id="{{guiId}}" title="DAS VLER: {{site.moniker}}" href="#">{{site.name}} </a></div>')(vlerData)

        }

        templateString += "</div>";

        return _.template(templateString)(data);

     },
    _generateDasVlerLoadingContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DASVLER',
                name: 'VLER',
                moniker: 'VA Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-vler-content"><div tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div><table class="jlv-dasvler-demographics-loading-label"><div class="tr"><div class="td jlv-vler-demographics-label">Community partner data retrieval in progress...</div><div class="td"><div style="width:22px;" class="jlv-table-data-loading"></div></div></div></table></div>')(data);
    },
    _generateDasVlerErrorContent: function () {

        var name = JLV.ActivePatientModel.get('name');
        var data = {
            site: {
                agency: 'DASVLER',
                name: 'VLER',
                moniker: 'VA Community Partners'
            },
            name: name
        };

        return _.template('<div class="jlv-vler-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
            JLV.common.formatSiteIconDisplay(data.site) + '</div><div class="jlv-vler-demographics-label">Connection unavailable. Community partner data is not available to display.</div>')(data);
    },
    _generateVlerDetailDemographicsContent: function (data) {
        try {
            return _.template('<div class="jlv-patient-demographics-content" tabindex="0">' +
                '<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
                JLV.common.formatSiteIconDisplay(data.site) +
                '</div>' +
                '<br/>' +
                '<table style="width:100%;margin:15px 15px 0 15px;box-sizing:border-box;" tabindex="0">' +
                '<tr>' +
                '<td style="vertical-align:top;width:50%;">' +
                '<div >{{-address1}}</div>' +
                '<div >{{-address2}}</div>' +
                '<div >{{-city}}, {{state}}</div>' +
                '<div >{{-zipCode}}</div>' +
                '<br/>' +
                '<table>' +
                '<tr><td class="jlv-demographic-item">Language:</td>' +
                '<td class="jlv-demographic-item-value">{{-languages}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Religious Affiliation:</td>' +
                '<td class="jlv-demographic-item-value">{{-religion}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Next of Kin:</td><td>' +
                '{[ _.each(nextOfKin, function(kin) { ]}' +
                '{{-kin.name}} <br/>' +
                'Relationship: {{-kin.relationship}} <br/>' +
                '{{-kin.street1}} {{-kin.street2}} {{-kin.street3}}<br/>' +
                '{{-kin.city}}, {{-kin.stateProvince}} {{-kin.postalCode}}<br/>' +
                    //'Phone: {{-kin.telecoms}}<br/>' +
                'Phone: ' +
                '{[ _.each(kin.telecoms, function(telecom) { ]}' +
                '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
                '{[ }) ]}' +
                '{[ }) ]}' +
                '</td></tr>' +
                '</table>' +
                '</td>' +
                '<td>' +
                '<table tabindex="0" style="min-width:120px;">' +
                '<tr>' +
                '<td class="jlv-demographic-item demographic-ssn">Patient ID:</td>' +
                '<td class="jlv-demographic-item-value"><b>{{-edipi}}</b></td>' +
                '</tr>' +
                '<tr><td class="jlv-demographic-item">Age:</td><td class="jlv-demographic-item-value"><b>{{-age}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">DOB:</td><td class="jlv-demographic-item-value"><b>{{-dob}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Race:</td><td class="jlv-demographic-item-value jlv-patient-race-display"><b>&nbsp;{{-race}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(h):</td><td class="jlv-demographic-item-value"><b>{{-phone1}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(w):</td><td class="jlv-demographic-item-value"><b>{{-phone2}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Gender:</td><td class="jlv-demographic-item-value"><b>{{-gender}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Marital Status:</td><td class="jlv-demographic-item-value"><b>{{-maritalStatus}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Source:</td><td class="jlv-demographic-item-value"><b>{{-sourceProtocol}}</b></td></tr>' +
                '<tr style="height:10px;"></tr>' +
                '<tr><td class="jlv-demographic-item">Emergency Contact:</td></tr>' +
                '<tr><td>' +
                '{[ _.each(emergencyContact, function(contact) { ]}' +
                '{{-contact.name}} <br/>' +
                'Relationship: {{-contact.relationship}} <br/>' +
                '{{-contact.street1}} {{-contact.street2}} {{-contact.street3}}<br/>' +
                '{{-contact.city}}, {{-contact.stateProvince}} {{-contact.postalCode}}<br/>' +
                'Phone: ' +
                '{[ _.each(contact.telecoms, function(telecom) { ]}' +
                '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
                '{[ }) ]}' +
                '{[ }) ]}' +
                '</td></tr></table>' +
                '</td></tr></table>' +
                '<button class="jlv-insurance-link" role="button" tabindex="0">Insurance</button><br/>' +
                '</div>')(data);
        } catch (e) {
        }

    },
    _generateDASVlerDetailDemographicsContent: function (data) {
        try {
            return _.template('<div class="jlv-patient-demographics-content" tabindex="0">' +
                '<div class="jlv-demographics-panel-content" tabindex="0"><label class="jlv-patient-name">{{-name}}</label>' +
                JLV.common.formatSiteIconDisplay(data.site) +
                '</div>' +
                '<br/>' +
                '<table style="width:100%;margin:15px 15px 0 15px;box-sizing:border-box;" tabindex="0">' +
                '<tr>' +
                '<td style="vertical-align:top;width:50%;">' +
                '<div >{{-address1}}</div>' +
                '<div >{{-address2}}</div>' +
                '<div >{{-city}}, {{state}}</div>' +
                '<div >{{-zipCode}}</div>' +
                '<br/>' +
                '<table>' +
                '<tr><td class="jlv-demographic-item">Language:</td>' +
                '<td class="jlv-demographic-item-value">{{-languages}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Religious Affiliation:</td>' +
                '<td class="jlv-demographic-item-value">{{-religion}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Next of Kin:</td><td>' +
                '{[ _.each(nextOfKin, function(kin) { ]}' +
                '{{-kin.name}} <br/>' +
                'Relationship: {{-kin.relationship}} <br/>' +
                '{{-kin.street1}} {{-kin.street2}} {{-kin.street3}}<br/>' +
                '{{-kin.city}}, {{-kin.stateProvince}} {{-kin.postalCode}}<br/>' +
                //'Phone: {{-kin.telecoms}}<br/>' +
                'Phone: ' +
                '{[ _.each(kin.telecoms, function(telecom) { ]}' +
                '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
                '{[ }) ]}' +
                '{[ }) ]}' +
                '</td></tr>' +
                '</table>' +
                '</td>' +
                '<td>' +
                '<table tabindex="0" style="min-width:120px;">' +
                '<tr>' +
                '<td class="jlv-demographic-item demographic-ssn">Patient ID:</td>' +
                '<td class="jlv-demographic-item-value"><b>{{-edipi}}</b></td>' +
                '</tr>' +
                '<tr><td class="jlv-demographic-item">Age:</td><td class="jlv-demographic-item-value"><b>{{-age}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">DOB:</td><td class="jlv-demographic-item-value"><b>{{-dob}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Race:</td><td class="jlv-demographic-item-value jlv-patient-race-display"><b>&nbsp;{{-race}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(h):</td><td class="jlv-demographic-item-value"><b>{{-phone1}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(w):</td><td class="jlv-demographic-item-value"><b>{{-phone2}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Gender:</td><td class="jlv-demographic-item-value"><b>{{-gender}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Marital Status:</td><td class="jlv-demographic-item-value"><b>{{-maritalStatus}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Source:</td><td class="jlv-demographic-item-value"><b>{{-site.name}}</b></td></tr>' +
                '<tr style="height:10px;"></tr>' +
                '<tr><td class="jlv-demographic-item">Emergency Contact:</td></tr>' +
                '<tr><td>' +
                '{[ _.each(emergencyContact, function(contact) { ]}' +
                '{{-contact.name}} <br/>' +
                'Relationship: {{-contact.relationship}} <br/>' +
                '{{-contact.street1}} {{-contact.street2}} {{-contact.street3}}<br/>' +
                '{{-contact.city}}, {{-contact.stateProvince}} {{-contact.postalCode}}<br/>' +
                'Phone: ' +
                '{[ _.each(contact.telecoms, function(telecom) { ]}' +
                '{{-telecom.usageType}} {{-telecom.value}} <br/>' +
                '{[ }) ]}' +
                '{[ }) ]}' +
                '</td></tr></table>' +
                '</td></tr></table>' +
                '<button class="jlv-insurance-link" role="button" tabindex="0">Insurance</button><br/>' +
                '</div>')(data);
        } catch (e) {
        }

    },
    _generatePatientDemographicsContent: function (data) {

        //compute age
        var dob;

        if (data.site.agency === JLV.common.AGENCY_DOD) {
            dob = moment(data.dob, 'YYYYMMDD');
        } else if (data.site.agency === JLV.common.AGENCY_DAS_VLER) {
            dob = moment(data.dob, 'YYYY-MM-DD');
        }
        else {
            dob = moment(data.dob, 'DD MMM YYYY');
        }

        var today = moment();
        data.age = today.diff(dob, 'years');

        if (data.site && data.site.agency === JLV.common.AGENCY_DOD_VLER)
            return this._generateVlerDetailDemographicsContent(data);
        if (data.site && data.site.agency === JLV.common.AGENCY_DAS_VLER)
            return this._generateDASVlerDetailDemographicsContent(data);

        data.hieSharingOptInLastUpdateDate = JLV.ActivePatientModel.get("hieSharingOptInLastUpdateDate");

        try {
            return _.template('<div class="jlv-patient-demographics-content">' +
                '<div style="margin:15px 15px 0 15px;" tabindex="0">' +
                '<label class="jlv-patient-name">{{-name}}</label>' +
                JLV.common.formatSiteIconDisplay(data.site) +
                '</div>' +
                '<table tabindex="0" style="width:100%;margin:15px 15px 0 15px;box-sizing:border-box;">' +
                '<tr>' +
                '<td style="vertical-align:top;width:50%;">' +
                '<div style="display:inline-block;">' +
                '<div >{{-address1}}</div>' +
                '<div >{{-address2}}</div>' +
                '<div >{{-city}}, {{state}}</div>' +
                '<div >{{-zipCode}}</div>' +
                '<br/>' +
                '<table>' +
                '{[ if(site.agency === JLV.common.AGENCY_DOD) { ]}' +
                '<tr><td class="jlv-demographic-item">Rank:</td><td class="jlv-demographic-item-value">{{-rank}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">FMP/SSN:</td><td class="jlv-demographic-item-value fmp-ssn">{{-fmp}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Sponsor Name:</td><td class="jlv-demographic-item-value">{{-sponsorName}}</td></tr>' +
                '<tr><td class="jlv-demographic-item"></td><td class="jlv-demographic-item-value"><label class="jlv-sponsor-link">Additional Sponsors</label></td></tr>' +
                '{[ } ]}' +
                '{[ if(site.agency === JLV.common.AGENCY_VA) { ]}' +
                '<tr><td class="jlv-demographic-item">SC Percent:</td><td class="jlv-demographic-item-value">{{-percentServiceConnected}}%</td></tr>' +
                '{[ } ]}' +
                '<tr><td class="jlv-demographic-item">PCM Name:</td><td class="jlv-demographic-item-value">{{-pcm}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">PCM Date:</td><td class="jlv-demographic-item-value">{{-pcmDate}}</td></tr>' +
                '{[ if(site.agency === JLV.common.AGENCY_VA) { ]}' +
                '<tr><td class="jlv-demographic-item">PCM Team:</td><td class="jlv-demographic-item-value">{{-pcTeam}}</td></tr>' +
                '{[ } else { ]}' +
                '<tr><td class="jlv-demographic-item">PCM Clinic:</td><td class="jlv-demographic-item-value">{{-pcmClinic}}</td></tr>' +
                '{[ } ]}' +
                '{[ if(JLV.ActivePatientModel.get("hieSharingOptIn")===false && JLV.ActivePatientModel.get("hieSharingOptInLastUpdateDate")) { ]}' +
                '<tr><td class="jlv-demographic-item">DoD Sharing Status:</td><td style="font-weight:bold;" class="jlv-demographic-item-value">Opted Out <br/> ({{hieSharingOptInLastUpdateDate}})</td></tr>' +
                '{[ } ]}' +
                '</table>' +
                '</td>' +
                '<td style="vertical-align:top;width:50%;">' +
                '<table style="float:right;margin-right:30px;min-width:120px;">' +
                '<tr>' +
                '{[ if(site.agency === JLV.common.AGENCY_DOD) { ]}' +
                '<td class="jlv-demographic-item">DoD ID:</td><td class="jlv-demographic-item-value demographic-ssn"><b>{{-edipi}}</b></td>' +
                '{[ } else { ]}' +
                '<td class="jlv-demographic-item">SSN:</td><td class="jlv-demographic-item-value demographic-ssn"><b>{{-fmpssn}}</b></td>' +
                '{[ } ]}' +
                '</tr>' +
                '<tr><td class="jlv-demographic-item">Age:</td><td class="jlv-demographic-item-value"><b>{{-age}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">DOB:</td><td class="jlv-demographic-item-value"><b>{{-dob}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Race:</td><td class="jlv-demographic-item-value jlv-patient-race-display"><b>&nbsp;{{-race}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(h):</td><td class="jlv-demographic-item-value"><b>{{-phone1}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">(w):</td><td class="jlv-demographic-item-value"><b>{{-phone2}}</b></td></tr>' +
                '<tr><td class="jlv-demographic-item">Gender:</td><td class="jlv-demographic-item-value"><b>{{-gender}}</b></td></tr>' +
                '{[ if(site.agency === JLV.common.AGENCY_DOD) { ]}' +
                '<tr><td class="jlv-demographic-item">Organ Donor: </td><td style="font-weight:bold;" class="jlv-demographic-item-value">{{-organDonorIndDateDisplay}}</td></tr>' +
                '<tr><td class="jlv-demographic-item">Blood Type: </td><td style="font-weight:bold;" class="jlv-demographic-item-value">{{-bloodType}}</td></tr>' +
                '{[ } ]}' +
                '</table>' +
                '</td>' +
                '</tr>' +
                '</table>' +
                '<button class="jlv-insurance-link" role="button" tabindex="0">Insurance</button>' +
                //'<button class="jlv-link-button add-to-rb" role="button" tabindex="0">Add to Report Builder</button>' +
                '</div>')(data);
        } catch (e) {
        }
    },
    showInterfaceStatus: function () {

        if (this._interfaceStatusWindow) {
            this._interfaceStatusWindow.close();
        }


        this._interfaceStatusWindow = new JLV.DetailsWindow({
            title: 'Interface Status ~ Patient Demographics',
            width: 600,
            height: 500,
            maximizable: true,
            minimizable: true,
            resizable: true,
            modal: false,
            className: 'jlv-interface-status'
        });

        var sourceStatus = JLV.common.DODADAPTER_STATUS_NO_REPORT_MSG;

        if (this.dodAdapterSourceReport.length) {
            sourceStatus = '';

            for (var i = 0; i < this.dodAdapterSourceReport.length; i++) {
                sourceStatus += this.dodAdapterSourceReport[i] + '<br/><br/>';
            }
        }

        if (Object.keys(this.connectionErrors).length > 0
            || (JLV.Session.dodVLERFail)
            || (JLV.Session.dasVLERFail)) {
            sourceStatus = "<table><thead><th>Site Name</th><th>Site Status</th></thead>";
            for (var i in this.connectionErrors) {
                if (this.connectionErrors[i].site.moniker != JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {
                    sourceStatus += "<tr><td>" + this.connectionErrors[i].site.moniker + '</td><td> Connection Unavailable </td></tr>';
                }
            }
            if (JLV.Session.dodVLERFail && this.get('vlerDomain')) {
                sourceStatus += "<tr><td>DOD VLER</td><td> Connection Unavailable </td></tr>";
            }

            if (JLV.Session.dasVLERFail) {
                sourceStatus += "<tr><td>VA VLER</td><td> Connection Unavailable </td></tr>";
            }
            sourceStatus += "</table>";

        }

        this._interfaceStatusWindow.$el.html('<div class="jlv-dodadapter-report">' + sourceStatus + '</div>');

        this._interfaceStatusWindow.center();
        this._interfaceStatusWindow.open();


    },
    _displayErrorAlerts: function (siteData) {

        var count = siteData.length,
            i = 0;

        var msg = "";

        this.connectionErrors = [];

        for (; i < count; i++) {

            var data = siteData[i],
                site = data.site,
                moniker = site.moniker,
                name = data.name,
                agency = site.agency;

            var siteCount = 0;


            var entry = '';
            if (name == "" || name.toLowerCase().search("connection unavailable") != -1) {

                siteCount++;
                if (agency === 'DOD' && moniker === 'PDWS') {
                    entry += ('<table class="jlv-site-alert"><tr><th>' + JLV.common.formatMonikerDisplay(site) + '&nbsp;Unavailable</th></tr>');
                    entry += ('<tr><td>PDWS service is unavailable: Some DoD demographics data may not display.</td></tr></table>');
                } else if (agency === 'DOD') {
                    entry += ('<table class="jlv-site-alert"><tr><th>' + JLV.common.formatMonikerDisplay(site) + '&nbsp;Unavailable</th></tr>');
                    entry += ('<tr><td>AHLTA data service is unavailable: DoD data may not display.</td></tr></table>');
                }
                else {
                    entry += ('<table class="jlv-site-alert"><tr><th>' + JLV.common.formatMonikerDisplay(site) + '&nbsp;Unavailable</th></tr>');
                    entry += ('<tr><td>An error occurred while connecting to ' + moniker + '.</td></tr></table>');
                }

                msg += entry;

                this.connectionErrors.push(data);
            }

        }

        if (siteCount > 0)
            AppKit.AlertDialog.alert('Sites Unavailable', msg, null, {height: 300});

    }
});

//used to save/load the last sessions sort info
JLV.PatientDemographics.InsuraceSortInfo = {
    id: 'planName',
    asc: true
};

JLV.PatientDemographics.HiddenColumns = [];

JLV.PatientDemographics.Fetch = function (data, callback) {

    jQuery.post('patientDemographics/list.json',
        {
            patToken: data.get('patientToken'),
            includeVLER: JLV.Session.dodVLERReady == true,
            includeDASVLER: JLV.Session.dasVLERReady == true
        },
        function (responseData) {
            if (data.get('fmpssn') === JLV.ActivePatientModel.get('fmpssn')) { //make sure data callback is for active patient
                if (!JLV.ActivePatientModel.get('fmpssn')) {
                    // SSN is either null or empty string. This usually happens coming from ccow (CPRS).
                    // This will try to fill in ssn, dob, and gender if the demographics for the site taken
                    // from the patientToken exists. Added to handle print issue with missing ssn coming from ccow.
                    $.ajax({
                        url: 'context/getPatientSiteCode',
                        type: 'POST',
                        data: {
                            patientToken: JLV.ActivePatientModel.get('patientToken')
                        },
                        success: function (data) {
                            var siteCode = data.siteCode;
                            if (siteCode && siteCode !== '') {
                                for (var i in responseData) {
                                    if (responseData[i].site.siteCode === siteCode) {
                                        JLV.ActivePatientModel.set('fmpssn', responseData[i].fmpssn)

                                        if (!JLV.ActivePatientModel.get('dob')) {
                                            JLV.ActivePatientModel.set('dob', responseData[i].dob)
                                        }
                                        if (!JLV.ActivePatientModel.get('gender')) {
                                            JLV.ActivePatientModel.set('gender', responseData[i].gender)
                                        }
                                        break;
                                    }
                                }
                            }
                        },
                        error: function () {
                        }
                    });
                }

                if (_.isFunction(callback))
                    callback(responseData);
            }
        });
};

JLV.PatientDemographics.VlerDemographicsDetail = JLV.DetailsWindow.extend({
    className: 'jlv-vler-demographics-details',
    options: {
        width: 460,
        height: 400,
        vlerDomain: true,
        dasvlerDomain: true,
        data: null
    },
    initialize: function (options) {

        this._super(options);

        var self = this;
        $('<div title="Add to Report Builder" tabindex="0" class="jlv-window-toolbutton" style="padding-top:5px;">')
            .append('<div class="icon-plus">')
            .button()
            .on('click', function () {
                self.addToReportBuilder(self.data);
            }).appendTo(this.$el.parent().find('.jlv-window-tools'));
    },
    addToReportBuilder: function (data) {
        var model = new Backbone.Model(data);
        JLV.ReportBuilderController.addDemographic(model, 'VLER');
    }
});
