//= require ./PatientWidget.js
//= require ./windows/PatientDocumentsWidgetWindow.js

JLV.PatientDocumentsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'noteTitle',
    url: 'patientDocuments/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['noteDate'],
        parse: function (data) {

            if (data.documentType === "HAIMS") {
                data.normDisplay = data.noteType;
            }

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("NOTES") === 0 ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0 ||
                            data.codes[i].system.toUpperCase().indexOf("CPT") >= 0 ||
                            data.codes[i].system === '2.16.840.1.113883.6.1' ||
                            data.codes[i].system === '2.16.840.1.113883.6.12' ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0 ||
                            data.codes[i].system.toUpperCase().indexOf("CPT") >= 0 ) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (data.documentType == "Outpatient Encounter" && data.outpatientEncounter && data.outpatientEncounter.codes) {
                data.normCode = "";
                data.normDisplay = "";
                if(data.outpatientEncounter.codes) {
                    if (data.outpatientEncounter.codes[0]) {
                        if (data.outpatientEncounter.codes[0].system) {
                            if (data.outpatientEncounter.codes[0].system.toUpperCase().indexOf("SNOMED") >= 0 ||
                                data.outpatientEncounter.codes[0].system === '2.16.840.1.113883.6.96') {
                                data.normCode = data.outpatientEncounter.codes[0].code;
                                data.normDisplay = data.outpatientEncounter.codes[0].display;
                            }
                        }
                    }
                }
            }

            if (data.documentType == "Consult Report" && data.noteTitle) {
                if (data.noteTitle.indexOf("<") === 0) {
                    data.noteTitle = data.noteTitle.replace("<", "").replace(">", "");
                }
            }

            if (!data.noteTitle) {
                data.noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
            }

            return this._super(data);
        }
    })
});


JLV.PatientDocuments = JLV.PatientWidget.extend({
    options: {
        widgetId: 'documents_widget',
        name: 'Documents',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientDocumentsWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                sortable: true,
                width: 90,
                formatter: function (value, model) {
                    if (model) {
                        if (model.get('documentType') == "Outpatient Encounter" && model.get('outpatientEncounter')) {
                            if (model.get('outpatientEncounter').clinic === 'Connection unavailable.') {
                                return JLV.common.dateFormatter(value);
                            }
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model)
                        }
                    }

                    return JLV.common.dateFormatter(value);
                }
            },
            {
                title: 'Description',
                id: 'noteTitle',
                width: 170,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    var noteType = "",
                        noteTitle = value,
                        docType = "",
                        outpatientEncounter = null;

                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    } else {
                        if(model.get('recordDetailsToken') && model.get('recordDetailsToken').indexOf('NOT_SUPPORTED') >= 0){
                            return value;
                        }
                        docType = model.get('documentType');
                        outpatientEncounter = model.get('outpatientEncounter');
                    }

                    if(model.get('noteType')) {
                        noteType = model.get('noteType');
                    }

                    if (docType
                        && docType.indexOf('Outpatient Encounter') >= 0) {
                        if (!noteTitle || noteTitle.length < 1) {
                            noteTitle = JLV.common.DIAGNOSIS_UNAVAILABLE_TXT;
                        }
                        if (outpatientEncounter && (outpatientEncounter.complexDataUrl || outpatientEncounter.noteText || outpatientEncounter.vaNote))
                            return JLV.common.linkFormatter(noteTitle, model);
                        else
                            return noteTitle;
                    }

                    if (!noteTitle || noteTitle.length < 1) {
                        noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if (docType
                        && docType.indexOf('Inpatient Note') >= 0) {
                        if (!noteType || noteType.length < 1) {
                            noteType = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                        } else {
                            noteTitle = noteType + ' / ' + noteTitle;
                        }
                    }

                    if (model.get('sensitive')) {
                        noteTitle = noteTitle.replace(/\n/g, '<br />');
                    }


                    return JLV.common.linkFormatter(noteTitle, model);

                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            docType = model.get('documentType');

                        if (normCode && docType) {
                            if (model.get('sourceProtocol')
                                && docType.indexOf('Radiology Exams') >= 0
                                && model.get('sourceProtocol').indexOf('RPC') >= 0) {
                                return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                    'Standardized Type: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if(docType.indexOf('Outpatient Encounter') >= 0){
                                return '<b>Systematized Nomenclature<br/>' +
                                    'of Medicine - Clinical Terms<br/>' +
                                    '(SNOMED CT)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if (docType.indexOf('HAIMS') < 0) {
                                return '<b>Logical Observation Identifiers<br/>' +
                                    'Names and Codes<br/>' +
                                    '(LOINC)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            }
                        }

                        if(model.get('recordDetailsToken') && model.get('recordDetailsToken').indexOf('NOT_SUPPORTED') >= 0){
                            return 'This document can only be opened in HAIMS.';
                        }
                    }
                }
            },
            //{
            //    title: 'DocType',
            //    id: 'documentType',
            //    width: 100,
            //    sortable: true
            //
            //},
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var study = model.get('vixStudy');

                        if (study && study.viewerUrl) {
                            var viewerUrl = study.viewerUrl;
                            if (forPrint) {
                                return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                            }

                            if (model.get('documentType') == "Radiology Exams") {
                                var viewerCmd = String.format("JLV.common.openRadVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));
                            } else {
                                var viewerCmd = String.format("JLV.common.openDocVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));
                            }

                            return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);

                        } else if (model.get('hasImage')) {
                            var siteCode = model.get('site').siteCode;

                            if (forPrint) {
                                return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                            }

                            var uniqueId = '';
                            var imageId = '';
                            var spinnerId = '';

                            if (model.get('documentType') == "Radiology Exams") {
                                var radiologyReport = model.get('radiologyReport');
                                var hasImage = radiologyReport.hasImage;
                                var caseNumber = radiologyReport.caseNumber;
                                var examId = radiologyReport.examId;
                                var facilityName = radiologyReport.facilityName;

                                uniqueId = examId.replace('.', '');
                                imageId = 'docRadWidgetImage' + uniqueId;
                                spinnerId = 'docRadWidgetSpinner' + uniqueId;

                                var viewerCmd = String.format("JLV.common.getUrlAndOpenRadVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\",\"{6}\",\"{7}\");return false;",
                                    siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);
                            } else {
                                var noteId = model.get('noteId');
                                var images = model.get('images');

                                uniqueId = noteId;
                                imageId = 'docNoteWidgetImage' + uniqueId;
                                spinnerId = 'docNoteWidgetSpinner' + uniqueId;

                                var viewerCmd = String.format("JLV.common.getUrlAndOpenDocVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\");return false;",
                                    siteCode, noteId, images, model.get('recordDetailsToken'), imageId, spinnerId);
                            }

                            return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id="' + spinnerId + '" style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);

                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 80,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        },
        data: JLV.PatientDocumentsCollection
    },
    itemClicked: function (model, item) {
        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function (agree, auditText) {
                if (agree) {
                    self._handleDetails(model, item, auditText);
                }
            });

        } else {

            this._handleDetails(model, item);
        }
    },
    _handleDetails: function (model, item, auditText) {

        var auditUrl;
        if ((model.get('documentType') == "Progress Note")
            || (model.get('documentType') == "Encounter Note")
            || (model.get('documentType') == "HAIMS")) {

            if (model.get('noteAsBinaryStream')) {
                if (auditText) {
                    auditUrl = String.format("patientDocuments/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), auditText);
                } else {
                    auditUrl = String.format("patientDocuments/getNoteDataStream?patToken={0}&recToken={1}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));
                }

                var noteURL = auditUrl;
                if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {
                    JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                        if (download) {
                            JLV.common.openAuxBrowserWindow(noteURL, 'JLV_PROGRESS_NOTE');
                        }
                        else if (rb) {
                            this._addToReportBuilder(model, true, 'GetNoteDataStream');
                        }

                    }, this));
                }
                else {
                    JLV.common.openAuxBrowserWindow(noteURL, 'JLV_PROGRESS_NOTE');
                }
                // $.post('auditUserAction/auditDetails', {
                //     recordType: "ProgressNoteDetail",
                //     patToken: JLV.ActivePatientModel.get('patientToken'),
                //     recordSiteMoniker: model.get('site').moniker,
                //     recordSiteAgency: model.get('site').agency,
                //     info: model.get('noteType') +'^' + model.get('noteTitle') + '^' + model.get('noteDate').toString(),
                //     complexTransaction: model.get('complexDataUrl')
                // });
            }

            if (model.get('noteAsPlainText')) {
                auditUrl = 'patientDocuments/details.json';

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Document Details ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('noteTitle')),
                    url: auditUrl,
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken'),
                        title: model.get('noteTitle'),
                        widgetName: 'Documents'
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response, forPrint) {
                        var thumbnail = "";
                        if (JLV.common.isVixDisplayEnabled && model) {
                            var study = model.get('vixStudy');
                            if (study && study.viewerUrl) {
                                var series;
                                var viewerCmd =
                                    String.format('JLV.common.openDocVixViewer(\'{0}\',\'{1}\');',
                                        study.viewerUrl, model.get('recordDetailsToken'));

                                if (study.series.length > 0) {
                                    series = study.series[0];

                                    if (forPrint) {
                                        thumbnail = '<div style="float:right;display: inline-block;">' +
                                            '<div>Associated image(s) not available for printing.</div>' +
                                            '</div>';

                                        response.images = '';
                                    }
                                    else {
                                        thumbnail = String.format(
                                            '<div style="float:right;display: inline-block;">' +
                                            '<a href="#" onClick="{0}">' +
                                            '<span style="position: relative;">' +
                                            '<img src="{1}" alt="{2}" title="{3}"/>' +
                                            '<div class="camera-icon">' +
                                            '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                            '</div>' +
                                            '</a></span>' +
                                            '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                            '</div>',
                                            viewerCmd, series.thumbnailUrl, series.caption, series.caption, viewerCmd);
                                    }


                                }
                            } else if (model.get('hasImage')) {
                                var siteCode = model.get('site').siteCode;
                                var noteId = model.get('noteId');
                                var images = model.get('images');

                                var uniqueId = noteId;
                                var imageId = 'docNoteDetailImage' + uniqueId;
                                var spinnerId = 'docNoteDetailSpinner' + uniqueId;

                                var viewerCmd = String.format('JLV.common.getUrlAndOpenDocVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\',\'{5}\');return false;',
                                    siteCode, noteId, images, model.get('recordDetailsToken'), imageId, spinnerId);

                                if (forPrint) {
                                    thumbnail = '<div style="float:right;display: inline-block;">' +
                                        '<div>Associated image(s) not available for printing.</div>' +
                                        '</div>';

                                    response.images = '';
                                } else {
                                    thumbnail = String.format(
                                        '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                        '<a href="#" onClick="{0}">' +
                                        '<span style="position: relative;">' +
                                        '<img src="{1}" alt="{2}" title="{3}"/>' +
                                        '<div class="camera-icon">' +
                                        '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                        '</div>' +
                                        '</a></span>' +
                                        '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                        '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                        viewerCmd, "assets/thumbnail.png", model.get('noteTitle'), model.get('noteTitle'), viewerCmd);
                                }
                            }
                        }

                        return thumbnail + '<pre>' + response.noteText + response.images + '</pre>';
                    }
                });

                dialog.center();
                dialog.set('visible', true);

            }
        }
        else if (model.get('documentType') == "Inpatient Note") {
            if (model.get('recordDetailsToken')) {


                if (model.get('noteAsBinaryStream')) {

                    var noteURL = String.format("patientDischargeSummaries/details?patToken={0}&recToken={1}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));


                    if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {
                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {
                                JLV.common.openAuxBrowserWindow(noteURL, "Discharge/Essentris Note Details");
                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetNoteDataStream');
                            }

                        }, this));
                    }
                    else {
                        JLV.common.openAuxBrowserWindow(noteURL, "Discharge/Essentris Note Details");
                    }


                }
                else {
                    var dialog = new JLV.DetailsWindow({
                        title: String.format("Discharge/Essentris Note Details ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            model.get('noteType')),
                        url: 'patientDischargeSummaries/details.json',
                        queryParams: {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recToken: model.get('recordDetailsToken'),
                            title: model.get('noteTitle'),
                            widgetName: 'Documents'
                        },
                        model: model,
                        widget: this,
                        reportBuilder: true,
                        renderer: function (response, forPrint) {
                            if (!response.images || forPrint) {
                                response.images = "";
                            }
                            return '<pre>' + response.noteText + response.images + '</pre>';
                        }
                    });

                    dialog.center();
                    dialog.set('visible', true);
                }


            }
        }
        else if (model.get('documentType') == "Consult Report") {
            if (model.get('recordDetailsToken')) {

                if (model.get('noteAsBinaryStream')) {

                    var consultURL = String.format("patientConsults/getNoteDataStream?patToken={0}&recToken={1}&widgetName={2}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'), this.name);


                    if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {

                                JLV.common.openAuxBrowserWindow(consultURL, "JLV_CONSULT_REPORT");
                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetNoteDataStream');
                            }
                        }, this));
                    }
                    else {
                        JLV.common.openAuxBrowserWindow(consultURL, "JLV_CONSULT_REPORT");
                    }
                }
                else {
                    var dialog = new JLV.DetailsWindow({
                        title: String.format("Consult Encounters Detail ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            model.get('noteTitle')),
                        url: 'patientConsults/details.json',
                        queryParams: {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recToken: model.get('recordDetailsToken'),
                            title: model.get('noteTitle'),
                            widgetName: 'Documents'
                        },
                        model: model,
                        widget: this,
                        reportBuilder: true,
                        renderer: function (response) {
                            return '<pre>' + response.report + '</pre>';
                        }
                    });

                    dialog.center();
                    dialog.set('visible', true);
                }


            }
        }
        else if (model.get('documentType') == "Outpatient Encounter") {

            if (item.text() !== 'Details') {

                var auditUrl;

                if (model.get('noteAsBinaryStream')) {


                    if (auditText) {
                        auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'),
                            auditText
                        );
                    } else {
                        auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}",
                            JLV.ActivePatientModel.get('patientToken'),
                            model.get('recordDetailsToken'));
                    }

                    var noteURL = auditUrl;

                    if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {
                                JLV.common.openAuxBrowserWindow(noteURL, "JLV_ENCOUNTER_NOTE");

                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetNoteDataStream');
                            }

                        }, this));
                    }
                    else {
                        JLV.common.openAuxBrowserWindow(noteURL, "JLV_ENCOUNTER_NOTE");

                    }

                }
                else {
                    auditUrl = 'patientEncounters/details.json';
                    if (auditText) {
                        auditUrl = String.format(auditUrl + "?auditText={0}", auditText);
                    }


                    var dialog = new JLV.DetailsWindow({
                        title: String.format("Outpatient Encounters Note Detail ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            JLV.common.dateFormatter(model.get('noteDate'))),
                        url: auditUrl,
                        queryParams: {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recToken: model.get('recordDetailsToken'),
                            title: model.get('noteTitle'),
                            widgetName: 'Documents'
                        },
                        model: model,
                        widget: this,
                        reportBuilder: true,
                        renderer: function (response, forPrint) {
                            if (forPrint) {
                                response.images = '';
                            }

                            return '<pre class="report-details">' +
                                response.noteText +
                                response.images + '</pre>'
                        }
                    });

                    dialog.center();
                    dialog.open();
                }
            }
            else {

                var widget = this;

                if (model.get('sensitive')) {
                    var auditUrl = String.format('patientEncounters/sensitiveEncounter.json?auditText={0}', auditText);

                    // need to retrieve the encounter from the cache
                    $.post(auditUrl, {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recToken: model.get('recordDetailsToken')
                        },
                        function (response) {

                            var encounterModel = new JLV.WidgetDataModel(response, {parse: true});

                            if (encounterModel.attributes.codes) {
                                for (var i in encounterModel.attributes.codes) {
                                    if (encounterModel.attributes.codes[i].system) {
                                        if (encounterModel.attributes.codes[i].system === '2.16.840.1.113883.6.96' ||
                                            encounterModel.attributes.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {

                                            encounterModel.attributes.normCode = encounterModel.attributes.codes[i].code;
                                            encounterModel.attributes.normDisplay = encounterModel.attributes.codes[i].display;
                                            break;
                                        }
                                    }
                                }
                            }

                            var dialog = new JLV.PatientEncounterDetailsWindow({
                                title: String.format("Outpatient Encounters Detail ~ {0} {1}",
                                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                    model.get('clinic')),
                                encounter: encounterModel,
                                model: model,
                                widget: widget,
                                reportBuilder: true,
                                patientToken: JLV.ActivePatientModel.get('patientToken')
                            });

                            dialog.center();
                            dialog.open();

                        })
                } else {
                    var dialog = new JLV.PatientEncounterDetailsWindow({
                        title: String.format("Outpatient Encounters Detail ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            model.get('outpatientEncounter').clinic),
                        encounter: new JLV.WidgetDataModel(model.get('outpatientEncounter')),
                        model: model,
                        widget: widget,
                        reportBuilder: true,
                        patientToken: JLV.ActivePatientModel.get('patientToken')
                    });

                    dialog.center();
                    dialog.open();

                }

            }
        }
        else if (model.get('documentType') == "Radiology Exams") {
            if (model.get('recordDetailsToken')) {
                var dialog = new JLV.DetailsWindow({
                    width: 780,
                    height: 540,
                    title: String.format("Radiology Reports Detail ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('noteTitle')),
                    url: 'patientRadiologyExams/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken'),
                        title: model.get('noteTitle'),
                        widgetName: 'Documents'
                        //,radPriority: model.get('priority')
                    },
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    renderer: function (response, forPrint) {

                        if (response.plainTextReport) {
                            var thumbnail = "";
                            if (JLV.common.isVixDisplayEnabled && model) {
                                var study = model.get('vixStudy');
                                if (study && study.viewerUrl) {
                                    var series;
                                    var viewerCmd =
                                        String.format('JLV.common.openRadVixViewer(\'{0}\',\'{1}\');',
                                            study.viewerUrl, model.get('recordDetailsToken'));

                                    if (study.series.length > 0) {
                                        series = study.series[0];
                                        if (forPrint) {
                                            thumbnail = '<div style="float:right;display: inline-block;">' +
                                                '<div>Associated image(s) not available for printing.</div>' +
                                                '</div>';

                                            response.images = '';
                                        }
                                        else {
                                            thumbnail = String.format(
                                                '<div style="float:right;display: inline-block;">' +
                                                '<a href="#" onClick="{0}">' +
                                                '<span style="position: relative;">' +
                                                '<img src="{1}" alt="{2}" title="{3}"/>' +
                                                '<div class="camera-icon">' +
                                                '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                                '</div>' +
                                                '</a></span>' +
                                                '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                                '</div>',
                                                viewerCmd, series.thumbnailUrl, series.caption, series.caption, viewerCmd);

                                        }
                                    }
                                } else if (model.get('hasImage')) {
                                    var siteCode = model.get('site').siteCode;
                                    var radiologyReport = model.get('radiologyReport');
                                    var hasImage = radiologyReport.hasImage;
                                    var caseNumber = radiologyReport.caseNumber;
                                    var examId = radiologyReport.examId;
                                    var facilityName = radiologyReport.facilityName;

                                    var uniqueId = examId.replace('.', '');
                                    var imageId = 'docRadDetailImage' + uniqueId;
                                    var spinnerId = 'docRadDetailSpinner' + uniqueId;

                                    var viewerCmd = String.format('JLV.common.getUrlAndOpenRadVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\',\'{5}\',\'{6}\',\'{7}\');return false;',
                                        siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);

                                    if (forPrint) {
                                        thumbnail = '<div style="float:right;display: inline-block;">' +
                                            '<div>Associated image(s) not available for printing.</div>' +
                                            '</div>';

                                        response.images = '';
                                    } else {
                                        thumbnail = String.format(
                                            '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                            '<a href="#" onClick="{0}">' +
                                            '<span style="position: relative;">' +
                                            '<img src="{1}" alt="{2}" title="{3}"/>' +
                                            '<div class="camera-icon">' +
                                            '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                            '</div>' +
                                            '</a></span>' +
                                            '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                            '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                            viewerCmd, "assets/thumbnail.png", model.get('noteTitle'), model.get('noteTitle'), viewerCmd);

                                    }
                                }
                            }

                            var html = "";
                            html += _.template(
                                thumbnail +
                                '<table class="radiology-report" style="float:left;" border="0">' +
                                '<tr>' +
                                '<td class="rad-header rad-td">Priority:</td>' +
                                '<td class="rad-td">{{priority}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td>&nbsp;</td>' +
                                '<td>&nbsp;</td>' +
                                '</tr>' +
                                '</table>'
                            )(response);

                            html += '<pre class="plain-text">' + response.reportText + '</pre>';
                            return html;
                        }
                        else {

                            response.amendedText = model.get('amendedText');
                            if (model.get('amendedDate')) {
                                response.amendedDate = moment(model.get('amendedDate')).format('M/D/YY');
                            }
                            else {
                                response.amendedDate = "";
                            }

                            var html = "";
                            if (JLV.ActiveProvider.agency !== JLV.common.AGENCY_DOD) {
                                html += _.template(
                                    '<table class="radiology-report" width="100%" border="0">' +
                                    '<tr>' +
                                    '<td class="rad-header">Priority:</td>' +
                                    '<td>{{priority}}</td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Exam Number:</td>' +
                                    '<td>{{examId}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header">Procedure:</td>' +
                                    '<td>{{procedure}}</td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Exam Date:</td>' +
                                    '<td>{{examDate}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header">Result Code:</td>' +
                                    '<td>{{resultCode}} {{abnormal}}</td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Report Status:</td>' +
                                    '<td>{{reportStatus}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{orderReason}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Impression:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{impressionText}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Report:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{reportText}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Transcription D/T:</span>' +
                                    '&nbsp;' +
                                    '<span>{{transcribeDate}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Interpreted by:</span>' +
                                    '&nbsp;' +
                                    '<span>{{interpretingHCP}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Approved by:</span>' +
                                    '&nbsp;' +
                                    '<span>{{approvingHCP}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Amended Date:</span>' +
                                    '&nbsp;' +
                                    '<span>{{amendedDate}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Amended Text:</span>' +
                                    '&nbsp;' +
                                    '<span>{{amendedText}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '{[ } ]}' +
                                    '</table>'
                                )(response);
                            } else {
                                html += _.template(
                                    '<table class="radiology-report" width="100%" border="0">' +
                                    '<tr>' +
                                    '<td class="rad-header"></td>' +
                                    '<td></td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Exam Number:</td>' +
                                    '<td>{{examId}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header">Procedure:</td>' +
                                    '<td>{{procedure}}</td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Exam Date:</td>' +
                                    '<td>{{examDate}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header">Result Code:</td>' +
                                    '<td>{{resultCode}} {{abnormal}}</td>' +
                                    '<td>&nbsp;</td>' +
                                    '<td class="rad-header">Report Status:</td>' +
                                    '<td>{{reportStatus}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{orderReason}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Impression:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{impressionText}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td class="rad-header" colspan="5">Report:</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">{{reportText}}</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                    '</tr>' +
                                    '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Transcription D/T:</span>' +
                                    '&nbsp;' +
                                    '<span>{{transcribeDate}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Interpreted by:</span>' +
                                    '&nbsp;' +
                                    '<span>{{interpretingHCP}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Approved by:</span>' +
                                    '&nbsp;' +
                                    '<span>{{approvingHCP}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Amended Date:</span>' +
                                    '&nbsp;' +
                                    '<span>{{amendedDate}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '<tr>' +
                                    '<td colspan="5">' +
                                    '<span class="rad-header">Amended Text:</span>' +
                                    '&nbsp;' +
                                    '<span>{{amendedText}}</span>' +
                                    '</td>' +
                                    '</tr>' +
                                    '{[ } ]}' +
                                    '</table>'
                                )(response);
                            }

                            return html;

                        }

                    }
                });

                dialog.center();
                dialog.open();
            }
        }
        //else if(model.get('documentType') == "HAIMS"){
        //
        //}
        else if ((model.get('documentType') == "Questionnaires")
            || (model.get('documentType') == "Deployment Form")) {
            if (model.get('recordDetailsToken')) {

                if (model.get('noteAsBinaryStream')) {

                    var noteURL = String.format("patientForms/getNoteDataStream?patToken={0}&recToken={1}",
                        JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

                    if (JLV.common.canAddTypeToReportBuilder(model.get('contentType'))) {

                        JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                            if (download) {
                                JLV.common.openAuxBrowserWindow(noteURL, 'JLV_DEPLOYMENT_FORMS');
                            }
                            else if (rb) {
                                this._addToReportBuilder(model, true, 'GetNoteDataStream');
                            }

                        }, this));
                    } else {
                        JLV.common.openAuxBrowserWindow(noteURL, 'JLV_DEPLOYMENT_FORMS');
                    }
                }
                else {
                    var dialog = new JLV.DetailsWindow({
                        title: String.format("Questionnaire ~ {0} {1}",
                            JLV.common.formatSiteMonikerDisplay(model.get('site')),
                            model.get('noteTitle')),
                        width: 700,
                        height: 500,
                        url: 'patientForms/details.json',
                        queryParams: {
                            patToken: JLV.ActivePatientModel.get('patientToken'),
                            recToken: model.get('recordDetailsToken'),
                            title: model.get('noteTitle'),
                            widgetName: 'Documents'
                        },
                        model: model,
                        widget: this,
                        reportBuilder: true,
                        renderer: function (response) {
                            return '<div class="report-details">' + response.reportText + '</div>';
                        }
                    });

                    dialog.center();
                    dialog.set('visible', true);

                }

            }

        }


    }

});








JLV.registerJLVWidget('documents_widget', JLV.PatientDocuments);
