//= require ./PatientWidget.js
//= require ./windows/PatientHistoriesWidgetWindow.js

JLV.PatientHistoriesCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'type',
    url: 'patientHistories/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['dateReported'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system === '2.16.840.1.113883.6.96' ||
                            data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            if(data.codes[i].display){
                                data.normDisplay = data.codes[i].display;
                            }
                            else {
                                data.normDisplay = '';
                            }
                            break;
                        }
                    }
                }
            }

            if (data.findings) {
                var list = "";

                for (var i in data.findings) {
                    if (data.findings[i] && data.findings[i].item) {
                        if (list.length > 0)
                            list += "; ";
                        list += data.findings[i].item;
                    }
                }
                data.findingList = list;
            }


            if (data.comments && data.comments.length > 0) {
                var list = "";

                for (var i in data.comments) {
                    if (data.comments[i]) {
                        if (list.length > 0)
                            list += "; ";
                        list += data.comments[i];
                    }
                }
                data.commentList = list;
            }



            return this._super(data);
        }
    }),
    doFetch: function (patient, callback) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                status: 'A'
            },
            reset: true,
            type: 'POST',
            success: callback
        });
    }
});

JLV.PatientHistories = JLV.PatientWidget.extend({
    options: {
        widgetId: 'histories_widget',
        name: 'Social, Family, and Other Past Histories',
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientHistoriesWidgetWindow',
        columns: [
            {
                width: 80,
                title: 'Date Reported',
                sortable: true,
                id: 'dateReported',
                formatter: JLV.common.dateFormatter
            },
            {
                id: 'type',
                width: 80,
                title: 'Type'
                // ,
                // tooltip: function (row, model) {
                //     if (model) {
                //         var normCode = model.get('normCode'),
                //             normDisplay = model.get('normDisplay');
                //
                //         if (normCode) {
                //             return '<b>Systematized Nomenclature<br/>' +
                //                 'of Medicine - Clinical Terms<br/>' +
                //                 '(SNOMED CT)</b><br/>' +
                //                 'Standardized Name: ' + normDisplay + '<br/>' +
                //                 'Standardized Code: ' + normCode;
                //         }
                //
                //     }
                //
                //     return '<b>Systematized Nomenclature<br/>' +
                //         'of Medicine - Clinical Terms<br/>' +
                //         '(SNOMED CT)</b><br/>' +
                //         'Standardized Name: none<br/>' +
                //         'Standardized Code: none'
                // }
            },
            {
                width: 80,
                title: 'Findings',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'findingList' //,
                //tooltip: function (row, model) {
                //    if (model) {
                //        var normCode = model.get('normCode'),
                //            normDisplay = model.get('normDisplay');
                //
                //        if (normCode) {
                //            return '<b>Systematized Nomenclature<br/>' +
                //                'of Medicine - Clinical Terms<br/>' +
                //                '(SNOMED CT)</b><br/>' +
                //                'Standardized Name: ' + normDisplay + '<br/>' +
                //                'Standardized Code: ' + normCode;
                //        }
                //    }
                //
                //    return '<b>Systematized Nomenclature<br/>' +
                //        'of Medicine - Clinical Terms<br/>' +
                //        '(SNOMED CT)</b><br/>' +
                //        'Standardized Name: none<br/>' +
                //        'Standardized Code: none';
                //}
            },
            {
                title: 'Status',
                width: 40,
                sortable: true,
                id: 'status'
            },
            {
                title: 'Site',
                id: 'site',
                width: 70,
                sortable: true,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'dateReported',
            asc: false
        },
        data: JLV.PatientHistoriesCollection
    }

});


JLV.registerJLVWidget('histories_widget', JLV.PatientHistories);
