//= require PatientWidget.js
//= require ./windows/PatientLabsWidgetWindow.js
//= require ./windows/PatientLabDetailsWindow.js

JLV.PatientLabsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'testName',
    url: 'patientLabs/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['collectionDate', 'orderedDate'],
        parse: function (data) {

            if (data.requestingProvider) {
                data.taxonomy = data.requestingProvider.providerType;
                data.taxonomyCode = data.requestingProvider.taxonomyCode;
                data.providerName = data.requestingProvider.name;
            }

            data.requestingLocation = "";
            if (data.labResults) {
                for (var i in data.labResults) {
                    if (data.labResults[i].requestingLocation) {
                        data.requestingLocation = data.labResults[i].requestingLocation;
                        break;

                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientLabs = JLV.PatientWidget.extend({
    options: {
        widgetId: 'labs_widget',
        name: 'Lab Panel Results',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientLabsWidgetWindow',
        columns: [
            {
                title: 'Collection Date',
                id: 'collectionDate',
                width: 120,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Order / Result',
                id: 'testName',
                width: 160,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && (model.get('labResults') || model.get('sensitive'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Site',
                id: 'site',
                width: 100,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'collectionDate',
            asc: false
        },
        data: JLV.PatientLabsCollection
    },
    itemClicked: function (model) {

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            };

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function (agree, auditText) {
                if (agree) {
                    self._handleDetails(model, auditText);
                }
            });
        } else {
            this._handleDetails(model);
        }

    },
    _handleDetails: function(model, auditText) {
        if (model.get('sensitive')) {
            // if sensitive, retreive the model from the cache first
            var labResultUrl = String.format('patientLabs/sensitiveOrder.json?auditText={0}', auditText)

            var widget = this;

            $.post(labResultUrl, {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                }, function (response) {

                    var labmodel = new JLV.WidgetDataModel(response, {parse: true});

                    if (labmodel.get('freeTextReport')) {
                        var dialog = new JLV.PatientLabReportWindow({
                            title: String.format("Lab Panel Results Detail ~ {0} {1}",
                                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                model.get('testName')),
                            model: model,
                            widget: widget,
                            reportBuilder: true,
                            patientToken: JLV.ActivePatientModel.get('patientToken'),
                            labResult: labmodel.get('labResults')[0]
                        });

                        dialog.center();
                        dialog.open();
                    }
                    else if (labmodel.get('labResults')) {

                        var dialog = new JLV.PatientLabResultsWindow({
                            width: 860,
                            height: 460,
                            title: String.format("Lab Panel Results ~ {0} {1}",
                                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                model.get('testName')),
                            model: model,
                            widget: widget,
                            reportBuilder: true,
                            patientToken: JLV.ActivePatientModel.get('patientToken'),
                            labOrder: labmodel
                        });
                        dialog.center();
                        dialog.open();
                    }
                    else {
                        AppKit.AlertDialog.alert('No Data', 'No lab report on ' + JLV.common.dateFormatter(model.get('collectionDate')) + ' for NONE');
                    }
                }
            )

        } else {
            if (model.get('freeTextReport')) {
                var dialog = new JLV.PatientLabReportWindow({
                    title: String.format("Lab Panel Results Detail ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('testName')),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    patientToken: JLV.ActivePatientModel.get('patientToken'),
                    labResult: model.get('labResults')[0]
                });

                dialog.center();
                dialog.open();
            }
            else if (model.get('labResults')) {

                var dialog = new JLV.PatientLabResultsWindow({
                    width: 860,
                    height: 460,
                    title: String.format("Lab Panel Results ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('testName')),
                    model: model,
                    widget: this,
                    reportBuilder: true,
                    patientToken: JLV.ActivePatientModel.get('patientToken'),
                    labOrder: model
                });
                dialog.center();
                dialog.open();
            }
            else {
                AppKit.AlertDialog.alert('No Data', 'No lab report on ' + JLV.common.dateFormatter(model.get('collectionDate')) + ' for NONE');
            }
        }

    }
});

JLV.registerJLVWidget('labs_widget', JLV.PatientLabs);

