//= require ./PatientWidget.js

JLV.MhsGenesisCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'document',
    url: 'mhsGenesis/list.json',
    model: JLV.PatientDataModel.extend({
        parse: function (data) {


            return this._super(data);
        }
    })
});

JLV.PatientMhsGenesis = JLV.PatientWidget.extend({
    options: {
        widgetId: 'mhs_genesis_widget',
        name: 'MHS GENESIS',
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        columns: [
            {
                title: 'Date',
                width: '25%',
                id: 'date',
                sortable: true,
                sorter: JLV.common.dateSortWithBlanksFirst,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Document Title',
                width: '40%',
                id: 'title',
                sortable: true,
                formatter: function (value, model) {
                    if (model.get('noteAsBinaryStream')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Document Type',
                width: '20%',
                id: 'type',
                sortable: true
            },
            {
                title: 'Site',
                width: '15%',
                id: 'site',
                sortable: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'date',
            asc: false
        },
        data: JLV.MhsGenesisCollection
    },
    itemClicked: function (model) {
        if (model.get('recordDetailsToken') && model.get('noteAsBinaryStream')) {
            var docUrl = String.format("mhsGenesis?patToken={0}&recToken={1}",
                JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

            JLV.common.openAuxBrowserWindow(docUrl, "JLV - " + model.get('document'));
        }
    },
    loadData: function(callback){
        if(JLV.common.disableGenesis) {
            this.addBanner({
                id: 'genesisBanner',
                bgcolor: '#efe3b3',
                fgcolor: '#FF0000',
                closable: false,
                text: 'MHS GENESIS data available at Go-Live',
                height: 34
            });
            this._clipboardBtn.tooltip({
                content: 'Clipboard is disabled until GENESIS data is available.'
            });

            this._clipboardBtn.css({opacity: 0.5});

            this._printBtn.tooltip({
                content: 'Printing is disabled until GENESIS data is available.'
            });

            this._printBtn.css({opacity: 0.5});

            this._refreshBtn.tooltip({
                content: 'Refresh is disabled until GENESIS data is available.'
            });

            this._refreshBtn.css({opacity: 0.5});

            this._interfaceStatusBtn.tooltip({
                content: 'Interface Status is disabled until GENESIS data is available.'
            });

            this._interfaceStatusBtn.css({opacity: 0.5});

            this._settingsBtn.tooltip({
                content: 'Column Settings is disabled until GENESIS data is available.'
            });

            this._settingsBtn.css({opacity: 0.5});
        } else {
            this._super(callback);
        }
    }
});

JLV.registerJLVWidget('mhs_genesis_widget', JLV.PatientMhsGenesis);
