//= require ./PatientWidget.js
//= require ./windows/PatientRadiologyExamsWidgetWindow.js

JLV.PatientRadiologyExamsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'study',
    url: 'patientRadiologyExams/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['examDate', 'amendedDate'],
        parse: function (data) {

            if (data.codes) {
                var loincSet = false;
                var cptSet = false;
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if ((data.codes[i].system === '2.16.840.1.113883.6.1' ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0 )&& !loincSet) {
                            data.normCodeLoinc = data.codes[i].code;
                            if(data.codes[i].display !== null){
                                data.normDisplayLoinc = data.codes[i].display;
                            } else {
                                data.normDisplayLoinc = '';
                            }
                            loincSet = true;
                        }

                        if ((data.codes[i].system === '2.16.840.1.113883.6.12' ||
                            data.codes[i].system.toUpperCase().indexOf("CPT") >= 0) && !cptSet) {
                            data.normCodeCpt = data.codes[i].code;
                            if(data.codes[i].display !== null){
                                data.normDisplayCpt = data.codes[i].display;
                            } else {
                                data.normDisplayCpt = '';
                            }
                            cptSet = true;
                        }
                    }
                }
            }

            if (data.examId) {
                data.examIdDisplay = data.examId;
                data.examId = parseInt(data.examId, 10);
            }

            return this._super(data);
        }
    })
});


JLV.PatientRadiologyExams = JLV.PatientWidget.extend({
    options: {
        widgetId: 'radiologyexams_widget',
        name: 'Radiology Reports',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientRadiologyExamsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Date',
                id: 'examDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Exam',
                id: 'study',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    var formattedString = value;

                    if (model && model.get('recordDetailsToken')) {
                        formattedString = JLV.common.linkFormatter(value, model);
                    }

                    /*if (model && model.get('abnormal')) {
                        if (model.get('abnormal').toLowerCase().indexOf('abnormal') > -1
                            || model.get('abnormal').toLowerCase() !== "normal")
                        {
                            formattedString = '<div class="jlv-alert-row">' + formattedString + '</div>';
                        }
                     } */

                    return formattedString;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplayLoinc + '<br/>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplayCpt + '<br/>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 60,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var study = model.get('vixStudy');
                        var hasImage = model.get('hasImage');

                        if (study && study.viewerUrl) {
                            var viewerUrl = study.viewerUrl;
                            if (viewerUrl) {
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var viewerCmd = String.format("JLV.common.openRadVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));

                                return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                            }
                        } else if (hasImage) {
                            var siteCode = model.get('site').siteCode;
                            var caseNumber = model.get('caseNumber');
                            var examId = model.get('examIdDisplay');
                            var facilityName = model.get('facilityName');
                            if (siteCode && caseNumber && examId && facilityName) {
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var uniqueId = examId.replace('.', '');
                                var imageId = 'radWidgetImage' + uniqueId;
                                var spinnerId = 'radWidgetSpinner' + uniqueId;

                                var viewerCmd = String.format("JLV.common.getUrlAndOpenRadVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\",\"{6}\",\"{7}\");return false;",
                                    siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);

                                return String.format('<div id=' + imageId + ' style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id=' + spinnerId + ' style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                            }
                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                sortable: true,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'examDate',
            asc: false
        },
        data: JLV.PatientRadiologyExamsCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.checkConnectionUnavailable();

            }, this)
        });

        //if (this._datagrid) {
        //    //make rows with abnormal results pink
        //    this._datagrid.on('afterRender', function () {
        //
        //        var $row = this.$el.find('.jlv-alert-row').parents('tr');
        //        $row.css(
        //            {
        //                'backgroundColor': '#FFC0CB',
        //                'color': '#000000'
        //            }
        //        );
        //        $row.find('a').css('color', '#000');
        //    });
        //}
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                width: 780,
                height: 540,
                title: String.format("Radiology Reports Detail ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('study')),
                url: 'patientRadiologyExams/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    radPriority: model.get('priority'),
                    procedure: model.get('procedure'),
                    recordDate: model.get('examDate')
                },
                model: model,
                widget: this,
                reportBuilder: true,
                renderer: function (response, forPrint) {
                    var thumbnail = "";
                    var study = model.get('vixStudy');
                    var hasImage = model.get('hasImage');
                    if (JLV.common.isVixDisplayEnabled) {
                        if (study && study.viewerUrl) {
                            var series;
                            var viewerCmd =
                                String.format('JLV.common.openRadVixViewer(\'{0}\',\'{1}\');',
                                    study.viewerUrl, model.get('recordDetailsToken'));

                            if (study.series.length > 0) {
                                series = study.series[0];
                                if (forPrint) {
                                    thumbnail = '<div style="float:right;display: inline-block;">' +
                                        '<div>Associated image(s) not available for printing.</div>' +
                                        '</div>';
                                }
                                else {

                                    thumbnail = String.format(
                                        '<div style="float:right;display: inline-block;">' +
                                        '<a href="#" onClick="{0}">' +
                                        '<span style="position:relative;">' +
                                        '<img src="{1}" alt="{2}" title="{3}"/>' +
                                        '<div class="camera-icon">' +
                                        '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                        '</div>' +
                                        '</a></span>' +
                                        '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                        '</div>',
                                        viewerCmd, "assets/thumbnail.png", model.get('study'), model.get('study'), viewerCmd);
                                }
                            }
                        } else if (hasImage) {
                            var siteCode = model.get('site').siteCode;
                            var caseNumber = model.get('caseNumber');
                            var examId = model.get('examIdDisplay');
                            var facilityName = model.get('facilityName');

                            if (siteCode && caseNumber && examId && facilityName) {
                                var uniqueId = examId.replace('.', '');
                                var imageId = 'radDetailImage' + uniqueId;
                                var spinnerId = 'radDetailSpinner' + uniqueId;

                                var viewerCmd = String.format('JLV.common.getUrlAndOpenRadVixViewer(\'{0}\',\'{1}\',\'{2}\',\'{3}\',\'{4}\',\'{5}\',\'{6}\',\'{7}\');return false;',
                                    siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);

                                if (forPrint) {
                                    thumbnail = '<div style="float:right;display: inline-block;">' +
                                        '<div>Associated image(s) not available for printing.</div>' +
                                        '</div>';
                                } else {
                                    thumbnail = String.format(
                                        '<div id="' + imageId + '" style="float:right;display: inline-block;">' +
                                        '<a href="#" onClick="{0}">' +
                                        '<span style="position:relative;">' +
                                        '<img src="{1}" alt="{2}" title="{3}"/>' +
                                        '<div class="camera-icon">' +
                                        '<img  src="assets/camera_icon.png" alt="camera"/>' +
                                        '</div>' +
                                        '</a></span>' +
                                        '<div class="vix-link"><a href="#" onClick="{4}">Click image to open viewer</a></div>' +
                                        '</div><div id=' + spinnerId + ' style="display: none;float:right;"><img src="assets/loading.gif"/></div>',
                                        viewerCmd, "assets/thumbnail.png", model.get('study'), model.get('study'), viewerCmd);
                                }
                            }
                        }
                    }

                    if (response.plainTextReport) {
                        var html = "";
                        html += _.template(
                            thumbnail +
                            '<table class="radiology-report" style="float:left;" border="0">' +
                            '<tr>' +
                            '<td class="rad-header rad-td">Priority:</td>' +
                            '<td class="rad-td">{{priority}}</td>' +
                            '<td>&nbsp;</td>' +
                            '<td>&nbsp;</td>' +
                            '<td>&nbsp;</td>' +
                            '</tr>' +
                            '</table>'
                        )(response);

                        html += '<pre class="plain-text">' + response.reportText + '</pre>';

                        return html;
                    }
                    else {

                        response.amendedText = model.get('amendedText');
                        if (model.get('amendedDate')) {
                            response.amendedDate = moment(model.get('amendedDate')).format('M/D/YY');
                        }
                        else {
                            response.amendedDate = "";
                        }

                        var html = "";
                        if (JLV.ActiveProvider.agency !== JLV.common.AGENCY_DOD) {
                            html += _.template(
                                thumbnail +
                                '<table class="radiology-report" width="100%" border="0">' +
                                '<tr>' +
                                '<td class="rad-header">Priority:</td>' +
                                '<td>{{priority}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Number:</td>' +
                                '<td>{{examId}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Procedure:</td>' +
                                '<td>{{procedure}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Date:</td>' +
                                '<td>{{examDate}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Result Code:</td>' +
                                '<td>{{resultCode}} {{abnormal}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Report Status:</td>' +
                                '<td>{{reportStatus}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{orderReason}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Impression:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{impressionText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '{[ if(status.toLowerCase().indexOf("amended") > -1 || reportStatus.toLowerCase().indexOf("amended") > -1 ) { ]}' +
                                '<td class="rad-header" colspan="5">Amended Report Text:</td>' +
                                '{[ } else { ]}' +
                                '<td class="rad-header" colspan="5">Report Text:</td>' +
                                '{[ } ]}' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{reportText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Transcription D/T:</span>' +
                                '&nbsp;' +
                                '<span>{{transcribeDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Interpreted by:</span>' +
                                '&nbsp;' +
                                '<span>{{interpretingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Approved by:</span>' +
                                '&nbsp;' +
                                '<span>{{approvingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Date:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Text:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedText}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '{[ } ]}' +
                                '</table>'
                            )(response);
                        } else {
                            html += _.template(
                                thumbnail +
                                '<table class="radiology-report" width="100%" border="0">' +
                                '<tr>' +
                                '<td class="rad-header"></td>' +
                                '<td></td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Number:</td>' +
                                '<td>{{examId}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Procedure:</td>' +
                                '<td>{{procedure}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Exam Date:</td>' +
                                '<td>{{examDate}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header">Result Code:</td>' +
                                '<td>{{resultCode}} {{abnormal}}</td>' +
                                '<td>&nbsp;</td>' +
                                '<td class="rad-header">Report Status:</td>' +
                                '<td>{{reportStatus}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Reason for Order:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{orderReason}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td class="rad-header" colspan="5">Impression:</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{impressionText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '<tr>' +
                                '{[ if(status.toLowerCase().indexOf("amended") > -1 || reportStatus.toLowerCase().indexOf("amended") > -1 ) { ]}' +
                                '<td class="rad-header" colspan="5">Amended Report Text:</td>' +
                                '{[ } else { ]}' +
                                '<td class="rad-header" colspan="5">Report Text:</td>' +
                                '{[ } ]}' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">{{reportText}}</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5" style="height:5px;">&nbsp;</td>' +
                                '</tr>' +
                                '{[ if(site.agency !== JLV.common.AGENCY_DOD) { ]}' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Transcription D/T:</span>' +
                                '&nbsp;' +
                                '<span>{{transcribeDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Interpreted by:</span>' +
                                '&nbsp;' +
                                '<span>{{interpretingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Approved by:</span>' +
                                '&nbsp;' +
                                '<span>{{approvingHCP}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Date:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedDate}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '<tr>' +
                                '<td colspan="5">' +
                                '<span class="rad-header">Amended Text:</span>' +
                                '&nbsp;' +
                                '<span>{{amendedText}}</span>' +
                                '</td>' +
                                '</tr>' +
                                '{[ } ]}' +
                                '</table>'
                            )(response);
                        }


                        return html;

                    }

                }
            });

            dialog.center();
            dialog.open();
        }

    }

});


JLV.registerJLVWidget('radiologyexams_widget', JLV.PatientRadiologyExams);