//= require ./PatientWidget.js
//= require ./windows/PatientVLERWidgetWindow.js


/*------ VA VLER Widget -------*/

JLV.VLERCollection = JLV.PatientDataCollection.extend({
    url: 'patientVLER/list.json',
    model: JLV.PatientDataModel.extend({
        parse: function (data) {

            if (data.authors && data.authors[0].institution) {
                var source = data.authors[0].institution;
                if (source.indexOf('^') > 0) {
                    source = source.substring(0, source.indexOf('^'));
                }
                data.institution = source;
            }

            //date rules
            if (data.codes && data.codes.length > 0
                && data.codes[0].code
                && data.codes[0].code.indexOf("34133-9") > -1) {

                //process C32/CCDA
                if (!data.creationTime) {
                    data.creationTime = new Date();
                }
                else {
                    data.creationTime = new Date(data.creationTime);
                    if (!data.creationTime.isValid()) {
                        data.creationTime = null;
                    }
                }

            } else {
                //process c62/others
                if (data.serviceStartTime) {
                    data.creationTime = new Date(data.serviceStartTime);
                } else if (!data.creationTime) {
                    data.creationTime = null;
                }

            }

            return this._super(data);
        }
    })
});

JLV.VLERSites = Backbone.Collection.extend({
    url: 'patientVLER/siteList.json'
});


JLV.PatientVLER = JLV.PatientWidget.extend({
    options: {
        widgetId: 'vler_widget',
        name: 'Community Health Summaries and Documents - VA',
        clipboard: false,
        print: true,
        refresh: true,

        columns: [
            {
                title: 'Date',
                width: '50%',
                id: 'creationTime',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 120
            },
            {
                title: 'Document',
                width: 140,
                id: 'name',
                sortable: true,
                formatter: function (value, model) {
                    if (!(/^Error/).test(value) && !(/available for/).test(value)) {
                        return JLV.common.linkFormatter(value, model);
                    } else {
                        return value;
                    }
                }
            },
            {
                title: 'Source',
                width: 150,
                id: 'institution',
                sortable: true,
                formatter: function (value) {
                    var author = "";
                    if (value) {
                        author = value;
                    } else {
                        author = "";
                    }
                    return JLV.common.formatVLERMonikerDisplay(author);
                }
            }
        ],
        defaultSort: {
            id: 'name',
            asc: false
        },
        data: JLV.VLERCollection
    },
    initialize: function (options) {
        this._super(options);

        this._siteListBtn = $('<div title="Site List" class="jlv-widget-toolbutton">')
            .append('<div class="icon-list">')
            .button()
            .insertAfter(this._settingsBtn)
            .tooltip({
                show: false,
                hide: false,
                content: '<ul>Fetching sites....</ul>',
                tooltipClass: 'vlerSiteList'
            }).off('mouseover')
            .on('focusout, mouseleave', function (evt) {
                evt.stopImmediatePropagation();
            })
            .on('click', function (evt) {
                evt.stopPropagation();
                $(this).tooltip('open');

                $(document).on('click.vlerSiteList, keydown.vlerSiteList', _.bind(function (evt) {
                    $(this).tooltip('close');
                    $(document).off('click.vlerSiteList, keydown.vlerSiteList');
                }, this));
            });


        var self = this;

        new JLV.VLERSites().fetch({
            reset: true,
            type: 'POST',
            success: function (data) {
                var content = '<div class="vlerSiteListHeader">Site List</div><ul>';
                data.each(function (model) {
                    if (model.get('sourceName')) {
                        content += '<li>' + model.get('sourceName') + '</li>';
                    }

                });

                content += '</ul>';
                self._siteListBtn.tooltip('option', 'content', content);
            },
            error: function (collection, err) {
                if (err.status !== 200)
                    self._siteListBtn.tooltip('option', 'content', '<ul>An error occurred when fetching the site list.</ul>');
            }
        });
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken') && model.get('sessionCachedRecord')) {
            var docURL = String.format("VLER/getDocument?patToken={0}&recToken={1}",
                JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

            JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                if (download) {
                    JLV.common.openAuxBrowserWindow(docURL, "JLV - " + model.get('institution') + " : " + model.get('name'));
                }
                else if (rb) {
                    this._addToReportBuilder(model, true, 'GetNoteDataStream');
                }

            }, this));

        } else if (model.get('recordDetailsToken')) {
            var docURL = String.format("VLER?patToken={0}&recToken={1}",
                JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

            JLV.Widget.DownloadConfirm(_.bind(function (rb, download) {

                if (download) {
                    JLV.common.openAuxBrowserWindow(docURL, "JLV - " + model.get('institution') + " : " + model.get('name'));
                }
                else if (rb) {
                    this._addToReportBuilder(model, true, 'GetNoteDataStream');
                }

            }, this));

        }
    }

});

JLV.registerJLVWidget('vler_widget', JLV.PatientVLER);


/*------ DoD VLER Widget -------*/

JLV.PatientDoDVLER = JLV.PatientWidget.extend({
    options: {
        widgetId: 'vler_dod_widget',
        name: 'Community Health Summaries and Documents - DoD',
        clipboard: false,
        print: false,
        refresh: true,
        moreWindowClass: 'JLV.PatientDoDVLERWidgetWindow',
        iframeUrl: undefined,
        settings: false
    },
    initialize: function (options) {

        this._iframe = $('<iframe>')
            .addClass("jlv-dod-vler-iframe");

        this._super(options);

        this._iframe.appendTo(this._body);
    },
    loadData: function () {

        this._body.find('.jlv-dod-vler-no-url').remove();

        //this._moreBtn.hide();

        var self = this;

        $.post('PatientDODVLER/getWidgetUrl.json', {
            patToken: JLV.ActivePatientModel.get('patientToken'),
            theme: JLV.Session.ui_conf.options[0].theme
        }, function (data) {
            //console.info("got url");

            if (data) {
                self._iframe.prop('src', data);
                self._moreBtn.show();

            } else {
                self._body.prepend($('<div class="jlv-dod-vler-no-url">').text("No URL set"));
            }

        }).fail(function (response) {
            self._body.prepend($('<div class="jlv-dod-vler-no-url">').text("No URL set"));
            AppKit.AlertDialog.alert('Warning', "An error occurred get the Community Health Summaries Viewer URL - " + response.responseText);
        });


    },
    iframeUrl: function (aString) {

        if (aString !== undefined) {
            this._iframeURL = aString;
        }

        return this._iframeURL;

    },
    doPrint: function () {

        var dataHTML = '<html><head><title>{{title}}</title>\n{{cssLink}}</head>';
        dataHTML += '<body><div style="text-align: center;font-weight: bold;">For Official Use Only</div>';

        if (JLV.common.isDoDProvider(JLV.ActiveProvider)) {
            dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>DoD ID: </b>{{edipi}}</div>';
        }
        else {
            dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>SSN: </b>{{fmpssn}}</div>';
        }

        dataHTML += '<div style="position:absolute;top:40px;left:0;right:0;bottom:0;overflow:hidden;">{{bodyHTML}}</div></body></html>';


        var printHTML = _.template(dataHTML)(_.extend({
            title: this.get('name') + " Print out - " + JLV.ActivePatientModel.get('name'),
            cssLink: '<link rel="stylesheet" type="text/css" href="assets/view/print.css">',
            bodyHTML: this._body.html()
        }, JLV.ActivePatientModel.toJSON()));


        var printWindow = window.open("", "Print");
        printWindow.document.write(printHTML);

        printWindow.document.close();
        setTimeout(function () {
            printWindow.print();
            printWindow.close();
        }, 2000);
    }
    //,
    ////TODO: NOT SURE WHAT THIS SHOULD DO
    //doClipboard: function () {
    //    if (window.clipboardData) {
    //        clipboardData.setData('text', "VLER DOD clipboard data here");
    //    }
    //}


});

JLV.registerJLVWidget('vler_dod_widget', JLV.PatientDoDVLER);
