//= require ../Widget.js

JLV.PatientDataModel = JLV.WidgetDataModel.extend({});

JLV.PatientDataCollection = Backbone.Collection.extend({
    initialize: function () {
        this.vlerErrors = {};
        this.dasvlerErrors = {};
        this.connectionErrors = {};
    },
    doFetch: function (patient, callback, error) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                includeVLER: JLV.Session.dodVLERReady === true,
                includeDASVLER: JLV.Session.dasVLERReady === true,
                forceRefresh: JLV.Session.forceRefresh === true
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    },
    parse: function (response) {

        this.dodAdapterSourceReport = [];
        this.dodAdapterStatusRec = [];
        this.connectionErrors = {}; //set

        var count = response.length,
            i = 0;

        while (i < count) {

            var model = response[i];
            var site = model.site;
            var modelStr = JSON.stringify(model).toLowerCase();

            if (site && this.dodAdapterSourceReportField
                && site.moniker === JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {

                this.dodAdapterSourceReport.push(model[this.dodAdapterSourceReportField]);
                this.dodAdapterStatusRec.push(model);

                if (modelStr.indexOf('no errors to report') === -1) {
                    this.connectionErrors[site.moniker] = model;
                }


                response.splice(i, 1);
                count = response.length;
            }
            else if (this.dodAdapterSourceReportField && model[this.dodAdapterSourceReportField] &&
                model[this.dodAdapterSourceReportField].toLowerCase().indexOf('connection unavailable') > -1) {
                this.connectionErrors[site.moniker] = model;

                response.splice(i, 1);
                count = response.length;

            }
            else if (site && site.moniker && modelStr.indexOf('connection unavailable') > -1) {
                this.connectionErrors[site.moniker] = model;
                response.splice(i, 1);
                count = response.length;
            }
            else {
                i++;
            }


        }

        return response;
    }
});

JLV.PatientDateFilteredDataCollection = JLV.PatientDataCollection.extend({
    doFetch: function (patient, callback, error) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                startDate: moment(this.startDate).format("M/D/YYYY"),
                endDate: moment(this.endDate).format("M/D/YYYY"),
                includeVLER: JLV.Session.dodVLERReady === true,
                includeDASVLER: JLV.Session.dasVLERReady === true
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});


JLV.PatientWidget = JLV.Widget.extend({

    initialize: function (options) {

        this._super(options);

        if (this._datagrid) {

            var self = this;
            //add report builder functionality
		    this._datagrid.on('afterRender', function () {

                //508 keyboard access to add record to RB
                this.$el.find('.ui-table-body tr').on('keydown', function (evt) {

                    if (evt.which === 82 && evt.ctrlKey) { //ctrl-R
                        evt.preventDefault();

                        var $row = $(this);
                        if ($row.text().indexOf('Connection unavailable') == -1) {
                            var modelId = $row.find('td').data('model-id'),
                                model = self._datagrid.get('source').get(modelId).clone();

                            self._addToReportBuilder(model);
                        }
                    }

                });

                this.$el.find('.ui-table-body tr').on('contextmenu', function (evt) {

                    var $row = $(this);
                    if ($row.text().indexOf('Connection unavailable') == -1) {
                        var modelId = $row.find('td').data('model-id'),
                            model = self._datagrid.get('source').get(modelId);

                        evt.preventDefault();

                        var menu = new AppKit.Menu({
                            menuItems: [
                                new AppKit.MenuItem({
                                    label: 'Add Record to Report',
                                    action: function () {
                                        model.set('details', null);
                                        model.set('detailsTitle', null);
                                        self._addToReportBuilder(model);
                                    }
                                })
                            ]
                        });

                        menu.open({
                            left: evt.clientX - 30,
                            top: evt.clientY
                        });

                        menu.on('close', function () {
                            setTimeout(_.bind(function () {
                                this.$el.remove();
                            }, this), 0);
                        });
                    }
                });

            });

        }

    },

    _addToReportBuilder: function (m, hasAttachment, auditQueryName) {

        if (m) {
            var model = m.clone();

            if (hasAttachment)
                model.set('hasAttachment', hasAttachment);

            if (auditQueryName)
                model.set('auditQueryName', auditQueryName);

            model.set('_name', this.get('name'));

            var moreWindow = this.get('moreWindow'),
                columns = moreWindow ? moreWindow.options.columns : this.options.columns;

            JLV.ReportBuilderController.addRecord(model, this.options.columns, columns);
        }

    },

    /**
     *
     * Fetch the data from the server using the Collection
     *
     **/
    loadData: function (callback) {

        if (this._dataIsLoading) {
            return;
        }

        var data = this.get('data'),
            patient = this.get('patient');

        if (!data && this._datagrid) {
            console.warn(this.get('name') + ' widget data not set.');
        }

        if (!patient) {
            console.warn(this.get('name') + ' widget patient not set.');
        }

        if (data && this._datagrid &&
            patient) {

            if (!_.isFunction(data.doFetch)) {
                console.warn('Widget data sources must be a Backbone collection implementing a doFetch method.');
                return;
            }

            data.reset();
            this._super();

            data.doFetch(patient, _.bind(function () {

                    if (this._datagrid.get('rowCount') === 0) {
                        this._datagrid.showMessage("No Data");
                    }
                    else if (typeof this._defaultSort !== 'undefined' && this._defaultSort !== null) {

                        var column = _.findWhere(this.options.columns, {id: this._defaultSort.id});
                        if (column) {
                            column.index = _.indexOf(this.options.columns, column);

                            var sortInfo = {
                                asc: this._defaultSort.asc,
                                column: column
                            };

                            this._datagrid.sortByColumn(sortInfo);
                            this._datagrid.render();

                            if (this.get('moreWindowClass')) {
                                this._moreBtn.show();
                            }
                        }
                        else {
                            this._datagrid.render();
                            if (this.get('moreWindowClass')) {
                                this._moreBtn.show();
                            }
                        }
                    }
                    else {
                        this._datagrid.render();

                        if (this.get('moreWindowClass')) {
                            this._moreBtn.show();
                        }
                    }

                    //render the expanded view
                    if (this._moreWindow) {
                        this._moreWindow.renderData();
                    }


                    //update the widget title to reflect the number of records

                    this.set('title', this.get('name') + ' (' + this._datagrid.get('rowCount') + ')');

                    if (_.isFunction(callback)) {
                        callback();
                    }

                    this._updatePageInfoDisplay();

                    //508 - if no more button, add in tab loop
                    this._datagrid.$el.find('.last-column').on('keydown', _.bind(function (evt) {
                        if (this.get('data').length === 0) {
                            evt.preventDefault();
                            this.$el.find('.jlv-widget-toolbutton:visible').first().focus();
                        }
                    }, this));

                    this.checkConnectionUnavailable();

                    /* loading is over, set flag to false, enable apply filter */
                    this._dataIsLoading = false;

                    if (this._dateFilterPanel) {
                        this._dateFilterPanel.set('disabled', false)
                    }

                    if (this._moreWindow) {
                        if (this._moreWindow) {
                            this._moreWindow.disableFilterPanel(false);
                        }
                    }

                }, this),
                /* handle error */
                _.bind(function (collection, err) {

                    console.info(err);
                    if (err.status !== 200) {

                        this._datagrid.showMessage(_.template(
                                '<div class="server-error-msg" style="color:red;cursor:pointer;text-decoration:underline;">Something went wrong: {{statusText}} ({{status}})</div>')(err)
                        );
                    }

                    this._dataIsLoading = false;

                }, this));
        }
    },
    checkConnectionUnavailable: function () {

        var collection = this.get('data'),
            errors = collection.connectionErrors;


        this.addConnectionUnavailableBanner((errors && Object.keys(errors).length > 0)
            || (JLV.Session.dodVLERFail && this.options.vlerDomain)
            || (JLV.Session.dasVLERFail && this.options.dasvlerDomain));

    },

    addConnectionUnavailableBanner: function (flag) {

        if (flag === undefined) {
            flag = true;
        }

        this.set('statusAlert', flag);

        if (flag) {
            this.addBanner({
                id: 'connUnavailableStatus',
                bgcolor: 'yellow',
                fgcolor: '#111',
                closable: true,
                text: 'One or more data sources could not be connected  <br/> and some entries will not appear',
                height: 34
            });
        }
        else {
            this.clearBanner('connUnavailableStatus');
        }

    },

    doPrint: function () {

        if (this._dataIsLoading) {
            return;
        }

        if (this._datagrid) {
            var data = this._datagrid.get('tableData'),
                patient = JLV.ActivePatientModel;

            if (data.length && patient) {

                var columns = this.get('columns');
                var hiddenColumns = this._datagrid.get('hiddenColumns');
                var visibleColumns = [],
                    formatters = {};

                var l = columns.length, i = 0;

                for (; i < l; i++) {

                    var column = columns[i]
                    if (hiddenColumns.indexOf(column.id) < 0) {
                        visibleColumns.push({
                            id: column.id,
                            title: column.title
                        });

                        formatters[column.id] = column.formatter;
                    }
                }

                var data = this._datagrid.get('tableData');
                //flatted grouped data
                if (!_.isArray(data)) {
                    var groups = data.data;
                    l = groups.length;
                    i = 0;

                    data = [];
                    for (; i < l; i++) {
                        var group = groups[i];
                        for (var key in group) {
                            data = _.union(data, group[key])
                        }
                    }
                }

                l = data.length;
                i = 0;

                var toPrint = [];
                for (; i < l; i++) {
                    var model = data[i].clone();

                    var vcl = visibleColumns.length,
                        j = 0;

                    for (; j < vcl; j++) {
                        var vc = visibleColumns[j];
                        var fvalue = model.get(vc.id);

                        if (_.isString(fvalue)) {
                            fvalue = _.escape(fvalue);
                        }

                        if (_.isFunction(formatters[vc.id])) {

                            fvalue = formatters[vc.id](
                                fvalue, model, true
                            );
                        }

                        model.set(vc.id, fvalue);
                    }

                    toPrint.push(model.toJSON());
                }

                var title = this.get('name');

                $.post('print/setup', {
                    patToken: patient.get('patientToken'),
                    data: JSON.stringify({
                        title: title,
                        columns: visibleColumns,
                        data: toPrint,
                        patient: patient.toJSON(),
                        datetime: moment().format('MMMM DD, YYYY h:mm a')
                    })
                }, function (response) {

                    var jobID = response;

                    JLV.Widget.PrintConfirm(function (ok) {

                        if (ok) {
                            var printUrl = 'print/table';
                            printUrl += ('?jobId=' + jobID);
                            printUrl += ('&patToken=' + patient.get('patientToken'));
                            window.open(printUrl, '_blank');
                        }
                    });

                });
            }
        }

    },
    savePortalUI: function () {
        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SavePatientPortalUINotification);
    }
});


