//= require ./PatientWidgetWindow.js

JLV.PatientAllergiesWidgetWindow = JLV.PatientWidgetWindow.extend({
    dodAdapterSourceReportField: 'allergyName',
    normCodeSystems: [JLV.common.CODING_SYSTEM_UMLS],
    options: {
        name: 'Allergies Expanded',
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        filter: false,
        clipboard: true,
        print: true,
        refresh: true,
        columns: [
            {
                id: 'enteredDate',
                title: 'Date Recorded',
                width: 130,
                sortable: true,
                formatter: JLV.common.dateFormatter
            },
            {
                id: 'allergyName',
                title: 'Allergen',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Allergen name was not supplied"
                    }
                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('codeNumber'),
                            normDisplay = model.get('codeName'),
                            systemDisplay = model.get('codeSystem');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                systemDisplay + ' Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                id: 'codeName',
                title: "Standardized Allergen",
                width: 200,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('codeNumber'),
                            normDisplay = model.get('codeName'),
                            systemDisplay = model.get('codeSystem');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                systemDisplay + ' Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                id: 'reaction',
                title: 'Reaction',
                width: 100,
                sorter: JLV.common.linkTextSorter,
                sortable: true
            },
            {
                id: 'severity',
                title: "Severity",
                width: 100,
                sorter: JLV.common.linkTextSorter,
                sortable: true
            },
            {
                id: 'comment',
                title: 'Comments',
                width: 120,
                sorter: JLV.common.linkTextSorter,
                sortable: true
            },
            {
                title: 'Site',
                width: 50,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'allergyName',
            asc: true
        }
    },


    _setup: function () {
        this._super();

        //add banner that TMDS allergies is not available
        this.addBanner({
            fgcolor: '#fff',
            bgcolor: '#ff0000',
            height: 25,
            text: 'Allergies from Theater Data Sources are not displayed at this time.'
        });
    }


});