//= require ./PatientWidgetWindow.js

JLV.PatientConsultsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Consult Encounters',
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['outCode', 'taxonomy'],
        columns: [
            {
                title: 'Date',
                id: 'requestDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm')
                },
                sortable: true,
                width: 80
            },
            {
                title: 'Consult Order',
                id: 'outCode',
                width: 210,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                id: 'providerName',
                width: 85,
                title: 'Provider',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 150,
                title: 'Provider Specialty',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }

            },
            {
                title: 'Status',
                id: 'outStatus',
                width: 100,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'requestDate',
            asc: false
        }
    }
});