//= require ./PatientWidgetWindow.js

JLV.PatientDischargeNotesWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Inpatient Summaries',
        width: 1040,
        minWidth: 1040,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['noteType', 'providerName', 'taxonomy', 'location'],
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Note Type / Title',
                id: 'noteType',
                width: 200,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || (model && !model.get('recordDetailsToken'))) {
                        if(model && model.get('noteTitle')){
                            return value + ' / ' + model.get('noteTitle');
                        }

                        return value;
                    }

                    var noteType = value;
                    if (!noteType || noteType.length < 1) {
                        noteType = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if(model.get('noteTitle')){
                        noteType = noteType + ' / ' + model.get('noteTitle');
                    }

                    return JLV.common.linkFormatter(noteType, model);
                }

            },
            {
                width: 150,
                title: 'Provider',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }

            },
            {
                width: 120,
                title: 'Provider Specialty',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }

            },
            {
                width: 100,
                title: 'Location',
                sortable: true,
                id: 'location'
            },
            {
                width: 120,
                title: 'Visit/Adm Date',
                sortable: true,
                id: 'visitAdm'
            },
            {
                width: 70,
                title: 'Status',
                sortable: true,
                id: 'status'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        }
    },
    toggleFilterPanel: function () {

        this._super();


        this.$el.find('.more-filter-date-msg').text('If the information you expect does not appear, expand the date range.');

    }
});
