//= require ./PatientWidgetWindow.js

JLV.PatientDocumentsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Documents',
        width: 900,
        minWidth: 840,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['noteTitle', 'normDisplay', 'taxonomy', 'location'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {
                    if (model) {
                        if (model.get('documentType') == "Outpatient Encounter" && model.get('outpatientEncounter')) {
                            if (model.get('outpatientEncounter').clinic === 'Connection unavailable.') {
                                return JLV.common.dateFormatter(value);
                            }
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model)
                        }
                    }

                    return JLV.common.dateFormatter(value);
                }
            },
            {
                title: 'Description',
                id: 'noteTitle',
                width: 160,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    var noteType = "",
                        noteTitle = value,
                        docType = "",
                        outpatientEncounter = null;

                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    } else {
                        if(model.get('recordDetailsToken') && model.get('recordDetailsToken').indexOf('NOT_SUPPORTED') >= 0){
                            return value;
                        }
                        docType = model.get('documentType');
                        outpatientEncounter = model.get('outpatientEncounter');
                    }

                    if(model.get('noteType')) {
                        noteType = model.get('noteType');
                    }

                    if (docType
                        && docType.indexOf('Outpatient Encounter') >= 0) {
                        if (!noteTitle || noteTitle.length < 1 || noteTitle === JLV.common.NOTE_TITLE_UNAVAILABLE_TXT) {
                            noteTitle = JLV.common.DIAGNOSIS_UNAVAILABLE_TXT;
                        }
                        if (outpatientEncounter && (outpatientEncounter.complexDataUrl || outpatientEncounter.noteText || outpatientEncounter.vaNote))
                            JLV.common.linkFormatter(noteTitle, model);
                        else
                            return noteTitle;
                    }

                    if (!noteTitle || noteTitle.length < 1) {
                        noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if (docType
                        && docType.indexOf('Inpatient Note') >= 0) {
                        if (!noteType || noteType.length < 1) {
                            noteType = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                        } else {
                            noteTitle = noteType + ' / ' + noteTitle;
                        }

                    }

                    if (model.get('sensitive')) {
                        noteTitle = noteTitle.replace(/\n/g, '<br />');
                    }

                    return JLV.common.linkFormatter(noteTitle, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            docType = model.get('documentType');

                        if (normCode && docType) {
                            if (model.get('sourceProtocol')
                                && docType.indexOf('Radiology Exams') >= 0
                                && model.get('sourceProtocol').indexOf('RPC') >= 0) {
                                return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                    'Standardized Type: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if(docType.indexOf('Outpatient Encounter') >= 0){
                                return '<b>Systematized Nomenclature<br/>' +
                                    'of Medicine - Clinical Terms<br/>' +
                                    '(SNOMED CT)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if ( docType.indexOf('HAIMS') < 0) {
                                return '<b>Logical Observation Identifiers<br/>' +
                                    'Names and Codes<br/>' +
                                    '(LOINC)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            }
                        }

                        if(model.get('recordDetailsToken') && model.get('recordDetailsToken').indexOf('NOT_SUPPORTED') >= 0){
                            return 'This document can only be opened in HAIMS.';
                        }

                    }
                }
            },
            {
                title: 'Standardized Description',
                id: 'normDisplay',
                width: 200,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            docType = model.get('documentType');

                        if (normCode && docType) {
                            if (model.get('sourceProtocol')
                                && docType.indexOf('Radiology Exams') >= 0
                                && model.get('sourceProtocol').indexOf('RPC') >= 0) {
                                return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                    'Standardized Type: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if(docType.indexOf('Outpatient Encounter') >= 0){
                                return '<b>Systematized Nomenclature<br/>' +
                                    'of Medicine - Clinical Terms<br/>' +
                                    '(SNOMED CT)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            } else if (docType.indexOf('HAIMS') < 0) {
                                return '<b>Logical Observation Identifiers<br/>' +
                                    'Names and Codes<br/>' +
                                    '(LOINC)</b><br/>' +
                                    'Standardized Name: ' + normDisplay + '<br/>' +
                                    'Standardized Code: ' + normCode;
                            }
                        }

                    }
                }
            },
            //{
            //    title: 'DocType',
            //    id: 'documentType',
            //    width: 100,
            //    sortable: true
            //
            //},
            {
                title: 'Provider',
                id: 'providerName',
                width: 110,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 110,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 150,
                title: 'Location',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'location'
            },
            {
                width: 130,
                title: 'Status',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'status',
                formatter: function (value, model) {

                    var docType = model.get('documentType');

                    if (docType) {
                        if (docType.indexOf('Progress Note') >= 0
                            || docType.indexOf('Encounter Note') >= 0
                            || docType.indexOf('Deployment Form') >= 0
                            || docType.indexOf('Questionnaires') >= 0
                            || docType.indexOf('HAIMS') >= 0) {
                            return ''
                        }
                    }

                    return value;
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var study = model.get('vixStudy');

                        if (study && study.viewerUrl) {
                            var viewerUrl = study.viewerUrl;
                            if (forPrint) {
                                return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                            }

                            if (model.get('documentType') == "Radiology Exams") {
                                var viewerCmd = String.format("JLV.common.openRadVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));
                            }
                            else {
                                var viewerCmd = String.format("JLV.common.openDocVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));
                            }

                            return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                        } else if (model.get('hasImage')) {
                            var siteCode = model.get('site').siteCode;

                            if (siteCode) {
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var uniqueId = '';
                                var imageId = '';
                                var spinnerId = '';

                                if (model.get('documentType') == "Radiology Exams") {
                                    var radiologyReport = model.get('radiologyReport');
                                    var hasImage = radiologyReport.hasImage;
                                    var caseNumber = radiologyReport.caseNumber;
                                    var examId = radiologyReport.examId;
                                    var facilityName = radiologyReport.facilityName;

                                    uniqueId = examId.replace('.', '');
                                    imageId = 'docRadWindowImage' + uniqueId;
                                    spinnerId = 'docRadWindowSpinner' + uniqueId;

                                    var viewerCmd = String.format("JLV.common.getUrlAndOpenRadVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\",\"{6}\",\"{7}\");return false;",
                                        siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);
                                }
                                else {
                                    var noteId = model.get('noteId');
                                    var images = model.get('images');

                                    uniqueId = noteId;
                                    imageId = 'docNoteWindowImage' + uniqueId;
                                    spinnerId = 'docNoteWindowSpinner' + uniqueId;

                                    var viewerCmd = String.format("JLV.common.getUrlAndOpenDocVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\");return false;",
                                        siteCode, noteId, images, model.get('recordDetailsToken'), imageId, spinnerId);
                                }

                                return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id="' + spinnerId + '" style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                            }
                        }
                    }

                    return '';
                }
            },
            {
                width: 80,
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        }
    }
});
