//= require ../../DetailsWindow.js

JLV.PatientEncounterDetailsWindow = JLV.DetailsWindow.extend({
    className: 'appointment-details',
    encounter: null,
    options: {
        height: 600
    },
    loadDetails: function () {
        var html = "";
        if (!this.encounter) {
            this.$el.html("No encounter details to display.");
            return;
        }

        if (!this.encounter.get('normDisplay') && this.encounter.get('codes')) {
            for (var i in this.encounter.get('codes')) {
                if (this.encounter.get('codes')[i].system) {
                    if (this.encounter.get('codes')[i].system.toUpperCase().indexOf("SNOMED") >= 0 ||
                        this.encounter.get('codes')[i].system === '2.16.840.1.113883.6.96') {
                        this.encounter.attributes.normCode = this.encounter.get('codes')[i].code;
                        this.encounter.attributes.normDisplay = this.encounter.get('codes')[i].display;
                        break;
                    }
                }
            }
        }

        if(this.encounter.get('apptDate')){
            this.encounter.attributes.apptDate = new Date(this.encounter.get('apptDate'));
            if (!this.encounter.attributes.apptDate.isValid()) {
                this.encounter.attributes.apptDate = null;
            }
        }
        else {
            this.encounter.set('apptDate', '');
        }

        //send audit details call
        $.post('patientEncounters/auditDetails', {
            patToken: this.patientToken,
            recordDate: this.encounter.get('apptDate'),
            recordSiteMoniker: this.encounter.get('site') ? this.encounter.get('site').moniker : '',
            recordSiteAgency: this.encounter.get('site') ? this.encounter.get('site').agency : '',
            info: this.encounter.get('diagnosisList'),
            auditText: this.encounter.get('clinic'),
            eventId: this.encounter.get('eventId'),
            encounterId: this.encounter.get('encounterId'),
            widgetName: this.widget.name
        });

        html += _.template([
            '<table style="background:#fff;color:#000;width:100%;">',
            '<tr><td colspan="3"><strong>Appt Date/Time:</strong>&nbsp;{{apptDate}}</td></tr>',
            '<tr><td colspan="3"><strong>Clinic:</strong>&nbsp;{{clinic}}</td></tr>',
            '{[ if(provider && typeof provider.name != "undefined") { ]}',
            '<tr><td colspan="3"><strong>User:</strong>&nbsp;{{provider.name}}</td></tr>',
            '{[ } else { ]}',
            '<tr><td colspan="3"><strong>User:</strong>&nbsp;</td></tr>',
            '{[ } ]}',
            '<tr><td colspan="3"><strong>Standardized Diagnosis:</strong>&nbsp;',
            '{[ if(typeof normDisplay != "undefined") { ]}',
            '{{normDisplay}}  <strong>Standardized Code:</strong> {{normCode}}',
            '{[ } ]}',
            '</td></tr>',
            '<tr><td colspan="3"><strong>Reason for Visit:</strong>&nbsp;{{reason}}</td></tr>',
            '<tr style="background-color:#003366;"><th><span style="color:white;"><b>Diagnosis Code</b></span></th><th><span style="color:white;"><b>Diagnosis</b></span></th><th><span style="color:white;"><b>Status</b></span></th></tr>',
        ].join(''))(this.encounter.toJSON());

        var diagnosis = this.encounter.get('diagnosis');
        if (diagnosis && diagnosis.length > 0) {
            for (var d in diagnosis) {
                html += _.template('<tr><td>{{code}}</td><td>{{description}}</td><td>{{status}}</td></tr>')(diagnosis[d]);
            }
        }
        else {
            html += '<tr><td></td><td>No Diagnosis Found</td><td></td></tr>';
        }

        html += '<tr style="background-color:#003366;"><th><span style="color:white;"><b>Procedure Code</b></span></th><th><span style="color:white;"><b>Procedure</b></span></th><th><span style="color:white;"><b>Status</b></span></th></tr>';

        var procedures = this.encounter.get('procedures');
        if (procedures && procedures.length > 0) {
            for (var p in procedures) {
                html += _.template('<tr><td>{{code}}</td><td>{{description}}</td><td>{{status}}</td></tr>')(procedures[p]);
            }
        }
        else {
            html += '<tr><td></td><td>No Procedures Found</td><td></td></tr>';
        }

        html += '</table>';

        this.$el.html(html);

    }
});