//= require ./PatientWidgetWindow.js

JLV.PatientImmunizationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Immunizations',
        width: 1000,
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        filter: true,
        dateFilter: false,
        clipboard: true,
        print: true,
        refresh: true,
        gridHeaderHeight: 30,
        filters: ['vaccine', 'normDisplay'],
        columns: [
            {
                title: 'Vaccine Administered <br /> Date',
                id: 'adminDate',
                formatter: function (aDate, model) {
                    return model.get('dateTime');
                },
                sortable: true,
                width: 120
            },
            {
                title: 'Vaccine Administered <br /> Product Type',
                id: 'vaccine',
                width: 160,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: JLV.common.linkFormatter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Vaccine<br/>' +
                                'Administered Code<br/>' +
                                '(CVX)</b><br/>' +
                                'National Name: ' + normDisplay + '<br/>' +
                                'National Code: ' + normCode;
                        }
                    }

                    return '<b>National Vaccine<br/>' +
                        'Administered Code<br/>' +
                        '(CVX)</b><br/>' +
                        'National Name: none<br/>' +
                        'National Code: none';
                }

            },
            {
                title: 'Standardized Vaccine <br/> Product Type',
                width: 180,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Vaccine<br/>' +
                                'Administered Code<br/>' +
                                '(CVX)</b><br/>' +
                                'National Name: ' + normDisplay + '<br/>' +
                                'National Code: ' + normCode;
                        }
                    }

                    return '<b>National Vaccine<br/>' +
                        'Administered Code<br/>' +
                        '(CVX)</b><br/>' +
                        'National Name: none<br/>' +
                        'National Code: none';
                }
            },
            {
                id: 'series',
                width: 80,
                title: 'Series',
                sortable: true
            },
            {
                id: 'result',
                width: 100,
                title: 'Immunization <br/> Result',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && (model.get('size'))) {
                        if (value == null || value.trim() == "") {
                            return String.format('{0}', model.get('size'));
                        } else {
                            return String.format('{0}/{1}', model.get('size'), value);
                        }
                    }

                    if (value === null) {
                        return null;
                    }


                    return "" + value;
                }
            },
            {
                title: 'Adverse Vaccine<br/>Reaction',
                id: 'reaction',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
            },
            {
                title: 'Exemption /<br/>Refusal Reason',
                id: 'exemptionRefusalReason',
                width: 120,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
            },
            {
                title: 'Exemption /<br/>Refusal Date',
                id: 'exemptionRefusalDate',
                width: 120,
                sortable: true,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 50,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'adminDate',
            asc: false
        }
    },

    _setup: function () {
        this._super();

        //add banner that VA data is not available for DoD user
        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {

            this.addBanner({
                fgcolor: '#fff',
                bgcolor: '#ff0000',
                height: 25,
                text: 'VA data is not complete and will not be displayed at this time.'
            });


        }

    }
});
