//= require ./PatientWidgetWindow.js

JLV.PatientInpatientMedicationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Inpatient Medications',
        width: 1000,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        groupBy: 'orderId',
        groupFormatter: function (value) {
            if (value && (value !== 'null')) {
                return 'Order Number: &nbsp;' + value;
            } else {
                return 'Order Number: none';
            }
        },
        filters: ['drugName', 'normDisplay', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 110
            },
            {
                title: 'Order<br/>Start Date',
                id: 'startDate',
                width: 90,
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm')
                },
                sortable: true
            },
            {
                title: 'Order<br/>Stop Date',
                id: 'stopDate',
                width: 90,
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm')
                },
                sortable: true
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 110,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized<br/>Drug Name',
                id: 'normDisplay',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 65,
                sortable: true
            },
            {
                width: 110,
                title: 'Ordering HCP',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 95,
                title: 'Ordering<br/>HCP Specialty',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    try {
                        if (model && model.attributes.taxonomy) {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: {{-taxonomy}}<br/>' +
                                'NUCC Taxonomy Code: {{-taxonomyCode}}'
                            )(model.toJSON());
                        }
                    } catch (e) {
                        console.info('template error');
                    }
                }
            },
            {
                title: 'Schedule',
                id: 'sig',
                sortable: true,
                width: 100
            },
            {
                width: 60,
                title: 'Quantity',
                sortable: true,
                id: 'quantity'
            },
            {
                title: 'Site',
                id: 'site',
                width: 50,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'orderId',
            asc: false
        }
        , data: JLV.InpatientMedicationsCollection
    },


    _setup: function () {
        this._super();

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            if (sortInfo.column.id === 'orderId') {
                if (this._datagrid.get('groupBy')) {
                    if (sortInfo.asc) {
                        this.set('groupSorter', function (val1, val2) {
                            return (val1 >= val2) ? 1 : -1;
                        });
                    }
                    else {
                        this.set('groupSorter', function (val1, val2) {
                            return (val1 <= val2) ? 1 : -1;
                        });
                    }
                }
            }
        }, this));


        this._datagrid.set('groupBy', 'orderId');

        this.on('afterFilter', _.bind(function (evt, value) {

            if (value.length !== 0) {
                this._datagrid.set('groupBy', null);
            }
            else {
                this._datagrid.set('groupBy', 'orderId');
            }


            var sort = this.get('defaultSort');
            var column = _.findWhere(this.options.columns, {id: sort.id});

            this._datagrid.sortByColumn({
                column: column,
                asc: sort.asc
            });

            this._datagrid.render();
        }, this));
    },
    renderData: function () {
        this._super();

        var defaultSort = this.get('defaultSort');
        if (defaultSort.id === 'orderId') {
            if (defaultSort.asc) {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 >= val2) ? 1 : -1;
                });
            }
            else {
                this.set('groupSorter', function (val1, val2) {
                    return (val1 <= val2) ? 1 : -1;
                });
            }
        }
    }
});
