//= require ./PatientWidgetWindow.js

JLV.PatientOutpatientMedicationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Outpatient Medications',
        width: 1024,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['drugName', 'normDisplay', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Last Fill',
                id: 'lastFilledDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 130,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && model.get('manuallyEntered')) {
                        return JLV.common.linkFormatter(value, model, null, "report") + ' <span class="jlv-widget-self-reported-med">(Patient/Family Reported)</span>';
                    }
                    else {
                        return JLV.common.linkFormatter(value, model, null, "report");
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/>Drug Name',
                id: 'normDisplay',
                width: 120,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br/>' +
                                'RxNorm Concept Unique Identifiers</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br/>' +
                        'RxNorm Concept Unique Identifiers</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 100,
                title: 'Sig',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'sig'
            },

            {
                width: 100,
                title: 'Quantity',
                sortable: true,
                id: 'quantity'
            },
            {
                width: 100,
                title: 'Days Supply',
                sortable: true,
                id: 'daysSupply'
            },
            {
                width: 100,
                title: 'Refills<br/>Left',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'refillsRemaining',
                formatter: function (value, model) {
                    var txt = "";

                    if (model && model.get("refills") && model.get("refillsRemaining"))
                        txt = model.get("refillsRemaining") + " of " + model.get("refills");
                    else if (model && model.get("refillsRemaining"))
                        txt = model.get("refillsRemaining");
                    else
                        txt = value;

                    if (model && model.get('site')) {
                        var site = model.get('site');
                        if (site.agency === JLV.common.AGENCY_VA) {
                            if (model && model.get('prescriptionFills')
                                && model.get('prescriptionFills').length
                                && model.get('lastFilledDate')) {
                                return JLV.common.linkFormatter(txt, model);
                            }
                        }
                    }
                    return txt;
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 100,
                sorter: JLV.common.linkTextSorter,
                sortable: true

            },
            {
                width: 120,
                title: 'Ordering HCP',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 120,
                title: 'Ordering HCP <br/> Specialty',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Expires',
                id: 'stopDate',
                width: 100,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                width: 50,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastFilledDate',
            asc: false
        }
    },


    _setup: function () {
        this._super();

        this.set('copyActiveMeds', true);
    },

    copyActiveMeds: function (aBool) {
        if (aBool === true) {

            if (!this._copyActiveMedsBtn) {
                var self = this;
                this._copyActiveMedsBtn = $('<div title="Copy Active Rxs to clipboard" tabindex="0" class="jlv-window-toolbutton">')
                    .append('<div class="icon-med-drugs">')
                    .button()
                    .insertBefore(self.$el.parent().find('.clipboard'))
                    .on('click', function (evt) {

                        if (self.widget) {
                            self.widget._doCopyActiveMeds(false, self.options.columns);
                        }
                    });
            }

            this._copyActiveMedsBtn.show();
        }
        else if (aBool === false) {
            this._copyActiveMedsBtn.hide();
        }

        return this._copyActiveMedsBtn.is(':visible');
    }
});
