//= require ./PatientWidgetWindow.js

JLV.PatientProceduresWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Procedures',
        width: 900,
        gridHeaderHeight: 30,
        filter: true,
        dateFilter: true,
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['taxonomy'],
        columns: [
            {
                title: 'Procedure Date',
                id: 'date',
                formatter: function (value, model) {
                    if (model) {
                        // Note link for VA Outpatient Encounter notes
                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote')) {
                            var links = JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>';
                            var documents = model.get('documents');
                            if (documents && documents.length > 1) {
                                for (var i = 0; i < documents.length; i++) {
                                    if (i > 0) {
                                        links += ' ';
                                    }
                                    links += JLV.common.linkFormatter("Note" + (i + 1), model);
                                }
                            } else {
                                links += JLV.common.linkFormatter("Note", model);
                            }
                            return links;
                        } else {
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                        }
                    }
                    return value;
                },
                sortable: true,
                width: 100
            },
            {
                width: 100,
                title: 'Location',
                sortable: true,
                id: 'location'
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 120,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br/>' +
                                '(NUCC)</b><br/>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br/>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br/>' +
                        'Taxonomy<br/>' +
                        '(NUCC)</b><br/>' +
                        'NUCC Taxonomy Description: none<br/>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 90,
                title: 'CPT Code',
                sortable: true,
                id: 'normCode'
            },
            {
                title: 'Procedure <br/> Description',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'description',
                formatter: function (value, model) {
                    // Only DoD VLER narratives have a link
                    if (model && model.get('narrativeId')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    return value;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br/>' +
                                '(CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br/>' +
                        '(CPT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/> Procedure Description',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplay',
                width: 160,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br/>' +
                                '(CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br/>' +
                        '(CPT)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Image',
                id: 'image',
                width: 75,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var documents = model.get('documents');
                        if (documents && documents.length > 0) {
                            var study = documents[0].vixStudy;
                            if (study && study.viewerUrl) {
                                var viewerUrl = study.viewerUrl;
                                if (viewerUrl) {

                                    if (forPrint) {
                                        return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                    }

                                    var viewerCmd = String.format("JLV.common.openProcedureVixViewer(\"{0}\",\"{1}\");return false;",
                                        viewerUrl, model.get('recordDetailsToken'));

                                    return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                        + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                                }
                            } else if (model.get('hasImage')) {  //procedure has image
                                var imageDocIndex = -1;
                                for (var i = 0; i < documents.length; i++) {
                                    if (documents[i].hasImage) {
                                        imageDocIndex = i;
                                        break;
                                    }
                                }

                                if (imageDocIndex >= 0) {
                                    var document = documents[imageDocIndex];
                                    var documentId = document.id;
                                    var documentSiteId = model.get('site').siteCode;

                                    if (documentId && documentSiteId) {
                                        if (forPrint) {
                                            return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                        }

                                        var uniqueId = documentId;
                                        var imageId = 'procedureWindowImage' + uniqueId;
                                        var spinnerId = 'procedureWindowSpinner' + uniqueId;

                                        var viewerCmd = String.format("JLV.common.getUrlAndOpenProcedureVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\");return false;",
                                            documentId, documentSiteId, model.get('recordDetailsToken'), imageId, spinnerId);

                                        return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                            + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id="' + spinnerId + '" style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                                    }
                                }
                            }
                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'date',
            asc: false
        }
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }
        else if (model.get('vaNote')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Outpatient Encounters Note Detail ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    JLV.common.dateFormatter(model.get('date'))),
                url: 'patientProcedures/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                model: model,
                renderer: function (response, forPrint) {

                    if (forPrint) {
                        response.images = '';
                    }

                    return '<pre class="report-details">' +
                        response.noteText +
                        response.images + '</pre>'
                }
            });

            dialog.center();
            dialog.open();
        }
    },
    _setup: function () {
        //add coding status for dod user
        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {
            var columns = this.options.columns;
            if (_.findIndex(columns, {id: 'codingStatus'}) < 0) {
                columns.splice(5, 0, {
                    title: 'Coding Status',
                    id: 'codingStatus',
                    sortable: true,
                    width: 110
                });
            }
        }

        this._super();
    }
});