//= require ./PatientWidgetWindow.js

JLV.PatientRadiologyExamsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Radiology Reports',
        width: 1000,
        gridHeaderHeight: 30,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['study', 'normDisplayLoinc'],
        columns: [
            {
                title: 'Date',
                id: 'examDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Exam<br/>Number',
                id: 'examId',
                sortable: true,
                formatter: function (value, model) {
                    return model.get('examIdDisplay');
                },
                width: 70
            },
            {
                title: 'Exam',
                id: 'study',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    var formattedString = value;

                    if (model && model.get('recordDetailsToken')) {
                        formattedString = JLV.common.linkFormatter(value, model);
                    }

                    /*if (model && model.get('abnormal')) {
                        if (model.get('abnormal').toLowerCase().indexOf('abnormal') > -1
                            || model.get('abnormal').toLowerCase() !== "normal")
                        {
                            formattedString = '<div class="jlv-alert-row">' + formattedString + '</div>';
                        }
                     } */
                    return formattedString;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplayLoinc + '<br/>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplayCpt + '<br/>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 120,
                title: 'Standardized <br/> Radiology Exam',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplayLoinc',
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplayLoinc + '<br/>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplayCpt + '<br/>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 120,
                title: 'CPT Description',
                id: 'normDisplayCpt',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCodeLoinc = model.get('normCodeLoinc'),
                            normDisplayLoinc = model.get('normDisplayLoinc'),
                            normCodeCpt = model.get('normCodeCpt'),
                            normDisplayCpt = model.get('normDisplayCpt');

                        if (normCodeLoinc) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplayLoinc + '<br/>' +
                                'Standardized Code: ' + normCodeLoinc;
                        }
                        else if (normCodeCpt) {
                            return '<b>Current Procedural Terminology (CPT)</b><br/>' +
                                'Standardized Name: ' + normDisplayCpt + '<br/>' +
                                'Standardized Code: ' + normCodeCpt;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }

            },
            {
                id: 'status',
                width: 100,
                title: 'Status',
                sortable: true,
                sorter: JLV.common.linkTextSorter
            },
            {
                title: 'Results',
                id: 'abnormal',
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                width: 100
            },
            {
                title: 'Image',
                id: 'image',
                width: 60,
                sortable: true,
                sorter: JLV.common.contentSorter,
                formatter: function (value, model, forPrint) {
                    if (JLV.common.isVixDisplayEnabled && model) {
                        var study = model.get('vixStudy');
                        var hasImage = model.get('hasImage');

                        if (study && study.viewerUrl) {
                            var viewerUrl = study.viewerUrl;
                            if (viewerUrl) {

                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var viewerCmd = String.format("JLV.common.openRadVixViewer(\"{0}\",\"{1}\");return false;",
                                    viewerUrl, model.get('recordDetailsToken'));

                                return String.format('<div style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div>', viewerCmd);
                            }
                        } else if (hasImage) {
                            var siteCode = model.get('site').siteCode;
                            var caseNumber = model.get('caseNumber');
                            var examId = model.get('examIdDisplay');
                            var facilityName = model.get('facilityName');

                            if (siteCode && caseNumber && examId && facilityName) {
                                if (forPrint) {
                                    return String.format('<div style="width:100%;text-align:center;">Yes</div>');
                                }

                                var uniqueId = examId.replace('.', '');
                                var imageId = 'radWindowImage' + uniqueId;
                                var spinnerId = 'radWindowSpinner' + uniqueId;

                                var viewerCmd = String.format("JLV.common.getUrlAndOpenRadVixViewer(\"{0}\",\"{1}\",\"{2}\",\"{3}\",\"{4}\",\"{5}\",\"{6}\",\"{7}\");return false;",
                                    siteCode, hasImage, caseNumber, examId, facilityName, model.get('recordDetailsToken'), imageId, spinnerId);

                                return String.format('<div id="' + imageId + '" style="width:100%;text-align:center;"><span class="imageMarker" title="View in VistA Imaging Viewer"'
                                    + ' onclick=\'{0}\' alt="Link View in VistA Image Viewer"></span></div><div id=' + spinnerId + ' style="display: none;text-align:center;""><img src="assets/loading_22.gif"/></div>', viewerCmd);
                            }
                        }
                    }

                    return '';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'examDate',
            asc: false
        }
    }/*,
     _setup : function() {
     this._super();

        //if (this._datagrid) {
        //    //make rows with abnormal results pink
        //    this._datagrid.on('afterRender', function () {
        //
        //        var $row = this.$el.find('.jlv-alert-row').parents('tr');
        //
        //        $row.css(
        //            {
        //                'backgroundColor': '#FFC0CB',
        //                'color': '#000000'
        //            }
        //        );
        //
        //        $row.find('a').css('color', '#000');
        //    });
        //}
     }*/
});