//= require ./PatientWidgetWindow.js
//= require ./VitalsGraphWindow.js

JLV.PatientVitalsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Vitals',
        filter: true,
        dateFilter: true,
        status: true,
        vlerDomain: true,
        dasvlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        groupBy: 'dateTaken',
        groupFormatter: function (value) {
            return 'Date Taken: &nbsp;' + JLV.common.dateFormatter(new Date(value));
        },
        filters: ['type', 'normDisplay'],
        columns: [
            {
             title: 'Date Taken',
             id: 'dateTaken',
             formatter: JLV.common.dateFormatter,
             sortable: true,
                width: 100
            },
            {
                title: 'Type',
                id: 'type',
                width: 100,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                formatter: function (value, model) {
                    if (model && (model.get('site').agency === JLV.common.AGENCY_DOD
                        || model.get('site').agency === JLV.common.AGENCY_DOD_VLER
                        || model.get('site').agency === JLV.common.AGENCY_DAS_VLER)) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    return value;
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }

            },
            {
                title: 'Standardized Type',
                width: 80,
                sortable: true,
                sorter: JLV.common.linkTextSorter,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br/>' +
                                'Names and Codes<br/>' +
                                '(LOINC)</b><br/>' +
                                'Standardized Name: ' + normDisplay + '<br/>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Logical Observation Identifiers<br/>' +
                        'Names and Codes<br/>' +
                        '(LOINC)</b><br/>' +
                        'Standardized Name: none<br/>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Result',
                id: 'result',
                width: 100,
                sortable: true
            },
            {
                title: 'Units',
                id: 'units',
                width: 60,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sortable: true,
                sorter: JLV.common.siteSorter,
                width: 70,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'type',
            asc: true
        }
    },
    _setup: function () {

        this._super();

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            if (sortInfo.column.id === 'dateTaken') {
                if (this._datagrid.get('groupBy')) {
                    if (sortInfo.asc) {
                        this.set('groupSorter', function (val1, val2) {

                            //for some reason date keys get converted to strings, make sure they are dates
                            val1 = new Date(val1);
                            val2 = new Date(val2);

                            return (val1 >= val2) ? 1 : -1;
                        });
                    }
                    else {
                        this.set('groupSorter', function (val1, val2) {

                            //for some reason date keys get converted to strings, make sure they are dates
                            val1 = new Date(val1);
                            val2 = new Date(val2);

                            return (val1 <= val2) ? 1 : -1;
                        });
                    }
                }
            }
        }, this));


        this.on('afterFilter', _.bind(function (evt, value) {

            if (value.length !== 0) {
                this._datagrid.set('groupBy', null);
            }

            var sort = this.get('defaultSort');
            var column = _.findWhere(this.options.columns, {id: sort.id});

            this._datagrid.sortByColumn({
                column: column,
                asc: sort.asc
            });

            this._datagrid.render();
        }, this));

    },
    toggleFilterPanel: function () {
        if (!this._filterPanel) {

            var filterIds = this.get('filters');
            var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));

            var data = this.get('data');

            if (data) {
                if (data.startDate !== undefined && data.endDate !== undefined) {
                    this._dateLabel.show();
                    this._filterPanel = new JLV.PatientVitalsWidgetWindow.FilterPanel({
                        window: this,
                        startDate: data.startDate.toDate(),
                        endDate: data.endDate.toDate(),
                        filters: filters,
                        filters: filters,
                        data: data
                    });

                    this._filterPanel.$el.prependTo(this._body);
                    this._filterPanel.$el.hide();
                }
            }
        }

        this._super();
    },

    loadData: function () {

        if (this._datagrid)
            this._datagrid.set('groupBy', 'dateTaken');

        this._super();
    },

    renderData: function () {
        this._super();

        var defaultSort = this.get('defaultSort');
        if (defaultSort.id === 'dateTaken') {
            if (defaultSort.asc) {
                this.set('groupSorter', function (val1, val2) {

                    //for some reason date keys get converted to strings, make sure they are dates
                    val1 = new Date(val1);
                    val2 = new Date(val2);

                    return (val1 >= val2) ? 1 : -1;
                });
            }
            else {
                this.set('groupSorter', function (val1, val2) {

                    //for some reason date keys get converted to strings, make sure they are dates
                    val1 = new Date(val1);
                    val2 = new Date(val2);

                    return (val1 <= val2) ? 1 : -1;
                });
            }
        }
    }
});


jQuery(function () {

    JLV.PatientVitalsWidgetWindow.FilterPanel = JLV.WidgetWindow.DateFilterPanel.extend({
        initialize: function (options) {

            this._super(options);

            //add table button
            this._tableButton = new JLV.PatientWidgetWindow.LinkButton({
                label: "Show Table",
                enabled: false,
                linkDisabledMessage: 'Enable “Show Table” by setting one item from either the "Filter by Type" or "Filter by Standardized Type" filter.',
                action: _.bind(function () {
                    //generate table html
                    if (this.window) {

                        var vitals = this.subview('.first-combo').get('selected')[0];
                        var stdVitals = this.subview('.second-combo').get('selected')[0];

                        var data = this.window.get('datagrid').tableData();
                        var i = 0,
                            count = data.length;

                        var html = '<table class="jlv-filtered-table-view"><tr><th style="background-color:#eee;">Date Taken</th>' +
                            '<th style="background-color:#eee;">Result</th>' +
                            '<th style="background-color:#eee;">Units</th>' +
                            '<th style="background-color:#eee;">Site</th>' +
                            '</tr>';

                        var itemCount = 0;
                        var units = '';
                        var stdVitalsCode = '';
                        for (; i < count; i++) {
                            var model = data[i];
                            units = model.get('units');
                            if (stdVitals) {
                                stdVitalsCode = model.get('normCode');
                            }

                            html += '<tr>';
                            html += '<td>' + JLV.common.dateFormatter(model.get('dateTaken')) + '</td>';
                            html += '<td>' + model.get('result') + '</td>';
                            html += '<td>' + units + '</td>';
                            html += '<td>' + JLV.common.formatMonikerDisplay(model.get('site')) + '</td>';
                            html += '</tr>';

                            itemCount++;

                        }

                        html += '</table>';
                        html += '<div><table class="jlv-filtered-summary">' +
                            '<tr><td><b>Vitals:</b></td><td>' + vitals + '</td></tr>' +
                            '<tr><td><b>Standardized Vitals:</b></td><td>' + stdVitals + '</td></tr>' +
                            '<tr><td><b>Standardized Vitals Code:</b></td><td>' + stdVitalsCode + '</td></tr>' +
                            '<tr><td><b>Units:</b></td><td>' + units + '</td></tr>' +
                        '</table>';


                        var tableDialog = new JLV.DetailsWindow({
                            title: String.format("Vitals - {0} ({1})", vitals ? vitals : stdVitals, itemCount),
                            width: 660,
                            renderer: function () {
                                return html;
                            }
                        });

                        tableDialog.center();
                        tableDialog.open();
                    }

                }, this)
            });


            //add graph button
            this._graphButton = new JLV.PatientWidgetWindow.LinkButton({
                label: "Show Graph",
                enabled: false,
                linkDisabledMessage: 'Enable “Show Graph” by setting one item from the "Filter by Type" filter.',
                action: _.bind(function () {

                    var vitals = this.subview('.first-combo').get('selected')[0];
                    var data = this.window.get('datagrid').tableData();
                    var count = data.length;

                    if (count) {
                        var graphDialog = new JLV.VitalsGraphWindow({
                            title: String.format("Vitals Graph - {0}", vitals),
                            data: data,
                            units: data[0].get('units'),
                            type: vitals
                        });
                        graphDialog.center();
                        graphDialog.open();
                    }
                }, this)
            });

            $("<div>").addClass('field-filter-buttons')
                .append(this._graphButton.$el)
                .append(this._tableButton.$el)
                .appendTo(this.$el);
        },
        clearFilters: function (skip) {

            if (this._graphButton) {
                this._graphButton.set('enabled', false);
            }
            if (this._tableButton) {
                this._tableButton.set('enabled', false);
            }

            this.window._datagrid.set('groupBy', 'dateTaken');

            this._super(skip);
        },
        applyFilters: function () {

            this._super();

            var firstCombo = this.subview('.first-combo'),
                firstSelected = firstCombo.get('selected');

            if (firstSelected.length === 1) {
                this._tableButton.set('enabled', true);

                if (this.window) {

                    var data = this.window.get('datagrid').tableData().data;

                    var i = 0,
                        count = data.length;
                    var models = [];
                    for (; i < count; i++) {
                        models.push(_.flatten(_.values(data[i])));
                    }

                    var dataArray = _.flatten(models);
                    if (dataArray.length) {
                        var model = dataArray[0];
                        var result = model.get('result');

                        this._graphButton.set('enabled', !_.isNaN(parseInt(result, 10)));
                    }
                }
            }
            else {
                this._tableButton.set('enabled', false);
                this._graphButton.set('enabled', false);
            }

            var secondCombo = this.subview('.second-combo'),
                secondSelected = secondCombo.get('selected');

            if (secondSelected.length === 1) {
                this._tableButton.set('enabled', true);
            }
            else if (firstSelected.length !== 1) {
                this._tableButton.set('enabled', false);
            }
        }
    });
});