//= require ProviderWidget.js


JLV.ProviderReferences = JLV.ProviderWidget.extend({
    initialize: function (options) {

        if (!JLV.Session.prov_ref || !JLV.Session.prov_ref.length) {
            JLV.Session.prov_ref = JLV.Session.defaultRefs;

            //default references are now loaded via .properties file through ViewController.groovy.  Leaving the
            // following code here, just in-case...also, removed VA-only links (load these via .properties file)
            //JLV.Session.prov_ref = [
            //    {id: _.uniqueId('default-'), linkDesc: 'CDC', linkUrl: 'http://www.cdc.gov/'},
            //    {id: _.uniqueId('default-'), linkDesc: 'FDA', linkUrl: 'http://www.fda.gov'},
            //    {id: _.uniqueId('default-'), linkDesc: 'Medscape', linkUrl: 'http://emedicine.medscape.com'},
            //    {id: _.uniqueId('default-'), linkDesc: 'webMD.com', linkUrl: 'http://www.webmd.com'},
            //    {id: _.uniqueId('default-'), linkDesc: 'DermAtlas', linkUrl: 'http://dermatlas.med.jhmi.edu/derm/'},
            //];
        } else {
            //remove unwanted links. "removeReference" is added via SQL Script...
            //delete any instance of UI_CONF in the JSON object
            for (var i = JLV.Session.prov_ref.length - 1; i >= 0; i--) {
                if (JLV.Session.prov_ref[i].linkUrl.indexOf("removeReference") > -1) {
                    JLV.Session.prov_ref.splice(i, 1);
                }
            }
        }

        var self = this;
        JLV.NotificationCenter.addObserverForNotification({
            name: 'JLV.ProviderReferences.Update',
            action: function () {
                self.set('_data', new Backbone.Collection(JLV.Session.prov_ref));
                self.loadData();
                self.savePortalUI();
            }
        });

        this._super(options);

        this.set('_data', new Backbone.Collection(JLV.Session.prov_ref));

        //add in delete context menu
        if (this._datagrid) {

            this._datagrid.on('afterRender', function(){



                //508 keyboard access to delete url
                this.$el.find('.ui-table-body tr').on('keydown', function (evt) {

                    if (evt.which === 68 && evt.ctrlKey) { //ctrl-D
                        evt.preventDefault();
                        //get the row
                        var $row = $(this);
                        var id = $row.find('a').attr('id');
                        JLV.ProviderReferences.AddReferencesController.onDelete(id);
                    }

                });


                this.$el.find('tr').on('contextmenu', function (evt) {

                    //get the row
                    var $row = $(this);

                    evt.preventDefault();

                    var custom = $row.find('.custom').length > 0;

                    //create the menu
                    var menu = new AppKit.Menu({
                        menuItems: [
                            new AppKit.MenuItem({
                                label: custom ? 'Delete URL' : 'This is a default Reference that cannot be deleted',
                                action: function(){
                                    if (custom) {
                                        var id = $row.find('a').attr('id');
                                        JLV.ProviderReferences.AddReferencesController.onDelete(id);
                                    }
                                },
                                enabled: custom
                            })
                        ]
                    });

                    //open the menu
                    menu.open({
                        left: evt.clientX - 30,
                        top: evt.clientY
                    });

                    //when the menu is closed, remove it from the DOM
                    menu.on('close', function(){
                        setTimeout(_.bind(function () {
                            this.$el.remove();
                        }, this), 0);
                    });

                });

            });
        }
    },
    options: {
        widgetId: 'prov_references_widget',
        name: 'References',
        plus: true,
        settings: false,
        dateFilter: false,
        status: false,
        clipboard: true,
        print: true,
        refresh: false,
        gridHeaderHeight: 50,
        columns: [
            {
                title: "Link - To delete a link, right-click a link and select <br/> \"Delete URL\". To use keyboard, focus on row and use ctrl-D",
                width: 55,
                sortable: true,
                id: 'linkDesc',
                formatter: function (value, model) {

                    var id = model.get('id');

                    if (id && id.indexOf('default') > -1) {
                        return String.format(
                            '<b><a id="{2}" class="referenceLink default" href="{1}" target="_blank" title="{1}">{0}</a></b>',
                            value, model.get('linkUrl'), id);
                    }
                    else {

                        if (!id) {
                            id = '';
                        }

                        return String.format(
                            '<b><a id="{2}" class="referenceLink custom" href="{1}" target="_blank" title="{1}">{0}</a></b>',
                            value, model.get('linkUrl'), id);
                    }


                }
            }
        ]
    },
    onAdd: function () {
        var addReferencesDialog = $('#AddReferencesDialog').widget();
        addReferencesDialog.center();
        addReferencesDialog.set('visible', true);
    }
});


JLV.registerJLVWidget('prov_references_widget', JLV.ProviderReferences);

JLV.ProviderReferences.AddReferencesController = {
    onDialogOpen: function () {
        //reset
        this.ReferenceNameInput.set('value', '');
        this.ReferenceURLInput.set('value', '');

        setTimeout(_.bind(function () {
            this.ReferenceNameInput.$el.focus();
        }, this), 0);
    },
    onAdd: function () {
        var name = _.escape(this.ReferenceNameInput.get('value'));
        if (name && name.trim().length > 0) {
            var url = _.escape(this.ReferenceURLInput.get('value'));

            if (url.substring(0, 7) !== 'http://') {
                url = 'http://' + url;
            }

            JLV.Session.prov_ref.push({
                linkDesc: name,
                linkUrl: url,
                id: _.uniqueId('link-')
            });

            JLV.NotificationCenter.postNotificationName('JLV.ProviderReferences.Update', this);
            JLV.NotificationCenter.postNotificationName('saveProviderPortalUI');

            $('#AddReferencesDialog').widget().set('visible', false);

        }
        else {
            AppKit.AlertDialog.alert('Form Error', 'Missing reference name.');
        }
    },
    onCancel: function (sender) {
        $('#AddReferencesDialog').widget().set('visible', false);
    },
    onDelete: function (id) {

     if (!id || id && id.indexOf('default') > -1)
            return;

        var i = 0;
        var found = false;

        foundObj:
            for (; i < JLV.Session.prov_ref.length; i++) {
                var pr = JLV.Session.prov_ref[i];
                if (pr.id === id) {
                    found = true;
                    break foundObj;
                }

            }

        if (!found) {
            console.log("Error finding link for deletion in References widget");
            return;
        }

        JLV.Session.prov_ref.splice(i, 1);
        JLV.NotificationCenter.postNotificationName('JLV.ProviderReferences.Update', this);
        JLV.NotificationCenter.postNotificationName('saveProviderPortalUI');
    }
};
