package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIMhsGenesis
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.NoteImage
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.codec.binary.Base64
import org.apache.commons.lang3.StringUtils
import org.springframework.context.ApplicationContextException
import org.w3c.dom.Document
import org.xml.sax.InputSource
import org.xml.sax.SAXException

import javax.servlet.ServletContext
import javax.xml.parsers.DocumentBuilder
import javax.xml.parsers.DocumentBuilderFactory
import javax.xml.transform.*
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.stream.StreamSource

class MhsGenesisController extends BaseController {

    def mhsGenesisDao;
    def auditDao;
    def noteDao;

    static allowedMethods = [list: 'POST']

    private static
    final String XPATH_CCDA_TEMPLATEID = "/ClinicalDocument/templateId[@root='2.16.840.1.113883.10.20.22.1.1']";
    private static final String CCDA_NODE_ID = "templateId root=\"2.16.840.1.113883.10.20.22.1.1\"";
    private static final String XSL_PATH_C32 = "/c32styles/nhin/cda2detail.xsl";
    private static final String XSL_PATH_CCDA = "/c32styles/nhin/cdaHFG.xsl";
    private static final String XSL_PATH_CCDA_DOD = "/c32styles/nhin/cdaHFG_dod.xsl";

    def beforeInterceptor = {
        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        }
    }

    def list = {
        def guiGenesisList;
        if (appConfig.getDisableGenesis()) {
            guiGenesisList = null;
        } else {
            guiGenesisList = mhsGenesisDao.getMhsGenesisDocuments(session.appUser, request.patient)

            if (guiGenesisList != null) {
                for (def guiGenesis : guiGenesisList) {
                    def queryMap = [(HttpKeys.VLER_DOCUMENT_ID)   : guiGenesis.getEventId(),
                                    (HttpKeys.VLER_DOCUMENT_NAME) : guiGenesis.getDocument(),
                                    (HttpKeys.RECORD_DATE)        : guiGenesis.getDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiGenesis.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiGenesis.getSite().getMoniker()]

                    guiGenesis.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiGenesis.getSite(),
                                    guiGenesis.getSourceProtocol(),
                                    guiGenesis.isSessionCachedRecord()))

                    if (guiGenesis.isSessionCachedRecord()) {
                        def docDetail = BeanUtils.cloneBean(guiGenesis)

                        session.appUser.genesisMap.put(
                                getCachedRecordKey(guiGenesis.getRecordDetailsToken()), docDetail)
                    }
                }
            }
        }
        sendResponse(guiGenesisList)
    }

    def index = {
        // Retrieve cached note
        GUIMhsGenesis guiGenesis = session.appUser.genesisMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        if (guiGenesis == null || !guiGenesis.noteAsBinaryStream) {
            response.sendError 500
            return false
        }

        String complexDataUrl = guiGenesis.getComplexDataUrl()
        if (complexDataUrl != null) {
            String[] parsedComplex = complexDataUrl.split("/");
            String recordID = parsedComplex[parsedComplex.length - 1]

            auditDao.auditDetailsQuery('GetMhsGenesisDocument', guiGenesis.getDate() + '^' + guiGenesis.getDocument(),
                    session.appUser, request.patient, complexDataUrl,
                    recordID,
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            NoteImage noteImage = null;
            try {
                noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl)
            }
            catch (DaoException e) {
                log.error(e)
            }

            PrintWriter out = response.getWriter();
            // setting some response headers
            response.setHeader("Expires", "0");
            response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
            response.setHeader("Pragma", "public");
            // setting the content type
            response.setContentType("text/html");

            if (noteImage != null && noteImage.getNoteBytes() != null) {
                try {

                    String dataXML = new String(noteImage.getNoteBytes(), "UTF-8");

                    String xslTemplate = XSL_PATH_C32;
                    if (isCcdaDoc(dataXML)) {
                        if (session.appUser && session.appUser.agency == "DOD")
                            xslTemplate = XSL_PATH_CCDA_DOD
                        else
                            xslTemplate = XSL_PATH_CCDA
                    }

                    DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
                    DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
                    InputSource is = new InputSource(new StringReader(dataXML));
                    Document doc = dBuilder.parse(is);

                    def queryMap = [(HttpKeys.COMPLEX_DATA_URL): complexDataUrl,
                                    (HttpKeys.RECORD_DATE)     : guiGenesis.getDate()]

                    GUISite guiSite = guiGenesis.getSite();
                    //guiSite.setAgency("DOD");

                    String recordDetailsToken = generateRecordDetailsToken(
                            params[HttpKeys.PATIENT_TOKEN],
                            controllerName,
                            queryMap,
                            guiSite,
                            guiGenesis.getSourceProtocol(),
                            true);

                    String patToken = params[HttpKeys.PATIENT_TOKEN];

                    String attachmentOutput = generateAttachmentTag(doc, patToken, recordDetailsToken);

                    StringBuilder title = new StringBuilder();
                    title.append('<title>JLV - ')
                    title.append(request.recordQuery[HttpKeys.VLER_DOCUMENT_NAME])
                    title.append('</title><script type=\'text/javascript\' src=\'resources/scripts/js/nhin.js\' ></script>')

                    String outputHTML = transform(dataXML, xslTemplate);
                    outputHTML = outputHTML.replace("<title>Veterans Affairs</title>", title.toString())

                    if (attachmentOutput != null && attachmentOutput.length() > 1) {
                        String sectionHeader = "<section id=\"attached-file\"><header><h2>Attached File</h2></header>";

                        if (outputHTML.contains("<section id=\"healthcare-providers\">")) {
                            outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"healthcare-providers\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"healthcare-providers\">").toString();
                        } else if (outputHTML.contains("<section id=\"contact-info\">")) {
                            outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("<section id=\"contact-info\">"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                        } else if (outputHTML.contains("</ul>")) {
                            outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</ul>") + 5, sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                        } else {
                            outputHTML = new StringBuffer(outputHTML).insert(outputHTML.lastIndexOf("</html>"), sectionHeader + attachmentOutput + "</section>\n<section id=\"contact-info\">").toString();
                        }
                    }

                    out.println(outputHTML);

                    return true;
                }
                catch (TransformerConfigurationException e) {
                    log.error(e);
                }
                catch (TransformerException e) {
                    log.error(e);
                }
                catch (SAXException e) {
                    log.error(e)
                }
                catch (IOException e) {
                    log.error(e)
                }
            }

            String errorMsg = "An error occurred while attempting to retrieve the document.";

            out.println(errorMsg);

            return true;
        }
    }

    public String transform(String xmlString, String xslTemplate) throws TransformerConfigurationException, TransformerException {
        try {

            ServletContext context = session.getServletContext();
            InputStream xslInput = context.getResourceAsStream(xslTemplate);

            String xslInputPath = context.getResource(xslTemplate).toExternalForm();
            String xslSystemId = xslInputPath.substring(0, xslInputPath.lastIndexOf('/') + 1);

            StreamSource xmlInput = new StreamSource(new StringReader(xmlString));

            Source xslStreamSource = null;
            if (XSL_PATH_C32.equalsIgnoreCase(xslTemplate)) {
                xslStreamSource = new StreamSource(xslInput, xslSystemId);
            } else {
                xslStreamSource = new StreamSource(xslInput, xslInputPath);
            }

            TransformerFactory factory = TransformerFactory.newInstance();
            try {
                //override Websphere's TransformerFactory
                Class<?> transformClass = Class.forName("org.apache.xalan.processor.TransformerFactoryImpl")
                factory = TransformerFactory.newInstance("org.apache.xalan.processor.TransformerFactoryImpl", null);
            } catch (ClassNotFoundException e) {
                //org.apache.xalan.processor.TransformerFactoryImpl doesn't exist (Websphere specific)
                log.debug(e);
            }
            Transformer transformer = factory.newTransformer(xslStreamSource);

            StringWriter outputWriter = new StringWriter();
            Result outputResult = new StreamResult(outputWriter);
            transformer.transform(xmlInput, outputResult);
            return outputWriter.toString();

        } catch (Exception e) {
            e.printStackTrace(System.out);
        }

        return "An error occurred while attempting to retrieve the document: Error during transform";
    }

    protected Source getStylesheetSource(String xslTemplate) throws ApplicationContextException {

        try {

            URL requestURL = new URL(request.getRequestURL().toString())
            URL xslURL = new URL(requestURL, xslTemplate);
            URL url = xslURL;

            String urlPath = url.toString();
            if (XSL_PATH_CCDA.equals(xslTemplate) || XSL_PATH_CCDA_DOD.equals(xslTemplate)) {
                return new StreamSource(url.openStream(), urlPath);
            } else {
                String systemId = urlPath.substring(0, urlPath.lastIndexOf('/') + 1);
                return new StreamSource(url.openStream(), systemId);
            }
        } catch (IOException ex) {
            throw new ApplicationContextException("Can't load XSLT stylesheet from " + xslTemplate, ex);
        }
    }

    protected String generateAttachmentTag(Document doc, String patToken, String recordToken) {
        StringBuffer attachmentOutput = new StringBuffer("");

        try {
            String encodedText = "";
            String decodedText = "";
            String mediaType = "";

            if (doc != null && doc.getElementsByTagName("text") != null
                    && doc.getElementsByTagName("text").item(0) != null
                    && !doc.getElementsByTagName("text").item(0).getTextContent().isEmpty()) {
                encodedText = doc.getElementsByTagName("text").item(0).getTextContent();
                decodedText = new String(Base64.decodeBase64(encodedText), "UTF-8");

                if (doc.getElementsByTagName("text").item(0).getAttribute("mediaType") != null) {
                    mediaType = doc.getElementsByTagName("text").item(0).getAttribute("mediaType")
                }

            }

            if (mediaType.contains("plain") || mediaType.contains("html")) {
                if (StringUtils.containsIgnoreCase(decodedText, "<html>")) {
                    attachmentOutput.append("<iframe id='vlerAttachedText' src='data:");
                    attachmentOutput.append(mediaType);
                    attachmentOutput.append(";base64,");
                    attachmentOutput.append(encodedText);
                    attachmentOutput.append("'");
                } else {
                    attachmentOutput.append("<pre>");
                    attachmentOutput.append(decodedText);
                    attachmentOutput.append("</pre>");
                }

            } else if (mediaType.contains("image")) {
                attachmentOutput.append("<img id='vlerAttachedImg' src='data:");
                attachmentOutput.append(mediaType);
                attachmentOutput.append(";base64,");
                attachmentOutput.append(encodedText);
                attachmentOutput.append("' />");

            } else if (mediaType.contains("application") || mediaType.contains("rtf")) {
                GUIProgressNote guiNote = new GUIProgressNote();
                guiNote.setNoteAsBinaryStream(true);
                guiNote.setContentType(mediaType);
                guiNote.setNoteText(encodedText);
                session.appUser.attachmentsMap.put(recordToken, guiNote);
                attachmentOutput.append("<a download='GENESIS_attachment' target='_blank' id='genesisAttachedFile' title='Attached Document' href='MhsGenesis/getNoteDataStream?patToken=");
                attachmentOutput.append(patToken);
                attachmentOutput.append("&recToken=");
                attachmentOutput.append(recordToken);
                attachmentOutput.append("'>" + mediaType + "</a>");
            }
        } catch (Exception e) {
            log.error("Could not parse the VLER attachment: " + e.getMessage());
            attachmentOutput.append("<span class='vlerError'>There was an error while attempting to parse the attached file.</span>");
        }
        return attachmentOutput.toString();
    }


    def getNoteDataStream = {

        //retrieve cached note
        GUIProgressNote note = session.appUser.attachmentsMap.get(params[HttpKeys.RECORD_DETAILS_TOKEN]);

        def auditQueryName = "GetNoteDataStream"

        auditDao.auditDetailsQuery(auditQueryName, note.getContentType(), session.appUser, request.patient,
                note.getComplexDataUrl() , note.getNoteId(),
                "GENESIS",
                "DOD")

        if ((note == null)) {
            response.sendError 500
            return false;
        }

        response.setHeader("Pragma", "public");
        response.setHeader("Cache-Control", "maxage=1");

        if (note != null) {
            response.setContentType(note.getContentType());

            String contentDisposition = note.getContentType();
            response.setHeader("Content-Disposition", contentDisposition.replace("attachment", "inline"))


            response.getOutputStream().write(Base64.decodeBase64(note.getNoteText()));
            response.getOutputStream().flush();
            response.getOutputStream().close();

            return true;
        }

        //if note attachment is null, send an error message

        String errorMsg = "An error occurred while attempting to retrieve note. - JLV";

        response.setContentType("text");
        response.setContentLength(errorMsg.length());
        response.getWriter() << errorMsg;
        response.getWriter().flush();

        response.sendError 500
        return false;
    }

    /*
 * Return true if XML document is CCDA document.
 *
 * @param ccdDoc VLER CCD XML document.
 * @return true if root CCDA template ID is present.  Otherwise, false.
 * @throws VlerDocumentUtilException If an error occurs while parsing the CCD.
 */

    public static boolean isCcdaDoc(String ccdXML) throws ApplicationContextException {
        if (ccdXML.indexOf(CCDA_NODE_ID) > 0) {
            return true;
        }
        return false;
    }
}
