package jlv

import gov.va.med.jmeadows.webservice.NarrativeV4
import gov.va.med.jmeadows.webservice.SiteStatusV4
import gov.va.med.jmeadows.webservice.Vlerccda
import org.apache.commons.collections.CollectionUtils
import org.apache.commons.lang3.StringUtils

class PatientDASVLERController extends BaseController {

    def DASVLERDao;
    def auditDao;

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {
        if (params["includeDASVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "VA") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            response.sendError 500
            return false;
        }
    }

    def list = {
        def forceRefresh = false;

        if (params["forceRefresh"] == "true") {
            forceRefresh = true;
        }

        session.appUser.partnersMap.clear();
        session.appUser.narrativesMap.clear();
        session.appUser.vlerSiteStatus = "";

        def vlerccdas = DASVLERDao.getDASVLERPatientCCDA(session.appUser, request.patient, forceRefresh);
        //one vler ccda is the error report
        if (vlerccdas != null && vlerccdas.size() > 1) {

            List<String> vlerAvailability = new ArrayList<String>();

            boolean hasData = false;

            //cache narratives in session
            for (Vlerccda ccda : vlerccdas) {
                if (ccda == null) continue;
                if (ccda.getNarratives() != null && ccda.getNarratives().size() > 0) {
                    for (NarrativeV4 narrative : ccda.getNarratives()) {
                        if (narrative == null) continue;
                        if (narrative.getNarrativeIdentifier() != null && narrative.getNarrativeText() != null)
                            session.appUser.narrativesMap.put(narrative.getNarrativeIdentifier(), narrative);
                    }
                }
                if(CollectionUtils.isNotEmpty(ccda.getDemographics())){
                    hasData = true;
                }
                if (ccda.getStatusList() != null && ccda.getStatusList().size() > 0) {
                    for (SiteStatusV4 status : ccda.getStatusList()) {
                        if (status.getPartner() != null
                                && status.getPartner().getPartnerIdentifier() != null
                                && status.getPartner().getPartnerName() != null) {
                            session.appUser.partnersMap.put(status.getPartner().getPartnerIdentifier(), status.getPartner().getPartnerName());
                            if(!"AVAILABLE".equals(status.getPartnerStatus().toUpperCase())){
                                StringBuilder siteError = new StringBuilder();
                                siteError.append("One or more partner sites was not able to complete the query: ");
                                siteError.append(status.getPartner().getPartnerName());
                                ccda.getErrorList().add(siteError.toString());
                            }
//                            if (status.getPartner().getPartnerSubIdentifier() != null) {
//                                session.appUser.partnersMap.put(status.getPartner().getPartnerSubIdentifier(), status.getPartner().getPartnerSubIdentifierName());
//                            }
                        }
                    }

                    String vlerSiteStatus = DASVLERDao.generateStatusReport(ccda.getStatusList(), ccda.getErrorList());

                    if (vlerSiteStatus != null && vlerSiteStatus.length() > 0) {
                        session.appUser.vlerSiteStatus = vlerSiteStatus;
                    }
                }

//                if (ccda.getResults() != null
//                        && (ccda.getResults().size() > 0)
//                    && ccda.getResults().get(0).getSite() != null
//                    && ccda.getResults().get(0).getSite().getMoniker() != null
//                    && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getResults().get(0).getSite().getMoniker()))
//                 {
//                    vlerAvailability.add("RESULTS");
//                }
//                if (ccda.getEncounters() != null && (ccda.getEncounters().size() > 0)
//                        && ccda.getEncounters().get(0).getSite() != null
//                        && ccda.getEncounters().get(0).getSite().getMoniker() != null
//                        && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getEncounters().get(0).getSite().getMoniker())) {
//                    vlerAvailability.add("ENCOUNTERS");
//                }
//                if (ccda.getMedications() != null && (ccda.getMedications().size() > 0)
//                        && ccda.getMedications().get(0).getSite() != null
//                        && ccda.getMedications().get(0).getSite().getMoniker() != null
//                        && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getMedications().get(0).getSite().getMoniker())) {
//                    vlerAvailability.add("MEDICATIONS");
//                }

            }

            if (hasData) {
                response.status = 200
                sendResponse(vlerAvailability);
//            render "SUCCESS!"
            } else {
                response.status = 500
                render "Failed to retrieve DAS VLER records: No data detected in the query."
            }
        } else {
            response.status = 500
            render "Failed to retrieve DAS VLER Records."
        }
        //sendResponse("place your return item(s) here")
    }

    def getNarrative = {
        NarrativeV4 narrative = session.appUser.narrativesMap.get(params.get("narrativeId"));

        if (narrative != null && narrative.getNarrativeText() != null) {
            response.status = 200
            render narrative.getNarrativeText()
        } else {
            response.status = 200
            render "This DAS VLER Narrative is no longer available. Please refresh this widget and try again."
        }
    }

}
