/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIFlag
import org.apache.commons.lang3.StringUtils

class PatientDemographicsController extends BaseController {

    def patientDao
    def auditDao

    static allowedMethods = [list: 'POST', details: 'POST', auditAlerts: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def list = {
        def includeVLER = false;
        def includeDASVLER = false;

        if (params["includeVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "DOD") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeVLER = true;
        }
        if (params["includeDASVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "VA") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeDASVLER = true;
        }

        def guiPatDemos = patientDao.getPatientDemographics(session.appUser, request.patient, includeVLER, includeDASVLER)
        for (def guiDemo : guiPatDemos) {

            if (guiDemo.getSite() != null && guiDemo.getSite().getName() != null
                    && (guiDemo.getSite().getName().contains("VLER"))
                    && guiDemo.getSite().getSiteCode() != null) {
                String siteName = session.appUser.partnersMap.get(guiDemo.getSite().getSiteCode());
                if (siteName != null) {
                    guiDemo.getSite().setName(siteName);
                }
                guiDemo.getSite().setAgency("VLER");
            }
            
            List<GUIFlag> flags = new ArrayList<GUIFlag>();
            for (def flag : guiDemo.getFlags()) {
                if (flag.getName() != null && flag.getName().toLowerCase().indexOf("connection unavailable") < 0) {
                    flags.add(flag);
                }
            }
            guiDemo.setFlags(flags);

        }
        sendResponse(guiPatDemos)
    }

    def details = {

        def guiPatDemoDetails = patientDao.getPatientDemographicDetails(session.appUser, request.patient)

        sendResponse(guiPatDemoDetails)
    }

    def auditAlerts = {
        def recordSiteAgency = params[HttpKeys.RECORD_SITE_AGENCY]
        def recordSiteMoniker = params[HttpKeys.RECORD_SITE_MONIKER]

        auditDao.auditDetailsQuery("patientFlags", session.appUser, request.patient, null, null, recordSiteMoniker, recordSiteAgency)

        sendResponse([success: "true"])
    }
}
