package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.*
import gov.va.med.jmeadows.webservice.NoteImage
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils

class PatientFormsController extends PatientNoteDetailBaseController {

    def questionnaireDao
    def noteDao
    def auditDao

    static allowedMethods = [list: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def list = {

        def guiForms = questionnaireDao.getPatientForms(session.appUser, request.patient)

        if (guiForms != null) {
            for (def guiForm : guiForms) {
                if (StringUtils.isNotEmpty(guiForm.formId)) {
                    def queryMap = [(HttpKeys.FORM_ID)            : guiForm.getFormId(),
                                    (HttpKeys.RECORD_DATE)        : guiForm.getFormDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiForm.site.getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiForm.site.getMoniker()]

                    guiForm.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiForm.getSite(),
                                    guiForm.getSourceProtocol(),
                                    guiForm.isSessionCachedRecord()))

                    if (guiForm.isSessionCachedRecord()) {
                        //cache consult report in session
                        def formDetail = BeanUtils.cloneBean(guiForm)

                        session.appUser.formMap.put(
                                getCachedRecordKey(guiForm.getRecordDetailsToken()), formDetail)

                        //don't send report text with note listings
                        guiForm.setDeploymentForm(null);
                        guiForm.setQuestionnaireBean(null);
                    }
                }
            }
        }

        sendResponse(guiForms)

    }

    def details = {

        GUIPatientForm guiForm = null
        GUIQuestionnaire result = null

        if (request.isSessionCachedRecord) {

            //audit details query



            guiForm = session.appUser.formMap.get(
                    getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))
        }

        if (guiForm != null && guiForm.getQuestionnaireBean() == null) {
            result = new GUIQuestionnaire();
            result.setName("This entry is empty.");
        } else {
            result = guiForm.getQuestionnaireBean();
        }


        if (result != null) {
            StringBuilder report = new StringBuilder();

            def statusScore = ''
            if (result.status) {
                statusScore += result.status;
            }
            if (result.score) {
                if (statusScore) {
                    statusScore += ' / ';
                }
                statusScore += 'Score: ' + result.score;
            }



            report.append("<div id='questionnaireTableWrapper'><table class='questionnaireTable'><tr>");
            report.append("<td>Document Title:</td><td>" + result.name + "</td></tr>");
            report.append("<td>Entered By:</td><td>" + result.enteredBy.name + "</td></tr>");
            report.append("<td>Status/Score</td><td>" + statusScore + "</td></tr>");
            report.append("<td>Response Date:</td><td>" + guiForm.formDate + "</td></tr>");
            report.append("<td>Version:</td><td>" + result.version + "</td></tr>");
            if(result.comments) {
                report.append("<td>Instructions:</td><td>" + result.comments + "</td></tr></table></div><br/><br/>");
            } else {
                report.append("<td>Instructions:</td>&nbsp;<td></td></tr></table></div><br/><br/>");
            }
            List<GUIQuestionnaireResponseComments> respCommentsList = result.getResponseComments();
            if (respCommentsList != null && !respCommentsList.isEmpty()) {
                String enteredBy = "";

                report.append("<table width='100%' border='1' class='questionnaire-table questionnaire-details'>")
                for (GUIQuestionnaireResponseComments respComment : respCommentsList) {
                    if(respComment.getEnteredBy() != null && respComment.getEnteredBy().getName() != null){
                        enteredBy = respComment.getEnteredBy().getName();
                    }

                    // Comments by
                    report.append("<tr class='questionnaire-highlighted'>")
                    report.append("<td>Comment entered by " + enteredBy + " @ " + respComment.getDateEntered() + "<br/>")
                    report.append(respComment.getComment() + "</td>")
                    report.append("</tr>")
                }
                report.append("</table>")
            }

            String selected = "";
            List<GUIQuestionnaireDetails> detailsList = result.getResponses();
            if (detailsList != null && !detailsList.isEmpty()) {
                report.append("<table width='100%' border='1' class='questionnaire-table questionnaire-details'>")
                for (GUIQuestionnaireDetails detail : detailsList) {
                    List<GUIQuestionnaireDetailChoices> choicesList = detail.getChoices();
                    List<GUIQuestionnaireDetailAnswers> answersList = detail.getAnswers();
                    List<GUIQuestionnaireDetailComments> commentsList = detail.getComments();
                    int choicesCount = 0;
                    int answersCount = 0;
                    int commentsCount = 0;

                    if (choicesList != null) {
                        choicesCount = choicesList.size();
                    }

                    if (answersList != null) {
                        answersCount = answersList.size();
                    }

                    if (commentsList != null) {
                        commentsCount = commentsList.size();
                    }

                    // Sequence column
                    report.append("<tr>")
                    report.append("<td valign='top' align='middle'>" + (int)Double.parseDouble(detail.getSequence()) + ".</td>")

                    // Yes/No, True/Falst, Multiple choice types
                    report.append("<td>")
                    report.append("<table width='100%' class='questionnaire-table questionnaire-choices'>")
                    if (choicesCount > 0) {
                        // Question
                        report.append("<tr>")
                        report.append("<td colspan='" + choicesCount + "'>" + detail.getQuestion() + "</td>");
                        report.append("</tr>")
                        // Choices & answers
                        if (choicesCount > 0) {
                            report.append("<tr>")
                            for (GUIQuestionnaireDetailChoices choice : choicesList) {
                                selected = "";
                                if (choicesCount > 1) {
                                    if (choice.isSelected()) {
//                                        selected = "&#x2612;";
                                        selected = "&#9746;";
                                    } else {
//                                        selected = '&#x2610;';
                                        selected = "&#9744;";
                                    }
                                }
                                selected = selected + " " + choice.getChoiceText();
                                report.append("<td width='" + Math.floor(100 / choicesCount).toInteger() + "%'>" + selected + "</td>");
                            }
                            report.append("</tr>")
                        }
                        if (commentsCount > 0) {
                            for (GUIQuestionnaireDetailComments comment : commentsList) {
                                // Comments by
                                report.append("<tr class='questionnaire-highlighted'>")
                                report.append("<td colspan='" + choicesCount + "'>Comment entered by " + comment.getCommentBy() + " @ " + comment.getDateEntered() + "</td>")
                                report.append("</tr>")
                                // Comment
                                report.append("<tr class='questionnaire-highlighted'>")
                                report.append("<td colspan='" + choicesCount + "'>" + comment.getNoteText() + "</td>")
                                report.append("</tr>")
                            }
                        }
                    }
                    report.append("</table>")
                    report.append("</td>")
                    report.append("</tr>")
                }
                report.append("</table>")
            }

            result.setReportText(report.toString());
        }

        def auditCategory = "PatQuestionnaireDetails";
        if (params[HttpKeys.WIDGET_NAME]) {
            auditCategory = params[HttpKeys.WIDGET_NAME] + '^' + auditCategory;
        }

        auditDao.auditDetailsQuery(auditCategory, result.getEnteredDate() + '^' + result.getName(),
                session.appUser, request.patient, null, result.getRecordId(),
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

        sendResponse(result)

    }

    def getNoteDataStream = {

        //retrieve cached note
        GUIPatientForm guiForm = session.appUser.formMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "PatDeploymentFormDetails"

        if ((guiForm == null) || !guiForm.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        //stream CDA data
        if (guiForm.getDeploymentForm() != null && StringUtils.isNotEmpty(guiForm.getDeploymentForm().getNoteText())) {
            auditDao.auditDetailsQuery(auditQueryName, request.recordQuery[HttpKeys.RECORD_DATE],
                    session.appUser, request.patient,
                    null, request.recordQuery[HttpKeys.FORM_ID],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            makeCIS_CDATextAndHeader(guiForm.getDeploymentForm().getNoteText());
            return true;
        }
        //fetch dod note image
        else if(guiForm.getDeploymentForm() != null && "PDF".equalsIgnoreCase(guiForm.getDeploymentForm().getNoteFormat())
                && StringUtils.isNotEmpty(guiForm.getDeploymentForm().getComplexDataUrl())) {

            String complexDataUrl =  guiForm.getDeploymentForm().getComplexDataUrl();

            NoteImage noteImage = null;

            if (complexDataUrl != null) {

                auditDao.auditDetailsQuery(auditQueryName, request.recordQuery[HttpKeys.RECORD_DATE] + '^' +
                        guiForm.getDocumentTitle(),
                        session.appUser, request.patient,
                        complexDataUrl,
                        guiForm.getEventId(),
                        request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                        request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

                try {
                    noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl);
                }
                catch (DaoException e) {
                    log.error(e);
                }

            }

            response.setHeader("Pragma", "public");
            response.setHeader("Cache-Control", "maxage=1");

            if (noteImage != null && noteImage.getNoteBytes() != null) {
                response.setContentType(noteImage.getContentType());

                String contentDisposition = noteImage.getContentDisposition()
                response.setHeader("Content-Disposition", contentDisposition.replace("attachment", "inline"))


                response.getOutputStream().write(noteImage.getNoteBytes());
                response.getOutputStream().flush();
                response.getOutputStream().close();

                return true;
            }

            //if note attachment is null, send an error message

            String errorMsg = "An error occurred while attempting to retrieve note. - JLV";

            response.setContentType("text");
            response.setContentLength(errorMsg.length());
            response.getWriter() << errorMsg;
            response.getWriter().flush();

            response.sendError 500
            return false;
        }

    }
}
