package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.dao.DaoException
import gov.va.med.guibeans.GUIProcedure
import gov.va.med.guibeans.GUIProgressNote
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.NarrativeV4
import gov.va.med.jmeadows.webservice.NoteImage
import gov.va.med.vhahon.common.Utils
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang3.StringUtils

class PatientProceduresController extends BaseController {

    def procedureDao
    def auditDao
    def noteDao
    def sensitiveRecordDao

    static allowedMethods = [list: 'POST', auditDetails: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }
    }

    def list = {
        def includeVLER = false;
        def includeDASVLER = false;

        if (params["includeVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "DOD") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeVLER = true;
        }
        if (params["includeDASVLER"] == "true" && (StringUtils.equals(session.appUser.agency, "VA") || StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT"))) {
            includeDASVLER = true;
        }

        HashSet narrativeIds = new HashSet();

        def guiProcedures = procedureDao.getPatientProcedures(
                session.appUser,
                request.patient,
                Utils.getCal(request.getParameter(HttpKeys.START_DATE)),
                Utils.getCal(request.getParameter(HttpKeys.END_DATE)),
                includeVLER,
                includeDASVLER);

        String ipAddress = request.getHeader("X-FORWARDED-FOR");
        if (ipAddress == null) {
            ipAddress = request.getRemoteAddr();
        }

        //generate record details token
        if (guiProcedures != null) {
            for (def guiProcedure : guiProcedures) {

                //-start note stuff
                String noteId = "";
                int dodNoteIdx = 0;

                if (guiProcedure.getDocuments() != null
                        && guiProcedure.getDocuments().get(0).getId() != null) {
                    noteId = guiProcedure.getDocuments().get(0).getId();
                    guiProcedure.setVaNote(true);

                    def queryMap = [(HttpKeys.NOTE_ID)            : noteId,
                                    (HttpKeys.RECORD_DATE)        : guiProcedure.getDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiProcedure.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiProcedure.getSite().getMoniker()]

                    guiProcedure.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiProcedure.getSite(),
                                    guiProcedure.getSourceProtocol(),
                                    guiProcedure.isSessionCachedRecord()))

                    guiProcedure.setGuiIpAddress(ipAddress)

                    if (guiProcedure.isSessionCachedRecord()) {
                        //cache procecure report in session
                        def procDetail = BeanUtils.cloneBean(guiProcedure)

                        session.appUser.proceduresMap.put(
                                getCachedRecordKey(guiProcedure.getRecordDetailsToken()), procDetail)
                    }
                } else if (guiProcedure.getComplexDataUrl() != null
                        && guiProcedure.getComplexDataUrl().toLowerCase().indexOf("http") > -1) {

                    def queryMap = [(HttpKeys.NOTE_ID)            : dodNoteIdx,
                                    (HttpKeys.RECORD_DATE)        : guiProcedure.getDate(),
                                    (HttpKeys.RECORD_SITE_AGENCY) : guiProcedure.getSite().getAgency(),
                                    (HttpKeys.RECORD_SITE_MONIKER): guiProcedure.getSite().getMoniker()]

                    guiProcedure.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiProcedure.getSite(),
                                    guiProcedure.getSourceProtocol(),
                                    guiProcedure.isSessionCachedRecord()))

                    guiProcedure.setGuiIpAddress(ipAddress)
                    guiProcedure.setNoteAsBinaryStream(true);

                    if (guiProcedure.isSessionCachedRecord()) {
                        //cache procecure report in session
                        def procDetail = BeanUtils.cloneBean(guiProcedure)

                        session.appUser.proceduresMap.put(
                                getCachedRecordKey(guiProcedure.getRecordDetailsToken()), procDetail)
                    }

                    dodNoteIdx++;
                }

                //-end note stuff

                if (guiProcedure.getNarrativeId() != null && !"".equalsIgnoreCase(guiProcedure.getNarrativeId())) {
                    narrativeIds.add(guiProcedure.getNarrativeId());

                    if(StringUtils.isNotEmpty(guiProcedure.getNarrative())){
                        NarrativeV4 narrativeV4 = new NarrativeV4();
                        narrativeV4.setNarrativeIdentifier(guiProcedure.getNarrativeId());
                        narrativeV4.setNarrativeText(guiProcedure.getNarrative());
                        session.appUser.narrativesMap.put(guiProcedure.getNarrativeId(), narrativeV4);
                        guiProcedure.setNarrative("");
                    }

                    if(guiProcedure.getSite() != null && guiProcedure.getSite().getName() != null
                            && "VLER".equalsIgnoreCase(guiProcedure.getSite().getName())
                            && guiProcedure.getSite().getSiteCode() != null){
                        String siteName = session.appUser.partnersMap.get(guiProcedure.getSite().getSiteCode());
                        if(siteName != null){
                            guiProcedure.getSite().setName(siteName);
                        }
                        guiProcedure.getSite().setAgency("VLER");
                    }
                }
            }
        }
        if (includeVLER && session.appUser.narrativesMap != null
                && !session.appUser.narrativesMap.isEmpty()) {
            Iterator narrativeIter = session.appUser.narrativesMap.entrySet().iterator();

            //loop through all narratives, check the allergy ones to see if they are referenced in this call
            //if they are not referenced, then create a record for them so the user is able to view this narrative
            while (narrativeIter.hasNext()) {
                Map.Entry narrative = (Map.Entry) narrativeIter.next();
                if (narrative.getKey().toString().contains("47519-4")) {
                    //check to see if record using this narrative exists
                    if (!narrativeIds.contains(narrative.getKey().toString())) {
                        NarrativeV4 narrativeItem = session.appUser.narrativesMap.get(narrative.getKey().toString());
                        String siteName = "VLER";
                        String siteMoniker = "VLER Partner";
                        if (narrativeItem != null)
                            if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                    && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                                siteName = session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier());
                            }
                        if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                            siteMoniker = session.appUser.partnersMap.get(narrativeItem.getNarrativeAssigningAuthorityIdentifier());
                        }
                        GUISite narrativeSite = new GUISite();
                        narrativeSite.setAgency("VLER");
                        narrativeSite.setName(siteName);
                        narrativeSite.setMoniker(siteMoniker);
                        narrativeSite.setSiteCode("VLER");

                        GUIProcedure narrativeRecord = new GUIProcedure();
                        narrativeRecord.setNarrativeId(narrative.getKey().toString());
                        narrativeRecord.setSite(narrativeSite);
                        narrativeRecord.setDescription("Narrative");
                        guiProcedures.add(narrativeRecord);

                        narrativeIds.add(narrative.getKey().toString());
                    }

                }
            }

        }
        sendResponse(guiProcedures)
    }

    def details = {

        GUIProgressNote guiNote = null

        //multipe note support using request param
        String noteId = StringUtils.isEmpty(params[HttpKeys.NOTE_ID]) ? request.recordQuery[HttpKeys.NOTE_ID] : params[HttpKeys.NOTE_ID];

        if (!request.isSessionCachedRecord) {
            guiNote = noteDao.getProgressNote(
                    session.appUser,
                    request.patient,
                    noteId,
                    request.recordQuery[HttpKeys.RECORD_SITE_CODE],
                    request.recordQuery[HttpKeys.RECORD_DATE] + '^' + params[HttpKeys.TITLE]);

            auditDao.auditDetailsQuery("ProcedureEncounterNote", request.recordQuery[HttpKeys.RECORD_DATE] + '^' +
                    params[HttpKeys.TITLE],
                    session.appUser, request.patient, null,
                    noteId,
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            sendResponse(guiNote)
            return true;
        }

//        //retrieve cached note
//        guiNote = session.appUser.outpatientNotesMap.get(
//                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))
//        if (guiNote.sensitive) {
//            String auditText = params[HttpKeys.AUDIT_TEXT]
//            // audit with break the glass
//            auditText = "btgPatientEncounters^" + auditText;
//
//            sensitiveRecordDao.auditSensitiveRecordAccess(session.appUser, request.patient, auditText, session.scId, session.scEmail)
//        }

//        auditDao.auditDetailsQuery("ProcedureEncounterNote", session.appUser, request.patient,
//                request.recordQuery[HttpKeys.RECORD_DATE],
//                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
//                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])
//
        sendResponse(guiNote)
    }

    def getNoteDataStream = {

        //retrieve cached note
        GUIProcedure guiProcedure = session.appUser.proceduresMap.get(
                getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))

        def auditQueryName = "GetEncounterDataStream"

        if ((guiProcedure == null) || !guiProcedure.noteAsBinaryStream) {
            response.sendError 500
            return false;
        }

        if (guiProcedure.getComplexDataUrl() && guiProcedure.getComplexDataUrl().length() > 0) {
            String complexDataUrl = guiProcedure.getComplexDataUrl();

            NoteImage noteImage = null;

            if (complexDataUrl != null) {

                StringBuilder info = new StringBuilder();
                info.append(params[HttpKeys.EXAM_DATE])
                info.append('^')
                info.append(params[HttpKeys.INFO])
                def recordID = request.recordQuery[HttpKeys.NOTE_ID]

                if (complexDataUrl != null && (recordID == null || recordID == '0')) {
                    String[] parsedComplex = complexDataUrl.split("/");
                    recordID = parsedComplex[parsedComplex.length - 1];
                }
                
                auditDao.auditDetailsQuery(auditQueryName, info.toString(), session.appUser, request.patient,
                        complexDataUrl,
                        recordID,
                        request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                        request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

                try {
                    noteImage = noteDao.getBHIENoteImage(session.appUser, complexDataUrl);
                }
                catch (DaoException e) {
                    log.error();
                }

                response.setHeader("Pragma", "public");
                response.setHeader("Cache-Control", "maxage=1");
                //response.setHeader("Expires", "0");

                if (noteImage != null && noteImage.getNoteBytes() != null) {
                    response.setContentType(noteImage.getContentType());
                    String contentDisposition = noteImage.getContentDisposition()
                    response.setHeader("Content-Disposition", contentDisposition.replace("attachment", "inline"))

                    response.getOutputStream().write(noteImage.getNoteBytes());

                    response.getOutputStream().flush();
                    response.getOutputStream().close();
                    return true;
                }
            }

            //if note attachment is null, send an error message

            String errorMsg = "An error occurred while attempting to retrieve note.";

            response.setContentType("text");
            response.setContentLength(errorMsg.length());
            response.getWriter() << errorMsg;
            response.getWriter().flush();

            response.sendError 500
            return false;
        }
    }

    def auditVixViewer = {
        auditDao.auditDetailsQuery(
                'ProcedureVixViewer',
                session.appUser,
                request.patient,
                null,
                'noteId=' + request.recordQuery[HttpKeys.NOTE_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                request.recordQuery[HttpKeys.RECORD_SITE_AGENCY]);
    }

    def getProcedureVixStudyViewerUrl = {
        def vixStudyViewerUrl = procedureDao.getProcedureVixStudyViewerUrl(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]),
                params.documentId,
                params.documentSiteId);
        sendResponse([vixStudyViewerUrl: vixStudyViewerUrl])
    }
}
