/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.Validators
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.IehrUserProfile
import org.apache.commons.lang3.StringUtils

class UserCfgController extends BaseController {
    def userDao

    static allowedMethods = [save: 'POST']

    def beforeInterceptor = {

        if (session.appUser == null) {
            response.sendError 500
            return false;
        }

    }

    def save = {

        String cfg = params['cfg'];

        if (StringUtils.equalsIgnoreCase(session.appUser.agency, "VA")
                && !StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT")) {
            cfg = Validators.validateCfg(cfg);

        }

        def result = userDao.setJanusGUIConfig(session.appUser, cfg)

        if (result.value) {
            session.appUser.cfg = cfg
            response.status = 200
            render "The config was saved successfuly."
        } else {
            response.status = 500
            render "Could not save the config: ${result} "
        }
        //sendResponse(result)
    }

    def show = {

        def IehrUserProfile userProfile
        def List<GUISite> sites

        if (session.sites == null) {
            try {
                sites = userDao.getSites()
                if (sites != null) {
                    def tempSites
                    Iterator<GUISite> i = sites.iterator()
                    while (i.hasNext()) {
                        GUISite site = i.next()
                        if (site.endpoints == null || site.endpoints.size() == 0) {
                            i.remove()
                        }
                    }
                    session.sites = sites
                }
            }
            catch (Exception e) {
                log.error("AuthenticateController::login - Unable to retrieve site list", e)
                String errorText = "There's an error retrieving the sites list.<br/>Please notify the system administrator";
            }
        } else {
            sites = session.sites
        }


        def result = userDao.getJanusGUIConfig(session.appUser)

        if (StringUtils.equals(session.appUser.agency, "VA") && !StringUtils.equals(grails.util.Environment.current.toString(), "DEVELOPMENT")) {
            result = Validators.validateCfg(result);
        }

        if (result != null) {
            response.status = 200
            render "${result}"
        } else {
            response.status = 500
            render "Could not retrieve the config: ${result} "
        }

        //sendResponse(result)
    }


}

