/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.guibeans.GUIEndpoint
import grails.converters.JSON

class ViewController {

    def index = {

        def siteIcon = "VA"
        def cfg = session.appUser.cfg
        def theme = "default"
        def jsonCfg = ""
        def defaultRefs = ""
        def refsExist = false;

        //generate default reference links
        StringBuilder sb = new StringBuilder();

        if (grailsApplication.config.grails.provReferences != null && grailsApplication.config.grails.provReferences.size() > 0) {
            String references = grailsApplication.config.grails.provReferences;
            StringTokenizer st = new StringTokenizer(references, ";");
            int i = 0;

            sb.append("[");
            while(st.hasMoreElements()){
                String[] ref = st.nextElement().split(",");
                if(i > 0){
                    sb.append(",");
                }
                sb.append("{id:'default-");
                sb.append(i);
                sb.append("', linkDesc:'");
                sb.append(ref[0]);
                sb.append("', linkUrl:'");
                sb.append(ref[1]);
                sb.append("'}")
                i++;
            }
            sb.append("]");
        }

        defaultRefs = sb.toString();

        if (cfg != null && cfg.length() > 0) {
            jsonCfg = JSON.parse(cfg)
            for (item in jsonCfg) {
                if (item.cfgItem == "UI_CONF") {
                    for (option in item.options) {
                        if (option.theme) {
                            theme = option.theme
                        } else if(option.prov_ref){
                            refsExist = true;
                        }
                    }
                }
            }
        }


        def hostSite = session.appUser.hostSite

        if (hostSite.agency == grailsApplication.config.grails.agencyDOD) {
            siteIcon = "DOD"
        }

        //only enable the provider profile for sites that include a cache site
        def enableProvProfile = false

        for (GUIEndpoint endpoint : hostSite.endpoints) {
            if (endpoint.protocol == 'CACHE' && endpoint.status == 'active') {
                enableProvProfile = true
                break
            }
        }

        //compute session timeout in minutes
        def sessionTimeout = session.maxInactiveInterval / 60

        def allowMedwebPlugin = false

        hostSite.permissions.each() {
            if (it.description == 'medweb_plugin') {
                allowMedwebPlugin = true
            }
        }

        // Ensure AHLTA link flag and object are valid if not coming from AHLTA link
        if (session.ahltaLoginFlag == null) {
            session.ahltaLoginFlag = false
            session.ahltaLoginPatient = []
        }

        //generate and set csrf token
        def csrfToken = "";
        if (session.getAttribute("X-CSRF-Token") == null) {
            csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
        } else {
            csrfToken = session.getAttribute("X-CSRF-Token")
        }

        // Vista Imaging Display Flag
        def isVixDisplayEnabled = Boolean.parseBoolean(grailsApplication.config.grails.isVixDisplayEnabled);

        [user             : session.appUser,
         theme            : theme,
         defaultRefs: defaultRefs,
         allowMedwebPlugin: allowMedwebPlugin,
         sessionTimeout   : sessionTimeout,
         csrfToken: csrfToken,
         siteIcon         : siteIcon,
         enableProvProfile: enableProvProfile,
         grailsConfig     : grailsApplication.config.grails,
         appVersion       : grailsApplication.metadata['app.version'],
         cfg              : cfg, theme: theme,
         ahltaLoginFlag   : session.ahltaLoginFlag,
         ahltaLoginPatient: session.ahltaLoginPatient as JSON,
         isVixDisplayEnabled: isVixDisplayEnabled,
        ]
    }

    def refresh = {
        def user = session.appUser
        render("OK");
    }

}
