/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.groovy.user

import gov.va.med.guibeans.GUIPermission
import gov.va.med.guibeans.GUIUser
import org.dozer.DozerBeanMapper

/**
 *
 *
 */
class AppUser extends GUIUser implements Serializable {
    public static final long serialVersionUID = 100

    /** AES security key, used to encrypt/decrypt secure session data (patient tokens, etc.) **/
    byte[] securityKey;

    def selectedPatientMap

    def problemListMap
    def consultMap
    def encounterMap
    def formMap
    def inpatientNotesMap
    def outpatientNotesMap
    def proceduresMap
    def narrativesMap
    def partnersMap
    def radiologyReportsMap
    def sensitiveLabResultsMap
    def sensitiveLabOrdersMap
    def genesisMap
    def vlerDocumentsMap
    def vlerSiteStatus
    def attachmentsMap

    def attributes

    /** counter to help with unique caching **/
    private int cacheCounter = 0;

    public AppUser() {
        init()
    }

    public synchronized int getCacheCounter() {
        return ++cacheCounter;
    }

    public AppUser(GUIUser user) {
        init()

        DozerBeanMapper dozerBeanMapper = new DozerBeanMapper();
        List<String> mappingFiles = new ArrayList<String>();
        mappingFiles.add("dozerBeanMappings.xml");
        dozerBeanMapper.setMappingFiles(mappingFiles);

        dozerBeanMapper.map(user, this)

        GUIPermission permission = new GUIPermission();
        permission.id = "1234"
        permission.description = "test permission"
        this.guiPermissions = []
        guiPermissions.add(permission)

        if (cfg == null) cfg = ""
    }

    private void init() {
        selectedPatientMap = [:]
        narrativesMap = [:]
        partnersMap = [:]
        problemListMap = [:]
        consultMap = [:]
        encounterMap = [:]
        formMap = [:]
        inpatientNotesMap = [:]
        outpatientNotesMap = [:]
        proceduresMap = [:]
        radiologyReportsMap = [:]
        sensitiveLabResultsMap = [:]
        sensitiveLabOrdersMap = [:]
        genesisMap = [:]
        attachmentsMap = [:]
        vlerDocumentsMap = [:]

        attributes = [:]
        cfg = ""
    }

    public byte[] getSecurityKey() {
        if (this.securityKey != null) {
            return this.securityKey.clone();
        }
        return null;

    }

    private void clearMaps(){
        narrativesMap = [:]
        partnersMap = [:]
        problemListMap = [:]
        consultMap = [:]
        encounterMap = [:]
        proceduresMap = [:]
        formMap = [:]
        inpatientNotesMap = [:]
        outpatientNotesMap = [:]
        radiologyReportsMap = [:]
        sensitiveLabResultsMap = [:]
        sensitiveLabOrdersMap = [:]
        genesisMap = [:]
        attachmentsMap = [:]
        vlerDocumentsMap = [:]
    }
}

