package gov.va.med.common;


import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import javax.servlet.ServletContext;
import javax.xml.XMLConstants;
import javax.xml.transform.*;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;

@Component
public class XMLUtils {
    private static final Logger logger = Logger.getLogger(XMLUtils.class.getName());

    static String XSL_PATH_C32 = "/c32styles/nhin/cda2detail.xsl";
    static String XSL_PATH_CCDA = "/c32styles/nhin/cdaHFG.xsl";

    public static String transform(ServletContext context, String xmlString, String xslTemplate) throws TransformerConfigurationException, TransformerException {
        InputStream xslInput = null;
        try {
            xslInput = context.getResourceAsStream(xslTemplate);

            String xslInputPath = context.getResource(xslTemplate).toExternalForm();
            String xslSystemId = xslInputPath.substring(0, xslInputPath.lastIndexOf('/') + 1);

            StreamSource xmlInput = new StreamSource(new StringReader(xmlString));

            Source xslStreamSource = null;
            if (XSL_PATH_C32.equalsIgnoreCase(xslTemplate)) {
                xslStreamSource = new StreamSource(xslInput, xslSystemId);
            } else {
                xslStreamSource = new StreamSource(xslInput, xslInputPath);
            }

            TransformerFactory factory = TransformerFactory.newInstance();
            factory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
            factory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
            factory.setFeature(XMLConstants.ACCESS_EXTERNAL_DTD, false);
            Transformer transformer = factory.newTransformer(xslStreamSource);

            StringWriter outputWriter = new StringWriter();
            Result outputResult = new StreamResult(outputWriter);
            transformer.transform(xmlInput, outputResult);
            return outputWriter.toString();

        } catch (MalformedURLException e) {
            logger.error(e.getMessage());
        } finally {
            if (xslInput != null) {
                try {
                    xslInput.close();
                } catch (IOException ioe) {
                    logger.error(ioe.getMessage());
                }
            }
        }

        return "An error occurred while attempting to retrieve the document: Error during transform";
    }


}
