/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 *
 *
 */
@Component
public class AppointmentDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(AppointmentDao.class.getName());

    public List<GUIAppointment> getPatientAppointments(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            PatientAppointments[] appts = getServiceHandler().getPatientAppointments(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAppointment> guiAppts = new ArrayList<GUIAppointment>();

            for (PatientAppointments appt : appts) {
                GUIAppointment guiAppt = new GUIAppointment();

                String apptDate = "";
                String clinic = "";
                String clinicId = "";
                String status = "";
                String statusReason = "";
                String type = "";
                String providerName = "";
                String specialty = "";
                String reason = "";
                String appointmentId = "";

                if (appt.getProvider() != null && appt.getProvider().getName() != null) {
                    guiAppt.setProvider(dozerBeanMapper.map(appt.getProvider(), GUIProvider.class));
                    providerName = appt.getProvider().getName();
                    if (appt.getProvider() != null)
                        specialty = appt.getProvider().getSpecialization();
                }

                if (appt.getClinic() != null) {
                    clinic = appt.getClinic();
                }

                if (appt.getClinicId() != null) {
                    clinicId = appt.getClinicId();
                }

                if (appt.getStatus() != null) {
                    status = appt.getStatus();
                }

                if (appt.getStatusReason() != null) {
                    statusReason = appt.getStatusReason();
                }

                if (appt.getApptId() != null) {
                    appointmentId = appt.getApptId();
                }

                if (appt.getApptDate() != null) {
                    apptDate = Utils.formatDisplayDateTime(appt.getApptDate());
                }

                if (appt.getApptType() != null) {
                    type = appt.getApptType();
                }

                if (appt.getReason() != null) {
                    reason = appt.getReason();
                }

                guiAppt.setAppointmentDate(apptDate);
                guiAppt.setAppointmentId(appointmentId);
                guiAppt.setClinic(clinic);
                guiAppt.setClinicId(clinicId);
                guiAppt.setProviderName(providerName);
                guiAppt.setReason(reason);
                guiAppt.setStatusReason(statusReason);
                guiAppt.setStatus(status);
                guiAppt.setType(type);

                if (appt.getProvider() != null) {
                    guiAppt.setProvider(dozerBeanMapper.map(appt.getProvider(), GUIProvider.class));
                }


                addSiteDataToGUIBean(guiAppt, appt);

                guiAppts.add(guiAppt);
            }

            return guiAppts;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIAppointmentDetail getAppointmentDetails(GUIUser user, GUIPatient patient, String appointmentId,
                                                      String recordSiteCode, Calendar startDate, Calendar endDate) {
        try {
            FreeTextReport report = getServiceHandler().getPatientEncountersReport(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .itemId(appointmentId)
                            .recordSiteCode(recordSiteCode)
                            .build());

            GUIAppointmentDetail apptDetail = new GUIAppointmentDetail();
            if (report != null) {
                apptDetail.setVisitDetails(StringEscapeUtils.escapeHtml4(report.getReportText()));
                addSiteDataToGUIBean(apptDetail, report);
            }

            return apptDetail;
        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIAppointment> getProviderAppointments(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            PatientAppointments[] appts = getServiceHandler().getProviderAppointments(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAppointment> guiAppts = new ArrayList<GUIAppointment>();

            for (PatientAppointments appt : appts) {
                GUIAppointment guiAppt = new GUIAppointment();

                String apptDate = "";
                String clinic = "";
                String status = "";
                String type = "";
                String providerName = "";
                String reason = "";
                String appointmentId = "";
                String patName = "";
                String patToken = "";

                if (appt.getProvider() != null && appt.getProvider().getName() != null) {
                    providerName = appt.getProvider().getName();
                }

                if (appt.getClinic() != null) {
                    clinic = appt.getClinic();
                }

                if (appt.getStatus() != null) {
                    status = appt.getStatus();
                }

                if (appt.getApptId() != null) {
                    appointmentId = appt.getApptId();
                }

                if (appt.getApptDate() != null) {
                    apptDate = Utils.formatDisplayDateTime(appt.getApptDate());
                }

                if (appt.getApptType() != null) {
                    type = appt.getApptType();
                }

                if (appt.getReason() != null) {
                    reason = appt.getReason();
                }

                if (appt.getPatientName() != null) {
                    patName = appt.getPatientName();
                }

                patToken = getProvQueryPatientToken(encryptSecurityKey, appt);

                guiAppt.setAppointmentDate(apptDate);
                guiAppt.setAppointmentId(appointmentId);
                guiAppt.setClinic(clinic);
                guiAppt.setProviderName(providerName);
                guiAppt.setReason(reason);
                guiAppt.setStatus(status);
                guiAppt.setType(type);
                addSiteDataToGUIBean(guiAppt, appt);
                guiAppt.setPatientName(patName);
                guiAppt.setPatientToken(patToken);

                guiAppts.add(guiAppt);
            }

            return guiAppts;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    /**
     * Retrieves patient encounters from AHLTA (only returns DoD data for now)
     *
     * @param user
     * @param patient
     * @param startDate
     * @param endDate
     * @return a list of patient encounters from AHLTA.
     */
    public List<GUIPatientEncounter> getPatientEncounters(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            Encounter[] patientEncounters = getServiceHandler().getPatientEncounters(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIPatientEncounter> guiEncounters = new ArrayList<GUIPatientEncounter>();
//            StringBuilder encounterId = new StringBuilder();
//            String newEncounterId;
            int i = 0;

            for (Encounter patientEncounter : patientEncounters) {
                GUIPatientEncounter guiEncounter = dozerBeanMapper.map(patientEncounter, GUIPatientEncounter.class);
                 guiEncounter = mapGUIEncounter(patientEncounter, guiEncounter, i);
                guiEncounters.add(guiEncounter);
                i++;
            }

            return guiEncounters;
        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIPatientEncounter mapGUIEncounter(Encounter encounter, GUIPatientEncounter guiEncounter, int docId){


            //guiEncounter = this.dozerBeanMapper.map(encounter, GUIPatientEncounter.class);

            StringBuilder encounterId = new StringBuilder();

            String newEncounterId = (encounter.getEncounterId() != null) ? encounter.getEncounterId() : "";

            //check for sensitive DoD adapter encounter
            if (guiEncounter.isSensitive()) {
                guiEncounter.setSensitive(true);
                guiEncounter.setClinic(guiEncounter.getClinic() + "  ** Sensitive **");
                guiEncounter.setReason(guiEncounter.getReason() + "  ** Sensitive **");
//                    guiEncounter.setDiagnosis(null);
//                    guiEncounter.setProcedures(null);
//                    guiEncounter.setEncounterNumber("");
//                    guiEncounter.setEncounterId("");
//                    guiEncounter.setComplexDataUrl("");
//                    guiEncounter.setReasonNarrative("");
//                    guiEncounter.setDischargeDisposition("");
//                    guiEncounter.setDocuments(null);
//                    guiEncounter.setReasonCode("");
//                    guiEncounter.setCodes(null);
            }

            //if rpc or dod adapter protocol then flag record for caching
            if ("DODADAPTER".equals(encounter.getSourceProtocol())) {
                guiEncounter.setSessionCachedRecord(true);
                encounterId.append("cache:");
                encounterId.append(docId);
            } else {
                encounterId.append(newEncounterId);
            }

            guiEncounter.setEncounterId(encounterId.toString());

            if (StringUtils.isNotBlank(guiEncounter.getComplexDataUrl())) {
                guiEncounter.setNoteAsBinaryStream(true);
            }
            guiEncounter.setApptDate(Utils.formatDisplayDateTime(encounter.getApptDate()));
            guiEncounter.setEventId(encounter.getCdrEventId());

            // Map VIX to Encounter GUI bean
            if (CollectionUtils.isNotEmpty(encounter.getDocuments())) {
                GUIVixStudy vixStudy;
                GUIEncounterDocument guiDocument;
                StudyDetail detail;
                int index = 0;

                for (EncounterDocument document : encounter.getDocuments()) {
                    if (document != null && document.getVixStudy() != null) {
                        detail = document.getVixStudy().getStudyDetail();
                        if (detail != null && detail.getStudies() != null && detail.getStudies().size() > 0) {
                            // Most likely only 1 study in the study detail
                            StudyDetailStudy detailStudy = detail.getStudies().get(0);
                            vixStudy = dozerBeanMapper.map(detailStudy, GUIVixStudy.class);
                            vixStudy.setViewerUrl(detail.getViewerUrl());

                            guiDocument = guiEncounter.getDocuments().get(index);
                            guiDocument.setVixStudy(vixStudy);
                        }
                    }

                    if (document != null && StringUtils.isNotEmpty(document.getImages())) {
                        try {
                            if (Integer.parseInt(document.getImages()) > 0) {
                                guiDocument = guiEncounter.getDocuments().get(index);
                                guiDocument.setHasImage(true);
                                guiDocument.setImages(document.getImages());
                                guiEncounter.setHasImage(true);
                            }
                        } catch (NumberFormatException nfe) {
                            logger.debug("Failed to convert document images count - " + document.getImages());
                        }
                    }

                    if (document != null && StringUtils.isNotEmpty(document.getContent())) {
                        guiDocument = guiEncounter.getDocuments().get(index);
                        guiDocument.setContent(StringEscapeUtils.escapeHtml4(document.getContent()));
                    }

                    index++;
                }
            }

//            return guiEncounter;
//        } catch (Exception e){
//            e.printStackTrace(System.out);
//        }
        return guiEncounter;
    }

    public String getEncounterVixStudyViewerUrl(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate, String documentId, String documentSiteId) {
        try {
            return getServiceHandler().getEncounterDocumentVixStudyViewerUrl(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build()
                    , documentId, documentSiteId);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

}
