/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.GUIClinicalReminder;
import gov.va.med.guibeans.GUIClinicalReminderDetail;
import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.ClinicalReminder;
import gov.va.med.jmeadows.webservice.FreeTextReport;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 *
 *
 */
@Component
public class ClinicalReminderDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(ClinicalReminderDao.class.getName());

    public List<GUIClinicalReminder> getPatientClinicalReminders(GUIUser user, GUIPatient patient, int activeVal) {
        try {
            ClinicalReminder[] reminders = getServiceHandler().getPatientClinicalReminders(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .active(activeVal)
                            .build());

            List<GUIClinicalReminder> guiReminders = new ArrayList<GUIClinicalReminder>();

            for (ClinicalReminder reminder : reminders) {
                GUIClinicalReminder guiReminder = dozerBeanMapper.map(reminder, GUIClinicalReminder.class);

                String dueDate = "";
                String clinicalReminderId = "";
                String lastDate = "";

                if (reminder.getDueDate() != null) {
                    dueDate = Utils.formatDisplayDate(reminder.getDueDate());
                }

                if (reminder.getId() != null) {
                    clinicalReminderId = reminder.getId();
                }

                if (reminder.getLastOccurrenceDate() != null) {
                    lastDate = Utils.formatDisplayDate(reminder.getLastOccurrenceDate());
                }

                guiReminder.setClinicalReminderId(clinicalReminderId);
                guiReminder.setDueDate(dueDate);
                guiReminder.setLastDate(lastDate);
                addSiteDataToGUIBean(guiReminder, reminder);

                guiReminders.add(guiReminder);
            }

            return guiReminders;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIClinicalReminderDetail getClinicalReminderDetails(GUIUser user, GUIPatient patient, String clinicalReminderId,
                                                                String recordSiteCode) {
        try {
            FreeTextReport report = getServiceHandler().getClinicalReminderDetail(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(clinicalReminderId)
                            .recordSiteCode(recordSiteCode)
                            .build());
            GUIClinicalReminderDetail guiReminderDetails = new GUIClinicalReminderDetail();
            if (report != null) {
                guiReminderDetails.setClinicalReminderText(StringEscapeUtils.escapeHtml4(report.getReportText()));
                addSiteDataToGUIBean(guiReminderDetails, report);
            }

            return guiReminderDetails;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }


    }
}
