package gov.va.med.dao;

import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.SiteStatusV4;
import gov.va.med.jmeadows.webservice.Vlerccda;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 5/19/14
 * Time: 1:42 PM
 * To change this template use File | Settings | File Templates.
 */
@Component
public class DASVLERDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(DASVLERDao.class.getName());

    /**
     * Fetches dod and stuffs it into a VLERPayload obj.
     *
     * @param user    GUI user
     * @param patient GUIPatient
     * @return VlerPayload
     */
    public Vlerccda[] getDASVLERPatientCCDA(GUIUser user, GUIPatient patient, boolean forceRefresh) {
        try {
            return getServiceHandler().getDASVLERPatientCCDA(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .forceRefresh(forceRefresh)
                            .build()
            );

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public String generateStatusReport(List<SiteStatusV4> siteStatusList, List<String> errorList) {
        StringBuilder report = new StringBuilder();

        Map<String, String> siteStatusMap = new LinkedHashMap<String, String>();

        report.append("<div class='statusWrap vlerStatus'><div class='title'>Community Partners Source Status List</div>");

        if (siteStatusList == null) {
            report.append("Source status list is unavailable.");
        } else {
            for (SiteStatusV4 siteStatus : siteStatusList) {
                if (siteStatus.getPartner() != null && siteStatus.getPartner().getPartnerName() != null) {
                    siteStatusMap.put(siteStatus.getPartner().getPartnerName(), siteStatus.getPartnerStatus());
                }
            }

            report.append("<table><thead><th scope='col' class='table-header'>Source Name</th><th scope='col' class='table-header'>Source Status</th></thead>");

            Set<String> keys = siteStatusMap.keySet();

            String[] keyArray = keys.toArray(new String[0]);
            Arrays.sort(keyArray);

            report.append("<tbody>");

            if (keyArray.length < 1) {
                report.append("<tr class='even'><td>Source status list is unavailable</td></tr>");
            } else {
                int cnt = 0;
                for (String siteName : keyArray) {
                    String statusStr = "";
                    String siteNameStr = "";

                    if (siteName != null) {
                        if (siteStatusMap.get(siteName) != null)
                            statusStr = siteStatusMap.get(siteName).toUpperCase();
                        siteNameStr = siteName.toUpperCase();
                    }
                    String trClass = "odd";

                    if (cnt % 2 == 0) trClass = "even";

                    report.append("<tr class='" + trClass + "'>");
                    report.append("<td>");
                    report.append(siteNameStr);
                    report.append("</td>");
                    report.append("<td>");
                    report.append(statusStr);
                    report.append("</td>");
                    report.append("</tr>");

                    cnt++;
                }
            }

            report.append("</tbody>");
            report.append("</table>");
        }

        report.append("<div class='title statusErrors'>Errors</div>");

        if (errorList == null) {
            report.append("Error report is unavailable.");
        } else {
            if (errorList.size() < 1) {
                report.append("No errors to report.");
            } else {
                report.append("<table><thead><th scope='col' colspan='2' class='table-header'>Error Code</th></thead>");

                int cnt = 0;
                for (String error : errorList) {
                    String trClass = "odd";

                    if (cnt % 2 == 0) trClass = "even";

                    report.append("<tr class='" + trClass + "'>");
                    report.append("<td align='center'><div style='margin:10px;background-color:red;width:15px;'></div></td>");
                    report.append("<td>");
                    report.append(error);
                    report.append("</td>");
                    report.append("</tr>");

                    cnt++;
                }

                report.append("</tbody>");
                report.append("</table>");
            }
        }
        report.append("</div>");
        return report.toString();
    }

}
