package gov.va.med.dao;


import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.EncounterDocument;
import gov.va.med.jmeadows.webservice.Procedure;
import gov.va.med.jmeadows.webservice.StudyDetail;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@Component
public class ProcedureDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(ProcedureDao.class.getName());

    /**
     * Retrieves a list of patient procedures from all available patient locations.
     *
     * @param user    {@link gov.va.med.guibeans.GUIUser} making the request.
     * @param patient {@link gov.va.med.guibeans.GUIPatient} patient retrieving procedures from this patient record.
     * @return A list of patient procedures.
     */
    public List<GUIProcedure> getPatientProcedures(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate, boolean includeVLER, boolean includeDASVLER) {
        try {
            Procedure[] procedures = getServiceHandler().getPatientProcedures(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .includeVLER(includeVLER)
                            .includeDASVLER(includeDASVLER)
                            .build());

            List<GUIProcedure> guiProcedures = new ArrayList<GUIProcedure>();

            GUIVixStudy vixStudy;
            GUIEncounterDocument guiDocument;
            StudyDetail detail;
            int index;

            for (Procedure procedure : procedures) {
                GUIProcedure guiProcedure = dozerBeanMapper.map(procedure, GUIProcedure.class);
                if (procedure.getDate() != null) {
                    String dateString = Utils.formatDisplayDate(procedure.getDate());
                    if (dateString.indexOf("1899") > -1) {
                        guiProcedure.setDate("");
                    } else {
                        guiProcedure.setDate(Utils.formatDisplayDate(procedure.getDate()));
                    }

                }

                // Map GUI VIXStudy to GUIProcedure Bean
                if (CollectionUtils.isNotEmpty(procedure.getDocuments())) {
                    index = 0;
                    for (EncounterDocument document : procedure.getDocuments()) {
                        if (document != null && document.getVixStudy() != null) {
                            detail = document.getVixStudy().getStudyDetail();
                            if (detail != null && detail.getStudies() != null && detail.getStudies().size() > 0) {
                                vixStudy = dozerBeanMapper.map(detail.getStudies().get(0), GUIVixStudy.class);
                                vixStudy.setViewerUrl(detail.getViewerUrl());

                                guiDocument = guiProcedure.getDocuments().get(index);
                                guiDocument.setVixStudy(vixStudy);
                            }
                        }

                        if (document != null && StringUtils.isNotEmpty(document.getImages())) {
                            try {
                                if (Integer.parseInt(document.getImages()) > 0) {
                                    guiDocument = guiProcedure.getDocuments().get(index);
                                    guiDocument.setHasImage(true);
                                    guiDocument.setImages(document.getImages());
                                    guiProcedure.setHasImage(true);
                                }
                            } catch (NumberFormatException nfe) {
                                logger.debug("Failed to convert document images count - " + document.getImages());
                            }
                        }

                        if (document != null && StringUtils.isNotEmpty(document.getContent())) {
                            guiDocument = guiProcedure.getDocuments().get(index);
                            guiDocument.setContent(StringEscapeUtils.escapeHtml4(document.getContent()));
                        }
                        
                        index++;
                    }
                }

                //if rpc or dod adapter protocol then flag record for caching
                if ("DODADAPTER".equals(procedure.getSourceProtocol())) {
                    guiProcedure.setSessionCachedRecord(true);
                }

                guiProcedures.add(guiProcedure);

            }

            return guiProcedures;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public String getProcedureVixStudyViewerUrl(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate, String documentId, String documentSiteId) {
        try {
            return getServiceHandler().getEncounterDocumentVixStudyViewerUrl(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build()
                    , documentId, documentSiteId);
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }
}
