
package gov.va.med.jlvqos;

import gov.va.med.common.AppConfig;
import gov.va.med.jlvqos.webservice.JLVQoSData;
import gov.va.med.jlvqos.webservice.JLVQoSDataService;
import gov.va.med.jmeadows.HeaderHandlerResolver;
import gov.va.med.jmeadows.webservice.JMeadowsQuery;
import gov.va.med.jmeadows.webservice.Patient;
import gov.va.med.jmeadows.webservice.Site;
import gov.va.med.jmeadows.webservice.User;
import gov.va.med.vhahon.common.Utils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.MTOMFeature;
import java.util.ArrayList;
import java.util.List;

@Component
public class JLVQoSDataServiceHandlerImpl implements JLVQoSDataServiceHandler {

    private static final Logger logger = Logger.getLogger(JLVQoSDataServiceHandlerImpl.class.getName());
    @Autowired
    protected AppConfig appConfig;
    private String serviceURL;
    private String m_ServiceName;
    private String m_namespace;
    private QName m_QserviceName;

    private String appName;

    private String username;
    private String password;

    private int connectionTimeoutMS;
    private int requestTimeoutMS;

    private JLVQoSDataService m_jlvqosDataService;

    public JLVQoSDataServiceHandlerImpl(String url) {
        serviceURL = url;
        init();
    }

    public JLVQoSDataServiceHandlerImpl() {
        init();
    }

    public void init() {
        m_ServiceName = "JLVQoSDataService";
        m_namespace = "http://webservice.jlvqos.med.va.gov/";
        m_QserviceName = new QName(m_namespace, m_ServiceName);

        connectionTimeoutMS = 30000;
        requestTimeoutMS = 30000;

    }

    public int getConnectionTimeoutMS() {
        return connectionTimeoutMS;
    }

    public void setConnectionTimeoutMS(int connectionTimeoutMS) {
        this.connectionTimeoutMS = connectionTimeoutMS;
    }

    public int getRequestTimeoutMS() {
        return requestTimeoutMS;
    }

    public void setRequestTimeoutMS(int requestTimeoutMS) {
        this.requestTimeoutMS = requestTimeoutMS;
    }

    public String getServiceURL() {
        return serviceURL;
    }

    public void setServiceURL(String serviceURL) {
        this.serviceURL = serviceURL;
    }

    private JLVQoSData getJLVQoSDataPort() {
        return getJLVQoSDataPort(false);
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    private JLVQoSData getJLVQoSDataPort(boolean includeMTOMFeature) {
        if (m_jlvqosDataService == null) {
            createService();
            if (appConfig.getEnableWSSecurityHeader()) {
                m_jlvqosDataService.setHandlerResolver(new HeaderHandlerResolver());
            }
        }

        JLVQoSData port = null;
        if (includeMTOMFeature) {
            port = m_jlvqosDataService.getJLVQoSDataPort(new MTOMFeature());
        } else port = m_jlvqosDataService.getJLVQoSDataPort();

        //set connection timeout
        BindingProvider bp = (BindingProvider) port;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        bp.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", connectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", requestTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.connect.timeout", connectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.request.timeout", requestTimeoutMS);

        return port;
    }

    private synchronized void createService() {
        if (m_jlvqosDataService == null) {
            try {
                m_jlvqosDataService = new JLVQoSDataService();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }

        }
    }

//    private void log(String methodName) {
//        log(methodName, null);
//    }

//    private void log(String methodName, JMeadowsQuery queryBean) {
//        if (logger.isDebugEnabled()) {
//            logger.debug("Invoking method: " + methodName);
//            if (queryBean != null) {
//                logQueryBeanParams(methodName, queryBean);
//            }
//        }
//    }

    private void logQueryBeanParams(String methodName, JMeadowsQuery queryBean) {
        if (!logger.isDebugEnabled()) return;

        String userId;
        Site hostSite;
        String patVAIcn;
        String patDoDEdipi;
        List<Site> vistaSites = new ArrayList<Site>();
        String itemId;
        String recordSiteCode;
        String status;
        Integer active;
        String startDate;
        String endDate;

        StringBuilder sb = new StringBuilder();
        sb.append(methodName);
        sb.append(" query bean params: ");

        if (queryBean != null) {
            User user = queryBean.getUser();
            Patient patient = queryBean.getPatient();

            if (user != null) {
                userId = user.getUserId();
                hostSite = user.getHostSite();

                sb.append("User - userId: ");
                sb.append(userId);
                sb.append(" , hostSite: ");
                if (hostSite != null) sb.append(hostSite.getSiteCode());
            } else {
                sb.append("User - null");
            }

            if (patient != null) {

                patVAIcn = patient.getICN();
                patDoDEdipi = patient.getEDIPI();

                sb.append("; Patient - ");
                sb.append(", ICN: ");
                sb.append(patVAIcn);
                sb.append(", EDIPI: ");
                sb.append(patDoDEdipi);

                sb.append(", vistaSites: ");

                if (vistaSites.size() < 1) sb.append("none");

                for (Site site : vistaSites) {
                    String siteCode = site.getSiteCode();
                    sb.append(siteCode);
                    sb.append(", ");
                }
            } else {
                sb.append("; Patient - null");
            }

            itemId = queryBean.getItemId();
            active = queryBean.getActive();
            recordSiteCode = queryBean.getRecordSiteCode();
            startDate = Utils.formatDisplayDate(queryBean.getStartDate());
            endDate = Utils.formatDisplayDate(queryBean.getEndDate());
            status = queryBean.getStatus();

            sb.append("; startDate: ");
            sb.append(startDate);
            sb.append(", endDate: ");
            sb.append(endDate);
            sb.append(", itemId: ");
            sb.append(itemId);
            sb.append(", recordSiteCode: ");
            sb.append(recordSiteCode);
            sb.append(", status: ");
            sb.append(status);
            sb.append(", active: ");
            sb.append(active);

        }

        logger.debug(sb.toString());
    }

    public String getAppName() {
        return this.appName;
    }

    public void setAppName(String appName) {
        this.appName = appName;
    }

    @Override
    public String verify() throws SOAPException {
        String retVal = null;
        try {
            JLVQoSData port = getJLVQoSDataPort();
            retVal = port.verify();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new SOAPException(e);
        }
        return retVal;
    }

    @Override
    public String getServiceErrors() throws SOAPException {
        String retVal = null;
        try {
            JLVQoSData port = getJLVQoSDataPort();
            retVal = port.getServiceErrors();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new SOAPException(e);
        }
        return retVal;
    }

}
