/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.bhie.*;
import gov.va.med.jmeadows.common.JMeadowsException;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.MTOMFeature;
import java.net.MalformedURLException;

/**
 * BHIEServiceHandler
 */
public class BHIERelayServiceHandler extends DataServiceHandler
{
    private BHIERelayService bhieRelayService;

    public BHIERelayServiceHandler(String url)
    {
        super();

        serviceURL = url;
        serviceName = "BHIERelayService";
        serviceNamespace = "http://webservice.bhie.med.va.gov/";
        qServiceName = new QName(serviceNamespace, serviceName);

        serviceUsername = APP_CONFIG.getBHIERelayServiceUsername();
        servicePassword = APP_CONFIG.getBHIERelayServicePassword();


    }

    private BHIERelay getBHIERelayPort() throws MalformedURLException
    {
        return getBHIERelayPort(false);
    }

    private BHIERelay getBHIERelayPort(boolean includeMTOMFeature) throws MalformedURLException
    {
        if (bhieRelayService == null)
        {
            createService();
        }

        BHIERelay port = null;

        if (includeMTOMFeature)
        {
            port = bhieRelayService.getBHIERelayPort(new MTOMFeature());
        }
        else port = bhieRelayService.getBHIERelayPort();

        //set port endpoint address
        BindingProvider bp = (BindingProvider)port;
        bp.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        bp.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, serviceUsername);
        bp.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, servicePassword);
        bp.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        bp.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        return port;
    }

    private synchronized void createService()
    {
        if (bhieRelayService == null)
        {
            try
            {
                bhieRelayService = new BHIERelayService();
            }
            catch(Exception e)
            {
                throw new RuntimeException(e);
            }
        }
    }

    // For BDA system check
    public V4ResultCollection getPatientProblemList(QueryBean queryBean) throws JMeadowsException
    {
        try
        {
            return getBHIERelayPort().getPatientProblemList(queryBean);
        }
        catch(Exception e)
        {
            throw new JMeadowsException(e);
        }
    }

    // For Snareworks system check
    public SnareUser  getSnareworksUser(String user, String credentials) throws JMeadowsException {
        try {
            return getBHIERelayPort().getSnareworksUser(null, user, credentials);
        }
        catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }
}
