package gov.va.med.monitor;

import gov.va.med.jmeadows.dataservicehandler.MVIServiceHandler;
import gov.va.med.jmeadows.enterprise.beans.ResponsePatientSelect;
import gov.va.med.jmeadows.webservice.beans.JMeadowsQuery;
import gov.va.med.vds.webservice.Patient;
import org.apache.commons.lang3.StringUtils;

public class MVIService extends ServiceBase
{
    private static final MVIService INSTANCE = new MVIService();
    private MVIServiceHandler serviceHandler;

    private MVIService()
    {
        super(ServiceType.MVIService);

        serviceHandler = new MVIServiceHandler(APP_CONFIG.getMVIURL());
    }

    public static MVIService getInstance()
    {
        return INSTANCE;
    }

    @Override
    public void run()
    {
        // Set up query bean
        JMeadowsQuery qb = new JMeadowsQuery();
        qb.setStatus("EDIPI");
        qb.setItemId(APP_CONFIG.getPatientEDIPIForMVI());

        // Retrieve VA identifiers for a patient with known VA identifiers
        ResponsePatientSelect response = serviceHandler.getPatientIds(qb);

        if (response.isSuccess())
        {
            Patient patient = response.getPatient();

            if (StringUtils.isBlank(patient.getICN()) && patient.getPatientIens().size() <= 0)
            {
                // Did not receive expected result
                status = ServiceStatus.ERROR;
                message.append(NO_RESULTS_MSG);
            }
            else
            {
                // Received expected result
                status = ServiceStatus.OK;
                message.append(SERVICE_OK_MSG);
            }
        }
        else
        {
            // Error was returned in response message
            status = ServiceStatus.ERROR;
            message.append(response.getErrorMsg());
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }
}
