package gov.va.med.monitor;

import gov.va.med.jmeadows.common.AppConfig;

public abstract class ServiceBase implements Runnable
{
    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();
    protected static final String LINE_SEPARATOR = System.getProperty("line.separator");
    protected static final String SERVICE_OK_MSG = "Running as expected.";
    protected static final String NO_RESULTS_MSG = "No results were returned.";
    protected ServiceType type;
    protected ServiceStatus status;
    protected StringBuilder message;
    private ServiceReport reportStr;
    private AbstractServiceMonitor serviceMonitorHandler;

    protected ServiceBase(ServiceType type, AbstractServiceMonitor... optionalServiceMonitorHandler)
    {
        this.type = type;

        status = ServiceStatus.NONE;
        message = new StringBuilder();

        serviceMonitorHandler = optionalServiceMonitorHandler.length > 0 ?
                optionalServiceMonitorHandler[0] : ServiceMonitor.getInstance();
    }

    protected void report(String type, String status, String message)
    {
        reportStr = new ServiceReport();

        reportStr.setType(type);
        reportStr.setStatus(status);
        reportStr.setMessage(message);

        if (serviceMonitorHandler != null) {
            serviceMonitorHandler.addReport(reportStr);
        }

        this.status = ServiceStatus.NONE;
        this.message = new StringBuilder();
    }

    public void init()
    {
        reportStr = null;
    }

    public ServiceReport getReport()
    {
        if(reportStr == null) {
            reportStr = new ServiceReport();

            reportStr.setType(type.toString());
            reportStr.setStatus(ServiceStatus.ERROR.toString());
            reportStr.setMessage("Failed to complete service test.");

            if (serviceMonitorHandler != null) {
                serviceMonitorHandler.addReport(reportStr);
            }
        }

        return reportStr;
    }

    public abstract void run();
}
