package gov.va.med.monitor;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dataservicehandler.ShareImagingServiceHandler;

public class ShareSiteService extends ServiceBase
{
    private ShareImagingServiceHandler serviceHandler;
    private String siteCode;
    private String siteName;

    public ShareSiteService(String host, String siteCode, String siteName, String encryptedUsername, String encryptedPassword)
    {
        super(ServiceType.ShareService);

        serviceHandler = new ShareImagingServiceHandler(host);
        serviceHandler.setServiceUsername(EncryptUtil.decrypt(encryptedUsername));
        serviceHandler.setServicePassword(EncryptUtil.decrypt(encryptedPassword));

        this.siteCode = siteCode;
        this.siteName = siteName;
    }

    @Override
    public void run()
    {
        try
        {
            // Pass fake access/verify code (because we don't have an official ac/vc pair to use) to invoke a
            // specific exception to indicate login failure. This way we know the service is running and working
            // as expected.
            serviceHandler.authenticateProvider("f@k3@cc355C0d3", "f@k3V3r!fyC0d3");
        }
        catch (JMeadowsException e)
        {
            String errorMessage = e.getMessage();

            // Checking for invalid access/verify code login to determine service is working as expected
            if (errorMessage.contains("msg=")                           // Old share service interface returns
                    || errorMessage.contains("AuthenticateProvider")    // New ShareImagingDataService interface returns
                    || errorMessage.contains("AuthenitcateProvider"))   // Misspelling in new interface returns
            {
                // The expected response for invalid login attempt.
                status = ServiceStatus.OK;
                message.append(SERVICE_OK_MSG);
            }
            else
            {
                // Not the response we were expecting for an invalid access/verify code login
                status = ServiceStatus.ERROR;
                message.append(errorMessage);
            }
        }

        report(type.toString() + ":" + siteCode + ":" + siteName, status.toString(), message.toString());
    }


    public String getSiteCode() {
        return siteCode;
    }

    public void setSiteCode(String siteCode) {
        this.siteCode = siteCode;
    }

    public String getSiteName() {
        return siteName;
    }

    public void setSiteName(String siteName) {
        this.siteName = siteName;
    }
}
