package gov.va.med.monitor;


import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dataservicehandler.VistaDataServiceHandler;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.Site;
import gov.va.med.vds.webservice.VistaStatus;

import java.util.*;

public class VistaStatusService extends ServiceBase {

    private static final VistaStatusService INSTANCE = new VistaStatusService();
    private VistaDataServiceHandler serviceHandler;

    private VistaStatusService()
    {
        super(ServiceType.VistaStatusService, VistaStatusServiceMonitor.getInstance());

        serviceHandler = new VistaDataServiceHandler(APP_CONFIG.getVistaWsURL());
    }

    public static VistaStatusService getInstance()
    {
        return INSTANCE;
    }

    @Override
    public void run()
    {
        try
        {

            List<VistaStatus> vistaStatusList = serviceHandler.getVistaStatusList(getVistaSites());
            Map<Site, String> badSites = new LinkedHashMap<Site, String>();
            for(VistaStatus vistaStatus : vistaStatusList) {
                if (!vistaStatus.isConnected()) {
                    badSites.put(vistaStatus.getSite(), vistaStatus.getErrorMsg());
                }
            }

            if (badSites.isEmpty()) {
                status = ServiceStatus.OK;
                message.append(SERVICE_OK_MSG);
            } else {

                status = ServiceStatus.ERROR;
                message.append("VistA Sites failing to connect:\n");

                Iterator<Map.Entry<Site, String>> iterator = badSites.entrySet().iterator();
                while(iterator.hasNext()) {
                    Map.Entry<Site, String> entry = iterator.next();
                    Site site = entry.getKey();
                    String errorMsg = entry.getValue();

                    message.append(String.format("SiteCode: %s, SiteName %s, Error Message: %s",
                            site.getSiteCode(), site.getName(), errorMsg));
                }
            }
        }
        catch (JMeadowsException e)
        {
            // An exception was caught
            status = ServiceStatus.ERROR;
            message.append(e.getMessage());
        }

        // Report status
        report(type.toString(), status.toString(), message.toString());
    }

    private List<Site> getVistaSites() {
        List<Site> allSites = Sites.getInstance().getSiteList();

        List<Site> vistaSites = new ArrayList<Site>();

        for(Site site : allSites) {
            if ("VA".equalsIgnoreCase(site.getAgency())) {
                vistaSites.add(site);
            }
        }

        return vistaSites;
    }
}
