/*
 * Janus 4.0 (c)
 * Copyright (c) 2016 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.rpc;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Dummy class that throws an exception for RPC connections that are not connected or initial socket timed out.
 */
public class ConnectionUnavailable {

    private static final Logger LOGGER = LoggerFactory.getLogger(ConnectionUnavailable.class);

    private RPCConnection cxn;

    public ConnectionUnavailable(RPCConnection cxn) {
        this.cxn = cxn;
    }

    public void throwException() throws RPCException {

        String errorMsg = "Connection Unavailable - ";

        if (cxn != null) {

            if (cxn.getSite() != null) {
                errorMsg += "SiteCode: " + cxn.getSite().getSiteCode() + ", SiteName: " + cxn.getSite().getName() + ", ";
            }

            if (cxn.getErrorMsg() != null) {
                errorMsg += "Error Message: " + cxn.getErrorMsg();
            }
        }

        throw new RPCException(errorMsg);
    }
}
