/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.rpc;


/**
 *
 *
 */
public class UtilityRPCS {
    public static String getWelcomeMessage(VistaQueryFactory vistaQueryFactory, RPCConnection cxn) throws RPCException {
        try {
            String request = buildGetWelcomeMessageRequest(vistaQueryFactory);
            return (String) cxn.query(request);
        } catch (Exception e) {
            throw new RPCException(e);
        }
    }

    private static String buildGetWelcomeMessageRequest(VistaQueryFactory vistaQueryFactory) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("XUS INTRO MSG");
        return vquery.buildMessage();
    }

    public static String getOptionIen(VistaQueryFactory vistaQueryFactory, RPCConnection cxn, String context) throws RPCException {
        try {
            String arg = "$O(^DIC(19,\"B\",\"" + context + "\",0))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            return RPCUtils.errMsgOrIen(response);
        } catch (Exception e) {
            throw new RPCException(e);
        }
    }

    public static boolean hasSecurityLock(VistaQueryFactory vistaQueryFactory, RPCConnection cxn, String contextIen) throws RPCException {
        String arg = "$P(^DIC(19," + contextIen + ",0),\"^\",6)";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return !"".equals(response);
    }

    public static String getSecurityKey(VistaQueryFactory vistaQueryFactory, RPCConnection cxn, String context) throws RPCException {
        String contextIen = getOptionIen(vistaQueryFactory, cxn, context);
        String arg = "$G(^DIC(19," + contextIen + ",0))";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return extractSecurityKey(response);
    }

    private static String extractSecurityKey(String response) {
        return StringUtils.piece(response, "^", 6);
    }

    public static String getSecurityKeyIen(VistaQueryFactory vistaQueryFactory, RPCConnection cxn, String securityKey) throws RPCException {
        try {
            String arg = "$O(^DIC(19.1,\"B\",\"" + securityKey + "\",0))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            return RPCUtils.errMsgOrIen(response);
        } catch (Exception e) {
            throw new RPCException(e);
        }
    }
}
