package gov.va.med.vds.das.beans;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import java.util.Date;


@JsonIgnoreProperties(ignoreUnknown = true)
public class Immunization extends Resource{
    @JsonProperty("status")
    private String status;
    @JsonProperty("vaccineCode")
    private CodedObject vaccineCode;
    @JsonDeserialize(using = DateDeserializer.class)
    @JsonProperty("date")
    private Date date;
    @JsonProperty("route")
    private CodeObj route;
    @JsonProperty("doseQuantity")
    private DoseQuantity doseQuantity;
    @JsonProperty("vaccinationProtocol")
    private VaccinationProtocol vaccinationProtocol;
    @JsonProperty("explanation")
    private Explanation explanation;
    @JsonProperty("note")
    private NoteObj note;
    @JsonProperty("lotNumber")
    private String lotNumber;
    @JsonProperty("manufacturer")
    private String manufacturer;
    @JsonProperty("reaction")
    private Reaction reaction;

    @JsonIgnoreProperties(ignoreUnknown = true)
    public class DoseQuantity {
        @JsonProperty("value")
        private double value;
        @JsonProperty("unit")
        private String unit;

        public double getValue() {
            return value;
        }

        public void setValue(double value) {
            this.value = value;
        }

        public String getUnit() {
            return unit;
        }

        public void setUnit(String unit) {
            this.unit = unit;
        }
    }

    @JsonIgnoreProperties(ignoreUnknown = true)
    public class VaccinationProtocol {
        @JsonProperty("series")
        private String series;

        public String getSeries() {
            return series;
        }

        public void setSeries(String series) {
            this.series = series;
        }
    }

    @JsonIgnoreProperties(ignoreUnknown = true)
    public class Explanation {
        @JsonProperty("reasonNotGiven")
        private String reasonNotGiven;

        public String getReasonNotGiven() {
            return reasonNotGiven;
        }

        public void setReasonNotGiven(String reasonNotGiven) {
            this.reasonNotGiven = reasonNotGiven;
        }
    }

    @JsonIgnoreProperties(ignoreUnknown = true)
    public class NoteObj {
        @JsonProperty("text")
        private String text;

        public String getText() {
            return text;
        }

        public void setText(String text) {
            this.text = text;
        }
    }

    @JsonIgnoreProperties(ignoreUnknown = true)
    public class Reaction {
        @JsonProperty("detail")
        private Detail detail;

        @JsonIgnoreProperties(ignoreUnknown = true)
        public class Detail{
            @JsonProperty("reference")
            private String reference;

            public String getReference() {
                return reference;
            }

            public void setReference(String reference) {
                this.reference = reference;
            }
        }

        public Detail getDetail() {
            return detail;
        }

        public void setDetail(Detail detail) {
            this.detail = detail;
        }
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public CodedObject getVaccineCode() {
        return vaccineCode;
    }

    public void setVaccineCode(CodedObject vaccineCode) {
        this.vaccineCode = vaccineCode;
    }

    public Date getDate() {
        if(date == null){
            return null;
        } else {
            return new Date(date.getTime());
        }
    }

    public void setDate(Date date) {
        if(date == null){
            this.date = null;
        } else {
            this.date = new Date(date.getTime());
        }
    }

    public CodeObj getRoute() {
        return route;
    }

    public void setRoute(CodeObj route) {
        this.route = route;
    }

    public DoseQuantity getDoseQuantity() {
        return doseQuantity;
    }

    public void setDoseQuantity(DoseQuantity doseQuantity) {
        this.doseQuantity = doseQuantity;
    }

    public VaccinationProtocol getVaccinationProtocol() {
        return vaccinationProtocol;
    }

    public void setVaccinationProtocol(VaccinationProtocol vaccinationProtocol) {
        this.vaccinationProtocol = vaccinationProtocol;
    }

    public Explanation getExplanation() {
        return explanation;
    }

    public void setExplanation(Explanation explanation) {
        this.explanation = explanation;
    }

    public NoteObj getNote() {
        return note;
    }

    public void setNote(NoteObj note) {
        this.note = note;
    }

    public String getLotNumber() {
        return lotNumber;
    }

    public void setLotNumber(String lotNumber) {
        this.lotNumber = lotNumber;
    }

    public String getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer(String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public Reaction getReaction() {
        return reaction;
    }

    public void setReaction(Reaction reaction) {
        this.reaction = reaction;
    }
}
