package gov.va.med.vds.das.beans;

import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import java.util.Date;
import java.util.List;


@JsonIgnoreProperties(ignoreUnknown = true)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.PROPERTY, property = "resourceType", visible = true, defaultImpl = Resource.class)
@JsonSubTypes({
        @JsonSubTypes.Type(value = AllergyIntolerance.class, name = "AllergyIntolerance"),
        @JsonSubTypes.Type(value = Observation.class, name = "Observation"),
        @JsonSubTypes.Type(value = Condition.class, name = "Condition"),
        @JsonSubTypes.Type(value = Immunization.class, name = "Immunization"),
        @JsonSubTypes.Type(value = Procedure.class, name = "Procedure"),
        @JsonSubTypes.Type(value = Patient.class, name = "Patient")
})
public class Resource {
    @JsonProperty("id")
    private String id;
    @JsonProperty("resourceType")
    private String resourceType;
    @JsonProperty("masterIdentifier")
    private SystemValue masterIdentifier;
    @JsonProperty("identifier")
    private List<SystemValue> identifier;
    @JsonProperty("total")
    private int total;
    @JsonProperty("entry")
    private List<Resource> entry;
    @JsonProperty("resource")
    private Resource resource;
    @JsonDeserialize(using = DateDeserializer.class)
    @JsonProperty("created")
    private Date created;
    @JsonDeserialize(using = DateDeserializer.class)
    @JsonProperty("recordedDate")
    private Date recordedDate;
    @JsonProperty("description")
    private String description;
    @JsonProperty("contained")
    private List<Entity> contained;
    @JsonProperty("author")
    private List<Author> author;
    @JsonProperty("text")
    private NarrativeObj text;
    @JsonProperty("code")
    private String code;
    @JsonProperty("contentType")
    private String contentType;
    @JsonProperty("content")
    private List<Content> content;

    public Resource() {
    }


    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getResourceType() {
        return resourceType;
    }

    public void setResourceType(String resourceType) {
        this.resourceType = resourceType;
    }

    public int getTotal() {
        return total;
    }

    public void setTotal(int total) {
        this.total = total;
    }

    public List<Resource> getEntry() {
        return entry;
    }

    public void setEntry(List<Resource> entry) {
        this.entry = entry;
    }

    public Resource getResource() {
        return resource;
    }

    public void setResource(Resource resource) {
        this.resource = resource;
    }

    public SystemValue getMasterIdentifier() {
        return masterIdentifier;
    }

    public void setMasterIdentifier(SystemValue masterIdentifier) {
        this.masterIdentifier = masterIdentifier;
    }

    public List<SystemValue> getIdentifier() {
        return identifier;
    }

    public void setIdentifier(List<SystemValue> identifier) {
        this.identifier = identifier;
    }

    public Date getCreated() {
        if(created == null){
            return null;
        } else {
            return new Date(created.getTime());
        }
    }

    public Date getRecordedDate() {
        if(recordedDate == null) {
            return null;
        } else {
           return new Date(recordedDate.getTime());
        }
    }

    public void setRecordedDate(Date recordedDate) {
        if(recordedDate == null){
            this.recordedDate = null;
        } else {
            this.recordedDate = new Date(recordedDate.getTime());
        }
    }

    public void setCreated(Date created) {
        if(created == null){
            this.created = null;
        } else {
            this.created = new Date(created.getTime());
        }
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public List<Entity> getContained() {
        return contained;
    }

    public void setContained(List<Entity> contained) {
        this.contained = contained;
    }

    public List<Author> getAuthor() {
        return author;
    }

    public void setAuthor(List<Author> author) {
        this.author = author;
    }

    public NarrativeObj getText() {
        return text;
    }

    public void setText(NarrativeObj text) {
        this.text = text;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getContentType() {
        return contentType;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public List<Content> getContent() {
        return content;
    }

    public void setContent(List<Content> content) {
        this.content = content;
    }
}
