package gov.va.med.vds.dataservicehandler;

import gov.va.med.nhin.adapter.announcepatient.AnnouncePatientPortType;
import gov.va.med.nhin.adapter.announcepatient.AnnouncePatientResponseType;
import gov.va.med.nhin.adapter.announcepatient.AnnouncePatientService;
import gov.va.med.nhin.adapter.announcepatient.AnnouncePatientType;
import gov.va.med.nhin.adapter.permission.UserInfoType;
import gov.va.med.vds.common.NHINHandlerException;
import gov.va.med.vds.vistabeans.QueryBean;

import java.net.MalformedURLException;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

/**
 * Created with IntelliJ IDEA.
 * User: melcolm
 * Date: 5/21/14
 * Time: 1:57 PM
 * To change this template use File | Settings | File Templates.
 */
public class AdapterPatientAnnounceHandler extends AdapterHandler {

//    private static final Logger LOGGER = LoggerFactory.getLogger(AdapterPatientAnnounceHandler.class);
    private AnnouncePatientService patAnnounceService;

    public AdapterPatientAnnounceHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "AnnouncePatientService";
        serviceNamespace = "http://announcepatientservice.nhinc.fha.hhs.gov";
        qServiceName = new QName(serviceNamespace, serviceName);

    }


    private AnnouncePatientPortType getAnnouncePatientPort() throws MalformedURLException
    {
        if (patAnnounceService == null) {
            createService();
        }

        AnnouncePatientPortType port = patAnnounceService.getAnnouncePatientPort();

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);

        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        // SOAP message logging - Enable to show logging in output window
        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()))
        {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    private void createService() {
        if (patAnnounceService == null) {
            synchronized (this) {
                try {
                    patAnnounceService = new AnnouncePatientService();

                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    public AnnouncePatientResponseType patientDiscovery(QueryBean qbean) throws NHINHandlerException
    {
        AnnouncePatientResponseType output = null;

        try {

            AnnouncePatientType apt = new AnnouncePatientType();
            apt.setPatientId(qbean.getPatient().getICN() + "^^^&2.16.840.1.113883.4.349.1&ISO");

            AnnouncePatientType.Facilities facilities = new AnnouncePatientType.Facilities();
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.190.6.100.1");
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.42.10001.100001.12");
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.715");
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.709.0");
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.488.1");
//            facilities.getFacility().add("urn:oid:1.3.6.1.4.1.12009.6.1");
//            facilities.getFacility().add("urn:oid:1.3.6.1.4.1.31523");
//            facilities.getFacility().add("urn:oid:2.16.840.1.113883.3.456");
            facilities.getFacility().add("2.16.840.1.113883.3.190.6.100.1");
            facilities.getFacility().add("2.16.840.1.113883.3.42.10001.100001.12");
            facilities.getFacility().add("2.16.840.1.113883.3.715");
            facilities.getFacility().add("2.16.840.1.113883.3.709.0");
            facilities.getFacility().add("2.16.840.1.113883.3.488.1");
            facilities.getFacility().add("1.3.6.1.4.1.12009.6.1");
            facilities.getFacility().add("1.3.6.1.4.1.31523");
            facilities.getFacility().add("2.16.840.1.113883.3.456");
            apt.setFacilities(facilities);

            UserInfoType uit = new UserInfoType();
            uit.setRole("MEDICAL DOCTOR");
            uit.setFullName("TESTING DOCTOR");
            uit.setUserId("123456789");
            //uit.setAuthenticationMethod("");
            apt.setUserInfo(uit);

            JAXBContext jaxbContext = JAXBContext.newInstance(AnnouncePatientType.class.getPackage().getName());
            Marshaller jaxbMarshaller = jaxbContext.createMarshaller();

            // for getting nice formatted output
            jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            jaxbMarshaller.marshal(new JAXBElement<AnnouncePatientType>(new QName("uri", "AnnouncePatientType"), AnnouncePatientType.class, apt), System.out);
//
            output = getAnnouncePatientPort().announcePatient(apt);
        } catch (Exception e) {
            throw new NHINHandlerException(e);
        }
        return output;
    }
}
