package gov.va.med.vds.dataservicehandler;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.common.NHINHandlerException;
import gov.va.med.vds.common.TokenUtils;
import gov.va.med.vds.das.beans.*;
import gov.va.med.vds.das.beans.Address;
import gov.va.med.vds.das.beans.Patient;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vds.vistabeans.Author;
import gov.va.med.vds.vistabeans.Immunization;
import gov.va.med.vds.vistabeans.Procedure;
import gov.va.med.vds.vistabeans.das.DASResultCollection;
import gov.va.med.vds.vistabeans.das.Narrative;
import gov.va.med.vds.vistabeans.das.SiteStatus;
import gov.va.med.vds.webservice.DASLimit;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.text.StrSubstitutor;
import org.apache.log4j.Logger;
import org.hl7.v3.Charset;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import javax.net.ssl.*;
import javax.ws.rs.core.MultivaluedMap;
import java.io.IOException;
import java.io.InputStream;
import java.net.*;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: melcolm
 * Date: 12/28/2016
 * Time: 12:41 PM
 * To change this template use File | Settings | File Templates.
 */
public class DASServiceHandler {
    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();
    protected static final TokenUtils tokenUtils = TokenUtils.getInstance();

    private static final Logger logger =
            Logger.getLogger(DASServiceHandler.class.getName());

    private static final String baseURL = APP_CONFIG.getVLER_DAS_URL();
    private static final String callbackURL = APP_CONFIG.getVLER_DAS_CALLBACK();

    private static final int CONNECTION_TIMEOUT_MS = AppConfig.getInstance().getRestConnectionTimeoutMS();
    private static final int READ_TIMEOUT_MS = AppConfig.getInstance().getRestReadTimeoutMS();
    private static final int REST_CONNECTION_TIMEOUT_MS = AppConfig.getInstance().getRestConnectionTimeoutMS();

    private static final String HOST_FQDN = getHostFQDN();
    private static final String SUBSCRIBE = "subscribe";

    //Resource Types
    public static final String DAS_DOCUMENTREFERENCE = "DocumentReference";
    public static final String DAS_DOCUMENTMANIFEST = "DocumentManifest";
    public static final String DAS_BASIC = "BASIC";
    public static final String DAS_ALLERGIES = "AllergyIntolerance";
    public static final String DAS_PROBLEMS = "Condition";
    public static final String DAS_VITALS = "Observation";
    public static final String DAS_DEMOGRAPHICS = "Patient";
    public static final String DAS_PROCEDURES = "Procedure";
    public static final String DAS_IMMUNIZATIONS = "Immunization";
    public static final String SUBSCRIPTION_ACTIVE = "Active";

    private static final DASServiceHandler INSTANCE = new DASServiceHandler();

    private ObjectMapper objectMapper;
    private Client restClient;

    final static byte[] encryptionKey = APP_CONFIG.getEncryptionKey();

    static {
        disableSslVerification();
    }

    private static String getHostFQDN() {
        String retVal = "localhost";

        try {
            Enumeration<NetworkInterface> interfaces = NetworkInterface.getNetworkInterfaces();

            interfaceLoop:
            while (interfaces.hasMoreElements()) {
                NetworkInterface ifc = interfaces.nextElement();
                if (ifc.isUp()) {
                    for (InterfaceAddress addr : ifc.getInterfaceAddresses()) {
                        if (!"127.0.0.1".equals(addr.getAddress().getCanonicalHostName())) {
                            logger.info("## DAS CallBack Network Interface HostName ##: " + addr.getAddress().getCanonicalHostName());
                            retVal = addr.getAddress().getCanonicalHostName();
                            break interfaceLoop;
                        }
                    }
                }

            }
        } catch (SocketException se) {
            logger.error("Could not retrieve HostName: " + se.getLocalizedMessage());
        } catch (Exception e) {
            logger.error("Could not retrieve HostName: " + e.getLocalizedMessage());
        }

        return retVal;
    }

    private static void disableSslVerification() {
        try {
            // Create a trust manager that does not validate certificate chains
            TrustManager[] trustAllCerts = new TrustManager[]{new X509TrustManager() {
                public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                    return null;
                }

                public void checkClientTrusted(X509Certificate[] certs, String authType) {
                }

                public void checkServerTrusted(X509Certificate[] certs, String authType) {
                }
            }
            };

            // Install the all-trusting trust manager
            SSLContext sc = SSLContext.getInstance("SSL");
            sc.init(null, trustAllCerts, new java.security.SecureRandom());
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());

            // Create all-trusting host name verifier
            HostnameVerifier allHostsValid = new HostnameVerifier() {
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            };

            // Install the all-trusting host verifier
            HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (KeyManagementException e) {
            e.printStackTrace();
        }
    }

    private byte[] getEncryptionKey() {
        return Arrays.copyOf(encryptionKey, encryptionKey.length);
    }

    public DASServiceHandler() {
        objectMapper = new ObjectMapper();

        //allow unknown data fields
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

        this.restClient = initRestClient(CONNECTION_TIMEOUT_MS, READ_TIMEOUT_MS);
    }

    public static DASServiceHandler getInstance() {
        return INSTANCE;
    }

    /**
     * Initializes REST client.
     *
     * @param connectionTimeout REST client connection timeout
     * @param readTimeout       REST client read timeout
     * @return Jersey REST Client
     */
    private Client initRestClient(Integer connectionTimeout, Integer readTimeout) {
        Client client = Client.create();
        client.setConnectTimeout(connectionTimeout);
        client.setReadTimeout(readTimeout);

        return client;
    }

    public DASResultCollection getVLERAllergies(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_ALLERGIES);
    }

    public DASResultCollection getVLERProblems(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_PROBLEMS);
    }

    public DASResultCollection getVLERVitals(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_VITALS);
    }

    public DASResultCollection getVLERProcedures(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_PROCEDURES);
    }

    public DASResultCollection getVLERImmunizations(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_IMMUNIZATIONS);
    }

    public DASResultCollection getVLERDemographics(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_DEMOGRAPHICS);
    }

    public DASResultCollection getVLERDocuments(QueryBean qBean) throws VDSException {
        return getVLER(qBean, DAS_DOCUMENTREFERENCE);
    }

    public DASResultCollection getVLER(QueryBean qBean, String domain) throws VDSException {
        DASResultCollection resultCollection = new DASResultCollection();

        if (StringUtils.isNotEmpty(APP_CONFIG.getDasTestPat())) {
            gov.va.med.vds.vistabeans.Patient p = qBean.getPatient();
            p.setICN(APP_CONFIG.getDasTestPat());
            qBean.setPatient(p);
        }

        if (APP_CONFIG.isDASCacheEnabled()) {

            logger.debug("DAS - Attempting to retrieve cached subscription");
            Subscription cachedSubscription = getCachedSubscription(qBean, domain);

            //first check to see if force refresh, if not let's check to see if there's an existing cache via subscriptions

            if (cachedSubscription != null && cachedSubscription.getResultCollection() != null ) {
                if (!qBean.isForceRefresh()) {
                    if(CollectionUtils.isNotEmpty(cachedSubscription.getResultCollection().getStatusList())){
                        logger.debug("Returning cached subscription.  Statuses detected.");
                        return cachedSubscription.getResultCollection();
                    }

                    else {
                        logger.debug("DAS - no valid data. continue to query. ");
//                        cachedSubscription.setResultCollection(null);
//                        cachedSubscription.setDocuments(null);
//                        cachedSubscription.setStatus("init");
//                        setCachedSubscription(qBean, domain, cachedSubscription);
                    }

                } else {
                    logger.debug("DAS - Clearing cached subscription");
                    cachedSubscription.setResultCollection(null);
                    cachedSubscription.setDocuments(null);
                    cachedSubscription.setStatus("init");
                    setCachedSubscription(qBean, domain, cachedSubscription);
                }
            }


        }

        //data doesn't exist in cache or forceRefresh is true...let's do a fresh subscription
        //subscribe to get data
        String jsonRequest = generateSubscription(qBean, domain);
        WebResource resource = restClient.resource(APP_CONFIG.getVLER_DAS_URL() + "subscribe");

        logger.debug("DAS Query: " + APP_CONFIG.getVLER_DAS_URL() + "subscribe");
        logger.debug("DAS - Valid JSON?: " + isJSONValid(jsonRequest));
        logger.debug("DAS - JSON Payload: \n" + jsonRequest);

        ClientResponse response = resource.accept("application/json").type("application/json").post(ClientResponse.class, jsonRequest);

        // check response status code
        if (!(response.getStatus() == 200 || response.getStatus() == 201)) {
            throw new VDSException("DAS - Failed : HTTP error code : "
                    + response.getStatus() + "\n" + response.getEntity(String.class));
        }

        // Map the response JSON to check status of subscription
        SubscriptionResponse subResponse = null;
        InputStream responseEntityInputStream = null;
        try {
            responseEntityInputStream = response.getEntityInputStream();
            subResponse = objectMapper.readValue(responseEntityInputStream, SubscriptionResponse.class);
        } catch (JsonParseException e) {
            logger.error("DAS - Failed to parse JSON", e);
            throw new VDSException(e);
        } catch (JsonMappingException e) {
            logger.error("DAS - Failed to map JSON", e);
            throw new VDSException(e);
        } catch (IOException e) {
            logger.error("DAS - Failed getting JSON data", e);
            throw new VDSException(e);
        } finally {
            if (responseEntityInputStream != null) {
                try {
                    responseEntityInputStream.close();
                } catch (IOException ioe) {
                    logger.error(ioe.getMessage());
                }
            }
        }

        if (subResponse != null && "success".equalsIgnoreCase(subResponse.getStatus())) {
            //subscription was success.  now start checking the subscription cache to see if data is available;
            //generate key for map ("icn|userId|domain")
            StringBuilder key = new StringBuilder();
            key.append(qBean.getPatient().getICN());
            key.append("|");
            key.append(qBean.getUser().getUserId());
            key.append("|");
            key.append(domain);

            boolean isQueryTimeout = false;
            boolean queryComplete = false;
            long cnt = 0;

            while (!queryComplete && !isQueryTimeout) {

                Subscription subscription = DASLimit.getInstance().getSubscription(key.toString());

                if (subscription == null) {
                    logger.error("DAS Query subscription not found.");
                    throw new VDSException("DAS Query subscription not found.");
                }
//                if(subscription.getStatus().equalsIgnoreCase(SUBSCRIPTION_ACTIVE)){
                if (!subscription.getStatus().equalsIgnoreCase("init")) {
                    queryComplete = true;

                    //retrieve data
                    resultCollection = getDASData(subscription);

                    //update subscription cache with this data
                    subscription.setCreationTime(new Date().getTime());
                    subscription.setResultCollection(resultCollection);
                    DASLimit.getInstance().putSubscription(key.toString(), subscription);
                }
                if (!queryComplete) {
                    logger.info("DAS Query not complete, sleeping: " + (200L + (cnt * 500L)));
                    try {
                        Thread.sleep(2000L + (cnt++ * 500L));
                    } catch (InterruptedException ex) {
                        logger.error("DAS QUERY EXCEPTION: " + ex.getLocalizedMessage(), ex);
                    }
                }
                //check for query timeout
                if ((System.currentTimeMillis() - subscription.getCreationTime()) > REST_CONNECTION_TIMEOUT_MS) {
                    isQueryTimeout = true;
                }
            }


        } else {
            logger.error("Query subscription failed");
            throw new VDSException("Query subscription failed");
        }

        return resultCollection;
    }

    public DASResultCollection getDASData(Subscription subscription) throws VDSException {
        DASResultCollection resultCollection = null;
        //subscribe to get data
        WebResource resource = restClient.resource(APP_CONFIG.getVLER_DAS_URL() + "readDocument/" + subscription.getDomain());

//        System.out.println("DAS Query: " + APP_CONFIG.getVLER_DAS_URL() + "readDocument/" + subscription.getDomain());

        MultivaluedMap queryParams = new MultivaluedMapImpl();
        //?Subject.Patient.identifier=106&_format=application/json%2Bfhir
        queryParams.add("Subject.Patient.identifier", subscription.getPatientICN());
        queryParams.add("_format", "application/json%2Bfhir");
        ClientResponse response = resource.queryParams(queryParams).accept("application/json").get(ClientResponse.class);

        // check response status code
        if (response != null && response.getStatus() != 200) {
            throw new RuntimeException("Failed : HTTP error code : "
                    + response.getStatus());
        }

        // Map the response JSON
        Resource dasResource;
        try {
            String json = "";
            if (response != null) {
                json = response.getEntity(String.class);
            }
            logger.debug("DAS Response to Read " + subscription.getDomain() + ": " + response.getStatus() + " :: " + json);
            dasResource = objectMapper.readValue(json, Resource.class);

        } catch (JsonParseException e) {
            logger.error("Failed to parse JSON", e);
            throw new VDSException(e);
        } catch (JsonMappingException e) {
            logger.error("Failed to map JSON", e);
            throw new VDSException(e);
        } catch (IOException e) {
            logger.error("Failed getting JSON data", e);
            throw new VDSException(e);
        }

        if (dasResource != null) {
//            switch (subscription.getDomain()) {
//                case DAS_ALLERGIES:
//                    resultCollection = mapVLERAllergies(dasResource, subscription);
//                    break;
//                case DAS_PROBLEMS:
//                    resultCollection = mapVLERProblems(dasResource, subscription);
//                    break;
//                case DAS_VITALS:
//                    resultCollection = mapVLERVitals(dasResource, subscription);
//                    break;
//                default:
//                    break;
//            }
            if (subscription.getDomain().equalsIgnoreCase(DAS_ALLERGIES)) {
                resultCollection = mapVLERAllergies(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_PROBLEMS)) {
                resultCollection = mapVLERProblems(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_VITALS)) {
                resultCollection = mapVLERVitals(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_IMMUNIZATIONS)) {
                resultCollection = mapVLERImmunizations(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_PROCEDURES)) {
                resultCollection = mapVLERProcedures(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_DEMOGRAPHICS)) {
                resultCollection = mapVLERDemographics(dasResource, subscription);
            } else if (subscription.getDomain().equalsIgnoreCase(DAS_DOCUMENTREFERENCE)) {
                resultCollection = mapVLERDocuments(dasResource, subscription);
            }
        }

        return resultCollection;
    }

//    public List<VLERDocument> documentQuery(QueryBean qbean) throws VDSException {
//        List<VLERDocument> output = new ArrayList<VLERDocument>();
//        try {
//
//
//        } catch (JsonParseException e) {
//            String name = "There was an error querying for documents";
//            String source = "Connection Unavailable";
//            String details = "The query was unsuccessful";
//            logger.error("VLER Document Query Error" + " :: " + e.getMessage());
//
//            output.add(generateErrorDoc(name, source, details));
//        } catch (RuntimeException e) {
//            if (e.getCause() instanceof ConnectException) {
//                output.add(generateErrorDoc("Connectivity Error", "Connection Unavailable", e.getLocalizedMessage()));
//                logger.error("VLER Document Query Error" + " :: " + e.getMessage());
////                return output;
//            } else {
//                throw new VDSException(e);
//            }
//        } catch (Exception e) {
//            logger.error("Error performing VLER Document Query: " + e.getMessage());
//            throw new VDSException(e);
//        }
//
//        return output;
//    }

    private VLERDocument generateErrorDoc(String name, String source, String detail) {
        VLERDocument vDoc = new VLERDocument();
        vDoc.setName("Error: " + name);
        vDoc.setHomeCommunityId(detail);
        Author author = new Author();
        author.setInstitution(source);
        List<Author> authorList = new ArrayList<Author>();
        authorList.add(author);
        vDoc.setAuthorList(authorList);

        return vDoc;
    }

    public boolean isJSONValid(String test) {
        try {
            new JSONObject(test);
        } catch (JSONException ex) {
            // edited, to include @Arthur's comment
            // e.g. in case JSONArray is valid as well...
            try {
                new JSONArray(test);
            } catch (JSONException ex1) {
                return false;
            }
        }
        return true;
    }

    public String generateSubscription(QueryBean qb, String domain) {

        String patId = "";
        String userNameGiven = "";
        String userNameFamily = "";
        String siteCode = "";

        if (qb.getUser() != null && qb.getUser().getHostSite() != null) {
            siteCode = qb.getUser().getHostSite().getSiteCode();
//            System.out.println("####  ####  siteCode: " + siteCode);
        }
        //qb.getUser().getHostSite().getSiteCode();

        if (StringUtils.isNotEmpty(qb.getUser().getName())) {
            PersonName pn = new PersonName(qb.getUser().getName());

            userNameFamily = pn.getFamily();
            userNameGiven = pn.getGiven();
        }

        if (qb.getPatient() != null) {
            patId = qb.getPatient().getICN();
        }

        //workaround for DAS interface not accepting DocumentReference as subscription
        String domainParam = domain;
        if(DAS_DOCUMENTREFERENCE.equals(domain)){
            domainParam = DAS_ALLERGIES;
        }

        String requestTemplate = "{\"id\":\"${requestID}\",\"resourceType\":\"Subscription\"," +
                "\"criteria\":\"${domain}?subject:Patient.identifier=${patID}&startDate=${startDate}&endDate=${endDate}&_format=application/json\u002Bfhir${forceRefresh}\"," +
                "\"reason\":\"Subscription?Org=JLV&RoleCode=112247003&FamilyName=${userNameFamily:-}&GivenName=${userNameGiven:-USER}&VaFacilityCode=${siteCode:-}&PurposeCode=TREATMENT\",\"text\":{" +
                "\"div\":\"<div>Null</div>\"},\"channel\":{\"type\":\"rest-hook\",\"endpoint\":\"${callbackURL}\",\"payload\":\"application/json\u002Bfhir\"},\"status\":\"Active\"}";

        Map<String, String> params = new HashMap<String, String>();

        String callbackToken = tokenUtils.generateCallbackToken(encryptionKey, qb.getPatient().getICN(), qb.getUser().getUserId(), domain, String.valueOf(qb.isForceRefresh()));

        logger.debug("**DEBUG** CallBackToken: " + callbackToken);

        String currentDate = Utils.getFormattedDate(Calendar.getInstance(), "yyyyMMddHHmmss");
        Calendar futureCal = Calendar.getInstance();
        futureCal.add(Calendar.YEAR, 2);
        String futureDate = Utils.getFormattedDate(futureCal, "yyyyMMddHHmmss");

        StringBuilder startDate = new StringBuilder();
        startDate.append("1867");
        startDate.append(currentDate.substring(4,8));
        startDate.append("000000");

        StringBuilder endDate = new StringBuilder();
        endDate.append(futureDate.substring(0,8));
        endDate.append("235959");


        params.put("requestID", callbackToken);
        params.put("domain", domainParam);
        params.put("patID", patId);
        params.put("userNameFamily", userNameFamily);
        params.put("userNameGiven", userNameGiven);
        params.put("siteCode", siteCode);
        params.put("startDate", startDate.toString());
        params.put("endDate", endDate.toString());

        if (qb.isForceRefresh()) {
            params.put("forceRefresh", ",{forceRefresh}");
            logger.debug("### DAS FORCE REFRESH ###");
        } else {
            params.put("forceRefresh", "");
        }

        String callback = callbackURL;
        if (callbackURL.contains("localhost")) {
            callback = callback.replace("localhost", HOST_FQDN);
        }
        params.put("callbackURL", callback + callbackToken);

        StrSubstitutor sub = new StrSubstitutor(params);
        String requestJSON = sub.replace(requestTemplate);

        cacheSubscription(callbackToken, Boolean.toString(qb.isForceRefresh()),
                qb.getPatient().getICN(), qb.getUser().getUserId(),
                qb.getUser().getName(), domain, "");

        return requestJSON;
    }

    public Subscription getCachedSubscription(QueryBean qb, String domain) {
        return getCachedSubscription(qb.getPatient().getICN(), qb.getUser().getUserId(), domain);
    }

    public Subscription getCachedSubscription(String patICN, String userID, String domain) {
        //generate key for map ("icn|userId|domain")
        StringBuilder key = new StringBuilder();
        key.append(patICN);
        key.append("|");
        key.append(userID);
        key.append("|");
        key.append(domain);

        return DASLimit.getInstance().getSubscription(key.toString());
    }

    public void setCachedSubscription(QueryBean qb, String domain, Subscription subscription) {
        setCachedSubscription(qb.getPatient().getICN(), qb.getUser().getUserId(), domain, subscription);
    }

    public void setCachedSubscription(String patICN, String userID, String domain, Subscription subscription) {
        //generate key for map ("icn|userId|domain")
        StringBuilder key = new StringBuilder();
        key.append(patICN);
        key.append("|");
        key.append(userID);
        key.append("|");
        key.append(domain);

        DASLimit.getInstance().putSubscription(key.toString(), subscription);
    }

    public Subscription cacheSubscription(String callbackToken, String forceRefresh,
                                          String patICN, String userID, String userName,
                                          String domain, String status) {

        //generate key for map ("icn|userId|domain")
        StringBuilder key = new StringBuilder();
        key.append(patICN);
        key.append("|");
        key.append(userID);
        key.append("|");
        key.append(domain);

        //check to see if there is already an existing subscription
        Subscription subscription = DASLimit.getInstance().getSubscription(key.toString());
        if (subscription == null) {
            subscription = new Subscription();
            logger.debug("### NEW subscription ###");
        } else {
            //subscription found, reset the clock for it
            subscription.setCreationTime(System.currentTimeMillis());
            logger.debug("### EXISTING subscription ###");
        }

        try {
            subscription.setSubscriptionID(callbackToken);
            subscription.setForceRefresh(Boolean.valueOf(forceRefresh));
            subscription.setPatientICN(patICN);
            subscription.setUserID(userID);
            subscription.setUserName(userName);
            subscription.setDomain(domain);
            if (StringUtils.isEmpty(status)) {
                subscription.setStatus("init");
            } else {
                subscription.setStatus(status);
            }

            DASLimit.getInstance().putSubscription(key.toString(), subscription);
        } catch (Exception ex) {
            //response.sendError(403, "Error attempting to cache the subscription");
            logger.debug("Error attempting to cache the subscription: " + ex.getMessage());
        }
        return subscription;

    }

    public DASResultCollection mapVLERAllergies(Resource dasResource, Subscription subscription) {
        DASResultCollection resultCollection = new DASResultCollection();

        List<Allergy> allergyList = new ArrayList<Allergy>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
        //List<SiteStatus> statusList = new ArrayList<SiteStatus>();
        try {

            for (Resource doc : dasResource.getEntry()) {

                List<Allergy> docAllergyList = new ArrayList<Allergy>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_ALLERGIES)) {

                            AllergyIntolerance ai = ((AllergyIntolerance) resource);
                            Allergy allergy = new Allergy();
                            allergy.setSeverity(ai.getCriticality());
                            allergy.setAllergyId(ai.getId());

                            if (ai.getRecordedDate() != null) {
                                allergy.setEnteredDate(Utils.getCalendar(ai.getRecordedDate()));
                            }

                            //process reactants
                            if (ai.getSubstance() != null) {
                                List<Code> allergyCodes = new ArrayList<Code>();
                                allergy.setAllergyName(ai.getSubstance().getText());
                                if (ai.getSubstance().getCoding() != null) {
                                    for (SCDBean coding : ai.getSubstance().getCoding()) {
//                                        SCDBean coding = ai.getSubstance().getCoding();
                                        Code allergyCode = new Code();
                                        if (coding.getCode() != null) {
                                            if (StringUtils.isEmpty(allergy.getAllergyName())) {
                                                allergy.setAllergyName(coding.getDisplay());
                                            }
                                            allergyCode.setDisplay(coding.getDisplay());
                                            allergyCode.setCode(coding.getCode());
                                            if (StringUtils.isNotEmpty(coding.getSystem())
                                                    && coding.getSystem().toUpperCase().indexOf("RXNORM") >= 0) {
                                                allergyCode.setSystem("RXNORM");
                                            } else {
                                                allergyCode.setSystem(coding.getSystem());
                                            }
                                            allergyCodes.add(allergyCode);
                                        }
                                    }
                                }
                                if (!allergyCodes.isEmpty()) {
                                    allergy.setCodes(allergyCodes.toArray(new Code[0]));
                                }
                            }

                            //process reaction
                            if (ai.getReaction() != null) {
                                List<Code> allergyReactions = new ArrayList<Code>();
                                if (ai.getReaction() != null) {
                                    for (Manifestation mani : ai.getReaction()) {
                                        if (mani.getManifestation() != null) {
                                            for (CodedObject co : mani.getManifestation()) {
                                                if (StringUtils.isNotEmpty(co.getText())) {
                                                    Code reaction = new Code();
                                                    reaction.setDisplay(co.getText());
                                                    reaction.setCode(co.getText());
                                                    reaction.setSystem("TEXT");
                                                    allergyReactions.add(reaction);
                                                }
//                                                if (co.getCoding() != null) {
//                                                    for (Coding coding : co.getCoding()) {
//                                                        if (coding != null && coding.getCode() != null) {
//                                                            Code reaction = new Code();
//                                                            reaction.setCode(coding.getCode().getCode());
//                                                            reaction.setDisplay(coding.getCode().getDisplay());
//                                                            if (StringUtils.isNotEmpty(coding.getCode().getSystem())
//                                                                    && coding.getCode().getSystem().toUpperCase().indexOf("SNOMED") >= 0) {
//                                                                reaction.setSystem("SNOMED");
//                                                            } else {
//                                                                reaction.setSystem(coding.getCode().getSystem());
//                                                            }
//                                                            allergyReactions.add(reaction);
//                                                        }
//                                                    }
//                                                }
                                            }
                                        }
                                    }
                                }
                                if (!allergyReactions.isEmpty()) {
                                    allergy.setReactions(allergyReactions.toArray(new Code[0]));
                                }

                            }

                            docAllergyList.add(allergy);
                        }
                    }
                }

                if (!docAllergyList.isEmpty()) {
                    //insert all common components into the allergies
                    for (Allergy allergy : docAllergyList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                            allergy.setNarrativeId(narrativeID);
                            allergy.setNarrative(narrativeText);
                        }
                        allergy.setSite(site);
                    }
                } else {
                    //no coded allergies found in this document.
                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                        //since a narrative exists, let's create a false record so the user is able to see the narrative
                        Allergy nonCoded = new Allergy();
                        nonCoded.setAllergyName("Narrative");
                        nonCoded.setSite(site);
                        nonCoded.setNarrativeId(narrativeID);
                        nonCoded.setNarrative(narrativeText);
                        docAllergyList.add(nonCoded);
                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docAllergyList)) {
                    allergyList.addAll(docAllergyList);
                }
            }
            if (!allergyList.isEmpty()) {
                resultCollection.setAllergies(allergyList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error mapping Allergies: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERProblems(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<Problem> problemList = new ArrayList<Problem>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
//        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        try {
            for (Resource doc : dasResource.getEntry()) {

                List<Problem> docProblemList = new ArrayList<Problem>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_PROBLEMS)) {

                            Condition condition = ((Condition) resource);
                            Problem problem = new Problem();
                            problem.setId(condition.getId());
                            problem.setStatus(condition.getClinicalStatus());

                            //process conditions
                            if (condition.getCondition() != null) {
                                List<Code> conditionCodes = new ArrayList<Code>();
                                problem.setDescription(condition.getCondition().getText());
                                if (condition.getCondition().getCoding() != null) {
                                    for (SCDBean coding : condition.getCondition().getCoding()) {
                                        if (coding != null && coding.getCode() != null) {
                                            Code conditionCode = new Code();
                                            //leave the display blank, to be filled in by terminology mapper
                                            //conditionCode.setDisplay(scdBean.getDisplay());
                                            conditionCode.setCode(coding.getCode());
                                            if (StringUtils.isNotEmpty(coding.getSystem())
                                                    && coding.getSystem().toUpperCase().indexOf("SNOMED") >= 0) {
                                                conditionCode.setSystem("SNOMED");
                                            } else {
                                                conditionCode.setSystem(coding.getSystem());
                                            }
                                            conditionCodes.add(conditionCode);
                                        }
                                    }
                                }
                                if (!conditionCodes.isEmpty()) {
                                    problem.setCodes(conditionCodes.toArray(new Code[0]));
                                }
                            }

                            if (StringUtils.isNotEmpty(condition.getDateAsserted())) {
                                problem.setLastModifiedDate(Utils.getCal(condition.getDateAsserted()));
                            }

                            if (StringUtils.isNotEmpty(condition.getOnsetDateTime())) {
                                problem.setOnsetDate(condition.getOnsetDateTime());
                            }


                            docProblemList.add(problem);
                        }
                    }
                }

                if (!docProblemList.isEmpty()) {
                    //insert all common components into the problem
                    for (Problem problem : docProblemList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                            problem.setNarrativeId(narrativeID);
                            problem.setNarrative(narrativeText);
                        }
                        problem.setSite(site);
                    }
                } else {
                    //no coded items found in this document.
                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                        //since a narrative exists, let's create a false record so the user is able to see the narrative
                        Problem nonCoded = new Problem();
                        nonCoded.setDescription("Narrative");
                        nonCoded.setSite(site);
                        nonCoded.setNarrativeId(narrativeID);
                        nonCoded.setNarrative(narrativeText);
                        docProblemList.add(nonCoded);
                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docProblemList)) {
                    problemList.addAll(docProblemList);
                }
            }


            if (!problemList.isEmpty()) {
                resultCollection.setProblems(problemList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error deserializing Problems payload: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERVitals(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<Vitals> vitalsList = new ArrayList<Vitals>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
//        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        try {
            for (Resource doc : dasResource.getEntry()) {

                List<Vitals> docItemList = new ArrayList<Vitals>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_VITALS)) {

                            Observation observation = ((Observation) resource);
                            Vitals vital = new Vitals();
                            vital.setVitalsIEN(observation.getId());

                            //process vital types
                            if (observation.getVitalType() != null) {
                                List<Code> vitalCodes = new ArrayList<Code>();
                                vital.setVitalType(observation.getVitalType().getText());
                                if (observation.getVitalType().getCoding() != null) {
                                    for (SCDBean coding : observation.getVitalType().getCoding()) {
                                        if (coding != null && coding.getCode() != null) {
                                            Code vitalCode = new Code();
                                            //vitalCode.setDisplay(scdBean.getDisplay());
                                            vitalCode.setCode(coding.getCode());
                                            if (StringUtils.isNotEmpty(coding.getSystem())
                                                    && coding.getSystem().toUpperCase().indexOf("LOINC") >= 0) {
                                                vitalCode.setSystem("LOINC");
                                            } else {
                                                vitalCode.setSystem(coding.getSystem());
                                            }
                                            vitalCodes.add(vitalCode);
                                        }
                                    }
                                }
                                if (!vitalCodes.isEmpty()) {
                                    vital.setCodes(vitalCodes.toArray(new Code[0]));
                                }
                            }

                            //process Values
                            if (observation.getValueQuantity() != null) {
                                vital.setRate(String.valueOf(observation.getValueQuantity().getValue()));
                                vital.setUnits(observation.getValueQuantity().getUnits());
                            }

                            if (observation.getIssued() != null) {
                                vital.setDateTimeTaken(Utils.getCalendar(observation.getIssued()));
                            }

                            docItemList.add(vital);
                        }
                    }
                }

                if (!docItemList.isEmpty()) {
                    //insert all common components into the problem
                    for (Vitals vital : docItemList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                            vital.setNarrativeId(narrativeID);
                            vital.setNarrative(narrativeText);
                        }
                        vital.setSite(site);
                    }
                } else {
                    //no coded items found in this document.
                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                        //since a narrative exists, let's create a false record so the user is able to see the narrative
                        Vitals nonCoded = new Vitals();
                        nonCoded.setVitalType("Narrative");
                        nonCoded.setSite(site);
                        nonCoded.setNarrativeId(narrativeID);
                        nonCoded.setNarrative(narrativeText);
                        docItemList.add(nonCoded);
                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docItemList)) {
                    vitalsList.addAll(docItemList);
                }
            }


            if (!vitalsList.isEmpty()) {
                resultCollection.setVitals(vitalsList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            e.printStackTrace(System.out);
            logger.error("Error deserializing Vitals payload: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERImmunizations(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<Immunization> immunizationList = new ArrayList<Immunization>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
//        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        try {
            for (Resource doc : dasResource.getEntry()) {

                List<Immunization> docItemList = new ArrayList<Immunization>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_IMMUNIZATIONS)) {

                            gov.va.med.vds.das.beans.Immunization dasImmunization = ((gov.va.med.vds.das.beans.Immunization) resource);
                            Immunization immunization = new Immunization();
                            immunization.setId(dasImmunization.getId());

                            //process immunization types
                            if (dasImmunization.getVaccineCode() != null) {
                                List<Code> immCodes = new ArrayList<Code>();
//                                if(dasImmunization.getNote() != null) {
//                                    immunization.setName(dasImmunization.getNote().getText());
//                                }
                                if (dasImmunization.getVaccineCode().getCoding() != null) {
                                    for (SCDBean coding : dasImmunization.getVaccineCode().getCoding()) {
                                        if (coding != null) {
                                            if (StringUtils.isNotEmpty(coding.getText())) {
                                                immunization.setName(coding.getText());
                                            } else if (StringUtils.isNotEmpty(coding.getDisplay())) {
                                                immunization.setName(coding.getDisplay());
                                            }
                                            Code immunizationCode = new Code();
                                            //vitalCode.setDisplay(scdBean.getDisplay());
                                            immunizationCode.setCode(coding.getCode());
                                            if (StringUtils.isNotEmpty(coding.getSystem())
                                                    && coding.getSystem().toUpperCase().indexOf("CVX") >= 0
                                                    && !"UNK".equalsIgnoreCase(coding.getCode())) {
                                                immunizationCode.setSystem("CVX");
                                                immunization.setCvx(coding.getCode());
                                            } else {
                                                immunizationCode.setSystem(coding.getSystem());
                                            }
                                            immCodes.add(immunizationCode);
                                        }
                                    }
                                }
                                if (!immCodes.isEmpty()) {
                                    immunization.setCodes(immCodes.toArray(new Code[0]));
                                }
                            }

                            //process Date
                            if (dasImmunization.getDate() != null) {
                                String date = Utils.formatDisplayDate(dasImmunization.getDate());
                                immunization.setDateTime(date);
                            }

                            if (StringUtils.isNotEmpty(dasImmunization.getStatus())
                                    && dasImmunization.getStatus().toUpperCase().indexOf("REFUSE") >= 0) {
                                immunization.setContraindicated(true);
                                Date refusalDate = DateDeserializer.getDate(immunization.getDateTime());
                                if (refusalDate != null) {
                                    immunization.setExemptionRefusalDate(Utils.getCalendar(refusalDate));
                                }
                            }

                            if (dasImmunization.getRoute() != null
                                    && dasImmunization.getRoute().getCoding() != null) {
                                immunization.setRoute(dasImmunization.getRoute().getCoding().getDisplay());
                            }

                            if (dasImmunization.getDoseQuantity() != null) {
                                immunization.setDose(String.valueOf(dasImmunization.getDoseQuantity().getValue()));
                                immunization.setUnits(dasImmunization.getDoseQuantity().getUnit());
                            }
                            if (dasImmunization.getExplanation() != null) {
                                immunization.setExemptionRefusalReason(dasImmunization.getExplanation().getReasonNotGiven());
                            }
                            if (dasImmunization.getVaccinationProtocol() != null) {
                                immunization.setSeries(dasImmunization.getVaccinationProtocol().getSeries());
                            }

                            if(dasImmunization.getReaction() != null && dasImmunization.getReaction().getDetail() != null){
                                immunization.setReaction(dasImmunization.getReaction().getDetail().getReference());
                            }

                            immunization.setLot(dasImmunization.getLotNumber());
                            immunization.setManufacturerName(dasImmunization.getManufacturer());

                            docItemList.add(immunization);
                        }
                    }
                }

                if (!docItemList.isEmpty()) {
                    //insert all common components into the problem
                    for (Immunization immunization : docItemList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                            immunization.setNarrativeId(narrativeID);
                            immunization.setNarrative(narrativeText);
                        }
                        immunization.setSite(site);
                    }
                } else {
                    //no coded items found in this document.
                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                        //since a narrative exists, let's create a false record so the user is able to see the narrative
                        Immunization nonCoded = new Immunization();
                        nonCoded.setName("Narrative");
                        nonCoded.setSite(site);
                        nonCoded.setNarrativeId(narrativeID);
                        nonCoded.setNarrative(narrativeText);
                        docItemList.add(nonCoded);
                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docItemList)) {
                    immunizationList.addAll(docItemList);
                }
            }


            if (!immunizationList.isEmpty()) {
                resultCollection.setImmunizations(immunizationList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error deserializing Immunizations payload: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERProcedures(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<Procedure> procedureList = new ArrayList<Procedure>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
//        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        try {
            for (Resource doc : dasResource.getEntry()) {

                List<Procedure> docProcedureList = new ArrayList<Procedure>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_PROCEDURES)) {

                            gov.va.med.vds.das.beans.Procedure dasProcedure = ((gov.va.med.vds.das.beans.Procedure) resource);
                            Procedure procedure = new Procedure();
                            procedure.setEncounterId(dasProcedure.getId());

                            //process provider
                            if (dasProcedure.getPerformer() != null && dasProcedure.getPerformer().get(0) != null
                                    && dasProcedure.getPerformer().get(0).getActor() != null
                                    && dasProcedure.getPerformer().get(0).getActor().getDisplay() != null
                                    && dasProcedure.getPerformer().get(0).getActor().getDisplay().get(0) != null) {
                                if (StringUtils.isEmpty(dasProcedure.getPerformer().get(0).getActor().getDisplay().get(0).getFirst())) {
                                    procedure.setProvider(dasProcedure.getPerformer().get(0).getActor().getDisplay().get(0).getLast());
                                } else {
                                    procedure.setProvider(dasProcedure.getPerformer().get(0).getActor().getDisplay().get(0).getLast() + ","
                                            + dasProcedure.getPerformer().get(0).getActor().getDisplay().get(0).getFirst());
                                }
                            }

                            //process procedures
                            //List<Code> procCodes = new ArrayList<Code>();
                            if (dasProcedure.getProcedure() != null && dasProcedure.getProcedure().getCode() != null
                                    && dasProcedure.getProcedure().getCode().getCoding() != null) {
                                procedure.setDescription(dasProcedure.getProcedure().getCode().getCoding().getText());
                                Code procCode = new Code();
                                if (StringUtils.isEmpty(dasProcedure.getProcedure().getCode().getCoding().getSystem())) {
                                    procCode.setSystem("CPT");
                                } else if (StringUtils.isNotEmpty(dasProcedure.getProcedure().getCode().getCoding().getSystem())
                                        && dasProcedure.getProcedure().getCode().getCoding().getSystem().toUpperCase().indexOf("CPT") >= 0) {
                                    procCode.setSystem("CPT");
                                } else {
                                    procCode.setSystem(dasProcedure.getProcedure().getCode().getCoding().getSystem());
                                }
                                procCode.setCode(dasProcedure.getProcedure().getCode().getCoding().getCode());
                                //procCodes.add(procCode);

                                procedure.setCode(procCode.getCode());
                                procedure.setCodeSystem(procCode.getSystem());
                            }

                            //process date
                            if (dasProcedure.getPerformedDateTime() != null) {
                                Date retDate = null;
                                try{
                                    retDate = DateDeserializer.getDate(dasProcedure.getPerformedDateTime());
                                    if(retDate != null){
                                        procedure.setDate(Utils.getCalendar(retDate));
                                    }
                                } catch (Exception e){
                                    logger.debug("DAS procedure date is not valid.");
                                }

                            }

                            //process location
                            if (dasProcedure.getLocation() != null && dasProcedure.getLocation().size() > 0) {
                                procedure.setLocation(dasProcedure.getLocation().get(0).getName());
                            }

                            docProcedureList.add(procedure);
                        }
                    }
                }

                if (!docProcedureList.isEmpty()) {
                    //insert all common components into the procedure
                    for (Procedure procedure : docProcedureList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                            procedure.setNarrativeId(narrativeID);
                            procedure.setNarrative(narrativeText);
                        }
                        procedure.setSite(site);
                    }
                } else {
                    //no coded items found in this document.
                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                        //since a narrative exists, let's create a false record so the user is able to see the narrative
                        Procedure nonCoded = new Procedure();
                        nonCoded.setDescription("Narrative");
                        nonCoded.setSite(site);
                        nonCoded.setNarrativeId(narrativeID);
                        nonCoded.setNarrative(narrativeText);
                        docProcedureList.add(nonCoded);
                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docProcedureList)) {
                    procedureList.addAll(docProcedureList);
                }
            }

            if (!procedureList.isEmpty()) {
                resultCollection.setProcedures(procedureList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error deserializing Procedures payload: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERDemographics(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<PatientDemographics> demographicsList = new ArrayList<PatientDemographics>();
        List<Narrative> narrativeList = new ArrayList<Narrative>();
//        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        try {
            for (Resource doc : dasResource.getEntry()) {

                List<PatientDemographics> docPatientList = new ArrayList<PatientDemographics>();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                //each doc has one narrative set
                Narrative docNarrative = new Narrative();
                String narrativeID = "";
                String narrativeText = "";
                String narrativeSystemID = "";
                String narrativeOrg = "";

                for (Resource res : doc.getEntry()) {
                    //each res is a resource in a doc

                    Resource resource = res.getResource();

                    if (resource.getResourceType() != null) {
                        if (resource.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                                || resource.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                            if (resource.getMasterIdentifier() != null) {
                                narrativeSystemID = resource.getMasterIdentifier().getSystem();
                                site.setSiteCode(narrativeSystemID);
                            }
                            if (resource.getContained() != null && resource.getContained().size() > 0) {
                                for (Entity contained : resource.getContained()) {
                                    if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                        narrativeOrg = contained.getName();
                                        site.setName(narrativeOrg);
                                        site.setMoniker(narrativeOrg);
                                    }
                                }
                            }

                        }
                        if (resource.getResourceType().equals(DAS_BASIC)) {
                            //narrative resource
                            if (resource.getText() != null && StringUtils.isEmpty(narrativeID)) {
                                //Narrative dasNarrative = new Narrative();
                                narrativeID = resource.getId();
                                narrativeText = resource.getText().getDiv();
                            }
                        }
                        if (resource.getResourceType().equals(DAS_DEMOGRAPHICS)) {

                            Patient patient = ((Patient) resource);
                            PatientDemographics demographics = new PatientDemographics();
                            if (patient.getIdentifier() != null) {
                                for (SystemValue value : patient.getIdentifier()) {
//                                    if (value != null && value.getSystem().endsWith("2.16.840.1.113883.4.349")) {
                                    if (value != null) {
                                        demographics.setICN(value.getValue());
                                        demographics.setEDIPI(value.getValue());
                                    }


                                }
                            }

                            if (patient.getName() != null && patient.getName().get(0) != null) {
                                StringBuilder sbName = new StringBuilder();
                                if (patient.getName().get(0).getFamily() != null) {
                                    sbName.append(patient.getName().get(0).getFamily().get(0));
                                }
                                if (patient.getName().get(0).getGiven() != null) {
                                    sbName.append(",");
                                    boolean hasMiddle = false;
                                    for (String firstName : patient.getName().get(0).getGiven()) {
                                        if (hasMiddle)
                                            sbName.append(" ");
                                        sbName.append(firstName);
                                        hasMiddle = true;
                                    }
                                }
                                demographics.setName(sbName.toString());
                            }

                            if (patient.getTelecom() != null && patient.getTelecom().get(0) != null) {
                                if(StringUtils.isNotEmpty(patient.getTelecom().get(0).getSystem()) && "phone".equalsIgnoreCase(patient.getTelecom().get(0).getSystem())) {
                                    demographics.setPhone1(patient.getTelecom().get(0).getValue());
                                }
                                if (patient.getTelecom().size() > 1 && patient.getTelecom().get(1) != null
                                        && StringUtils.isNotEmpty(patient.getTelecom().get(1).getSystem())
                                        && "phone".equalsIgnoreCase(patient.getTelecom().get(1).getSystem())) {
                                    demographics.setPhone2(patient.getTelecom().get(1).getValue());
                                }
                            }

                            demographics.setGender(patient.getGender());
                            demographics.setDob(patient.getBirthDate());

                            if (patient.getAddress() != null) {
                                List<gov.va.med.vds.vistabeans.Address> addressList = new ArrayList<gov.va.med.vds.vistabeans.Address>();
                                for (Address dasAddress : patient.getAddress()) {
                                    gov.va.med.vds.vistabeans.Address address = new gov.va.med.vds.vistabeans.Address();
                                    if (dasAddress.getLine() != null) {
                                        address.setStreet1(dasAddress.getLine().get(0));
                                        if (dasAddress.getLine().size() > 1)
                                            address.setStreet2(dasAddress.getLine().get(1));
                                        if (dasAddress.getLine().size() > 2)
                                            address.setStreet3(dasAddress.getLine().get(2));
                                    }
                                    address.setCity(dasAddress.getCity());
                                    address.setStateProvince(dasAddress.getState());
                                    address.setPostalCode(dasAddress.getPostalCode());
                                    address.setType(dasAddress.getUse());
                                    addressList.add(address);
                                }
                                demographics.setAddress(addressList);
                            }
//                            if(patient.getMaritalStatus() != null && patient.getMaritalStatus().getCoding() != null ){
//                                demographics.setMaritalStatus(patient.getMaritalStatus().getCoding().get(0).getDisplay());
//                            }
                            if (patient.getMaritalStatus() != null) {
                                demographics.setMaritalStatus(patient.getMaritalStatus().getText());
                            }

                            if (patient.getExtension() != null) {
                                List<Code> raceCodeList = new ArrayList<Code>();
                                List<String> religionList = new ArrayList<String>();
                                for (Extension extension : patient.getExtension()) {
                                    if (extension.getValueCodableConcept() != null && extension.getValueCodableConcept().getCoding() != null) {
                                        for (SCDBean coding : extension.getValueCodableConcept().getCoding()) {
                                            if (coding != null && coding.getCode() != null) {
                                                if (coding.getSystem().endsWith("2.16.840.1.113883.6.238")) {
                                                    Code code = new Code();
                                                    code.setSystem("CDC");
                                                    code.setCode(coding.getCode());
                                                    code.setDisplay(coding.getDisplay());
                                                    raceCodeList.add(code);
                                                } else if (coding.getSystem().contains("Religious")) {
                                                    religionList.add(coding.getDisplay());
                                                }

                                            }
                                        }
                                    }
                                }
                                demographics.setReligion(religionList);
                                demographics.setRace(raceCodeList.toArray(new Code[0]));
                            }
                            docPatientList.add(demographics);
                        }
                    }
                }

                if (!docPatientList.isEmpty()) {
                    //insert all common components into the problem
                    for (PatientDemographics demographics : docPatientList) {
                        if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
//                            demographics.setNarrativeId(narrativeID);
//                            demographics.setNarrative(narrativeText);
                        }
                        demographics.setSite(site);
                    }
                } else {
                    //no coded items found in this document.
//                    if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
//                        //since a narrative exists, let's create a false record so the user is able to see the narrative
//                        PatientDemographics nonCoded = new Problem();
//                        nonCoded.setDescription("Narrative");
//                        nonCoded.setSite(site);
//                        nonCoded.setNarrativeId(narrativeID);
//                        nonCoded.setNarrative(narrativeText);
//                        docPatientList.add(nonCoded);
//                    }
                }

                if (StringUtils.isNotEmpty(narrativeID) && StringUtils.isNotEmpty(narrativeText)) {
                    docNarrative.setNarrativeId(narrativeID);
                    docNarrative.setNarrativeText(narrativeText);
                    docNarrative.setOrganization(narrativeOrg);
                    docNarrative.setSourceSystemID(narrativeSystemID);
                    narrativeList.add(docNarrative);
                }

                if (CollectionUtils.isNotEmpty(docPatientList)) {
                    demographicsList.addAll(docPatientList);
                }
            }


            if (!demographicsList.isEmpty()) {
                resultCollection.setDemographics(demographicsList);
            }
            if (!narrativeList.isEmpty()) {
                resultCollection.setNarratives(narrativeList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error deserializing Demographics payload: " + e.getMessage());
        }
        return resultCollection;
    }

    public DASResultCollection mapVLERDocuments(Resource dasResource, Subscription subscription) {

        DASResultCollection resultCollection = new DASResultCollection();

        List<VLERDocument> docList = new ArrayList<VLERDocument>();

        try {
            for (Resource resource : dasResource.getEntry()) {

                VLERDocument vDoc = new VLERDocument();

                Resource doc = resource.getResource();

                //each doc will have one site
                Site site = new Site();
                site.setAgency("DASVLER");

                if (doc.getResourceType() != null) {
                    if (doc.getResourceType().equals(DAS_DOCUMENTMANIFEST)
                            || doc.getResourceType().equals(DAS_DOCUMENTREFERENCE)) {
                        if (doc.getMasterIdentifier() != null && StringUtils.isNotEmpty(doc.getMasterIdentifier().getSystem())) {
                            site.setSiteCode(doc.getMasterIdentifier().getSystem());
                            vDoc.setHomeCommunityId(doc.getMasterIdentifier().getSystem());
                            vDoc.setRepositoryUniqueId(doc.getMasterIdentifier().getSystem());
                            vDoc.setDocumentUniqueId(doc.getMasterIdentifier().getValue());
                        }
                        if (doc.getContained() != null && doc.getContained().size() > 0) {
                            List<Author> authorList = new ArrayList<Author>();

                            for (Entity contained : doc.getContained()) {
                                if (StringUtils.isNotEmpty(contained.getResourceType()) && contained.getResourceType().equalsIgnoreCase("Organization")) {
                                    site.setName(contained.getName());
                                    site.setMoniker(contained.getName());

                                    Author author = new Author();
                                    author.setName(contained.getName());
                                    author.setInstitution(contained.getName());
                                    authorList.add(author);
                                }
                            }
                            vDoc.setAuthorList(authorList);
                        }

                        if(CollectionUtils.isNotEmpty(resource.getContent())){
                            for(Content content : resource.getContent()){
                                if(content.getAttachment() != null && content.getAttachment().getData() != null){
                                    vDoc.setDocument(content.getAttachment().getData().getBytes("UTF-8"));
                                }
                            }
                        }

                    }
                }

                vDoc.setSite(site);

                if(doc.getCreated() != null){
                    vDoc.setCreationTime(Utils.getCalendar(doc.getCreated()));
                } else {
                    vDoc.setCreationTime(Calendar.getInstance());
                }
                vDoc.setName(doc.getDescription());

                docList.add(vDoc);
            }

            if (!docList.isEmpty()) {
                resultCollection.setDocuments(docList);
            }

            processStatus(subscription, resultCollection);

        } catch (Exception e) {
            logger.error("Error deserializing Documents payload: " + e.getMessage());
        }
        return resultCollection;
    }

    private void processStatus(Subscription subscription, DASResultCollection resultCollection) {
        List<SiteStatus> statusList = new ArrayList<SiteStatus>();

        if (subscription.getDocuments() != null) {

            Set<String> siteErrorSet = new HashSet<String>();

            for (Document document : subscription.getDocuments()) {
                if (document != null) {
                    SiteStatus siteStatus = new SiteStatus();
                    siteStatus.setStatus(document.getStatus());
                    siteStatus.setSiteName(document.getPartnerName());
                    siteStatus.setDocumentId(document.getPartnerDocumentID());
                    statusList.add(siteStatus);

                    if (StringUtils.isNotEmpty(document.getStatus())) {
                        if ("AVAILABLEOLD".equals(document.getStatus().toUpperCase())) {
                            siteErrorSet.add("At least one partner site delivered old or partial data. \n");
                        } else if ("NOTAVAILABLE".equals(document.getStatus().toUpperCase())) {
                            siteErrorSet.add("At least one partner site does not have data for this patient. \n");
                        } else if (!"AVAILABLE".equals(document.getStatus().toUpperCase())) {
                            siteErrorSet.add("At least one partner site was not able to complete the query. \n");
                        }

                    }
                }
            }

            if (siteErrorSet.size() > 0) {
                for (String error : siteErrorSet) {
                    if (resultCollection.getErrorList() != null) {
                        resultCollection.getErrorList().add(error);
                    } else {
                        ArrayList<String> errorList = new ArrayList<String>();
                        errorList.add(error);
                        resultCollection.setErrorList(errorList);
                    }
                }
            }

        }
        resultCollection.setStatusList(statusList);
    }
}
