/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.ehr.common;

import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.ehr.common.domain.ResponseCode;
import gov.va.med.vds.ehr.common.dto.EHRRequestType;
import gov.va.med.vds.ehr.common.dto.EHRResponseType;
import gov.va.med.vds.vistabeans.Patient;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

/**
 * EHR Data Service abstract class
 */
public abstract class EHRDataServiceBase extends DataServiceBase {
    /**
     * Helper method that logs a service request.
     * @param req EHRRequestType instance.
     */
    protected String ehrRequestToString(EHRRequestType req) {
        StringBuilder logStr = new StringBuilder();

        logStr.append("EHRRequestType [");

        if (req == null) {
            logStr.append("EHR Request Type is null]");
        } else {
    
            if (req.getUser() == null) {
                logStr.append("User is null");
            } else {
                logStr.append("User login site code: " + req.getUser().getHostSite().getSiteCode());
                logStr.append(", User ID: " + req.getUser().getUserIen());
                logStr.append(", User NPI: " + req.getUser().getNPI());
            }
    
            if (req.getPatient() == null) {
                logStr.append(", Patient is null");
            } else {
                Patient patient = req.getPatient();
                logStr.append(", Patient ICN: " + patient.getICN());
            }
    
            logStr.append("]");
        }
        return logStr.toString();
    }

    /**
     * Helper method that logs a service response.
     * @param resp EHRResponseType instance.
     */
    protected String ehrResponseToString(EHRResponseType resp) {
        StringBuilder logStr = new StringBuilder();

        logStr.append("EHRResponseType [");

        if (resp == null) {
            logStr.append("EHR Response Type is null]");
        } else {
    
            if (resp.getResponseCode() != null 
                    && resp.getResponseCode().getCode() != null) {
                String code = resp.getResponseCode().getCode().toString();
                String message = resp.getResponseCode().getMessage();
    
                logStr.append("Response Code: " + code + " - " + message);
            } else {
                logStr.append("Response Code is null");
            }
    
            logStr.append("]");
        }
        return logStr.toString();
    }

    protected void toIllegalArgumentsResponse(EHRResponseType resp, String message) {
        ResponseCode code = new ResponseCode();
        code.setCode(ResponseCode.ILLEGAL_ARGUMENTS.getCode());
        code.setMessage(message);
        resp.setResponseCode(code);
    }

}
