/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.ehr.common.domain.validation;

import gov.va.med.foundations.utilities.Messages;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.ehr.common.dto.EHRRequestType;

import java.util.ArrayList;
import java.util.List;

public abstract class EHRCommand {

    /**
     * Helper method that validates required params of an EHRRequestType instance.
     * @param requestType EHRRequestType
     * @param isPatientRequired Check for required fields on patient object.
     */
    public void validate(EHRRequestType requestType, boolean isPatientRequired)
    {
        illegalArgCheck(validateBaseParams(requestType, isPatientRequired));
    }

    protected List<String> validateBaseParams(EHRRequestType requestType, boolean isPatientRequired)
    {
        List<String> invalidParams = new ArrayList<String>();

        if (requestType == null) {
            invalidParams.add("EHRRequestType is null");
//            return invalidParams;
        } else {
    
            if (requestType.getUser() == null) {
                invalidParams.add("user");
            } else {
                if (StringUtils.isEmpty(requestType.getUser().getUserIen())) {
                    invalidParams.add("user userId");
                }
    
                if (requestType.getUser().getHostSite() == null) {
                    invalidParams.add("user host site");
                }
    
                if (StringUtils.isEmpty(requestType.getUser().getEncryptedFederatedUid())) {
                    invalidParams.add("federated user id");
                }
            }
    
            if (isPatientRequired) {
                if (requestType.getPatient() == null) {
                    invalidParams.add("patient");
                } else {
                    if (StringUtils.isEmpty(requestType.getPatient().getICN())) {
                        invalidParams.add("patient: ICN");
                    }
                }
            }
        }
        return invalidParams;
    }

    protected void illegalArgCheck(List<String> invalidParams) {
        if (!invalidParams.isEmpty()) {
            throw new IllegalArgumentException(Messages.get("invalid.parameters") + invalidParams.toString());
        }
    }

    protected List<String> validatePatientId(String patientId) {
        return validateId("patientId", patientId);
    }

    protected List<String> validateUserId(String userId) {
        return validateId("userId", userId);
    }

    private List<String> validateId(String idName, String id) {
        List<String> invalidParams = new ArrayList<String>();
        if (StringUtils.isEmpty(id)) {
            invalidParams.add(idName);
        }

        return invalidParams;
    }
}
