/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.DateUtils;
import gov.va.med.rpc.KeyValuePair;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.RPCUtils;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.Timestamp;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.RPCUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.*;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 *
 */
public class PatientRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(PatientRPC.class);

    public PatientRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Patient[] match(String target) throws RPCException {
        String request = buildMatchRequest(target);
        String response = (String) cxn.query(request);
        return toMatches(response);
    }

    private String buildMatchRequest(String intarget) {
        String target = intarget;
        String rtc;
        int targetLength = target.length();
        if (targetLength < 2) {
            throw new IllegalArgumentException("Invalid search criteria");
        }
        VistaQuery vquery = vistaQueryFactory.getVistaQuery();

        // If first char is numeric, must be SSN
        if (StringUtils.isNumericChar(target.charAt(0))) {
            /*if (!SocSecNum.isWellFormed(target))
            {
                throw new RPCException("Invalid SSN");
            }*/
            StringBuilder sbld = new StringBuilder();
            sbld.append(target.charAt(0));
            for (int i = 1; i < targetLength; i++) {
                if (StringUtils.isNumericChar(target.charAt(i))) {
                    sbld.append(target.charAt(i));
                }
            }
            if (sbld.length() != 9) {
                throw new IllegalArgumentException("Invalid SSN");
            }
            vquery.setRpcName("ORWPT FULLSSN");
            vquery.addParameter(VistaQuery.LITERAL, sbld.toString());
            rtc = vquery.buildMessage();
        } else {
            // First char was not numeric.  Make sure it's alpha.
            target = target.toUpperCase(Locale.getDefault());
            if (!StringUtils.isAlphaChar(target.charAt(0))) {
                throw new IllegalArgumentException("Invalid search criteria");
            }
    
            // First char was alpha.  If second char is numeric, must be last5.
            if (StringUtils.isNumericChar(target.charAt(1))) {
                if (target.length() != 5) {
                    throw new IllegalArgumentException("Invalid last 5 identifier");
                }
                for (int i = 2; i < 5; i++) {
                    if (!StringUtils.isNumericChar(target.charAt(i))) {
                        throw new IllegalArgumentException("Invalid last 5 identifier");
                    }
                }
                vquery.setRpcName("ORWPT LAST5");
                vquery.addParameter(VistaQuery.LITERAL, target);
                rtc = vquery.buildMessage();
            } else {
        
                // Second char was not numeric.  Assume it's a preferredTerm.
                /*if (!PersonName.isValid(target))
                {
                    throw new RPCException("Invalid person name");
                }*/
                vquery.setRpcName("ORWPT LIST ALL");
                vquery.addParameter(VistaQuery.LITERAL, RPCUtils.adjustForNameSearch(target));
                vquery.addParameter(VistaQuery.LITERAL, "1");
                rtc = vquery.buildMessage();
            }
        }
        return rtc;
    }

    private Patient[] toMatches(String response) {
        String[] lines = StringUtils.split(response, StringUtils.CRLF);
        ArrayList<Patient> lst = new ArrayList<Patient>(lines.length);
        for (int i = 0; i < lines.length; i++) {
            if ("".equals(lines[i])) {
                continue;
            }
            Patient patient = toMatch(lines[i]);
            if (patient != null) {
                lst.add(patient);
            }
        }

//        for(Patient patient : lst)
//        {
//            patient.setPatientIdSiteCode((String) cxn.getSiteCode().getKey());
//        }

        return lst.toArray(new Patient[0]);
    }

    private Patient toMatch(String line) {
        Patient patient = new Patient();
        String[] fields = StringUtils.split(line, StringUtils.CARET);
        if (fields.length > 1) {
            patient.setName(fields[1]);
//            if (fields.length == 2) {
//                return patient;
//            } 
            if (fields.length != 2) {
                if (!"".equals(fields[2])) {
                    patient.setDob(Timestamp.toUtcString(fields[2]));
                }
                if (!"".equals(fields[3])) {
                    patient.setSSN(fields[3]);
                }
    
                //patient ien
                if (!StringUtils.isEmpty(fields[0])) {
                    Site site = cxn.getSite();
                    List<Site> vistaSites = new ArrayList<Site>();
                    vistaSites.add(site);
    
                    List<KeyValuePair<String, String>> patientIens = new ArrayList<KeyValuePair<String, String>>();
                    KeyValuePair<String, String> patIdKVP = new KeyValuePair<String, String>();
                    patIdKVP.setKey(cxn.getSite().getSiteCode());
                    patIdKVP.setValue(fields[0]);
                    patientIens.add(patIdKVP);
    
                    patient.setVistaSites(vistaSites);
                    patient.setPatientIens(patientIens);
                }
            }
        }
        return patient;
    }

    public Patient select(String patientIen) throws RPCException {
        return selectPlus(patientIen);
    }

    public PatientDemographics patientDemoSelect(String patientIen, User user) throws RPCException {
        if (StringUtils.isEmpty(patientIen) || !StringUtils.isNumeric(patientIen)) {
            throw new RPCException("Empty or non-numeric patientIen");
        }
        String request = buildSelectByRpcRequest(patientIen);
        String response = (String) cxn.query(request);
        PatientDemographics demographics = toPatientDemoSelectedByRPC(response/*, patientIen*/);

        demographics = getExtraDemoDetailsFromVPR(patientIen, demographics, user);
        return demographics;
    }


    public PatientDemographics patientDemoSelectVPR(String patientIen, User user) throws RPCException {

        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, Calendar.getInstance(), Calendar.getInstance(), VPRUtil.VPRType.DEMOGRAPHICS);
        String response = (String) cxn.query(request);

        PatientDemographics demographics = parseVPRDemographics(response);

//        Calendar startDate = Calendar.getInstance();
//        Calendar endDate = Calendar.getInstance();
        request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, VPRUtil.VPRType.FLAGS);
        response = (String) cxn.query(request);

        demographics = parseVPRFlags(response, demographics);

        return demographics;
    }


    public Boolean issueSensitivePatientBulletin(String patientIen) throws RPCException {
        Boolean rtc;
        String request = buildSensitivePatientBulletinRequest(patientIen);
        String response = (String) cxn.query(request);
        if ("1".equals(response)) {
            rtc = Boolean.TRUE;
        } else {
            rtc = Boolean.FALSE;
        }
        return rtc;
    }

    private String buildSensitivePatientBulletinRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery();
        vquery.setRpcName("DG SENSITIVE RECORD BULLETIN");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "1");

        return vquery.buildMessage();
    }

    //This call uses no DDR LISTER, just regular RPCs and GET VARIABLE VALUE.

    private Patient selectPlus(String patientIen) throws RPCException {
        Patient result = selectByRpc(patientIen);
        //SB 2/6/13: Janus will utilize VA MVI to retrieve and set patient locations
//        result.setLocations(getSiteIDs(result.getPatientId()));
        //SB: 2/7/13: Setting connection site as default location (so patient select does not break)
        result.setVistaSites(Arrays.asList(cxn.getSite()));

        if (result.isSensitive()) {
            boolean isIssueSecurityBulletin = false;

            try {
                isIssueSecurityBulletin = issueSensitivePatientBulletin(patientIen);
            } catch (RPCException e) {
                isIssueSecurityBulletin = false;
                LOGGER.error("An error occurred while issuing sensitive patient bulletin: " + e.getMessage());
            }

            if (!isIssueSecurityBulletin) {
                LOGGER.info("Failed to issue sensitive patient security bulletin at site: " + cxn.getSiteCode() + ".");
            }

        }

        return result;
    }
    //SB 2/6/13: Janus will utilize VA MVI to retrieve and set patient locations
//    private List<Site> getSiteIDs(String patientIen) throws RPCException
//    {
//        String request = buildGetRemoteSitesRequest(patientIen);
//        String response = (String) cxn.query(request);
//        if (response.startsWith("-1^"))
//        {
//            ArrayList<String> sites = new ArrayList<String>();
//            sites.add((String) cxn.getSiteCode().getKey());
//            return sites;
//        }
//        String[] lines = StringUtils.split(response, StringUtils.CRLF);
//        lines = StringUtils.trimArray(lines);
//        ArrayList<String> result = new ArrayList<String>();
//        result.add((String)cxn.getSiteCode().getKey());
//        for (int i = 0, j = 1; i < lines.length; i++, j++)
//        {
//            String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
//            //SB: do not include station 200 in patient site id list
//            if (flds[0].equals("200"))
//            {
//                continue;
//            }
//            result.add(flds[0]);
//        }
//        return result;
//    }

//    private String buildGetRemoteSitesRequest(String patientIen)
//    {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORWCIRN FACLIST");
//        vq.addParameter(vq.LITERAL, patientIen);
//        return vq.buildMessage();
//    }

    private Patient selectByRpc(String patientIen) throws RPCException {
        if (!StringUtils.isNumeric(patientIen)) {
            throw new RPCException("Non-numeric patientIen");
        }
        String request = buildSelectByRpcRequest(patientIen);
        String response = (String) cxn.query(request);
        return toPatientSelectedByRpc(response/*, patientIen*/);
    }

    private PatientDemographics toPatientDemoSelectedByRPC(String response/*, String patientIen*/) throws RPCException {
        if ("".equals(response)) {
            throw new RPCException("No such patient");
        }
        String[] fields = StringUtils.split(response, StringUtils.CARET);
        if (fields.length > 0 && "-1".equals(fields[0])) {
            throw new RPCException(fields[5]);
        }
        PatientDemographics patientDemo = new PatientDemographics();
        setPatSelectVals(fields, /*patientIen, */patientDemo);

        //set service connected
        if (fields.length > 12 && "1".equals(fields[11]) && !StringUtils.isEmpty(fields[12])) {
            patientDemo.setPercentServiceConnected(fields[12]);
        }

        addHomeData(patientDemo);
        addPCPData(patientDemo);
//        addPatientFlags(patientDemo);

        fillCommonDataBeanValues(patientDemo);

        return patientDemo;
    }

    private Patient toPatientSelectedByRpc(String response/*, String patientIen*/) throws RPCException {
        if ("".equals(response)) {
            throw new RPCException("No such patient");
        }
        String[] fields = StringUtils.split(response, StringUtils.CARET);
        Patient patient = new Patient();
        setPatSelectVals(fields, /*patientIen, */patient);

        return patient;
    }

    private void setPatSelectVals(String[] fields/*, String patientIen*/, Patient toPatient) throws RPCException {
        if (fields.length > 0 && "-1".equals(fields[0])) {
            throw new RPCException(fields[5]);
        }
        if (fields.length > 0) {
            toPatient.setName(fields[0]);
        }
        if (fields.length > 1) {
            toPatient.setGender(fields[1]);
        }
        if (fields.length > 2) {
            toPatient.setDob(Utils.formatDate(Utils.getDate(DateUtils.trimTime(Timestamp.toUtcString(fields[2])))));
        }
        if (fields.length > 3) {
            toPatient.setSSN(fields[3]);
        }
        if (fields.length > 7) {
            toPatient.setCWAD(fields[7]);
        }
//        toPatient.setPatientId(patientIen);
//        toPatient.setPatientIdSiteCode((String) cxn.getSiteCode().getKey());
        if (fields.length > 13) {
            toPatient.setICN(fields[13]);
        }
        if (fields.length > 14 && !StringUtils.isEmpty(fields[14])) {
            toPatient.setAge(fields[14]);
        }

        if (fields.length > 8 && "1".equals(fields[8])) {
            toPatient.setSensitive(true);
        }

        /*if (!fields[4].equals("") && !fields[5].equals(""))
        {
            HospitalLocation hl = new HospitalLocation(fields[4], fields[5]);
            hl.Room = fields[6];
            if (fields[15] != "")
            {
                hl.Specialty = new KeyValuePair<String, String>(fields[15], "");
            }
            patient.Location = hl;
            patient.IsInpatient = true;
        }
        patient.Cwad = fields[7];
        patient.IsRestricted = (fields[8] == "1");
        if (fields[9] != "")
        {
            patient.AdmitTimestamp = VistaTimestamp.toUtcString(fields[9]);
        }
        patient.IsServiceConnected = (fields[11] == "1");
        if (fields[12] != "")
        {
            patient.ScPercent = Convert.ToInt16(fields[12]);
        }*/
    }

    private String buildSelectByRpcRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT SELECT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

//    public Patient selectBySSN(String ssn) throws RPCException {
//        if (ssn.equals(""))
//        {
//            throw new IllegalArgumentException("Invalid SSN");
//        }
//        Patient[] matches = match(ssn);
//        if (matches == null || matches.length == 0)
//        {
//            return null;
//        }
//        if (matches.length > 1)
//        {
//            throw new RPCException("Multiple SSNs!");
//        }
//        return select(matches[0].getPatientId());
//    }

    public String getLocalPid(String mpiPID) throws RPCException {
        String arg = "$O(^DPT(\"AICN\"," + mpiPID + ",0))";
        String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        return response;
    }

    public PatientDemographics getPatientDemographics(String patientIen, User user) throws RPCException {
        return patientDemoSelect(patientIen, user);
    }

    public PatientDemographics getPatientDemographicsVPR(String patientIen, User user) throws RPCException {
        return patientDemoSelectVPR(patientIen, user);
    }


    private void addHomeData(PatientDemographics patientDemo) {
        try {
            String arg = "$G(^DPT(" + cxn.getPatientIen() + ",.11))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            if ("".equals(response)) {
                return;
            }
            String[] flds = StringUtils.split(response, StringUtils.CARET);

            Address homeAddress = new Address();

            homeAddress.setType("HOME");
            if (flds.length > 0) {
                homeAddress.setStreet1(flds[0]);
            }
            if (flds.length > 1) {
                homeAddress.setStreet2(flds[1]);
            }
            if (flds.length > 2) {
                homeAddress.setCity(flds[3]);
            }
            if (flds.length > 3 && !"".equals(flds[4])) {
                String stateName = getStateName(flds[4]);

                if (!StringUtils.isEmpty(stateName)) {
                    homeAddress.setStateProvince(stateName);
                }
            }

            if (flds.length > 4) {
                homeAddress.setPostalCode(flds[5]);
            }

            List<Address> addressList = new ArrayList<Address>();
            addressList.add(homeAddress);
            patientDemo.setAddress(addressList);

            String[] phones = getPersonalPhones(cxn.getPatientIen());

            if (phones != null) {
                if (phones.length > 0 && !"".equals(phones[0])) {
                    patientDemo.setPhone1(phones[0]);
                }

                if (phones.length > 1 && !"".equals(phones[1])) {
                    patientDemo.setPhone2(phones[1]);
                }
            }
        } catch (RPCException e) {
            //SB: error pulling patient home data, data my not be empty, log as a warning
            LOGGER.warn(e.getMessage(), e);
        }
    }

    private String getStateName(String stateIen) throws RPCException {
        String rtc;
        if (!StringUtils.isNumeric(stateIen)) {
            rtc = "";
        } else {
            String arg = "$P($G(^DIC(5," + stateIen + ",0)),U,1)";
    
            rtc = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
        }
        return rtc;
    }

    private String[] getPersonalPhones(String patientIen) throws RPCException {
        String[] rtc;
        if (!StringUtils.isNumeric(patientIen)) {
            rtc = null;
        } else {
            String arg = "$G(^DPT(" + patientIen + ",.13))";
            String response = RPCUtils.getVariableValue(vistaQueryFactory, cxn, arg);
            if ("".equals(response)) {
                rtc = null;
            } else {
                String[] flds = StringUtils.split(response, StringUtils.CARET);
                String[] phoneNumbers = new String[3];
        
                //home phone
                if (flds.length > 0) {
                    phoneNumbers[0] = flds[0];
                }
                //work phone
                if (flds.length > 1) {
                    phoneNumbers[1] = flds[1];
                }
                //cell phone
                if (flds.length > 3) {
                    phoneNumbers[2] = flds[3];
                }
        
        
                rtc = phoneNumbers;
            }
        }
        return rtc;
    }

    private void addPCPData(PatientDemographics patientDemo) {
        try {
            VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT1 PRCARE");
            vquery.addParameter(VistaQuery.LITERAL, cxn.getPatientIen());
            String response = (String) cxn.query(vquery.buildMessage());
            if (StringUtils.isEmpty(response) || "^^".equals(response) || "^^^".equals(response)) {
                return;
            }

            String[] flds = StringUtils.split(response, StringUtils.CARET);

            if (flds.length > 0) {
                patientDemo.setClinic(flds[0]);
            }
            if (flds.length > 1) {
                patientDemo.setPrimaryProvider(flds[1]);
            }
        } catch (RPCException e) {
            //SB: error pulling patient pcp data, data my not be empty, log as a warning
            LOGGER.warn(e.getMessage(), e);
        }
    }

//    private void addPatientFlags(PatientDemographics patientDemo) throws RPCException {
//        Map<String, String> patFlags = getPatientFlags(cxn.getPatientIen());
//
//        addPatientFlags(patientDemo, patFlags);
//    }

    private void addPatientFlags(PatientDemographics patientDemo, Map<String, String> patFlags) {
        if (patFlags != null) {
    
            Set<Map.Entry<String, String>> entries = patFlags.entrySet();
    
            int count = 0;
    
            Iterator entryIterator = entries.iterator();
    
            while (entryIterator.hasNext()) {
                Map.Entry next = (Map.Entry) entryIterator.next();
    
                if (patientDemo.getCodeGreen() == null) {
                    patientDemo.setCodeGreen("");
                }
    
                if (count > 0) {
                    patientDemo.setCodeGreen(patientDemo.getCodeGreen() + "\r\n\r\n");
                }
    
                patientDemo.setCodeGreen(patientDemo.getCodeGreen() + (String) next.getValue());
                count++;
            }
        }
    }


//    private Map<String, String> getPatientFlags(String patientIen) throws RPCException {
//        String request = buildGetHasPatientFlagsRequest(patientIen);
//        String response = (String) cxn.query(request);
//        Map<String, String> patFlags = toPatientFlags(response);
//
//        if (patFlags == null) return null;
//
//        Set<Map.Entry<String, String>> entries = patFlags.entrySet();
//        Map<String, String> flags = new LinkedHashMap<String, String>();
//        for (Map.Entry<String, String> entry : entries) {
//            String flagQuery = buildGetPatientFlagRequest(patientIen, entry.getKey());
//            String flagResp = (String) cxn.query(flagQuery);
//            flags.put(entry.getKey(), flagResp);
//        }
//
//        return flags;
//    }

//    private String buildGetHasPatientFlagsRequest(String patientIen) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORPRF HASFLG");
//        vq.addParameter(vq.LITERAL, patientIen);
//        return vq.buildMessage();
//    }

//    private String buildGetPatientFlagRequest(String patientIen, String flagRecordId) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORPRF GETFLG");
//        vq.addParameter(vq.LITERAL, patientIen);
//        vq.addParameter(vq.LITERAL, flagRecordId);
//        return vq.buildMessage();
//    }

//    private Map<String, String> toPatientFlags(String response) {
//        if (response.equals("")) {
//            return null;    //Error?
//        }
//        String[] lines = StringUtils.split(response, StringUtils.CRLF);
//        Map<String, String> result = new LinkedHashMap<String, String>();
//        for (int i = 0; i < lines.length; i++) {
//            if (lines[i].equals("")) {
//                continue;
//            }
//            String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
//            result.put(flds[0], flds[1]);
//        }
//        return result;
//    }

    public PatientDemographics getPatientDemographics(User user) throws RPCException {
        return getPatientDemographics(cxn.getPatientIen(), user);
    }

    public PatientDemographics getPatientDemographicsVPR(User user) throws RPCException {
        return getPatientDemographicsVPR(cxn.getPatientIen(), user);
    }


    public PatientDemographicsDetail getPatientDemographicsDetail(String patientIen) throws RPCException {
        String query = buildDemographicDetailsQuery(patientIen);
        String response = (String) cxn.query(query);
        PatientDemographicsDetail demoDetails = new PatientDemographicsDetail();
        demoDetails.setReport(response);
        fillCommonDataBeanValues(demoDetails);
        return demoDetails;
    }

    public PatientDemographicsDetail getPatientDemographicsDetail() throws RPCException {
        return getPatientDemographicsDetail(cxn.getPatientIen());
    }

    private String buildDemographicDetailsQuery(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWPT PTINQ");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    public Alert getSensitivePatientAlert() throws RPCException {
        String query = buildSensitivePatientQuery(cxn.getPatientIen());
        String response = (String) cxn.query(query);
        String[] lines = StringUtils.split(response, StringUtils.CRLF);
        Alert sensitiveAlert = new Alert();
        fillCommonDataBeanValues(sensitiveAlert);

        sensitiveAlert.setAlertId(lines[0].trim());
        if (lines.length > 1) {
            StringBuilder alertMessageSB = new StringBuilder();
            for (int i = 1; i < lines.length; i ++) {
                alertMessageSB.append(lines[i]);
                alertMessageSB.append("\n");
            }
            sensitiveAlert.setMessageText(alertMessageSB.toString().trim());
        }

//        -1-RPC/API failed
//        Required variable not defined
//        0-No display/action required
//        Not an employee, not sensitive or not accessing own Patient
//        record
//        1-Display warning message
//        Sensitive - inpatient or a DG SENSITIVITY key holder
//        or Employee and DG SECURITY OFFICER key holder
//        2-Display warning message, require OK to continue and call
//        DG SENSITIVE RECORD BULLETIN RPC to update DG Security Log
//        file and generate Sensitive Record Access mail message.
//        Sensitive - not an inpatient and not a key holder
//        or Employee/not a DG SECURITY OFFICER key holder
//        3-Access to record denied
//        Accessing own Patient file record
//        4-Access to Patient file (#2) records denied
//        SSN not defined

        //return true if response is greater than 0
        return sensitiveAlert;
    }

    private String buildSensitivePatientQuery(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("DG SENSITIVE RECORD ACCESS");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    private PatientDemographics getExtraDemoDetailsFromVPR(String patientIen, PatientDemographics demographics, User user) throws RPCException {
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, VistaConstants.VPR_CONTEXT, true);

        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.DEMOGRAPHICS);
        String response = (String) cxn.query(request);

//        PatientDemographics returnDemographics = parseVPRDemographics(response, demographics);
//        return returnDemographics;
        return  parseVPRDemographics(response, demographics);
    }


    private PatientDemographics parseVPRFlags(String response, PatientDemographics demographics) throws RPCException {
        PatientDemographics rtc;
        if ("".equals(response)) {
            rtc = demographics;
        } else {
    
            final String XPATH_FLAG = "//flag";
    
            Document doc = null;
    
            try {
                doc = parseXMLDocument(response);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
    
            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();
    
            XPathExpression exprFlag = null;
    
            try {
                exprFlag = xpath.compile(XPATH_FLAG);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList flagsNodeList = null;
            try {
                flagsNodeList = (NodeList) exprFlag.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            List<VPRFlagParser> flagParserList = new ArrayList<VPRFlagParser>();
            int listLength = flagsNodeList.getLength();
            for (int flagNum = 0; flagNum < listLength; flagNum++) {
                Node flagNode = flagsNodeList.item(flagNum);
                flagNode.getParentNode().removeChild(flagNode);
    
                flagParserList.add(new VPRFlagParser(flagNode));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<Flag>> futures = null;
            try {
                futures = executorService.invokeAll(flagParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
    
                if (futures == null) {
                    throw new RPCException("Failed to parse flags results: flag node parser futures list is null.");
                }
    
            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }
    
            Iterator<Future<Flag>> futuresItr = futures.iterator();
            List<Flag> flagsList = new ArrayList<Flag>();
            while (futuresItr.hasNext()) {
                try {
                    Future<Flag> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Flag Data");
                    } else {
                        Flag flg = future.get();
                        if (flg != null && !StringUtils.isEmpty(flg.getName())) {
                            if(demographics.getSite() != null){
                                flg.setSite(demographics.getSite());
                            }
                            flagsList.add(flg);
                        }
                    }
    
                } catch (InterruptedException e) {
                    LOGGER.error(e.getMessage(), e);
                } catch (ExecutionException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
    
            if (!flagsList.isEmpty()) {
                demographics.setFlags(flagsList.toArray(new Flag[0]));
            }
    
            rtc = demographics;
        }
        return rtc;
    }

    private static class VPRFlagParser implements Callable<Flag> {
        private Node flagNode;

        public VPRFlagParser(Node flagNode) {
            this.flagNode = flagNode;
        }


        public Flag call() throws RPCException {
            final String XPATH_CATEGORY = "./category/@value";
            final String XPATH_CONTENT = "./content";
            final String XPATH_ID = "./id/@value";
            final String XPATH_NAME = "./name/@value";
            final String XPATH_ORIG_SITE = "./origSite/@name";
            final String XPATH_OWN_SITE = "./ownSite/@name";
            final String XPATH_REVIEW_DUE = "./reviewDue/@value";
            final String XPATH_TYPE = "./type/@value";
            final String XPATH_APPROVED_BY = "./approvedBy/@name";
            final String XPATH_ASSIGNED = "./assigned/@value";
            final String XPATH_DOCUMENT_CODE = "./document/@code";
            final String XPATH_DOCUMENT_NAME = "./document/@name";

            Flag flag = new Flag();
            XPathExpression exprCategory = null;
            XPathExpression exprContent = null;
            XPathExpression exprId = null;
            XPathExpression exprName = null;
            XPathExpression exprOrigSite = null;
            XPathExpression exprOwnSite = null;
            XPathExpression exprReviewDate = null;
            XPathExpression exprType = null;
            XPathExpression exprApprovedBy = null;
            XPathExpression exprAssigned = null;
            XPathExpression exprDocumentCode = null;
            XPathExpression exprDocumentName = null;
            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();

            try {
                exprCategory = xpath.compile(XPATH_CATEGORY);
                exprContent = xpath.compile(XPATH_CONTENT);
                exprId = xpath.compile(XPATH_ID);
                exprName = xpath.compile(XPATH_NAME);
                exprOrigSite = xpath.compile(XPATH_ORIG_SITE);
                exprOwnSite = xpath.compile(XPATH_OWN_SITE);
                exprReviewDate = xpath.compile(XPATH_REVIEW_DUE);
                exprType = xpath.compile(XPATH_TYPE);
                exprApprovedBy = xpath.compile(XPATH_APPROVED_BY);
                exprAssigned = xpath.compile(XPATH_ASSIGNED);
                exprDocumentCode = xpath.compile(XPATH_DOCUMENT_CODE);
                exprDocumentName = xpath.compile(XPATH_DOCUMENT_NAME);

            } catch (Exception e) {
                throw new RPCException(e);
            }

            String category = RPCUtil.evaluateNodeToString(flagNode, exprCategory, LOGGER);
            String content = RPCUtil.evaluateNodeToString(flagNode, exprContent, LOGGER);
            String id = RPCUtil.evaluateNodeToString(flagNode, exprId, LOGGER);
            String name = RPCUtil.evaluateNodeToString(flagNode, exprName, LOGGER);
            String origSite = RPCUtil.evaluateNodeToString(flagNode, exprOrigSite, LOGGER);
            String ownSite = RPCUtil.evaluateNodeToString(flagNode, exprOwnSite, LOGGER);
            String reviewDate = RPCUtil.evaluateNodeToString(flagNode, exprReviewDate, LOGGER);
            String type = RPCUtil.evaluateNodeToString(flagNode, exprType, LOGGER);
            String approvedBy = RPCUtil.evaluateNodeToString(flagNode, exprApprovedBy, LOGGER);
            String assigned = RPCUtil.evaluateNodeToString(flagNode, exprAssigned, LOGGER);
            String documentCode = RPCUtil.evaluateNodeToString(flagNode, exprDocumentCode, LOGGER);
            String documentName = RPCUtil.evaluateNodeToString(flagNode, exprDocumentName, LOGGER);

            flag.setCategory(category);
            flag.setContent(content);
            flag.setId(id);
            flag.setName(name);
            flag.setOriginalSite(origSite);
            flag.setOwnerSite(ownSite);
            flag.setType(type);
            flag.setApprovedBy(approvedBy);
            flag.setDocumentName(documentName);
            flag.setDocumentId(documentCode);

            if (reviewDate != null) {
                Calendar reviewDateCal = Timestamp.toDateTime(reviewDate);
                flag.setReviewDue(reviewDateCal);
            }
            if (assigned != null) {
                Calendar assignedCal = Timestamp.toDateTime(assigned);
                flag.setAssigned(assignedCal);
            }

            return flag;
        }
    }

    private PatientDemographics parseVPRDemographics(String response) throws RPCException {
        PatientDemographics demographics = new PatientDemographics();
        fillCommonDataBeanValues(demographics);
        return parseVPRDemographics(response, demographics);
    }

    private PatientDemographics parseVPRDemographics(String response, PatientDemographics demographics) throws RPCException {
        PatientDemographics rtc;
        if (!"".equals(response)) {
    
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            final String XPATH_FULLNAME = "./fullName/@value";
            final String XPATH_GENDER = "./gender/@value";
            final String XPATH_DOB = "./dob/@value";
            final String XPATH_SSN = "./ssn/@value";
            final String XPATH_ID = "./id/@value";
            // AGE??
            final String XPATH_SENSITIVE = "./sensitive/@value";
            final String XPATH_SC_PERCENT = "./scPercent/@value";
            final String XPATH_ADDRESS = "./address";
            final String XPATH_STREET1 = "./@streetLine1";
            final String XPATH_STREET2 = "./@streetLine2";
            final String XPATH_STREET3 = "./@streetLine3";
            final String XPATH_CITY = "./@city";
            final String XPATH_STATE_PROVINCE = "./@stateProvince";
            final String XPATH_POSTAL_CODE = "./@postalCode";
            final String XPATH_TELECOM_LIST = "./telecomList";
            final String XPATH_TELECOM = "./telecom";
//            final String XPATH_USAGE_TYPE = "./@usageType";
    
    
            final String XPATH_PATIENT = "//patient";
            final String XPATH_ETHNICITIES = "./ethnicities";
            final String XPATH_ETHNICITY = "./ethnicity";
            final String XPATH_VALUE = "./@value";
            final String XPATH_RACES = "./races";
            final String XPATH_RACE = "./race";
    
            final String XPATH_FLAGS = "./flags";
            final String XPATH_FLAG = "./flag";
            final String XPATH_NAME = "./@name";
            final String XPATH_TEXT = "./@text";
    
            final String XPATH_ELIGIBILITIES = "./eligibilities";
            final String XPATH_ELIGIBILITY = "./eligibility";
            final String XPATH_PRIMARY = "./@primary";
            final String XPATH_ELIGIBILITY_STATUS = "./eligibilityStatus/@value";
            final String XPATH_PC_ASSIGNED = "./pcAssigned/@value";
            final String XPATH_PC_TEAM_NAME = "./pcTeam/@name";
    
            final String XPATH_PC_PROVIDER = "./pcProvider";
            final String XPATH_CODE = "./@code";
            final String XPATH_OFFICE_PHONE = "./@officePhone";
            final String XPATH_ANALOG_PAGER = "./@analogPager";
            final String XPATH_FAX = "./@fax";
            final String XPATH_EMAIL = "./@email";
            final String XPATH_TAXOMOMY_CODE = "./@taxonomyCode";
            final String XPATH_PROVIDER_TYPE = "./@providerType";
            final String XPATH_CLASSIFICATION = "./@classification";
            final String XPATH_SPECIALIZATION = "./@specialization";
            final String XPATH_SERVICE = "./@service";
    
    
            Document document = null;
            XPathExpression exprPatient = null;
            XPathExpression exprEthnicities = null;
            XPathExpression exprEthnicity = null;
            XPathExpression exprRaces = null;
            XPathExpression exprRace = null;
            XPathExpression exprValue = null;
    
            XPathExpression exprFullName = null;
            XPathExpression exprGender = null;
            XPathExpression exprDob = null;
            XPathExpression exprSsn = null;
            XPathExpression exprId = null;
            XPathExpression exprSensitive = null;
            XPathExpression exprScPercent = null;
            XPathExpression exprAddress = null;
            XPathExpression exprStreetLine2 = null;
            XPathExpression exprStreetLine1 = null;
            XPathExpression exprStreetLine3 = null;
            XPathExpression exprCity = null;
            XPathExpression exprStateProvince = null;
            XPathExpression exprPostalCode = null;
            XPathExpression exprTelecomList = null;
            XPathExpression exprTelecom = null;
//            XPathExpression exprUsageType = null;
            XPathExpression exprFlags = null;
            XPathExpression exprFlag = null;
            XPathExpression exprName = null;
            XPathExpression exprText = null;
    
            XPathExpression exprEligibility = null;
            XPathExpression exprPrimary = null;
            XPathExpression exprEligibilityStatus = null;
            XPathExpression exprPcAssigned = null;
            XPathExpression exprPcTeamName = null;
            XPathExpression exprPcProvider = null;
            XPathExpression exprCode = null;
            XPathExpression exprTaxonomyCode = null;
            XPathExpression exprProviderType = null;
            XPathExpression exprClassification = null;
            XPathExpression exprSpecialization = null;
            XPathExpression exprService = null;
            XPathExpression exprEligibilities = null;
            XPathExpression exprOfficePhone = null;
            XPathExpression exprAnalogPager = null;
            XPathExpression exprFax = null;
            XPathExpression exprEmail = null;
    
            try {
                document = parseXMLDocument(sbld.toString());
                XPathFactory xPathFactory = XPathFactory.newInstance();
                XPath xpath = xPathFactory.newXPath();
                exprPatient = xpath.compile(XPATH_PATIENT);
                exprEthnicities = xpath.compile(XPATH_ETHNICITIES);
                exprEthnicity = xpath.compile(XPATH_ETHNICITY);
                exprRaces = xpath.compile(XPATH_RACES);
                exprRace = xpath.compile(XPATH_RACE);
                exprValue = xpath.compile(XPATH_VALUE);
    
                exprFullName = xpath.compile(XPATH_FULLNAME);
                exprGender = xpath.compile(XPATH_GENDER);
                exprDob = xpath.compile(XPATH_DOB);
                exprSsn = xpath.compile(XPATH_SSN);
                exprId = xpath.compile(XPATH_ID);
                exprSensitive = xpath.compile(XPATH_SENSITIVE);
                exprScPercent = xpath.compile(XPATH_SC_PERCENT);
                exprAddress = xpath.compile(XPATH_ADDRESS);
                exprStreetLine1 = xpath.compile(XPATH_STREET1);
                exprStreetLine2 = xpath.compile(XPATH_STREET2);
                exprStreetLine3 = xpath.compile(XPATH_STREET3);
                exprCity = xpath.compile(XPATH_CITY);
                exprStateProvince = xpath.compile(XPATH_STATE_PROVINCE);
                exprPostalCode = xpath.compile(XPATH_POSTAL_CODE);
                exprTelecomList = xpath.compile(XPATH_TELECOM_LIST);
                exprTelecom = xpath.compile(XPATH_TELECOM);
//                exprUsageType = xpath.compile(XPATH_USAGE_TYPE);
    
                exprFlags = xpath.compile(XPATH_FLAGS);
                exprFlag = xpath.compile(XPATH_FLAG);
                exprName = xpath.compile(XPATH_NAME);
                exprText = xpath.compile(XPATH_TEXT);
    
                exprEligibilities = xpath.compile(XPATH_ELIGIBILITIES);
                exprEligibility = xpath.compile(XPATH_ELIGIBILITY);
                exprPrimary = xpath.compile(XPATH_PRIMARY);
                exprEligibilityStatus = xpath.compile(XPATH_ELIGIBILITY_STATUS);
                exprPcAssigned = xpath.compile(XPATH_PC_ASSIGNED);
                exprPcTeamName = xpath.compile(XPATH_PC_TEAM_NAME);
                exprPcProvider = xpath.compile(XPATH_PC_PROVIDER);
                exprCode = xpath.compile(XPATH_CODE);
                exprTaxonomyCode = xpath.compile(XPATH_TAXOMOMY_CODE);
                exprProviderType = xpath.compile(XPATH_PROVIDER_TYPE);
                exprClassification = xpath.compile(XPATH_CLASSIFICATION);
                exprSpecialization = xpath.compile(XPATH_SPECIALIZATION);
                exprService = xpath.compile(XPATH_SERVICE);
                exprOfficePhone = xpath.compile(XPATH_OFFICE_PHONE);
                exprAnalogPager = xpath.compile(XPATH_ANALOG_PAGER);
                exprFax = xpath.compile(XPATH_FAX);
                exprEmail = xpath.compile(XPATH_EMAIL);
    
            } catch (XPathExpressionException xe) {
                throw new RPCException(xe);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            Node patientNode = RPCUtil.evaluateNodeToNode(document, exprPatient, LOGGER);
            if (patientNode != null) {
                Node racesNode = RPCUtil.evaluateNodeToNode(patientNode, exprRaces, LOGGER);
                if (racesNode != null) {
                    NodeList raceNodeList = RPCUtil.evaluateNodeToNodeList(racesNode, exprRace, LOGGER);
                    if (raceNodeList != null) {
                        int raceNodeListLength = raceNodeList.getLength();
                        List<Code> raceList = new ArrayList<Code>();
                        for (int raceNum = 0; raceNum < raceNodeListLength; raceNum++) {
                            Node raceNode = raceNodeList.item(raceNum);
                            raceNode.getParentNode().removeChild(raceNode);

                            Code race = new Code();
                            race.setSystem("CDC");

                            String raceCode = RPCUtil.evaluateNodeToString(raceNode, exprValue, LOGGER);
                            race.setCode(raceCode);
                            raceList.add(race);
                        }

                        if (!raceList.isEmpty()) {
                            demographics.setRace(raceList.toArray(new Code[0]));
                        }
                    }
                }

                Node ethnicitiesNode = RPCUtil.evaluateNodeToNode(patientNode, exprEthnicities, LOGGER);
                if (ethnicitiesNode != null) {
                    NodeList ethnicityNodeList = RPCUtil.evaluateNodeToNodeList(ethnicitiesNode, exprEthnicity, LOGGER);
                    if (ethnicityNodeList != null) {
                        int ethnicityNodeListLength = ethnicityNodeList.getLength();
                        List<Code> ethnicityList = new ArrayList<Code>();
                        for (int ethnicityNum = 0; ethnicityNum < ethnicityNodeListLength; ethnicityNum++) {
                            Node ethnicityNode = ethnicityNodeList.item(ethnicityNum);
                            ethnicityNode.getParentNode().removeChild(ethnicityNode);

                            Code ethnicity = new Code();
                            ethnicity.setSystem("CDC");

                            String ethnicityCode = RPCUtil.evaluateNodeToString(ethnicityNode, exprValue, LOGGER);
                            ethnicity.setCode(ethnicityCode);
                            ethnicityList.add(ethnicity);
                        }

                        if (!ethnicityList.isEmpty()) {
                            demographics.setEthnicity(ethnicityList.toArray(new Code[0]));
                        }
                    }
                }

                String fullName = RPCUtil.evaluateNodeToString(patientNode, exprFullName, LOGGER);
                demographics.setName(fullName);
                String gender = RPCUtil.evaluateNodeToString(patientNode, exprGender, LOGGER);
                demographics.setGender(gender);
                String dobString = RPCUtil.evaluateNodeToString(patientNode, exprDob, LOGGER);
                dobString = Utils.formatDate(Utils.getDate(DateUtils.trimTime(Timestamp.toUtcString(dobString))));
                demographics.setDob(dobString);
                String ssnString = RPCUtil.evaluateNodeToString(patientNode, exprSsn, LOGGER);
                demographics.setSSN(ssnString);
                String id = RPCUtil.evaluateNodeToString(patientNode, exprId, LOGGER);
                demographics.setICN(id);
                String sensitive = RPCUtil.evaluateNodeToString(patientNode, exprSensitive, LOGGER);
                if (sensitive != null && "1".equals(sensitive)) {
                    demographics.setSensitive(true);
                }
                String scPercent = RPCUtil.evaluateNodeToString(patientNode, exprScPercent, LOGGER);
                demographics.setPercentServiceConnected(scPercent);

                Node addressNode = RPCUtil.evaluateNodeToNode(patientNode, exprAddress, LOGGER);
                if (addressNode != null) {
                    String streetLine1 = RPCUtil.evaluateNodeToString(addressNode, exprStreetLine1, LOGGER);
                    String streetLine2 = RPCUtil.evaluateNodeToString(addressNode, exprStreetLine2, LOGGER);
                    // String streetLine3 = RPCUtil.evaluateNodeToString(addressNode, exprStreetLine3, logger);
                    String city = RPCUtil.evaluateNodeToString(addressNode, exprCity, LOGGER);
                    String stateProvince = RPCUtil.evaluateNodeToString(addressNode, exprStateProvince, LOGGER);
                    String postalCode = RPCUtil.evaluateNodeToString(addressNode, exprPostalCode, LOGGER);

                    Address address = new Address();
                    address.setStreet1(streetLine1);
                    address.setStreet2(streetLine2);
                    address.setCity(city);
                    address.setStateProvince(stateProvince);
                    address.setPostalCode(postalCode);

                    List<Address> addressList = new ArrayList<Address>();
                    addressList.add(address);
                    demographics.setAddress(addressList);
                }

                Node telecomListNode = RPCUtil.evaluateNodeToNode(patientNode, exprTelecomList, LOGGER);
                if (telecomListNode != null) {
                    NodeList telecomNodeList = RPCUtil.evaluateNodeToNodeList(telecomListNode, exprTelecom, LOGGER);
                    if (telecomNodeList != null) {
                        int telecomNodeListLength = telecomNodeList.getLength();
                        List<String> phoneStringList = new ArrayList<String>();
                        for (int telecomNum = 0; telecomNum < telecomNodeListLength; telecomNum++) {
                            Node telecomNode = telecomNodeList.item(telecomNum);
                            telecomNode.getParentNode().removeChild(telecomNode);

                            String telecomString = RPCUtil.evaluateNodeToString(telecomNode, exprValue, LOGGER);
//                        String usageType = RPCUtil.evaluateNodeToString(telecomNode, exprUsageType, LOGGER);

                            phoneStringList.add(telecomString);
                        }

                        if (!phoneStringList.isEmpty()) {
                            demographics.setPhone1(phoneStringList.get(0));
                            if (phoneStringList.size() > 1) {
                                demographics.setPhone2(phoneStringList.get(1));
                            }
                        }
                    }
                }

                Node flagsNode = RPCUtil.evaluateNodeToNode(patientNode, exprFlags, LOGGER);
                if (flagsNode != null) {
                    NodeList flagNodeList = RPCUtil.evaluateNodeToNodeList(flagsNode, exprFlag, LOGGER);
                    if (flagNodeList != null) {
                        int flagNodeListLength = flagNodeList.getLength();
                        Map<String, String> flags = new LinkedHashMap<String, String>();
                        for (int flagNum = 0; flagNum < flagNodeListLength; flagNum++) {
                            Node flagNode = flagNodeList.item(flagNum);
                            flagNode.getParentNode().removeChild(flagNode);

                            String flagName = RPCUtil.evaluateNodeToString(flagNode, exprName, LOGGER);
                            String flagText = RPCUtil.evaluateNodeToString(flagNode, exprText, LOGGER);

                            StringBuilder flagSB = new StringBuilder("Alert/Flag: ");
                            flagSB.append(flagName);
                            flagSB.append("\r\n");
                            flagSB.append(flagText);
                            flags.put(flagName, flagSB.toString());
                        }

                        if (!flags.isEmpty()) {
                            addPatientFlags(demographics, flags);
                        }
                    }
                }

                Node eligibilitiesNode = RPCUtil.evaluateNodeToNode(patientNode, exprEligibilities, LOGGER);
                if (eligibilitiesNode != null) {
                    NodeList eligibilitiesNodeList = RPCUtil.evaluateNodeToNodeList(eligibilitiesNode, exprEligibility, LOGGER);
                    if (eligibilitiesNodeList != null) {
                        int eligibilitiesNodeListLength = eligibilitiesNodeList.getLength();
                        for (int i = 0; i < eligibilitiesNodeListLength; i++) {
                            Node eligibilityNode = eligibilitiesNodeList.item(i);
                            eligibilityNode.getParentNode().removeChild(eligibilityNode);

                            String eligibilityPrimary = RPCUtil.evaluateNodeToString(eligibilityNode, exprPrimary, LOGGER);

                            if (eligibilityPrimary != null && "1".equals(eligibilityPrimary)) {
                                String eligibilityName = RPCUtil.evaluateNodeToString(eligibilityNode, exprName, LOGGER);
                                String eligibilityStatus = RPCUtil.evaluateNodeToString(patientNode, exprEligibilityStatus, LOGGER);
                                demographics.setEligibility(eligibilityName);
                                demographics.setEligibilityStatus(eligibilityStatus);
                                break;
                            }
                        }
                    }
                }
                String pcTeamName = RPCUtil.evaluateNodeToString(patientNode, exprPcTeamName, LOGGER);
                demographics.setPcTeam(pcTeamName);
                String pcAssignedString = RPCUtil.evaluateNodeToString(patientNode, exprPcAssigned, LOGGER);
                Calendar pcAssigned = null;
                if (!StringUtils.isEmpty(pcAssignedString)) {
                    pcAssigned = Timestamp.toDateTime(pcAssignedString);
                    demographics.setPcAssigned(pcAssigned);
                    demographics.setPcmDates(pcAssigned.getTime());
                }

                Node pcProviderNode = RPCUtil.evaluateNodeToNode(patientNode, exprPcProvider, LOGGER);
                if (pcProviderNode != null) {
                    Provider pcProvider = new Provider();

                    String pcProviderCode = RPCUtil.evaluateNodeToString(pcProviderNode, exprCode, LOGGER);
                    String pcProviderName = RPCUtil.evaluateNodeToString(pcProviderNode, exprName, LOGGER);
                    String pcProviderTaxonomyCode = RPCUtil.evaluateNodeToString(pcProviderNode, exprTaxonomyCode, LOGGER);
                    String pcProviderProviderType = RPCUtil.evaluateNodeToString(pcProviderNode, exprProviderType, LOGGER);
                    String pcProviderClassification = RPCUtil.evaluateNodeToString(pcProviderNode, exprClassification, LOGGER);
                    String pcProviderSpecialization = RPCUtil.evaluateNodeToString(pcProviderNode, exprSpecialization, LOGGER);
                    String pcProviderService = RPCUtil.evaluateNodeToString(pcProviderNode, exprService, LOGGER);
                    String pcProviderOfficePhone = RPCUtil.evaluateNodeToString(pcProviderNode, exprOfficePhone, LOGGER);
                    String pcProviderAnalogPager = RPCUtil.evaluateNodeToString(pcProviderNode, exprAnalogPager, LOGGER);
                    String pcProviderFax = RPCUtil.evaluateNodeToString(pcProviderNode, exprFax, LOGGER);
                    String pcProviderEmail = RPCUtil.evaluateNodeToString(pcProviderNode, exprEmail, LOGGER);

                    Node pcProviderAddressNode = RPCUtil.evaluateNodeToNode(pcProviderNode, exprAddress, LOGGER);
                    if (pcProviderAddressNode != null) {
                        String pcProviderStreetLine1 = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprStreetLine1, LOGGER);
                        String pcProviderStreetLine2 = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprStreetLine2, LOGGER);
                        String pcProviderStreetLine3 = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprStreetLine3, LOGGER);
                        String pcProviderCity = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprCity, LOGGER);
                        String pcProviderStateProvince = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprStateProvince, LOGGER);
                        String pcProviderPostalCode = RPCUtil.evaluateNodeToString(pcProviderAddressNode, exprPostalCode, LOGGER);

                        pcProvider.setStreet1(pcProviderStreetLine1);
                        pcProvider.setStreet2(pcProviderStreetLine2);
                        pcProvider.setStreet3(pcProviderStreetLine3);
                        pcProvider.setCity(pcProviderCity);
                        pcProvider.setStateProvince(pcProviderStateProvince);
                        pcProvider.setPostalCode(pcProviderPostalCode);
                    }
                    pcProvider.setCode(pcProviderCode);
                    pcProvider.setName(pcProviderName);
                    demographics.setPrimaryProvider(pcProviderName);
                    pcProvider.setTaxonomyCode(pcProviderTaxonomyCode);
                    pcProvider.setProviderType(pcProviderProviderType);
                    pcProvider.setClassification(pcProviderClassification);
                    pcProvider.setSpecialization(pcProviderSpecialization);
                    pcProvider.setService(pcProviderService);
                    pcProvider.setPhone(pcProviderOfficePhone);
                    demographics.setPcmPhone(pcProviderOfficePhone);
                    pcProvider.setAnalogPager(pcProviderAnalogPager);
                    pcProvider.setFax(pcProviderFax);
                    pcProvider.setEmail(pcProviderEmail);

                    demographics.setPcProvider(pcProvider);
                }
            }
        }
        rtc = demographics;
        
        return rtc;
    }

}
