/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.Procedure;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;

/**
 *
 *
 */
public class ProceduresRPC extends BaseRPC {

//    private static final Logger LOGGER = LoggerFactory.getLogger(ProceduresRPC.class);
//    private static final String CPT_URN_PREFIX = "urn:cpt:";

    public ProceduresRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Procedure[] getPatientEncounterProcedures(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        EncountersRPC encountersRPC = new EncountersRPC(cxn);

        Encounter[] visits = encountersRPC.getPatientEncounters(patientIen, startDate, endDate);

        return getProceduresFromEncounters(visits);
    }

    public Procedure[] getPatientEncounterProcedures(Calendar startDate, Calendar endDate) throws RPCException {
        return getPatientEncounterProcedures(cxn.getPatientIen(), startDate, endDate);
    }

    public Procedure[] getProceduresFromEncounters(Encounter[] encounters) {
        ArrayList<Procedure> procedureList = new ArrayList<Procedure>();
        for (Encounter encounter : encounters) {
            Procedure[] procedures = encounter.getProcedures();
            if (procedures != null && procedures.length > 0) {
                for (Procedure procedure : procedures) {

                    procedure.setDocuments(encounter.getDocuments());

                    if (procedure.getDate() == null) {
                        procedure.setDate(encounter.getApptDate());
                    }

                    if (procedure.getCode() != null
                            && !Utils.isEmptyString(procedure.getCode())
                            && procedure.getDescription() != null
                            && !Utils.isEmptyString(procedure.getDescription())) {
                        ArrayList<Code> procCodes = new ArrayList<Code>();

                        Code procCode = new Code();
                        procCode.setSystem("CPT");
                        procCode.setDisplay(procedure.getDescription());
                        procCode.setCode(procedure.getCode());

                        procCodes.add(procCode);

                        procedure.setCodes(procCodes.toArray(new Code[0]));
                    }
                }
                procedureList.addAll(Arrays.asList(procedures));
            }
        }

        return procedureList.toArray(new Procedure[0]);
    }

    /**
     * Builds the JSON VPR RPC request string using "cpt" as the type. The problems contain the list of
     * procedures. The RPC will fail if the "VPR APPLICATION PROXY" context is not set. That is set when
     * setting up the connection.
     *
     * @param patientIen Patient IEN
     * @param startDate  Start date of range
     * @param endDate    End date of range
     * @return The RPC response message string
     */
//    private String buildGetProceduresRequest(String patientIen, Calendar startDate, Calendar endDate) {
//        return VPRUtil.buildVPRJSONRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.CPT);
//    }

//    private void logErrorMessage(Exception e) {
//        LOGGER.error(e.getMessage());
//    }

//    private static Calendar toCalendarFromTextTS(String textTS) {
//        // pad up until seconds
//        StringBuilder sbld = new StringBuilder(textTS);
//        if (textTS.length() == 8) {
//            sbld.append("000000");
//        } else if (textTS.length() == 12) {
//            sbld.append("00");
//        }
//        String newString = sbld.toString();
//
//        return new GregorianCalendar(
//                //year
//                Integer.parseInt(StringUtils.CSubstring(newString, 0, 4)),
//                //month
//                Integer.parseInt(StringUtils.CSubstring(newString, 4, 2)) - 1,
//                //day
//                Integer.parseInt(StringUtils.CSubstring(newString, 6, 2)),
//                //hour
//                Integer.parseInt(StringUtils.CSubstring(newString, 8, 2)),
//                //minute
//                Integer.parseInt(StringUtils.CSubstring(newString, 10, 2)),
//                //second
//                Integer.parseInt(StringUtils.CSubstring(newString, 12, 2)));
//    }

}
