/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.RPCUtils;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.Timestamp;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Code;
import gov.va.med.vds.vistabeans.Vitals;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 *
 */
public class VitalsRPC extends BaseRPC {
    private static final Logger LOGGER = LoggerFactory.getLogger(VitalsRPC.class);

//    private static final String VITAL_SIGN = "VITAL SIGN";
    private static final String TEMPERATURE = "TEMPERATURE";
    private static final String PULSE = "PULSE";
    private static final String RESPIRATION = "RESPIRATION";
    private static final String BLOOD_PRESSURE = "BLOOD PRESSURE";
    private static final String HEIGHT = "HEIGHT";
    private static final String WEIGHT = "WEIGHT";
    private static final String PAIN = "PAIN";
    private static final String PULSE_OXIMETRY = "PULSE OXIMETRY";
    private static final String CIRCUMFERENCE_GIRTH = "CIRCUMFERENCE / GIRTH";
    private static final String CENTRAL_VENOUS_PRESSURE = "CENTRAL VENOUS PRESSURE";
    private static final String BODY_MASS_INDEX = "BODY MASS INDEX";

    private static final int VITAL_DETAILS_UNITS = 0;
    private static final int VITAL_DETAILS_QUALIFIERS = 1;

    public VitalsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public Vitals[] getVitals(Calendar startDate, Calendar endDate) throws RPCException {
        return getVitals(cxn.getPatientIen(), startDate, endDate);
    }

    public Vitals[] getVitals(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetVitalSignsRdvRequest(patientIen, Timestamp.fromDateTimeToTextTS(startDate), Timestamp.fromDateTimeToTextTS(endDate), 0);
        String response = (String) cxn.query(request);
        return toVitalSignsFromRdv(response/*, patientIen*/);
    }

    private String buildGetVitalSignsRdvRequest(String patientIen, String fromDate, String toDate, int maxRex) {
        return RPCUtils.buildReportTextRequest(vistaQueryFactory, patientIen, fromDate, toDate, maxRex, "OR_VS:VITAL SIGNS~VS;ORDV04;47");
    }

    private Vitals createVital(String type, String rate, Calendar timeTaken/*, String patientIen*/) {
        Vitals vital = new Vitals();
        vital.setVitalType(type);
        vital.setRate(rate);
        vital.setDateTimeTaken(timeTaken);
        fillCommonDataBeanValues(vital);

        return vital;
    }

    private Vitals[] toVitalSignsFromRdv(String response/*, String patientIen*/) {
        Vitals[] rtc;
        if ("".equals(response)) {
            rtc = new Vitals[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            lines = StringUtils.trimArray(lines);
            ArrayList<Vitals> subList = null;
            ArrayList<Vitals> vitalList = new ArrayList<Vitals>();
            Calendar vitalTimeTaken = null;
    
            String qualifiers = null;
            String units = null;
    
            for (int i = 0; i < lines.length; i++) {
                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
                int fldnum = Integer.parseInt(flds[0]);
                String vitalRate = null;
    
                switch (fldnum) {
                    case 1:
    
                        //if the beginning of a new vital results sequence, add the previous vital results to master list
                        if (subList != null) {
                            if (qualifiers != null) {
                                setVitalSignDetails(VITAL_DETAILS_QUALIFIERS, subList, qualifiers);
                            }
    
                            if (units != null) {
                                setVitalSignDetails(VITAL_DETAILS_UNITS, subList, units);
                            }
    
                            vitalList.addAll(subList);
    
                            subList = null;
                            vitalTimeTaken = null;
                            qualifiers = null;
                            units = null;
                        }
    
                        subList = new ArrayList<Vitals>();
    
    //                    String[] subflds = StringUtils.split(flds[1], StringUtils.SEMICOLON);
    //                    if (subflds.length == 1)
    //                    {
    //                        vitalSite = new KeyValuePair<String, String>((String) cxn.getSiteCode().getKey(), flds[1]);
    //                    }
    //                    else
    //                    {
    //                        vitalSite = new KeyValuePair<String, String>(subflds[1], subflds[0]);
    //                    }
                        break;
                    case 2:
                        if (flds.length == 2) {
                            vitalTimeTaken = Utils.getCal(Timestamp.toUtcFromRdv(flds[1]));
                        }
                        break;
                    case 3:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(TEMPERATURE, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 4:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(PULSE, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 5:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(RESPIRATION, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 6:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(BLOOD_PRESSURE, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 7:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(HEIGHT, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 8:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(WEIGHT, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 9:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(PAIN, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 10:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(PULSE_OXIMETRY, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 11:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(CENTRAL_VENOUS_PRESSURE, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 12:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(CIRCUMFERENCE_GIRTH, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 15:
                        if (flds.length == 2) {
                            qualifiers = flds[1];
                        }
                        break;
                    case 16:
                        if (flds.length == 2) {
                            vitalRate = flds[1];
                            if (subList != null) {
                                subList.add(createVital(BODY_MASS_INDEX, vitalRate, vitalTimeTaken/*, patientIen*/));
                            }
                        }
    
                        break;
                    case 17:
                        if (flds.length == 2) {
                            units = flds[1];
                        }
                        break;
                    default:
                        break;
                }
    
            }
    
            rtc = vitalList.toArray(new Vitals[0]);
        }
        return rtc;
    }

    private void setVitalSignDetails(int detailsType, List<Vitals> vitalsList, String detailsStr) {
        String[] detailsPiece = detailsStr.split(",");

        //if (detailsPiece == null) return;

        LinkedHashMap<String, String> detailsData = new LinkedHashMap<String, String>();

        for (String unit : detailsPiece) {
            String[] detailsPair = unit.split(":");

            if (detailsPair == null || detailsPair.length < 2) {
                continue;
            }

            String vitalType = detailsPair[0];
            String detailsVal = detailsPair[1];

            if ("BP".equalsIgnoreCase(vitalType)) {
                detailsData.put(BLOOD_PRESSURE, detailsVal);
            } else if ("PULSE".equalsIgnoreCase(vitalType)) {
                detailsData.put(PULSE, detailsVal);
            } else if ("PAIN".equalsIgnoreCase(vitalType)) {
                detailsData.put(PAIN, detailsVal);
            } else if ("RESP".equalsIgnoreCase(vitalType)) {
                detailsData.put(RESPIRATION, detailsVal);
            } else if ("TEMP".equalsIgnoreCase(vitalType)) {
                detailsData.put(TEMPERATURE, detailsVal);
            } else if ("WT".equalsIgnoreCase(vitalType)) {
                detailsData.put(WEIGHT, detailsVal);
            } else if ("HT".equalsIgnoreCase(vitalType)) {
                detailsData.put(HEIGHT, detailsVal);
            } else if ("POx".equalsIgnoreCase(vitalType) || "O2".equalsIgnoreCase(vitalType)) {
                detailsData.put(PULSE_OXIMETRY, detailsVal);
            } else if ("CG".equalsIgnoreCase(vitalType)) {
                detailsData.put(CIRCUMFERENCE_GIRTH, detailsVal);
            } else if ("CVP".equalsIgnoreCase(vitalType)) {
                detailsData.put(CENTRAL_VENOUS_PRESSURE, detailsVal);
            } else if ("BMI".equalsIgnoreCase(vitalType)) {
                detailsData.put(BODY_MASS_INDEX, detailsVal);
            }
        }

        for (Vitals vital : vitalsList) {
            if (detailsData.containsKey(vital.getVitalType())) {
                String detailsValue = detailsData.get(vital.getVitalType());

                if (detailsType == VITAL_DETAILS_UNITS) {
                    vital.setUnits(detailsValue);
                } else if (detailsType == VITAL_DETAILS_QUALIFIERS) {
                    vital.setQualifiers(detailsValue);
                }
            }
        }
    }

    public Vitals[] getLatestVitals() throws RPCException {
        return getLatestVitals(cxn.getPatientIen());
    }

    public Vitals[] getLatestVitals(String patientIen) throws RPCException {
        String request = buildGetLatestVitalsRequest(patientIen);
        String response = (String) cxn.query(request);
        return toVitalSignsLatest(response/*, patientIen*/);
    }

    private String buildGetLatestVitalsRequest(String patientIen) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORQQVI VITALS");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        return vquery.buildMessage();
    }

    private Vitals[] toVitalSignsLatest(String response/*, String patientIen*/) {
        Vitals[] rtc;
        if ("".equals(response)) {
            rtc = new Vitals[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<Vitals> lst = new ArrayList<Vitals>(lines.length);
    
            for (int i = 0; i < lines.length; i++) {
                if ("".equals(lines[i])) {
                    continue;
                }
                String[] flds = StringUtils.split(lines[i], StringUtils.CARET);
                String vitalType = null;
                if ("T".equals(flds[1])) {
                    vitalType = TEMPERATURE;
                } else if ("P".equals(flds[1])) {
                    vitalType = PULSE;
                } else if ("R".equals(flds[1])) {
                    vitalType = RESPIRATION;
                } else if ("BP".equals(flds[1])) {
                    vitalType = BLOOD_PRESSURE;
                } else if ("HT".equals(flds[1])) {
                    vitalType = HEIGHT;
                } else if ("WT".equals(flds[1])) {
                    vitalType = WEIGHT;
                } else if ("PN".equals(flds[1])) {
                    vitalType = PAIN;
                }
                if (vitalType == null) {
                    continue;
                }
                Vitals vital = new Vitals();
                vital.setVitalType(vitalType);
                vital.setRate(flds[4]);
//                if (flds.length == 6) {
                    //observation.Value2 = flds[5];
//                }
                vital.setDateTimeTaken(Utils.getCal(Timestamp.toUtcString(flds[3])));
    
                fillCommonDataBeanValues(vital);
                lst.add(vital);
    
            }
            rtc = lst.toArray(new Vitals[0]);
        }
        return rtc;
    }

    /**
     * Uses the VPR RPC with "vitals" as the type.
     *
     * @param patientIen The patient IEN
     * @param startDate  The start of the date range for the list of vitals
     * @param endDate    The end of the date range for the list of vitals
     * @return The list of vitals for the specified date range
     * @throws RPCException
     */
    public Vitals[] getVitalsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.VITALS);
        String response = (String) cxn.query(request);

        return parseVitalsFromVPR(response);
    }

    /**
     * Uses the VPR RPC with "vitals" as the type.
     *
     * @param startDate The start of the date range for the list of vitals
     * @param endDate   The end of the date range for the list of vitals
     * @return The list of vitals for the specified date range
     * @throws RPCException
     */
    public Vitals[] getVitalsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getVitalsVPR(cxn.getPatientIen(), startDate, endDate);
    }

    private Vitals[] parseVitalsFromVPR(String response) throws RPCException {
        Vitals[] rtc;
        if ("".equals(response)) {
            rtc = new Vitals[0];
        } else {
    
            List<Vitals> vitalList = new ArrayList<Vitals>();
    
            final String XPATH_VITAL = "//vital";
    
            Document doc = null;
    
            try {
                doc = parseXMLDocument(response);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();
    
            XPathExpression exprVital;
    
            try {
                exprVital = xpath.compile(XPATH_VITAL);
    
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList vitalsNodeList = null;
            try {
                vitalsNodeList = (NodeList) exprVital.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            List<VPRVitalParser> vitalParserList = new ArrayList<VPRVitalParser>();
            int listLength = vitalsNodeList.getLength();
            for (int vitalNum = 0; vitalNum < listLength; vitalNum++) {
                Node vitalNode = vitalsNodeList.item(vitalNum);
                vitalNode.getParentNode().removeChild(vitalNode);
    
                vitalParserList.add(new VPRVitalParser(vitalNode));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<Vitals[]>> futures = null;
            try {
                futures = executorService.invokeAll(vitalParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
    
                if (futures == null) {
                    throw new RPCException("Failed to parse vitals results : accession node parser futures list is null.");
                }
    
            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }
    
            Iterator<Future<Vitals[]>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<Vitals[]> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Vital Data");
                    } else {
                        vitalList.addAll(Arrays.asList(future.get()));
                    }
    
                } catch (InterruptedException e) {
                    LOGGER.error(e.getMessage(), e);
                } catch (ExecutionException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
    
            rtc = vitalList.toArray(new Vitals[0]);
        }
        return rtc;
    }

    /**
     * Uses the VPR RPC with "vitals" as the type to retrieve the most vitals
     * measurement (within 5 years ago).
     *
     * @param patientIen The patient IEN
     * @return The list of vitals for the specified date range
     * @throws RPCException
     */
    public Vitals[] getLatestVitalsVPR(String patientIen) throws RPCException {
        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -5);

        String request = VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.VITALS);
        String response = (String) cxn.query(request);

        return parseLatestVitalsFromVPR(response);
    }

    /**
     * Uses the VPR RPC with "vitals" as the type to retrieve the last vitals
     * measurement (within 5 years ago).
     *
     * @return The list of vitals for the specified date range
     * @throws RPCException
     */
    public Vitals[] getLatestVitalsVPR() throws RPCException {
        return getLatestVitalsVPR(cxn.getPatientIen());
    }

    private Vitals[] parseLatestVitalsFromVPR(String response) throws RPCException {
        Vitals[] rtc;
        if ("".equals(response)) {
            rtc = new Vitals[0];
        } else {
    
            List<Vitals> vitalList = new ArrayList<Vitals>();
    
            final String XPATH_VITAL = "//vital";
    
            Document doc = null;
    
            try {
                doc = parseXMLDocument(response);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();
    
            XPathExpression exprVital;
    
            try {
                exprVital = xpath.compile(XPATH_VITAL);
    
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList vitalsNodeList = null;
            try {
                vitalsNodeList = (NodeList) exprVital.evaluate(doc, XPathConstants.NODESET);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            List<VPRVitalParser> vitalParserList = new ArrayList<VPRVitalParser>();
    
            // For the current Vitals take only the first node.
            if (vitalsNodeList.getLength() > 0) {
                Node vitalNode = vitalsNodeList.item(0);
                vitalNode.getParentNode().removeChild(vitalNode);
    
                vitalParserList.add(new VPRVitalParser(vitalNode));
            }
    
            ExecutorService executorService = cxn.getParserThreadPool().getThreadPool();
            List<Future<Vitals[]>> futures = null;
            try {
                futures = executorService.invokeAll(vitalParserList, cxn.getParserThreadPool().getPoolTimeoutSeconds(), TimeUnit.SECONDS);
    
                if (futures == null) {
                    throw new RPCException("Failed to parse vitals results : accession node parser futures list is null.");
                }
    
            } catch (InterruptedException e) {
                LOGGER.error(e.getMessage(), e);
                throw new RPCException(e);
            }
    
            Iterator<Future<Vitals[]>> futuresItr = futures.iterator();
    
            while (futuresItr.hasNext()) {
                try {
                    Future<Vitals[]> future = futuresItr.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Vital Data");
                    } else {
                        vitalList.addAll(Arrays.asList(future.get()));
                    }
    
                } catch (InterruptedException e) {
                    LOGGER.error(e.getMessage(), e);
                } catch (ExecutionException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
    
            rtc = vitalList.toArray(new Vitals[0]);
        }
        return rtc;
    }


    private class VPRVitalParser implements Callable<Vitals[]> {
        private Node vitalNode;

        public VPRVitalParser(Node vitalNode) {
            this.vitalNode = vitalNode;
        }

        public Vitals[] call() throws RPCException {
//            final String XPATH_ENTERED = "./entered/@value";
//            final String XPATH_LOCATION_NAME = "./location/@name";
//            final String XPATH_LOCATION_CODE = "./location/@code";
            final String XPATH_TAKEN = "./taken/@value";

            final String XPATH_MEASUREMENTS = "./measurements";
            final String XPATH_MEASUREMENT = "./measurement";
            final String XPATH_MEASUREMENT_NAME = "./@name";
            final String XPATH_MEASUREMENT_ID = "./@id";
            final String XPATH_MEASUREMENT_VUID = "./@vuid";
            final String XPATH_MEASUREMENT_VALUE = "./@value";
            final String XPATH_MEASUREMENT_UNITS = "./@units";
//            final String XPATH_MEASUREMENT_METRICVALUE = "./@metricValue";
//            final String XPATH_MEASUREMENT_METRICUNITS = "./@metricUnits";
            final String XPATH_MEASUREMENT_HIGH = "./@high";
            final String XPATH_MEASUREMENT_LOW = "./@low";
            final String XPATH_MEASUREMENT_BMI = "./@bmi";

            List<Vitals> vitalsList = new ArrayList<Vitals>();

//            XPathExpression exprEntered = null;
//            XPathExpression exprLocationName = null;
//            XPathExpression exprLocationCode = null;
            XPathExpression exprTaken = null;
            XPathExpression exprMeasurements = null;
            XPathExpression exprMeasurement = null;
            XPathExpression exprMeasurementName = null;
            XPathExpression exprMeasurementVuid = null;
            XPathExpression exprMeasurementValue = null;
            XPathExpression exprMeasurementUnits = null;
//            XPathExpression exprMeasurementMetricValue = null;
//            XPathExpression exprMeasurementMetricUnits = null;
            XPathExpression exprMeasurementId = null;
            XPathExpression exprMeasurementHigh = null;
            XPathExpression exprMeasurementLow = null;
            XPathExpression exprMeasurementBmi = null;

            XPathFactory xPathfactory = XPathFactory.newInstance();
            XPath xpath = xPathfactory.newXPath();

            try {
//                exprEntered = xpath.compile(XPATH_ENTERED);
//                exprLocationCode = xpath.compile(XPATH_LOCATION_CODE);
//                exprLocationName = xpath.compile(XPATH_LOCATION_NAME);
                exprTaken = xpath.compile(XPATH_TAKEN);

                exprMeasurements = xpath.compile(XPATH_MEASUREMENTS);
                exprMeasurement = xpath.compile(XPATH_MEASUREMENT);
                exprMeasurementName = xpath.compile(XPATH_MEASUREMENT_NAME);
                exprMeasurementId = xpath.compile(XPATH_MEASUREMENT_ID);
                exprMeasurementVuid = xpath.compile(XPATH_MEASUREMENT_VUID);
                exprMeasurementValue = xpath.compile(XPATH_MEASUREMENT_VALUE);
                exprMeasurementUnits = xpath.compile(XPATH_MEASUREMENT_UNITS);
//                exprMeasurementMetricValue = xpath.compile(XPATH_MEASUREMENT_METRICVALUE);
//                exprMeasurementMetricUnits = xpath.compile(XPATH_MEASUREMENT_METRICUNITS);
                exprMeasurementHigh = xpath.compile(XPATH_MEASUREMENT_HIGH);
                exprMeasurementLow = xpath.compile(XPATH_MEASUREMENT_LOW);
                exprMeasurementBmi = xpath.compile(XPATH_MEASUREMENT_BMI);

            } catch (Exception e) {
                throw new RPCException(e);
            }

            //String entered = null, locationCode = null, locationName = null;
            String taken = null;

//            try {
//                entered = (String) exprEntered.evaluate(vitalNode, XPathConstants.STRING);
//            } catch (XPathExpressionException e) {
//                entered = null;
//                logger.error(e.getMessage(), e);
//            }
//            try {
//                locationCode = (String) exprLocationCode.evaluate(vitalNode, XPathConstants.STRING);
//            } catch (XPathExpressionException e) {
//                locationCode = null;
//                logger.error(e.getMessage(), e);
//            }
//            try {
//                locationName = (String) exprLocationName.evaluate(vitalNode, XPathConstants.STRING);
//            } catch (XPathExpressionException e) {
//                locationName = null;
//                logger.error(e.getMessage(), e);
//            }
            try {
                taken = (String) exprTaken.evaluate(vitalNode, XPathConstants.STRING);
            } catch (XPathExpressionException e) {
                taken = null;
                LOGGER.error(e.getMessage(), e);
            }

            Calendar takenDate = null;
            if (!StringUtils.isEmpty(taken)) {
                takenDate = Timestamp.toDateTime(taken);
            }

            Node measurementsNode = null;
            try {
                measurementsNode = (Node) exprMeasurements.evaluate(vitalNode, XPathConstants.NODE);
            } catch (XPathExpressionException e) {
                LOGGER.error(e.getMessage(), e);
            }

            if (measurementsNode != null) {
                NodeList measurementNodeList = null;
                try {
                    measurementNodeList = (NodeList) exprMeasurement.evaluate(measurementsNode, XPathConstants.NODESET);
                } catch (XPathExpressionException e) {
                    LOGGER.error(e.getMessage(), e);
                }

                if (measurementNodeList != null) {
                    StringBuilder qualifiersSB = new StringBuilder();
                    int listLength = measurementNodeList.getLength();
                    for (int measurementNum = 0; measurementNum < listLength; measurementNum++) {
                        Node measurementNode = measurementNodeList.item(measurementNum);
                        measurementNode.getParentNode().removeChild(measurementNode);

                        Vitals measurement = new Vitals();
                        fillCommonDataBeanValues(measurement);

                        String measurementName = null;
                        try {
                            measurementName = (String) exprMeasurementName.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementId = null;
                        try {
                            measurementId = (String) exprMeasurementId.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementVuid = null;
                        try {
                            measurementVuid = (String) exprMeasurementVuid.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementValue = null;
                        try {
                            measurementValue = (String) exprMeasurementValue.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementUnits = null;
                        try {
                            measurementUnits = (String) exprMeasurementUnits.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
//                        String measurementMetricValue = null;
//                        try {
//                            measurementMetricValue = (String) exprMeasurementMetricValue.evaluate(measurementNode, XPathConstants.STRING);
//                        } catch (XPathExpressionException e) {
//                            LOGGER.error(e.getMessage(), e);
//                        }
//                        String measurementMetricUnits = null;
//                        try {
//                            measurementMetricUnits = (String) exprMeasurementMetricUnits.evaluate(measurementNode, XPathConstants.STRING);
//                        } catch (XPathExpressionException e) {
//                            LOGGER.error(e.getMessage(), e);
//                        }
                        String measurementHigh = null;
                        try {
                            measurementHigh = (String) exprMeasurementHigh.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementLow = null;
                        try {
                            measurementLow = (String) exprMeasurementLow.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        String measurementBmi = null;
                        try {
                            measurementBmi = (String) exprMeasurementBmi.evaluate(measurementNode, XPathConstants.STRING);
                        } catch (XPathExpressionException e) {
                            LOGGER.error(e.getMessage(), e);
                        }
                        measurement.setBmi(measurementBmi);

                        measurement.setRate(measurementValue);
                        measurement.setUnits(measurementUnits);
                        measurement.setVitalType(measurementName);
                        measurement.setVitalsIEN(measurementId);
                        if (takenDate != null) {
                            measurement.setDateTimeTaken(takenDate);
                        }
                        measurement.setUnits(measurementUnits);
                        qualifiersSB.setLength(0);
                        if (!StringUtils.isEmpty(measurementHigh)) {
                            qualifiersSB.append("high: ");
                            qualifiersSB.append(measurementHigh);
                        }
                        if (!StringUtils.isEmpty(measurementHigh) && !StringUtils.isEmpty(measurementLow)) {
                            qualifiersSB.append("; ");
                        }
                        if (!StringUtils.isEmpty(measurementLow)) {
                            qualifiersSB.append("low: ");
                            qualifiersSB.append(measurementLow);
                        }
                        measurement.setQualifiers(qualifiersSB.toString());

                        Code measurementCode = new Code();
                        measurementCode.setCode(measurementVuid);
                        measurementCode.setSystem("VUID");
                        Code[] codeArray = new Code[]{measurementCode};
                        measurement.setCodes(codeArray);

                        vitalsList.add(measurement);
                    }
                }
            }

            return vitalsList.toArray(new Vitals[0]);
        }
    }

}
