/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.Timestamp;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.Immunization;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vhahon.common.Utils;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class ImmunizationsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(ImmunizationsDataService.class.getName());

    public List<Immunization> getPatientImmunizations(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);
        VistaRPCConnectionManager msq = null;
        VistaRPCConnectionManager msqVPR = null;

        try {
            List<Immunization> immunizationList = new ArrayList<Immunization>();

            msqVPR = setupVPRContextMultiSourceQuery(queryBean);

            LinkedHashMap<Site, Object> immsMap = VistaRPCConnectionManager.execute(msqVPR.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_IMMUNIZATION,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_IMMUNIZATIONS, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> entries = immsMap.entrySet();

            for (Map.Entry<Site, Object> entry : entries) {
                Object immunizations = entry.getValue();
                if (immunizations instanceof Exception) {
                    immunizations = generateConnectionUnavailableBean(entry.getKey(), (Exception) immunizations);
                }
                immunizationList.addAll(Arrays.asList((Immunization[]) immunizations));
            }

            msq = setupMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> immSkinTestMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_IMMUNIZATION,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_IMMUNIZATION_SKINTESTS, new Object[]{queryBean});
            Set<Map.Entry<Site, Object>> skinTests = immSkinTestMap.entrySet();

            for (Map.Entry<Site, Object> entry : skinTests) {
                Object immunizations = entry.getValue();
                if (immunizations instanceof Exception) {
                    immunizations = generateConnectionUnavailableBean(entry.getKey(), (Exception) immunizations);
                }
                immunizationList.addAll(Arrays.asList((Immunization[]) immunizations));
            }

            return immunizationList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
            if (msqVPR != null) {
                try {
                    msqVPR.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }

        }
    }

    private Immunization[] generateConnectionUnavailableBean(Site site, Exception except) {
        
        Immunization immun = new Immunization();
        immun.setName(CONNECTION_UNAVAILABLE);
        immun.setDateTime(Utils.formatDisplayDate(Utils.backDate().toGregorianCalendar()));
        immun.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            immun.setName(UNABLE_TO_RETRIEVE);
        } else {
            if(except.getCause() != null && except.getCause().getCause() != null
                    && except.getCause().getCause() instanceof InvocationTargetException
                    && ((InvocationTargetException) except.getCause().getCause()).getTargetException() != null ){
                LOGGER.error("VDS Error InvocationTargetException: " + ((InvocationTargetException) except.getCause().getCause()).getTargetException().toString());
            } else {
                LOGGER.error(except.getLocalizedMessage(), except);        //log if any other exception type
            }
        }
        return new Immunization[] {immun};
    }
}
