/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.provider.dao;

import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.DaoBase;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.patient.dao.rpc.UserAuthenticationException;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Alert;
import gov.va.med.vds.vistabeans.ProviderAppointmentBean;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.User;

/**
 *
 *
 */
public class UserDao extends DaoBase {
    public UserDao(RPCConnection cxn) {
        super(cxn);
    }

    public User login(String accessCode, String verifyCode, String context) throws RPCException, UnsupportedRPCMethodException {
        return login(accessCode, verifyCode, context, false);
    }

    public User login(String accessCode, String verifyCode, String context, boolean defaultToRPC) throws RPCException, UnsupportedRPCMethodException {
        User user = null;
        try {
            UserRPC rpc = new UserRPC(cxn);
            user = rpc.login(accessCode, verifyCode, context);

            if (user != null) {

                user.setHostSite(cxn.getSite());

                String aCodeDecrypt = VistaKernelHash.decrypt(accessCode);
                String vCodeDecrypt = VistaKernelHash.decrypt(verifyCode);
                try {
                    user.setEncryptedAvCode(VistaKernelHash.encrypt(aCodeDecrypt + ";" + vCodeDecrypt, false));
                } catch (Exception e) {
                    throw new RPCException("Setting encrypted AV code failed.");
                }
            }
        } catch (UserAuthenticationException e) {
            user = new User();
            user.setUserIen("^" + e.getMessage());
        }

        return user;
    }

    public User[] lookupUser(String lookupstr, String userId, int maxReturn) throws UnsupportedRPCMethodException {
        throw new UnsupportedRPCMethodException("No supported RPC method");
    }

    public User getProviderProfile(String userIen, String providerNpi) throws UnsupportedRPCMethodException
    {
        throw new UnsupportedRPCMethodException("No supported RPC method");
    }

    public ProviderAppointmentBean[] getProviderWorkload(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {
        throw new UnsupportedRPCMethodException("No supported RPC method");
    }

    public Alert[] getUserAlerts(QueryBean queryBean) throws RPCException, UnsupportedRPCMethodException {
        throw new UnsupportedRPCMethodException("No supported RPC method");
    }
}
