package gov.va.med.vds.servlet;

import com.fasterxml.jackson.databind.ObjectMapper;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.common.TokenUtils;
import gov.va.med.vds.das.beans.DateDeserializer;
import gov.va.med.vds.das.beans.Notification;
import gov.va.med.vds.das.beans.Subscription;
import gov.va.med.vds.patient.dao.DASDao;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.das.DASResultCollection;
import gov.va.med.vds.webservice.DASLimit;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.*;
import java.util.*;

/**
 * Created by melcolm on 1/5/17.
 */
@WebServlet(name = "Notifications", urlPatterns = {"/notify/*"})
public class Notifications extends HttpServlet {

    private static final long serialVersionUID = -8484848484411657451L;
    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    private static final Logger logger = Logger.getLogger(Notifications.class.getName());

    final static byte[] encryptionKey = APP_CONFIG.getEncryptionKey();

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        logger.debug("### CALLBACK Recieved ###");

//        PrintWriter out = response.getWriter();
        StringBuilder jsonString = new StringBuilder();

        String jsonToken = null;

        BufferedReader reader = request.getReader();

        String pathInfo = request.getPathInfo(); // /{value}/test
        String[] pathParts = pathInfo.split("/");

        String token = "";

        if (pathParts.length > 0) {
            token = pathParts[1]; // {value}
            logger.debug("### CALLBACK token ### : " + token);
        }

        if (token.length() > 0) {
            Map<String, String> tokenParams = null;

            try {
                tokenParams = TokenUtils.getInstance().decryptTokenToMap(encryptionKey, token);
            } catch (Exception ex) {
                response.sendError(403, "Invalid Token");
                logger.debug("Login error decrypting token: " + ex.getMessage());
                return;
            }

            if (tokenParams != null && (
                    StringUtils.isBlank(tokenParams.get("patientICN")) ||
                            StringUtils.isBlank(tokenParams.get("userID")) ||
                            StringUtils.isBlank(tokenParams.get("domain")) ||
                            StringUtils.isBlank(tokenParams.get("forceRefresh")))
                    ) {
                response.sendError(403, "Invalid Token");
                return;
            }

            StringBuilder key = new StringBuilder();
            if(tokenParams != null) {
                key.append(tokenParams.get("patientICN"));
                key.append("|");
                key.append(tokenParams.get("userID"));
                key.append("|");
                key.append(tokenParams.get("domain"));
            }

            Subscription subscription = DASLimit.getInstance().getSubscription(key.toString());
            if (subscription == null) {
                logger.debug("### Creating a new subscription on callback ### key: " + key.toString());
                //no subscription found. Let's add a new one to the cache just in case
                subscription = new Subscription();
                subscription.setPatientICN(tokenParams.get("patientICN"));
                subscription.setUserID(tokenParams.get("userID"));
                subscription.setDomain(tokenParams.get("domain"));
                subscription.setForceRefresh(Boolean.valueOf(tokenParams.get("forceRefresh")));
            }

//            System.out.println("This is a test: " + subscription.getStatus());
//            Subscription subscription = new Subscription();
//
//            try {
//                subscription.setForceRefresh(Boolean.valueOf(tokenParams.get("forceRefresh")));
//                subscription.setPatientICN(tokenParams.get("patientICN"));
//                subscription.setUserID(tokenParams.get("userID"));
//                subscription.setDomain(tokenParams.get("domain"));
//                subscription.setStatus("Available");
//                DASLimit.getInstance().putSubscription(token, subscription);
//            } catch (Exception ex){
//                response.sendError(403, "Error attempting to cache the subscription");
//                logger.debug("Error attempting to cache the subscription: " + ex.getMessage());
//            }


            while ((jsonToken = reader.readLine()) != null) {
                jsonString.append(jsonToken);
            }
            // converting the JSON String into a JSONObject, which almost work like a Map
            try {
                ObjectMapper mapper = new ObjectMapper();

                logger.debug("### key: " + key.toString() + " ###");
                logger.debug("### CALLBACK JSON ### : " + jsonString.toString());

                Notification notification = mapper.readValue(jsonString.toString(), Notification.class);
                if (StringUtils.isNotEmpty(notification.getStatus())) {
                    subscription.setStatus(notification.getStatus());
                }

                //process document statuses
                StringBuilder divXML = new StringBuilder();
                if (notification.getText() != null && StringUtils.isNotEmpty(notification.getText().getDiv())) {
                    //wrap xml since there is no root
                    divXML.append("<wrapper>");
                    divXML.append(notification.getText().getDiv());
                    divXML.append("</wrapper>");

                    final String XPATH_DOCUMENTS = "//document";
                    final String XPATH_PARTNERNAME = "./PartnerName";
                    final String XPATH_DOCUMENTID = "./PartnerDocumentID";
                    final String XPATH_STATUS = "./status";
                    final String XPATH_TIMESTAMP = "./timestamp";

                    InputSource is = new InputSource(new StringReader(divXML.toString()));
                    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
                    documentBuilderFactory.setValidating(true);
                    documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
                    documentBuilderFactory.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
                    documentBuilderFactory.setExpandEntityReferences(false);
                    Document xmlDoc = documentBuilderFactory.newDocumentBuilder().parse(is);

                    XPath xpath = XPathFactory.newInstance().newXPath();
                    XPathExpression exprDocs = xpath.compile(XPATH_DOCUMENTS);
                    XPathExpression exprStatus = xpath.compile(XPATH_STATUS);
                    XPathExpression exprPartnerName = xpath.compile(XPATH_PARTNERNAME);
                    XPathExpression exprDocID = xpath.compile(XPATH_DOCUMENTID);
                    XPathExpression exprTimestamp = xpath.compile(XPATH_TIMESTAMP);

                    NodeList documents = (NodeList) exprDocs.evaluate(xmlDoc, XPathConstants.NODESET);

                    List<gov.va.med.vds.das.beans.Document> documentList = new ArrayList<gov.va.med.vds.das.beans.Document>();

                    for (int i = 0; i < documents.getLength(); i++) {
                        Node doc = documents.item(i);
                        gov.va.med.vds.das.beans.Document document = new gov.va.med.vds.das.beans.Document();
                        String status = (String) exprStatus.evaluate(doc, XPathConstants.STRING);
                        document.setPartnerDocumentID((String) exprDocID.evaluate(doc, XPathConstants.STRING));
                        document.setPartnerName((String) exprPartnerName.evaluate(doc, XPathConstants.STRING));
                        String timeStamp = (String) exprTimestamp.evaluate(doc, XPathConstants.STRING);
                        if (DateDeserializer.getDate(timeStamp) != null) {
                            timeStamp = String.valueOf(DateDeserializer.getDate(timeStamp).getTime());
                        }
                        if (StringUtils.isNotEmpty(timeStamp)) {
                            try {
                                document.setTimestamp(Long.parseLong(timeStamp));
                            } catch (NumberFormatException nfe){
                                Date currentDate = new Date();
                                document.setTimestamp(currentDate.getTime());
                                logger.error("ERROR parsing DAS Notification - Invalid timeStamp");
                            }
                        }
                        document.setStatus(status);
                        documentList.add(document);
                    }

                    subscription.setDocuments(documentList);
                }

                DASLimit.getInstance().putSubscription(key.toString(), subscription);

            } catch (Exception ex) {
                logger.error("There was an error updating the subscription.");
                ex.printStackTrace();
            }
        } else {
            //no token, return error
            response.sendError(403, "Invalid Token");
        }

    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {


        logger.debug("### invalid notify via GET ### : ");

        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment())) {

//            try {
//                DASServiceHandler sh = DASServiceHandler.getInstance();
//                System.out.println("## DAS CALLBACK FQDN ##: " + sh.getHostFqdn());
//            } catch (Exception e){
//                System.out.println("Error attempting to get server info: " + e.getLocalizedMessage());
//            }

//            QueryBean qb = new QueryBean();
//            Patient pat = new Patient();
//            pat.setICN("1012581676V377802");
//
//            User user = new User();
//            user.setName("TestUser");
//            user.setUserId("987");
//            Site hostSite = new Site();
//            hostSite.setSiteCode("984");
//            user.setHostSite(hostSite);
//
//            qb.setPatient(pat);
//            qb.setUser(user);
//            qb.setForceRefresh(true);
//            try {
//                DASDao dDao = new DASDao();
//                DASResultCollection rtc = dDao.getVLERDemographics(qb);
//            } catch (VDSException ex) {
//                logger.error("Error retrieving DAS Data", ex);
//            }
        } else {
            throw new ServletException("GET method is not supported.");
        }

    }

}
