/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * This class represents the inmmunization data within Vista.
 */
public class Immunization extends DataBean {
    /**
     * Internal Id of the immunization.
     */
    private String id;
    /**
     * Date/Time of the immunzation.
     */
    private String dateTime;
    /**
     * Name of the Immunization.
     */
    private String name;
    /**
     * Reaction to the immunization if any.
     */
    private String reaction;
    /**
     * the user who was responsible for this immunization encounter.
     */
    private String encounterProvider;
    /**
     * User ordering the immunization.
     */
    private String orderingProvider;
    /**
     * The sequence if the immunizations is part of a series.
     */
    private String series;

    private String manufacturerName;
    private String lot;
    private String dose;
    private String units;
    private Calendar lotExpirationDate;
    private String productType;
    private String route;
    private String result;
    private boolean contraindicated;
    private String bodySite;
    private String cvx;
    private String documentedBy;
    private String source;
    private String comment;
    private String visitLocation;
    private String size;

    /**
     * The reason for exemption or refusal
     */
    private String exemptionRefusalReason;

    private Calendar exemptionRefusalDate;

    /**
     * Internal Id of the immunization.
     */
    public String getId() {
        return id;
    }

    public void setId(String newId) {
        id = newId;
    }

    /**
     * Date/Time of Immunization.
     */
    public String getDateTime() {
        return dateTime;
    }


    public void setDateTime(String newDateTime) {
        dateTime = newDateTime;
    }

    /**
     * Name of the immunization.
     */
    public String getName() {
        return name;
    }

    public void setName(String newName) {
        name = newName;
    }

    /**
     * reaction associated with the immunization.
     */
    public String getReaction() {
        return reaction;
    }

    public void setReaction(String newReaction) {
        reaction = newReaction;
    }

    /**
     * User associated with this immunization encounter.
     */
    public String getEncounterProvider() {
        return encounterProvider;
    }

    public void setEncounterProvider(String newEncounterProvider) {
        encounterProvider = newEncounterProvider;
    }

    /**
     * User who ordered the immunization.
     */
    public String getOrderingProvider() {
        return orderingProvider;
    }

    public void setOrderingProvider(String newOrderingProvider) {
        orderingProvider = newOrderingProvider;
    }

    /**
     * Sequence of series of this immunization is part of a series.
     */
    public String getSeries() {
        return series;
    }

    public void setSeries(String newSeries) {
        series = newSeries;
    }

    public String getManufacturerName() {
        return manufacturerName;
    }

    public void setManufacturerName(String manufacturerName) {
        this.manufacturerName = manufacturerName;
    }

    public String getLot() {
        return lot;
    }

    public void setLot(String lot) {
        this.lot = lot;
    }

    public String getDose() {
        return dose;
    }

    public void setDose(String dose) {
        this.dose = dose;
    }

    public String getUnits() {
        return units;
    }

    public void setUnits(String units) {
        this.units = units;
    }

    public Calendar getLotExpirationDate() {
        return lotExpirationDate;
    }

    public void setLotExpirationDate(Calendar lotExpirationDate) {
        this.lotExpirationDate = lotExpirationDate;
    }

    public String getProductType() {
        return productType;
    }

    public void setProductType(String productType) {
        this.productType = productType;
    }

    public String getRoute() {
        return route;
    }

    public void setRoute(String route) {
        this.route = route;
    }

    public String getResult() {
        return result;
    }

    public void setResult(String result) {
        this.result = result;
    }

    public String getExemptionRefusalReason() {
        return exemptionRefusalReason;
    }

    public void setExemptionRefusalReason(String exemptionRefusalReason) {
        this.exemptionRefusalReason = exemptionRefusalReason;
    }

    public Calendar getExemptionRefusalDate() {
        return exemptionRefusalDate;
    }

    public void setExemptionRefusalDate(Calendar exemptionRefusalDate) {
        this.exemptionRefusalDate = exemptionRefusalDate;
    }

    public boolean isContraindicated() {
        return contraindicated;
    }

    public void setContraindicated(boolean contraindicated) {
        this.contraindicated = contraindicated;
    }

    public String getBodySite() {
        return bodySite;
    }

    public void setBodySite(String bodySite) {
        this.bodySite = bodySite;
    }

    public String getCvx() {
        return cvx;
    }

    public void setCvx(String cvx) {
        this.cvx = cvx;
    }

    public String getDocumentedBy() {
        return documentedBy;
    }

    public void setDocumentedBy(String documentedBy) {
        this.documentedBy = documentedBy;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public String getVisitLocation() {
        return visitLocation;
    }

    public void setVisitLocation(String visitLocation) {
        this.visitLocation = visitLocation;
    }

    public String getSize() {
        return size;
    }

    public void setSize(String size) {
        this.size = size;
    }
}