/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

/**
 * This class represents fill data associated with an outpatient prescription.
 */
public class PrescriptionFill extends DataBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(PrescriptionFill.class);

    /**
     * Date the fill was dispensed.
     */
    private Calendar dispenseDate;
    /**
     * Name of pharmacy dispensing the fill
     */
    private String dispensingPharmacy;
    /**
     * Amount dispensed.
     */
    private String dispensingQuantity;

    private Calendar releaseDate;
    private String routing;
    private String daysSupply;
    private boolean partial;

    public PrescriptionFill() {
        partial = false;
    }

    /**
     * Calendar this fill was dispensed.
     */
    public Calendar getDispenseDate() {
        return dispenseDate;
    }

    public void setDispenseDate(String newDispenseDate) {
        Calendar date = null;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.getDefault());
        date = new java.util.GregorianCalendar();
        try {
            date.setTime(sdf.parse(newDispenseDate));
        } catch (ParseException e) {
            LOGGER.error(e.getMessage());
        }


        dispenseDate = date;
    }

    public void setDispenseDate(Calendar newDispenseDate) {
        dispenseDate = newDispenseDate;
    }

    /**
     * Name of pharmacy that dispensed this fill.
     */
    public String getDispensingPharmacy() {
        return dispensingPharmacy;
    }

    public void setDispensingPharmacy(String newDispensingPharmacy) {
        dispensingPharmacy = newDispensingPharmacy;
    }

    /**
     * Quantity dispensed for this fill.
     */
    public String getDispensingQuantity() {
        return dispensingQuantity;
    }

    public void setDispensingQuantity(String newDispensingQuantity) {
        dispensingQuantity = newDispensingQuantity;
    }

    public Calendar getReleaseDate() {
        return releaseDate;
    }

    public void setReleaseDate(Calendar releaseDate) {
        this.releaseDate = releaseDate;
    }

    public String getRouting() {
        return routing;
    }

    public void setRouting(String routing) {
        this.routing = routing;
    }

    public String getDaysSupply() {
        return daysSupply;
    }

    public void setDaysSupply(String daysSupply) {
        this.daysSupply = daysSupply;
    }

    public boolean isPartial() {
        return partial;
    }

    public void setPartial(boolean partial) {
        this.partial = partial;
    }
}