/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.webservice;

import gov.va.med.vhahon.common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.Text;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.util.Set;

public class InvalidXMLCharSOAPHandler implements SOAPHandler<SOAPMessageContext> {

    private static final Logger LOGGER = LoggerFactory.getLogger(InvalidXMLCharSOAPHandler.class.getName());

    public Set<QName> getHeaders() {
        return null;
    }

    public boolean handleMessage(SOAPMessageContext smc) {

        Boolean outboundProperty = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        //remove any invalid xml characters from outbound SOAP messages    
        if (outboundProperty) {
            try {
                SOAPMessage message = smc.getMessage();
                SOAPBody body = message.getSOAPBody();
                removeInvalidXMLCharacters(body.getChildNodes());
            } catch (SOAPException ex) {
                LOGGER.error(ex.getMessage(), ex);
            }
        }

        return true;
    }

    private void removeInvalidXMLCharacters(NodeList nodeList) {
        int listLength = nodeList.getLength();
        for (int i = 0; i < listLength; i++) {
            Node node = nodeList.item(i);
            if (node instanceof Text) {
                Text textNode = (Text) node;
                textNode.setData(Utils.stripInvalidXMLCharacters(textNode.getData()));
            }
            removeInvalidXMLCharacters(node.getChildNodes());
        }
    }

    public boolean handleFault(SOAPMessageContext smc) {
        return true;
    }

    public void close(MessageContext context) {
        // Nothing to do.
    }
}
