/*
 * Janus 4.0 (c)
 * Copyright (c) 2016 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.dataservice.patient;


import gov.va.med.rpc.KeyValuePair;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.AllergyDataService;
import gov.va.med.vds.vistabeans.Allergy;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class MultiSiteQueryTest extends DataServiceBaseTest {

    @Test
    public void testGetPatientAllergies()
    {
        try
        {
            addSiteVistaKai(patient);

            QueryBean queryBean = setupQuery(user, patient, null, null);
            AllergyDataService allergyService = new AllergyDataService();
            List<Allergy> allergies = allergyService.getPatientAllergies(queryBean);
            assertTrue(allergies.size() > 0);
            for(Allergy allergy : allergies)
            {
                assertFalse(StringUtils.isEmpty(allergy.getSourceProtocol()));
                assertFalse(allergy.getSite() == null);
                assertFalse(allergy.getAllergyName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    private void addSiteVistaKai(Patient p) {
        //vista kai
        Site vistaKai = TestSiteUtils.getInstance().getSite("500");
        p.getVistaSites().add(vistaKai);
        p.getPatientIens().add(new KeyValuePair<String, String>("500", "227"));
    }
}
