/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.NotesDataService;
import gov.va.med.vds.vistabeans.*;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class NotesDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientProgressNotes()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            NotesDataService noteService = new NotesDataService();
            List<ProgressNote> notes = noteService.getPatientProgressNotes(queryBean);
            assertTrue(notes.size() > 0);

            for(ProgressNote noteInpt : notes)
            {
                assertFalse(StringUtils.isEmpty(noteInpt.getSourceProtocol()));
                assertFalse(noteInpt.getSite() == null);
                assertFalse(noteInpt.getNoteType().equals(DataServiceBase.CONNECTION_UNAVAILABLE));               
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientDischargeSummaries()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);
            endDate.set(2015, 0, 30);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            NotesDataService noteService = new NotesDataService();
            List<ProgressNote> notes = noteService.getPatientDischargeSummaries(queryBean);
            assertTrue(notes.size() > 0);
            for(ProgressNote noteOutpt : notes)
            {
                assertFalse(StringUtils.isEmpty(noteOutpt.getSourceProtocol()));
                assertFalse(noteOutpt.getSite() == null);
                assertFalse(noteOutpt.getNoteType().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientConsultRequests()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);
            endDate.set(2015, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            
            NotesDataService noteService = new NotesDataService();
            List<Consult> consults = noteService.getPatientConsultRequests(queryBean);
            assertTrue(consults.size() > 0);
            for(Consult consult : consults)
            {
                assertFalse(StringUtils.isEmpty(consult.getSourceProtocol()));
                assertFalse(consult.getSite() == null);
                assertFalse(consult.getService().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientActiveConsultRequests()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient);

            NotesDataService noteService = new NotesDataService();
            List<Consult> consults = noteService.getPatientActiveConsultRequests(queryBean);
            assertTrue(consults.size() > 0);
            for(Consult consult : consults)
            {
                assertTrue("ACTIVE".equalsIgnoreCase(consult.getStatus()));
                assertFalse(StringUtils.isEmpty(consult.getSourceProtocol()));
                assertFalse(consult.getSite() == null);
                assertFalse(consult.getService().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }



    @Test
    public void testGetPatientConsultRequest()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);
            endDate.set(2015, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            NotesDataService noteService = new NotesDataService();
            List<Consult> consults = noteService.getPatientConsultRequests(queryBean);
            assertTrue(consults.size() > 0);
            Consult consult = consults.get(0);

            assertFalse(StringUtils.isEmpty(consult.getId()));
            queryBean.setItemId(consult.getId());
            queryBean.setRecordSite(consult.getSite());
            Consult consult1 = noteService.getPatientConsultRequest(queryBean);

            assertTrue(consult1.getId().equals(consult.getId()));
            assertTrue(consult1.getService().equals(consult.getService()));
            assertTrue(consult1.getStatus().equals(consult.getStatus()));
            assertFalse(StringUtils.isEmpty(consult.getSourceProtocol()));
            assertFalse(consult.getSite() == null);
            assertFalse(consult.getService().equals(DataServiceBase.CONNECTION_UNAVAILABLE));

        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }


    @Test
    public void testGetConsultReport() {
        try {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);
            endDate.set(2015, 0, 1);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            NotesDataService noteService = new NotesDataService();
            List<Consult> notesConsults = noteService.getPatientConsultRequests(queryBean);
            assertTrue(notesConsults.size() > 0);
            for (Consult consult: notesConsults) {
                queryBean = setupQuery(user, patient, null, null);
                queryBean.setItemId(consult.getId());
                queryBean.setRecordSite(consult.getSite());
                List<FreeTextReport> reports = noteService.getConsultReport(queryBean);
                assertFalse(StringUtils.isEmpty(reports.get(0).getReportText()));
            }
        } catch (VDSException ex) {
            fail(ex.getMessage());
        }

    }

    @Test
    public void testGetProgressNote()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            NotesDataService noteService = new NotesDataService();
            List<ProgressNote> notes = noteService.getPatientProgressNotes(queryBean);
            assertTrue(notes.size() > 0);

            for(ProgressNote noteInpt : notes)
            {
                queryBean = setupQuery(user, patient);
                queryBean.setItemId(noteInpt.getNoteId());
                queryBean.setRecordSite(noteInpt.getSite());
                FreeTextReport progressNoteText = noteService.getProgressNote(queryBean);

                assertTrue(progressNoteText != null);
                assertTrue(progressNoteText.getReportText() != null);
                assertFalse(progressNoteText.getReportText().isEmpty());

            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientPostingsList()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient);

            NotesDataService noteService = new NotesDataService();
            List<PatientPostings> postings = noteService.getPatientPostingsList(queryBean);
            assertTrue(postings.size() > 0);

            for(PatientPostings patientPostings : postings)
            {
                assertFalse(StringUtils.isEmpty(patientPostings.getSourceProtocol()));
                assertFalse(patientPostings.getSite() == null);
                assertFalse(patientPostings.getCwad().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }


    @Test
    public void testGetPatientPostingsAllergiesSummary()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient);
            queryBean.setRecordSite(user.getHostSite());

            NotesDataService noteService = new NotesDataService();
            FreeTextReport summary = noteService.getPatientPostingsAllergiesSummary(queryBean);
            assertNotNull(summary.getReportText());
            assertTrue(summary.getReportText().length() > 0);

        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

}
