/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.VitalsDataService;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Vitals;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class VitalsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientVitals()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2000, 0, 1);
            endDate.set(2010, 10, 10);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            VitalsDataService vitalsService = new VitalsDataService();
            List<Vitals> vitals = vitalsService.getPatientVitals(queryBean);
            assertTrue(vitals.size() > 0);
            for(Vitals vital : vitals)
            {
                assertFalse(StringUtils.isEmpty(vital.getSourceProtocol()));
                assertFalse(vital.getSite() == null);
                assertFalse(vital.getVitalType().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetLatestVitals()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            VitalsDataService vitalsService = new VitalsDataService();
            List<Vitals> vitals = vitalsService.getPatientCurrentVitals(queryBean);
            assertTrue(vitals.size() > 0);
            for(Vitals vital : vitals)
            {
                assertFalse(StringUtils.isEmpty(vital.getSourceProtocol()));
                assertFalse(vital.getSite() == null);
                assertFalse(vital.getVitalType().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
