/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.ehr.common;


import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.junit.After;
import org.junit.Before;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class RPCBaseTest
{
    private static final Logger logger = Logger.getLogger(RPCBaseTest.class.getName());

    protected RPCConnection vistaCxn;
    protected String patientId;
    protected User user;

    protected void vistaConnect()
    {
        Site site = new Site();
        site.setAgency("VA");
        site.setMoniker("HON");
        site.setSiteCode(TestConfig.get("testconfig.ehr.datasource.sitecode"));
        site.setName(TestConfig.get("testconfig.ehr.datasource.sitename"));
        Endpoint endpoint = new Endpoint();
        endpoint.setEnvironment("test");
        endpoint.setHost(TestConfig.get("testconfig.ehr.datasource"));
        endpoint.setPort(Integer.valueOf(TestConfig.get("testconfig.ehr.datasource.port")));
        endpoint.setProtocol("RPC");
        endpoint.setStatus("active");
        List<Endpoint> endpointList = new ArrayList<Endpoint>();
        endpointList.add(endpoint);
        site.setEndpoints(endpointList);

        vistaCxn = new RPCConnection(site);
        try
        {
            vistaCxn.connect();
        }
        catch (RPCException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
    }

    protected void testUserLogin()
    {
        UserRPC userRPC = new UserRPC(vistaCxn);
        try
        {
            String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
            String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
            user = userRPC.login(VistaKernelHash.encrypt(accessCode, false),
                VistaKernelHash.encrypt(verifyCode, false), VistaConstants.CPRS_CONTEXT);

            user.setEncryptedAvCode(VistaKernelHash.encrypt(accessCode + ";" + verifyCode, false));
//            user.setHostSiteId(TestConfig.get(TestConfig.PROVIDER_LOGIN_SITE_CODE));
//            rpcClient = new RPCClient(user.getEncryptedAvCode(),
//                    RPCClient.CPRS_CONTEXT, DataServiceBase.getSiteTable(), user.getHostSite().getSiteCode());
        }
        catch (RPCException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        } catch (VistaKernelHashCountLimitExceededException e) {
            logger.severe(e.getMessage());
        }
    }

    @Before
    public void setUp()
    {
        vistaConnect();
        testUserLogin();

        vistaCxn.setUserIen(user.getUserIen());

        patientId = TestConfig.get(TestConfig.EHR_PATIENT_IEN);

        vistaCxn.setPatientIen(patientId);
    }

    @After
    public void tearDown()
    {
        try
        {
            vistaCxn.disconnect();
        }
        catch (RPCException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
    }
}
