/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.AdmissionsRPC;
import gov.va.med.vds.vistabeans.AdmissionDetail;
import gov.va.med.vds.vistabeans.PatientAdmission;
import org.junit.Before;
import org.junit.Test;

import java.lang.reflect.Method;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.*;

/**
 *
 *
 */
public class AdmissionsRPCTest extends BaseRPCTest
{

    private Calendar startDate;
    private Calendar endDate;

    @Before
    public void setUp() {
        super.setUp();

        startDate = Calendar.getInstance();
        endDate = Calendar.getInstance();

        startDate.set(1990, 9, 16);
        endDate.set(2013, 2, 1);
    }

    @Test
    public void testGetAdmissions() throws RPCException {
        AdmissionsRPC admissionsRPC = new AdmissionsRPC(vistaCxn);

        PatientAdmission[] admissions = admissionsRPC.getAdmissions(patientId, startDate, endDate);
        assertTrue(admissions.length > 0);
    }

    @Test
    public void testGetAdmissionDetailReport() throws RPCException {
        AdmissionsRPC admissionsRPC = new AdmissionsRPC(vistaCxn);

        PatientAdmission[] admissions = admissionsRPC.getAdmissions(patientId, startDate, endDate);
        assertTrue(admissions.length > 0);

        AdmissionDetail detail = admissionsRPC.getAdmissionDetailsReport(patientId, startDate, endDate);
        assertNotNull(detail);
    }
//
//    @Test
//    public void testGetPatientVisitsVPR() throws RPCException {
//        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
//        cxn.connect();
//
//        UserRPC userRPC = new UserRPC (cxn);
//        userRPC.visit(user, "VPR APPLICATION PROXY", true);
//
//        AdmissionsRPC admsVPRRPC = new AdmissionsRPC(cxn);
//
//        PatientAdmission[] admResults = admsVPRRPC.getPatientAdmissionsVPR(patientId, startDate, endDate);
//
//        assertTrue(admResults.length > 0);
//    }

    private static final SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy");
    @Test
    public void testParseAdmissionsDetail() throws Exception {

        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        AdmissionsRPC rpc = new AdmissionsRPC(mockRPCConnection);
        String methodName = "parseAdmissionsDetail";
        Class[] parameterTypes = new Class[2];
        parameterTypes[0] = PatientAdmission[].class;
        parameterTypes[1] = String.class;
        Object[] parameters = new Object[2];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        ArrayList<PatientAdmission> admissions = new ArrayList<PatientAdmission>();
        for (String dateString: ADMISSION_DATES_1) {
            try {
                PatientAdmission adm = new PatientAdmission();
                Date date = sdf.parse(dateString);
                Calendar cal = Calendar.getInstance();
                cal.setTime(date);
                adm.setAdmissionDate(cal);
                admissions.add(adm);
            } catch (ParseException pe) {

            }
        }

        parameters[0] = (PatientAdmission[]) admissions.toArray(new PatientAdmission[0]);
        parameters[1] = ADMISSIONS_DETAIL_1;
//        rpc.parseAdmssionsDetail((PatientAdmission[]) admissions.toArray(new PatientAdmission[0]), ADMISSIONS_DETAIL_1);
        method.invoke(rpc, parameters);

        for (PatientAdmission admission : admissions) {
            assertNotNull(admission.getProvider());
//            assertTrue(admission.getProvider().length() > 0);
            assertNotNull(admission.getSpecialty());
//            assertTrue(admission.getSpecialty().length() > 0);
        }

        admissions = new ArrayList<PatientAdmission>();

        for (String dateString: ADMISSION_DATES_2) {
            try {
                PatientAdmission adm = new PatientAdmission();
                Date date = sdf.parse(dateString);
                Calendar cal = Calendar.getInstance();
                cal.setTime(date);
                adm.setAdmissionDate(cal);
                admissions.add(adm);
            } catch (ParseException pe) {

            }
        }

        parameters[0] = (PatientAdmission[]) admissions.toArray(new PatientAdmission[0]);
        parameters[1] = ADMISSIONS_DETAIL_2;
//        rpc.parseAdmssionsDetail((PatientAdmission[]) admissions.toArray(new PatientAdmission[0]), ADMISSIONS_DETAIL_2);
        method.invoke(rpc, parameters);

        assertTrue(admissions.size() > 0);
    }

    private static final String[] ADMISSION_DATES_1 = {"05/13/2013", "05/10/2013", "04/23/2013"};

    private static final String ADMISSIONS_DETAIL_1 = "\n" +
            "Printed for data from 01/01/2000 to 06/17/2014                 06/17/2014 16:20\n" +
            "***********************  CONFIDENTIAL  SUMMARY   pg. 1 ************************\n" +
            "VHAPATIENT,AMANDA    100-55-0087                                DOB: 03/27/1975\n" +
            "                                 131-2AO 227-2\n" +
            " \n" +
            "-------------------------- ADC - Admission/Discharge --------------------------\n" +
            "                                 \n" +
            "05/13/2013 - Present     131-2AO 227-2                  LOS: 400\n" +
            "  Last Tr Specialty: MEDICAL OBSERVATION          Last Prov: KOTHA,SREEVALLI\n" +
            "Admitting Diagnosis: TESTING THE RPC\n" +
            " \n" +
            "05/10/2013 - 05/13/2013                                 LOS: 3\n" +
            "  Last Tr Specialty: MEDICAL OBSERVATION          Last Prov: KOTHA,SREEVALLI\n" +
            "         Bedsection: MEDICAL OBSERVATION\n" +
            " \n" +
            "04/23/2013 - 04/29/2013                                 LOS: 6\n" +
            "  Last Tr Specialty: GENERAL SURGERY              Last Prov:\n" +
            "         Bedsection: GENERAL SURGERY\n" +
            " \n" +
            "*** END ***************  CONFIDENTIAL  SUMMARY   pg. 1 ***********************\n";

    private static final String[] ADMISSION_DATES_2 = {"06/05/2014", "06/10/2014"};

    private static final String ADMISSIONS_DETAIL_2 = "\n" +
            "Printed for data from 01/01/2000 to 06/17/2014                 06/17/2014 16:25\n" +
            "***********************  CONFIDENTIAL  SUMMARY   pg. 1 ************************\n" +
            "IPOABOOD,VANCE EUGENE    101-94-2147                            DOB: 11/03/1948\n" +
            "                                 W7U W7U-218-A\n" +
            " \n" +
            "-------------------------- ADC - Admission/Discharge --------------------------\n" +
            "                                 \n" +
            "06/05/2014 - Present     W7U W7U-218-A                  LOS: 12\n" +
            "  Last Tr Specialty: GENERAL CWT/TR               Last Prov: MYERS,ANGELA\n" +
            "Admitting Diagnosis: DEMENTIA\n" +
            " \n" +
            "*** END ***************  CONFIDENTIAL  SUMMARY   pg. 1 ***********************\n";

}
