/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.patient.dao.rpc.AllergyRPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Allergy;
import gov.va.med.vds.vistabeans.AllergyDetail;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 *
 *
 */
public class AllergyRPCTest extends BaseRPCTest {

    @Test
    public void testGetAllergies() throws RPCException
    {
        AllergyRPC allergyRPC = new AllergyRPC(vistaCxn);
        Allergy[] allergies = allergyRPC.getAllergies(patientId);
        assertTrue(allergies.length > 0);
    }

    @Test
    public void testGetAllergyDetail() throws RPCException
    {
        AllergyRPC allergyRPC = new AllergyRPC(vistaCxn);
        Allergy[] allergies = allergyRPC.getAllergies(patientId);
        assertTrue(allergies.length > 0);

        AllergyDetail allergyDetail = allergyRPC.getAllergyDetail(patientId, allergies[0].getAllergyId());
        assertFalse(StringUtils.isEmpty(allergyDetail.getReportText()));
    }

    @Test
    public void testGetAllergiesVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        AllergyRPC allsVPRRPC = new AllergyRPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -200);
        Allergy[] allergies= allsVPRRPC.getAllergiesVPR(patientId, startDate, endDate);

        assertTrue(allergies.length > 0);
    }
}