/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;

import java.util.logging.Level;
import java.util.logging.Logger;


/**
 *
 *
 */
public abstract class BaseRPCTest {

    private static final Logger logger = Logger.getLogger(BaseRPCTest.class.getName());

    protected RPCConnection vistaCxn;
    protected String patientId;

    protected Site site;

    protected User user;

    public BaseRPCTest()
    {
    }

    @BeforeClass
    public static void setUpClass() throws RPCException
    {
    
    }

    @AfterClass
    public static void tearDownClass() throws RPCException {
    }

    protected void vistaConnect()
    {
        site = TestSiteUtils.getInstance().getSite(TestConfig.get(TestConfig.EHR_SITECODE));

        vistaCxn = new RPCConnection(site);

        try
        {
            vistaCxn.connect();
        }
        catch (RPCException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
    }

    protected void testUserLogin()
    {
        UserRPC userRPC = new UserRPC(vistaCxn);
        String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
        String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);
        try {
            user = userRPC.login(VistaKernelHash.encrypt(accessCode, false),
                    VistaKernelHash.encrypt(verifyCode, false), VistaConstants.CPRS_CONTEXT);
        } catch (RPCException e) {
            logger.severe(e.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            logger.severe(e.getMessage());
        }
    }

    @Before
    public void setUp()
    {
        vistaConnect();
        testUserLogin();

        patientId = TestConfig.get(TestConfig.EHR_PATIENT_IEN);
    }

    @After
    public void tearDown() 
    {
        try
        {
            vistaCxn.disconnect();
        }
        catch (RPCException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
    }

}