/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.vds.patient.dao.rpc.InsuranceRPC;
import gov.va.med.vds.vistabeans.InsuranceBean;
import org.junit.Test;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 *
 *
 */
public class InsuranceRPCTest
{
    private static final Logger logger = Logger.getLogger(InsuranceRPCTest.class.getName());

    @Test
    public void testParseInsurance() throws Exception
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        InsuranceRPC insRPC = new InsuranceRPC(mockRPCConnection);
        String methodName = "parseInsurance";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = insRPC.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        parameters[0] = INSURANCE_POLICIES_VPR_RESPONSE;
        InsuranceBean[] policies = (InsuranceBean[]) method.invoke(insRPC, parameters);

        assertTrue(policies.length == 3);
    }

    private static final String INSURANCE_POLICIES_VPR_RESPONSE = "<results version='1.1' timeZone='-0400' >\n" +
            "<insurancePolicies total='3' >\n" +
            "<insurancePolicy>\n" +
            "<company code='7168777.868' name='BLUE CROSS/BS WY' >\n" +
            "<address streetLine1='PO BOX 2266' city='CHEYENNE' stateProvince='56' postalCode='WYOMING' />\n" +
            "<telecomList>" +
            "<telecom usageType='WP' value='888-999-0000' />\n" +
            "</telecomList>\n" +
            "</company>\n" +
            "<effectiveDate value='2940109' />\n" +
            "<facility code='451' name='IPOTEST 1' />\n" +
            "<groupName value='KIESOW' />\n" +
            "<groupNumber value='GRP NUM 6033' />\n" +
            "<id value='34458;7168777.868;6033' />\n" +
            "<insuranceType code='40' name='PREFERRED PROVIDER ORGANIZATION (PPO)' />\n" +
            "<relationship value='SPOUSE' />\n" +
            "<subscriber code='R57793861' name='ADGER,BRENDAN' />\n" +
            "</insurancePolicy>\n" +
            "<insurancePolicy>\n" +
            "<company code='7170593' name='CIGNA' >\n" +
            "<address streetLine1='PO BOX 5909' city='SCRANTON' stateProvince='42' postalCode='PENNSYLVANIA' />\n" +
            "</company>\n" +
            "<effectiveDate value='3090311' />\n" +
            "<facility code='451' name='IPOTEST 1' />\n" +
            "<groupName value='BRAINERD' />\n" +
            "<groupNumber value='GRP NUM 13525' />\n" +
            "<id value='34458;7170593;13525' />\n" +
            "<insuranceType code='40' name='PREFERRED PROVIDER ORGANIZATION (PPO)' />\n" +
            "<relationship value='PATIENT' />\n" +
            "<subscriber code='101300999' name='ADGER,VANCE J' />\n" +
            "</insurancePolicy>\n" +
            "<insurancePolicy>\n" +
            "<company code='7170115' name='EXPRESS SCRIPTS' >\n" +
            "<address streetLine1='PO BOX 390873-BL0470' city='BLOOMINGTON' stateProvince='MINNESOTA' postalCode='39807' />\n" +
            "</company>\n" +
            "<effectiveDate value='3090311' />\n" +
            "<facility code='451' name='IPOTEST 1' />\n" +
            "<groupName value='BEAUMONTE' />\n" +
            "<groupNumber value='GRP NUM 14441' />\n" +
            "<id value='34458;7170115;14441' />\n" +
            "<insuranceType code='57' name='PRESCRIPTION' />\n" +
            "<relationship value='PATIENT' />\n" +
            "<subscriber code='101300999' name='ADGER,VANCE J' />\n" +
            "</insurancePolicy>\n" +
            "</insurancePolicies>\n" +
            "</results>\n";

    private static final String SINGLE_INSURANCE_POLICY_VPR = "<insurancePolicy>\n" +
            "<company code='7170115' name='EXPRESS SCRIPTS' >\n" +
            "<address streetLine1='PO BOX 390873-BL0470' city='BLOOMINGTON' stateProvince='27' postalCode='MINNESOTA' />\n" +
            "</company>\n" +
            "<effectiveDate value='3090311' />\n" +
            "<facility code='451' name='IPOTEST 1' />\n" +
            "<groupName value='BEAUMONTE' />\n" +
            "<groupNumber value='GRP NUM 14441' />\n" +
            "<id value='34458;7170115;14441' />\n" +
            "<insuranceType code='57' name='PRESCRIPTION' />\n" +
            "<relationship value='PATIENT' />\n" +
            "<subscriber code='101300999' name='ADGER,VANCE J' />\n" +
            "</insurancePolicy>\n";


}
