/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.LabsRPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.LabOrder;
import gov.va.med.vds.vistabeans.LabResult;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertTrue;

/**
 *
 *
 */
public class LabsRPCTest extends BaseRPCTest
{
    private Calendar startDate;
    private Calendar endDate;
    private LabsRPC labsRPC;

    @Before
    public void setUp() {
        super.setUp();

        labsRPC = new LabsRPC(vistaCxn);

        startDate = Calendar.getInstance();
        endDate = Calendar.getInstance();

        startDate.set(1990, 9, 16);
        endDate.set(2013, 2, 1);

    }

    @Test
    public void testGetLabOrders() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientLabs(patientId, startDate, endDate);

        assertTrue(labs.length > 0);

    }

    @Test
    public void testGetMicroLabOrders() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientMicroLabOrders(patientId, startDate, endDate);

        assertTrue(labs.length > 0);

    }

    @Test
    public void testGetAnaPathLabOrders() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientAnaPathLabOrders(patientId, startDate, endDate);

        assertTrue(labs.length > 0);

    }

    @Test
    public void testGetMicroLabResults() throws RPCException
    {
        LabResult[] labs = labsRPC.getPatientMicroLabResults(patientId, startDate, endDate);

        assertTrue(labs.length > 0);

    }

    @Test
    public void testGetAnaPathLabResults() throws RPCException
    {
        LabResult[] labs = labsRPC.getPatientAnaPathLabResults(patientId, startDate, endDate);

        assertTrue(labs.length > 0);

    }

    @Test
    public void testGetLabOrderResult() throws RPCException
    {
        LabResult[] results = labsRPC.getLabOrderResult(patientId, startDate, endDate);

        assertTrue(results.length > 0);

    }

    @Test
    public void testGetLabOrderResultCollectionDateAccession() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientLabs(patientId, startDate, endDate);
        assertTrue(labs.length > 0);

        Calendar collectionDate = labs[1].getCollectionDate();
        String accessionNumber = labs[1].getAccessionNumber();

        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        LabsRPC labsVPRRPC = new LabsRPC(cxn);

        LabResult[] results = labsVPRRPC.getLabOrderResult(patientId, collectionDate, accessionNumber);
        assertTrue(results.length > 0);

//        for (LabResult result : results) {
//            assertTrue(collectionDate.equals(result.getResultDate()));
//        }
    }


    @Test
    public void testGetLabTests() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientLabs(patientId, startDate, endDate);
        assertTrue(labs.length > 0);

        Calendar collectionDate = labs[1].getCollectionDate();
        String accessionNumber = labs[1].getAccessionNumber();

        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        LabsRPC labsVPRRPC = new LabsRPC(cxn);

        LabResult[] results = labsVPRRPC.getLabOrderResult(patientId, collectionDate, accessionNumber);
        assertTrue(results.length > 0);

        //get the first testName
        String testName = results[1].getTestName();
//        String testName = "GLUCOSE";
        assertTrue(testName.length() > 0);

        LabResult[] testResults = labsVPRRPC.getPatientLabTestResults(patientId, startDate, endDate, testName);
        assertTrue(testResults.length > 0);
    }


    @Test
    public void testGetLabOrderResultCollectionDateSpecimen() throws RPCException
    {
        LabOrder[] labs = labsRPC.getPatientLabs(patientId, startDate, endDate);
        assertTrue(labs.length > 0);

        Calendar collectionDate = labs[0].getCollectionDate();
        String specimenType = labs[0].getCollectionSample();

        LabResult[] results = labsRPC.getLabOrderResult(patientId, collectionDate, specimenType);
        assertTrue(results.length > 0);

        for (LabResult result : results) {
            assertTrue(specimenType.equals(result.getSpecimen()));
            assertTrue(collectionDate.equals(result.getResultDate()));
        }

    }
//
//    @Test
//    public void testGetLabResultsReport() throws RPCException
//    {
//        String report = labsRPC.getLabResultsReport(patientId, startDate, endDate);
//
//        assertTrue(report.length() > 0);
//
//    }

    @Test
    public void testGetPatientLabTestResultsByName() throws RPCException
    {
        final String testName = "LDL";

        LabResult[] labs = labsRPC.getPatientLabTestResults(patientId, startDate, endDate, testName);

        assertTrue(labs.length > 0);

        for (LabResult lab : labs) {
            assertTrue(testName.equals(lab.getTestName()));
        }
    }

    @Test
    public void testGetPatientLabsVPR() throws  RPCException
    {
//        vistaCxn.setCurrentContext("VPR APPLICATION PROXY");

        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        LabsRPC labsVPRRPC = new LabsRPC(cxn);

        LabOrder[] labOrders = labsVPRRPC.getVPRLabOrders(patientId, startDate, endDate);

        assertTrue(labOrders.length > 0);
    }

    @Test
    public void testGetPatientLabResultsVPR() throws  RPCException
    {
//        vistaCxn.setCurrentContext("VPR APPLICATION PROXY");

        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        LabsRPC labsVPRRPC = new LabsRPC(cxn);

        LabResult[] labResults = labsVPRRPC.getVPRLabResults(patientId, startDate, endDate);

        assertTrue(labResults.length > 0);
    }

    @Test
    public void testGetLabReport() throws RPCException
    {
//        final String PARSE_DATE_FORMAT = "MMM dd, yyyy HH:mm";
//        SimpleDateFormat df = new SimpleDateFormat(PARSE_DATE_FORMAT);
//        String collectedDate = df.format(startDate.getTime());


        startDate.set(1995, Calendar.OCTOBER, 18, 10, 40);
        String accessionNumber = "CH 1016 19";

        FreeTextReport labReport = labsRPC.getLabReport(patientId, startDate, accessionNumber);

        assertTrue(labReport.getReportText() != null);

    }

}
