/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.PCERPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.PCE;
import org.junit.Test;

import java.lang.reflect.Method;
import java.util.Calendar;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 *
 *
 */
public class PCERPCTest extends BaseRPCTest {
//
//    @Test
//    public void testGetSkinTests() throws RPCException
//    {
//        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
//        cxn.connect();
//        UserRPC userRPC = new UserRPC(cxn);
//        userRPC.visit(user, "VPR APPLICATION PROXY", true);
//
//        PCERPC pceRPC = new PCERPC(cxn);
//
//        Calendar endDate = Calendar.getInstance();
//        Calendar startDate = Calendar.getInstance();
//        startDate.add(Calendar.YEAR, -5);
//        PCE[] skinTests = pceRPC.getSkinTests(patientId, startDate, endDate);
//
//        assertTrue(skinTests.length > 0);
//
//    }

    @Test
    public void testGetSkinTests() throws Exception {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);
        when(mockRPCConnection.isConnected()).thenReturn(true);

        PCERPC rpc = new PCERPC(mockRPCConnection);
        String methodName = "toPCESkinTests";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = rpc.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);
        parameters[0] = SKIN_TEST_REPORT;

        Object pces = method.invoke(rpc, parameters);

        assertNotNull(pces);
    }

    @Test
    public void testGetExams() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        PCERPC pceRPC = new PCERPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -5);
        PCE[] exams = pceRPC.getExams(patientId, startDate, endDate);

        assertTrue(exams.length > 0);

    }

    @Test
    public void testGetEducationTopics() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        PCERPC pceRPC = new PCERPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -5);
        PCE[] educationTopics = pceRPC.getEducationTopics(patientId, startDate, endDate);

        assertTrue(educationTopics.length > 0);

    }

    private static final String SKIN_TEST_REPORT = "\n" +
            "Printed for data from 06/14/1879 to 05/06/2016                 05/06/2016 14:11\n" +
            "***********************  CONFIDENTIAL  SUMMARY   pg. 1 ************************\n" +
            "EIGHT,PATIENT    666-00-0008    7A GEN MED 722-B                DOB: 04/07/1935\n" +
            "\n" +
            "------------------------------- ST - Skin Tests -------------------------------\n" +
            "                                  \n" +
            "                                     -  Date  - \n" +
            "Skin Test   Reading  Results     Admin.      Reading     Facility\n" +
            "\n" +
            "CANDIDA      1 mm    DOUBTFUL    08/14/2015  09/01/2015  CAMP BEE\n" +
            "     Comments: boo \n" +
            "             2 mm    NO TAKE     09/10/2013              CAMP BEE\n" +
            "COCCI        1 mm                08/14/2015              CAMP BEE\n" +
            "     Comments: sdfa adsf a fcada dfr a adfa dar ae adf aswer adc acdasdc\n" +
            "     ear aecda cadr weaac daf aer acd adf aer.  \n" +
            "HISTO       11 mm    POSITIVE    09/10/2013              CAMP BEE\n" +
            "PPD          0 mm    NEGATIVE    08/14/2014              CAMP BEE\n" +
            "SCHICK      12 mm    NO TAKE     09/10/2013              CAMP BEE\n" +
            "TETANUS      0 mm                08/14/2015              CAMP BEE\n" +
            "\n" +
            "*** END ***************  CONFIDENTIAL  SUMMARY   pg. 1 ***********************";


}