/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.RadiologyRPC;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.RadiologyExam;
import gov.va.med.vds.vistabeans.RadiologyReport;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertTrue;
/**
 *
 *
 */
public class RadiologyRPCTest extends BaseRPCTest
{
    private RadiologyRPC radiologyRPC;
    private Calendar startDate;
    private Calendar endDate;

    @Before
    public void setUp () {
        super.setUp();

        radiologyRPC = new RadiologyRPC(vistaCxn);
        startDate = Calendar.getInstance();
        endDate = Calendar.getInstance();
        startDate.set(1999, 0, 1);
        endDate.set(2013, 0, 1);

    }


    @Test
    public void testGetRadiologyExams() throws RPCException
    {
        RadiologyExam[] exams = radiologyRPC.getRadiologyExams(patientId, startDate, endDate);
        assertTrue(exams.length > 0);
    }

    @Test
    public void testGetRadiologyReport() throws RPCException {
        RadiologyExam[] exams = radiologyRPC.getRadiologyExams(patientId, startDate, endDate);
        assertTrue(exams.length > 0);

        RadiologyReport report = radiologyRPC.getRadiologyReport(exams[0].getExamId(), exams[0].getAccessionNumber());
        assertTrue(report.getReportText().length() > 0);
    }

    @Test
    public void testGetRadiologyVPR() throws RPCException {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();

        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        RadiologyRPC radVPRRPC = new RadiologyRPC(cxn);

        RadiologyReport[] reports = radVPRRPC.getRadiologyVPR(patientId, startDate, endDate);

        assertTrue(reports.length > 0);
    }
}
