/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.patient.dao.rpc.*;
import gov.va.med.vds.vistabeans.*;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.File;
import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Scanner;

import static org.junit.Assert.assertTrue;

/**
 * Created with IntelliJ IDEA.
 * User: sishii
 * Date: 8/28/13
 * Time: 1:37 PM
 * To change this template use File | Settings | File Templates.
 */
public class VPRStaticFileParsingTest {

    @Test
    public void testParseMedsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String medsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\zzmousehismeds.xml")).useDelimiter("\\Z").next();
        MedicationsRPC medicationsRPC = new MedicationsRPC(cxn);


        Method parseMethod = null;
        Method[] methods = medicationsRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseMedsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            Medication[] meds = (Medication[]) parseMethod.invoke(medicationsRPC, "patientID", medsVPRexample, "O");

            assertTrue(meds.length > 0);
        }
    }

    @Test
    public void testParseLabsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String labsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\testaccessionorder.xml")).useDelimiter("\\Z").next();
        LabsRPC labsVPRRPC = new LabsRPC(cxn);


        Method parseMethod = null;
        Method[] methods = labsVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseLabsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            LabOrder[] labs = (LabOrder[]) parseMethod.invoke(labsVPRRPC, "patientID", labsVPRexample);

            assertTrue(labs.length > 0);
        }
    }

    @Test
    public void testParseLabResultsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String labsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\testaccession.xml")).useDelimiter("\\Z").next();
        LabsRPC labsVPRRPC = new LabsRPC(cxn);

        Method parseMethod = null;
        Method[] methods = labsVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseLabResultsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            LabResult[] labs = (LabResult[]) parseMethod.invoke(labsVPRRPC, "patientID", labsVPRexample);

            assertTrue(labs.length > 0);
        }
    }

    @Test
    public void testParseImmunizationsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String immunizationsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\rpdvi.xml")).useDelimiter("\\Z").next();
        ImmunizationsRPC immVPRRPC = new ImmunizationsRPC(cxn);


        Method parseMethod = null;
        Method[] methods = immVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseImmunizationsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            Immunization[] imms = (Immunization[]) parseMethod.invoke(immVPRRPC, immunizationsVPRexample);

            assertTrue(imms.length > 0);
        }
    }

    @Test
    public void testParseVitalsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String vitalsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\rpdvi.xml")).useDelimiter("\\Z").next();
        VitalsRPC vitalsVPRRPC = new VitalsRPC(cxn);


        Method parseMethod = null;
        Method[] methods = vitalsVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseVitalsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            Vitals[] vitals = (Vitals[]) parseMethod.invoke(vitalsVPRRPC, vitalsVPRexample);

            assertTrue(vitals.length > 0);
        }
    }

    @Test
    public void testParseAllergiesFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String allergiesVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\rpdvi.xml")).useDelimiter("\\Z").next();
        AllergyRPC allergiesVPRRPC = new AllergyRPC(cxn);


        Method parseMethod = null;
        Method[] methods = allergiesVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseAllergiesFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            Allergy[] allergies = (Allergy[]) parseMethod.invoke(allergiesVPRRPC, allergiesVPRexample);

            assertTrue(allergies.length > 0);
        }
    }

    @Test
    public void testParseProgressNotesFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String pnVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\rpdvi.xml")).useDelimiter("\\Z").next();
        NotesRPC notesVPRRPC = new NotesRPC(cxn);


        Method parseMethod = null;
        Method[] methods = notesVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseProgressNotesFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            ProgressNote[] pns = (ProgressNote[]) parseMethod.invoke(notesVPRRPC, pnVPRexample);

            assertTrue(pns.length > 0);
        }
    }

    @Test
         public void testParseProblemsFromVPR() throws RPCException, FileNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {

        RPCConnection cxn = Mockito.mock(RPCConnection.class);

        String probsVPRexample = new Scanner(new File("C:\\Users\\sishii\\Documents\\Work\\interop\\RPC docs\\VPR Examples\\rpdvi.xml")).useDelimiter("\\Z").next();
        ProblemsRPC probsVPRRPC = new ProblemsRPC(cxn);


        Method parseMethod = null;
        Method[] methods = probsVPRRPC.getClass().getDeclaredMethods();
        for(Method m: methods) {
            String mname = m.getName();
            if (mname.startsWith("parseProblemsFromVPR")) {
                parseMethod = m;
            }
        }
        if (parseMethod != null) {
            parseMethod.setAccessible(true);

            ProblemDetail[] probs = (ProblemDetail[]) parseMethod.invoke(probsVPRRPC, "patientID", probsVPRexample);

            assertTrue(probs.length > 0);
        }
    }


}
