/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */

package gov.va.med.des.common;

import gov.va.med.foundations.utilities.EncryptUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class AppConfig {

    private static final Logger logger = LoggerFactory.getLogger(AppConfig.class);
    /**
     * jMeadows version number *
     */
    private static final String APP_VERSION = "app.version";
    /**
     * jMeadows app environment *
     */
    private static final String APP_ENVIRONMENT = "app.environment";
//    /**
//     * log4j properties file path *
//     */
//    private static final ServiceType LOG4J_PROPERTIES = "LOG4J_PROPERTIES";
//

    /**
     * SITE_STATUS_PATH endpoint  *
     */
    private static final String SITE_STATUS_PATH = "SITE_STATUS_PATH";
    /**
     * DES_TRANSACTION_SERVICE_PATH endpoint  *
     */
    private static final String DES_TRANSACTION_SERVICE_PATH = "DES_TRANSACTION_SERVICE_PATH";
    /**
     * SYSTEM_MONITOR_PATH endpoint  *
     */
    private static final String SYSTEM_MONITOR_PATH = "SYSTEM_MONITOR_PATH";

    /**
     * REST_CONNECTION_TIMEOUT_MS timeout  *
     */
    private static final String REST_CONNECTION_TIMEOUT_MS = "REST_CONNECTION_TIMEOUT_MS";
    /**
     * REST_READ_TIMEOUT_MS timeout  *
     */
    private static final String REST_READ_TIMEOUT_MS = "REST_READ_TIMEOUT_MS";
    /**
     * REST_POLLER_TIMEOUT_MS timeout  *
     */
    private static final String REST_POLLER_TIMEOUT_MS = "REST_POLLER_TIMEOUT_MS";

    private static final String SITE_STATUS_EHX_ENTITY = "SITE_STATUS_EHX_ENTITY";
    private static final String SITE_STATUS_EHX_ENTITY_TOKEN = "SITE_STATUS_EHX_ENTITY_TOKEN";
    private static final String SITE_STATUS_ENTITY = "SITE_STATUS_ENTITY";
    private static final String SITE_STATUS_ENTITY_TOKEN = "SITE_STATUS_ENTITY_TOKEN";
    private static final String DES_TRANSACTION_SERVICE_ENTITY = "DES_TRANSACTION_SERVICE_ENTITY";
    private static final String DES_TRANSACTION_SERVICE_ENTITY_TOKEN = "DES_TRANSACTION_SERVICE_ENTITY_TOKEN";
    private static final String SYSTEM_MONITOR_ENTITY = "SYSTEM_MONITOR_ENTITY";
    private static final String SYSTEM_MONITOR_ENTITY_TOKEN = "SYSTEM_MONITOR_ENTITY_TOKEN";

    private static final String SITE_STATUS_AUTH = "SITE_STATUS_AUTH";
    private static final String DES_TRANSACTION_SERVICE_AUTH = "DES_TRANSACTION_SERVICE_AUTH";
    private static final String SYSTEM_MONITOR_AUTH = "SYSTEM_MONITOR_AUTH";
    private static final String PURGE_INTERVAL = "PURGE_INTERVAL";
    /**
     * true to utilize C3PO for as the sql connection pooling api.
     * If false, defaults to utilizing JNDI lookup for connection pool.
     */
    private static final String USE_C3PO = "USE_C3PO";
    /**
     * SQL server JDBC connection string (for use with C3PO) *
     */
    private static final String SQL_URL = "SQL_URL";
    /**
     * SQL server driver (for use with C3PO) *
     */
    private static final String SQL_DRIVER = "SQL_DRIVER";
    /**
     * JDNI connection pooling lookup name *
     */
    private static final String JNDI_JDBC = "JNDI_JDBC";
    /**
     * General service request timeout setting in milliseconds *
     */
    private static final String REQUEST_TIMEOUT_MS = "REQUEST_TIMEOUT_MS";
    /**
     * General service connection timeout setting in milliseconds *
     */
    private static final String CONNECTION_TIMEOUT_MS = "CONNECTION_TIMEOUT_MS";
    /**
     * General thread timeout in milliseconds *
     */
    private static final String THREAD_TIMEOUT_MS = "THREAD_TIMEOUT_MS";
    /**
     * Default error message string *
     */
    private static final String ERROR_MSG_CONNECTION_UNAVAILABLE = "ERROR_MSG_CONNECTION_UNAVAILABLE";

    /**
     * VA agency name string Should be configured as VA
     */
    private static final String AGENCY_VA = "AGENCY_VA";
    /**
     * DOD agency name string. Should be configured as DOD
     */
    private static final String AGENCY_DOD = "AGENCY_DOD";
    /**
     * SiteMap's EHCache name
     */
    private static final String EHCACHE_NAME_SITEMAP = "EHCACHE_NAME_SITEMAP";
    /**
     * RPC endpoint environment (enterprise, hrg, etc.)
     */
    private static final String RPC_ENDPOINT_ENVIRONMENT = "RPC_ENDPOINT_ENVIRONMENT";
    /**
     * CACHE endpoint environment (enterprise, hrg, etc.)
     */
    private static final String CACHE_ENDPOINT_ENVIRONMENT = "CACHE_ENDPOINT_ENVIRONMENT";
    /**
     * Share endpoint environment
     */
    private static final String SHARE_ENDPOINT_ENVIRONMENT = "SHARE_ENDPOINT_ENVIRONMENT";
    /**
     * Medweb endpoint environment
     */
    private static final String MEDWEB_ENDPOINT_ENVIRONMENT = "MEDWEB_ENDPOINT_ENVIRONMENT";
    private static final String AHLTA_ENDPOINT_ENVIRONMENT = "AHLTA_ENDPOINT_ENVIRONMENT";
    private static final AppConfig INSTANCE = new AppConfig();
    private Properties appConfigProp;

    private AppConfig() {
        appConfigProp = new Properties();

        ClassLoader classLoader = AppConfig.class.getClassLoader();

        if (classLoader == null) {
            throw new NullPointerException("ClassLoader is null!");
        }

        InputStream inputStream = null;

        try {
            inputStream = classLoader.getResourceAsStream("application.properties");
            appConfigProp.load(inputStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {

            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                logger.error("IOException in AppConfig");
            }
        }

        try {
            inputStream = classLoader.getResourceAsStream(String.format("appconfig-%s.properties", appConfigProp.get(APP_ENVIRONMENT)));
            appConfigProp.load(inputStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {

            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                logger.error("IOException in AppConfig");
            }
        }
    }

    public static final AppConfig getInstance() {
        return INSTANCE;
    }

    /**
     * jMeadows version number *
     */
    public final String getAppVersion() {
        return appConfigProp.getProperty(APP_VERSION);
    }

    /**
     * jMeadows app environment *
     */
    public final String getAppEnvironment() {
        return appConfigProp.getProperty(APP_ENVIRONMENT);
    }

    /**
     * true to utilize C3PO for as the sql connection pooling api.
     * If false, defaults to utilizing JNDI lookup for connection pool.
     */
    public final boolean useC3PO() {
        return Boolean.valueOf(appConfigProp.getProperty(USE_C3PO));
    }

    /**
     * SQL server JDBC connection string (for use with C3PO) *
     */
    public final String getSQLURL() {
        return appConfigProp.getProperty(SQL_URL);
    }

    /**
     * SQL server driver (for use with C3PO) *
     */
    public final String getSQLDriver() {
        return appConfigProp.getProperty(SQL_DRIVER);
    }

    /**
     * JDNI connection pooling lookup name *
     */
    public final String getJDBC_JNDILookup() {
        return appConfigProp.getProperty(JNDI_JDBC);
    }

    /**
     * General service request timeout setting in milliseconds *
     */
    public final int getRequestTimeoutMS() {
        return Integer.parseInt(appConfigProp.getProperty(REQUEST_TIMEOUT_MS));
    }

    /**
     * General service connection timeout setting in milliseconds *
     */
    public final int getConnectionTimeoutMS() {
        return Integer.parseInt(appConfigProp.getProperty(CONNECTION_TIMEOUT_MS));
    }


    public final long getServiceMonitorTaskTimerDelay() {
        return Long.parseLong(appConfigProp.getProperty("SERVICE_MONITOR_TASK_TIMER_DELAY"));
    }

    public final long getServiceMonitorTaskTimerPeriod() {
        return Long.parseLong(appConfigProp.getProperty("SERVICE_MONITOR_TASK_TIMER_PERIOD"));
    }

    public final Boolean isServiceMonitorTest() {
        return Boolean.parseBoolean(appConfigProp.getProperty("SERVICE_MONITOR_TEST"));
    }

    public final Boolean isServiceMonitorTestServices() {
        return Boolean.parseBoolean(appConfigProp.getProperty("SERVICE_MONITOR_TEST_SERVICES"));
    }

    public final Boolean isServiceMonitorTestSystem() {
        return Boolean.parseBoolean(appConfigProp.getProperty("SERVICE_MONITOR_TEST_SYSTEM"));
    }

    public final String getEmailUsername() {
        return EncryptUtil.decrypt(appConfigProp.getProperty("EMAIL_USERNAME"));
    }

    public final String getEmailPassword() {
        return EncryptUtil.decrypt(appConfigProp.getProperty("EMAIL_PASSWORD"));
    }

    public final String getEmailHost() {
        return appConfigProp.getProperty("EMAIL_HOST");
    }

    public final String getEmailSSLPort() {
        return appConfigProp.getProperty("EMAIL_SSL_PORT");
    }

    public final String getEmailFrom() {
        return appConfigProp.getProperty("EMAIL_FROM");
    }

    public final String getEmailTo() {
        return appConfigProp.getProperty("EMAIL_TO");
    }

    public final String getEmailSrcSys() {
        return appConfigProp.getProperty("EMAIL_SRC_SYS");
    }

    public final Boolean isEmailStatusChangeOnly() {
        return Boolean.parseBoolean(appConfigProp.getProperty("EMAIL_STATUS_CHANGE_ONLY"));
    }

    public final Boolean isEmailErrorAtEveryCheck() {
        return Boolean.parseBoolean(appConfigProp.getProperty("EMAIL_ERROR_AT_EVERY_CHECK"));
    }

    public final long getEmailSendTaskTimerPeriod() {
        return Long.parseLong(appConfigProp.getProperty("EMAIL_SEND_TASK_TIMER_PERIOD"));
    }

    public final String getUserName() {
        return EncryptUtil.decrypt(appConfigProp.getProperty("ENCRYPTED_USER_NAME"));
    }

    public final String getUserIEN() {
        return EncryptUtil.decrypt(appConfigProp.getProperty("ENCRYPTED_USER_IEN"));
    }

    public final String getEncryptedUserAVCode() {
        return appConfigProp.getProperty("ENCRYPTED_USER_AV_CODE");
    }

    public final String getEncryptedUserFederatedUID() {
        return appConfigProp.getProperty("ENCRYPTED_USER_FEDERATED_UID");
    }

    public final String getUserSiteCode() {
        return appConfigProp.getProperty("USER_SITE_CODE");
    }

    public final String getPatientEDIPIForMVI() {
        return appConfigProp.getProperty("PATIENT_EDIPI_MVI");
    }

    public final String getPatientEDIPIForBHIE() {
        return appConfigProp.getProperty("PATIENT_EDIPI_BHIE");
    }

    public final String getEnv() {
        return appConfigProp.getProperty("ENV");
    }

    public final String getSiteStatusPath() {
        return appConfigProp.getProperty("SITE_STATUS_PATH");
    }

    public final String getDesTransactionServicePath() {
        return appConfigProp.getProperty("DES_TRANSACTION_SERVICE_PATH");
    }

    public final String getSystemMonitorPath() {
        return appConfigProp.getProperty("SYSTEM_MONITOR_PATH");
    }

    public final String getDesMonitorUrl() {
        return appConfigProp.getProperty("DES_MONITOR_URL");
    }

    public final int getRestConnectionTimeoutMs() {
        return Integer.parseInt(appConfigProp.getProperty("REST_CONNECTION_TIMEOUT_MS"));
    }

    public final int getRestReadTimeoutMs() {
        return Integer.parseInt(appConfigProp.getProperty("REST_READ_TIMEOUT_MS"));
    }

    public final String getSiteStatusEntity() {
        return appConfigProp.getProperty("SITE_STATUS_ENTITY");
    }

    public final String getSiteStatusEntityToken() {
        return appConfigProp.getProperty("SITE_STATUS_ENTITY_TOKEN");
    }

    public final String getDesTransactionServiceEntity() {
        return appConfigProp.getProperty("DES_TRANSACTION_SERVICE_ENTITY");
    }

    public final String getDesTransactionServiceEntityToken() {
        return appConfigProp.getProperty("DES_TRANSACTION_SERVICE_ENTITY_TOKEN");
    }

    public final String getSystemMonitorEntity() {
        return appConfigProp.getProperty("SYSTEM_MONITOR_ENTITY");
    }

    public final String getSystemMonitorEntityToken() {
        return appConfigProp.getProperty("SYSTEM_MONITOR_ENTITY_TOKEN");
    }

    public final Boolean isSiteStatusAuth() {
        return Boolean.parseBoolean(appConfigProp.getProperty("SITE_STATUS_AUTH"));
    }

    public final Boolean isDesTransactionServiceAuth() {
        return Boolean.parseBoolean(appConfigProp.getProperty("DES_TRANSACTION_SERVICE_AUTH"));
    }

    public final Boolean isSystemMonitorAuth() {
        return Boolean.parseBoolean(appConfigProp.getProperty("SYSTEM_MONITOR_AUTH"));
    }

    public final int getPurgeInterval() {
        return Integer.parseInt(appConfigProp.getProperty("PURGE_INTERVAL"));
    }


}
