/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.jmeadows.common;

import gov.va.med.foundations.utilities.EncryptUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Application configuration.
 * 
 * User: VHAHONUYEHAC
 * Date: 8/1/11
 * Time: 10:24 AM
 */
public class AppConfig {

    /**
     * Logger.
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(AppConfig.class);

    /**
     * CHCS site code.
     */
    private static final String LOCAL_CHCS_SITECODE = "LOCAL_CHCS_SITECODE";
    
    /**
     * VistA site code.
     */
    private static final String LOCAL_VISTA_SITECODE = "LOCAL_VISTA_SITECODE";

    /**
     * jMeadows version number.
     */
    private static final String APP_VERSION = "app.version";
    
    /**
     * jMeadows app environment.
     */
    private static final String APP_ENVIRONMENT = "app.environment";

    /**
     * Datasource.
     */
    private static final String APP_DATASOURCE = "app.datasource";

    /**
     * Environment.
     */
    private static final String ENV = "ENV";
    
    /**
     * log4j properties file path.
     */
    private static final String LOG4J_PROPERTIES = "LOG4J_PROPERTIES";

    /**
     * CHCSWS endpoint.
     */
    private static final String CHCS_URL = "CHCS_URL";

    /**
     * VistaDataService endpoint.
     */
    private static final String VISTA_URL = "VISTA_URL";

    /**
     * MVI endpoint.
     */
    private static final String MVI_URL = "MVI_URL";

    /**
     * MVI HL7 Message Sender Device ID.
     */
    private static final String MVI_SND_DEV_ID = "MVI_SND_DEV_ID";

    /**
     * MVI search result quantity.
     */
    private static final String MVI_SEARCH_QTY = "MVI_SEARCH_QTY";

    /**
     * Restrict VA user access to DoD-only patient information.
     */
    private static final String VA_USER_RESTRICT = "VA_USER_RESTRICT";

    /**
     * Allow VA user access to DoD-only patient information (Audited and prompted).
     */
    private static final String VA_USER_ALLOW_RESTRICT_ACCESS = "VA_USER_ALLOW_RESTRICT_ACCESS";

    /**
     * PIX service endpoint.
     */
    private static final String PIX_URL = "PIX_URL";
    
    /**
     * BHIERelayService endpoint.
     */
    private static final String BHIE_RELAY_SERVICE_URL = "BHIE_RELAY_SERVICE_URL";
    
    /**
     * ESSENTRIS custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_ESS = "BHIE_CUSTODIAN_OID_ESS";
    
    /**
     * TMDS custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_CHCS = "BHIE_CUSTODIAN_OID_CHCS";
    /**
     * SHARE custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_SHARE = "BHIE_CUSTODIAN_OID_SHARE";
    /**
     * PDWSService endpoint *
     */
    private static final String PDWS_PS_URL = "PDWS_PS_URL";
    private static final String PDWS_PR_URL = "PDWS_PR_URL";
    private static final String PDWS_FS_URL = "PDWS_FS_URL";
    private static final String PDWS_FR_URL = "PDWS_FR_URL";
    /**
     * PDWSService HL7 Message Header Processing Code.
     */
    private static final String PDWS_PROC_CODE = "PDWS_PROC_CODE";
    /**
     * PDWSService HL7 Message //Sender/Device/id.
     */
    private static final String PDWS_SND_DEV_ID = "PDWS_SND_DEV_ID";

    /**
     * PDWSService SOAP DEBUG FLAG.
     */
    private static final String DEBUG_SOAP_OUTPUT = "DEBUG_SOAP_OUTPUT";

    /**
     * CHCSWS encrypted username.
     */
    private static final String CHCSWS_USERNAME = "CHCSWS_USERNAME";
    /**
     * CHCSWS encrypted pwd.
     */
    private static final String CHCSWS_PWD = "CHCSWS_PWD";
    /**
     * BHIERelayService encrypted username.
     */
    private static final String BHIE_RELAY_SERVICE_USERNAME = "BHIE_RELAY_SERVICE_USERNAME";
    /**
     * BHIERelayService encrypted pwd.
     */
    private static final String BHIE_RELAY_SERVICE_PWD = "BHIE_RELAY_SERVICE_PWD";

    /**
     * true to relax SSL v3 security requirements. (Use this for debugging purposes only!).
     */
    private static final String IS_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS = "IS_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS";
    /**
     * true to utilize C3PO for as the sql connection pooling api.
     * If false, defaults to utilizing JNDI lookup for connection pool.
     */
    private static final String USE_C3PO = "USE_C3PO";
    /**
     * SQL server JDBC connection string (for use with C3PO).
     */
    private static final String SQL_URL = "SQL_URL";
    /**
     * SQL server JDBC connection string (for use with C3PO).
     */
    private static final String SQL_URL_ENCRYPTED = "SQL_URL_ENCRYPTED";
    /**
     * SQL server driver (for use with C3PO).
     */
    private static final String SQL_DRIVER = "SQL_DRIVER";
    /**
     * JDNI connection pooling lookup name.
     */
    private static final String JNDI_JDBC = "JNDI_JDBC";
    /**
     * JDNI connection pooling lookup name.
     */
    private static final String JNDI_JDBC_ENCRYPTED = "JNDI_JDBC_ENCRYPTED";

    /**
     * General service request timeout setting in milliseconds.
     */
    private static final String REQUEST_TIMEOUT_MS = "REQUEST_TIMEOUT_MS";
    /**
     * General service connection timeout setting in milliseconds.
     */
    private static final String CONNECTION_TIMEOUT_MS = "CONNECTION_TIMEOUT_MS";
    /**
     * General thread timeout in milliseconds.
     */
    private static final String THREAD_TIMEOUT_MS = "THREAD_TIMEOUT_MS";

    /**
     * VLER service request timeout setting in milliseconds.
     */
    private static final String VLER_REQUEST_TIMEOUT_MS = "VLER_REQUEST_TIMEOUT_MS";
    /**
     * VLER service connection timeout setting in milliseconds.
     */
    private static final String VLER_CONNECTION_TIMEOUT_MS = "VLER_CONNECTION_TIMEOUT_MS";

    /**
     * Default error message string.
     */
    private static final String ERROR_MSG_CONNECTION_UNAVAILABLE = "ERROR_MSG_CONNECTION_UNAVAILABLE";
    /**
     * Default edr service related error message string.
     */
    private static final String ERROR_MSG_EDR_DOCS = "ERROR_MSG_EDR_DOCS";

    /**
     * AHLTA custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_AHLTA = "BHIE_CUSTODIAN_OID_AHLTA";
    /**
     * VA custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_VA = "BHIE_CUSTODIAN_OID_VA";
    /**
     * TMDS custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_TMDS = "BHIE_CUSTODIAN_OID_TMDS";

    /**
     * FHIE custodian OID.
     */
    private static final String BHIE_CUSTODIAN_OID_FHIE = "BHIE_CUSTODIAN_OID_FHIE";
    /**
     * Site code to return with BHIE error beans (utilized when BHIE connection is unavailable).
     */
    private static final String BHIE_ERROR_SITE_CODE = "BHIE_ERROR_SITE_CODE";
    /**
     * Site name to return with BHIE error bean (utilized when BHIE connection is unavailable).
     */
    private static final String BHIE_ERROR_SITE_NAME = "BHIE_ERROR_SITE_NAME";
    /**
     * Site moniker to return with BHIE error bean (utilized when BHIE connection is unavailable).
     */
    private static final String BHIE_ERROR_SITE_MONIKER = "BHIE_ERROR_SITE_MONIKER";

    /**
     * The local VA facility's patient IEN OID - for use with the PIX Service patient identifier map.
     * (2.16.840.1.113883.3.42.10006.100005.8 for North Chicago VA)
     */
    private static final String LOCAL_VA_IEN_OID = "LOCAL_VA_IEN_OID";
    /**
     * Vista enterprise ICN OID - for use with the PIX Service patient identifier map.
     */
    private static final String VISTA_ICN_OID = "VISTA_ICN_OID";
    /**
     * The DoD enterprise EDIPN (EDIPI) OID - for use with the Pix Service identifier map.
     */
    private static final String DOD_EDIPN_OID = "DOD_EDIPN_OID";

    /**
     * VA agency name string Should be configured as VA,
     */
    private static final String AGENCY_VA = "AGENCY_VA";
    /**
     * DOD agency name string. Should be configured as DOD,
     */
    private static final String AGENCY_DOD = "AGENCY_DOD";

    /**
     * SiteMap's EHCache name,
     */
    private static final String EHCACHE_NAME_SITEMAP = "EHCACHE_NAME_SITEMAP";

    /**
     * RPC endpoint environment (enterprise, hrg, etc.),
     */
    private static final String RPC_ENDPOINT_ENVIRONMENT = "RPC_ENDPOINT_ENVIRONMENT";

    /**
     * CACHE endpoint environment (enterprise, hrg, etc.),
     */
    private static final String CACHE_ENDPOINT_ENVIRONMENT = "CACHE_ENDPOINT_ENVIRONMENT";

    /**
     * Share endpoint environment,
     */
    private static final String SHARE_ENDPOINT_ENVIRONMENT = "SHARE_ENDPOINT_ENVIRONMENT";

    /**
     * Medweb endpoint environment,
     */
    private static final String MEDWEB_ENDPOINT_ENVIRONMENT = "MEDWEB_ENDPOINT_ENVIRONMENT";
    private static final String AHLTA_ENDPOINT_ENVIRONMENT = "AHLTA_ENDPOINT_ENVIRONMENT";
    /** static constants **/
    private static final String CONST_DODADAPTER_SOURCE_STATUS_REPORT_FLAG = "DODADAPTER_SOURCE_STATUS_REPORT";
    private final static String CONST_SYSTEM_TYPE_CHCS = "CHCS";
    private final static String CONST_SYSTEM_TYPE_VISTA = "VISTA";
    private final static String CONST_SYSTEM_TYPE_SHARE = "SHARE";
    private final static String CONST_SYSTEM_TYPE_DOD_ADAPTER = "DODADAPTER";
    private final static String IDENTIFIER_UNAVAILABLE_FLAG = "IDENTIFIER_UNAVAILABLE";

    private final static String CONST_SOURCE_PROTOCOL_CACHE = "CACHE";
    private final static String CONST_SOURCE_PROTOCOL_RPC = "RPC";
    private final static String CONST_SOURCE_PROTOCOL_SHARE = "SHARE";
    private final static String CONST_SOURCE_PROTOCOL_MEDWEB = "MEDWEB";
    private final static String CONST_SOURCE_PROTOCOL_DOD_ADAPTER = "DODADAPTER";

    private final static String USE_TERM_MAPPING_CACHE = "USE_TERM_MAPPING_CACHE";

    private final static String LOAD_VLER_SITE_CACHE = "LOAD_VLER_SITE_CACHE";

    private final static String USE_SNAREWORKS = "USE_SNAREWORKS";

    /**
     * DMIS IDs and facilities used for mapping DMIS IDs to clinic names for patient demographic's PCM clinic.
     */
    private final static String DMIS_FILE_NAME = "DMIS_FILE_NAME";
    private final static String DMIS_TOKEN_DELIM = "DMIS_TOKEN_DELIM";
    private final static String DMIS_ID_TOKEN_POS = "DMIS_ID_TOKEN_POS";
    private final static String DMIS_NAME_TOKEN_POS = "DMIS_NAME_TOKEN_POS";

    private final static String LAB_TYPE_CH = "LAB_TYPE_CH";
    private final static String LAB_TYPE_MI = "LAB_TYPE_MI";
    private final static String LAB_TYPE_AP = "LAB_TYPE_AP";
    private final static String LAB_TYPE_EM = "LAB_TYPE_EM";
    private final static String LAB_TYPE_SP = "LAB_TYPE_SP";
    private final static String LAB_TYPE_ST = "LAB_TYPE_ST";
    private final static String LAB_TYPE_CY = "LAB_TYPE_CY";
    private final static String LAB_TYPE_AU = "LAB_TYPE_AU";
    private final static String LAB_TYPE_BB = "LAB_TYPE_BB";
    private final static String LAB_TYPE_HIV = "LAB_TYPE_HIV";

    private final static String ENABLE_VIX_DOD_RADIOLOGY_RETRIEVE = "ENABLE_VIX_DOD_RADIOLOGY_RETRIEVE";
    private final static String VIX_DOD_RAD_EXAM_ID_DELIM = "VIX_DOD_RAD_EXAM_ID_DELIM";

    private final static String USE_ENHANCED_DEBUG = "USE_ENHANCED_DEBUG";
    
    private static final AppConfig INSTANCE = new AppConfig();

    private Properties appProperties;

    private AppConfig() {

        appProperties = new Properties();

        loadAppMetaProperties();
        loadAppConfigProperties();
    }

    private void loadAppMetaProperties() {
        String propertiesFile = "application.properties";

        InputStream inputStream = null;

        ClassLoader classLoader = AppConfig.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        try {
            appProperties.load(inputStream);
        } catch (IOException e) {
            LOGGER.error("Failed load application.properties: " + e.getMessage());
            throw new RuntimeException(e);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
            } catch (IOException e) {
                LOGGER.error(e.getMessage());
            }
        }
    }

    private void loadAppConfigProperties() {
        String propertiesFile = "jmeadows.properties";

        InputStream inputStream = null;

        ClassLoader classLoader = AppConfig.class.getClassLoader();
        if (classLoader != null) {
            inputStream = classLoader.getResourceAsStream(propertiesFile);
        }

        if (inputStream != null) {
            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load appconfig properties: " + e.getMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        try {

            inputStream = new FileInputStream(propertiesFile);

            try {
                appProperties.load(inputStream);
            } catch (IOException e) {
                LOGGER.error("Failed load appconfig properties: " + e.getLocalizedMessage());
                throw new RuntimeException(e);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }

        } catch (FileNotFoundException e) {
            LOGGER.info("Failed to load appconfig from file (jmeadows.properties) -"
                    + " utilizing default appconfig properties.");

        }
    }

//    private void logConfigs()
//    {
//        //log some jMeadows configuration values
//        logger.info("Local VA site code: {}", getLocalVASiteCode());
//        logger.info("Local VA site name: {}", getLocalVASiteName());
//        logger.info("Local VA site moniker: {}", getLocalVASiteMoniker());
//
//        logger.info("Local DoD site code: {}", getLocalDODSiteCode());
//        logger.info("Local DoD site name: {}", getLocalDODSiteName());
//        logger.info("Local DoD site moniker {}", getLocalDODSiteName());
//
//        logger.info("MPI enabled: {}", isMPIEnabled());
//        logger.info("Pix service enabled: {}", isPIXServiceEnabled());

//        logger.info("Utilize MSI location's url as webservice endpoint value: {}", useMSIEndpoint());
//        logger.info("Query user location using NPI enabled: {}", isProviderNPILocationQueryEnabled());
//
//        logger.info("CHCSWS endpoint: {}", getCHCSWSURL());
//        logger.info("VistaDataService endpoint: {}", getVistaWsURL());
//        logger.info("PIX service endpoint: {}", getPIXURL());
//        logger.info("BHIERelayService endpoint: {}", getBHIERelayServiceURL());
//        logger.info("JALFHCC alert service endpoint: {}", getAlertServiceURL());
//        logger.info("DREnj enabled: {}", isDREnjServiceEnabled());
//        logger.info("DREnj URL: {}", getDREnjURL());
//        logger.info("SHARE enabled: {}", isShareServiceEnabled());
//        logger.info("SHARE URL: {}", getShareURL());
//        logger.info("PDWS URL: {}", getPDWSURL());
//
//        logger.info("BHIE allergies enabled: {}", isBHIEAllergiesEnabled());
//        logger.info("BHIE problems list enabled: {}", isBHIEProblemsEnabled());
//        logger.info("BHIE progress notes enabled: {}", isBHIEProgressNotesEnabled());
//        logger.info("BHIE discharge summaries enabled: {}" + isBHIEDischargeSummariesEnabled());
//        logger.info("BHIE vitals enabled: {}", isBHIEVitalsEnabled());
//        logger.info("BHIE appointments enabled: {}", isBHIEAppointmentsEnabled());
//        logger.info("BHIE consults enabled: {}", isBHIEConsultsEnabled());
//        logger.info("BHIE demographics enabled: {}", isBHIEDemographicDetailsEnabled());
//        logger.info("BHIE demographic details enabled: {}", isBHIEDemographicDetailsEnabled());
//        logger.info("BHIE outpatient medications enabled: {}", isBHIEOutpatientMedicationsEnabled());
//        logger.info("BHIE inpatient medications enabled: {}" + isBHIEInpatientMedicationsEnabled());
//        logger.info("BHIE radiology results enabled: {}" + isBHIERadiologyEnabled());
//
//        logger.info("BHIE utilize patient EDIPN (EDIPI): {}", isBHIEUtilizePatientEDIPN());
//        logger.info("BHIE CHCS namespace: {}", getBHIECHCSNamespace());
//    }

    public static final AppConfig getInstance() {
        return INSTANCE;
    }

    /**
     * Site moniker to return with BHIE error bean (utilized when BHIE connection is unavailable).
     */
    public final String getBHIEErrorSiteMoniker() {
        return appProperties.getProperty(BHIE_ERROR_SITE_MONIKER);
    }

    /**
     * jMeadows version number.
     */
    public final String getAppVersion() {
        return appProperties.getProperty(APP_VERSION);
    }

    public final String getDataSource() {
        return appProperties.getProperty(APP_DATASOURCE);
    }

    /**
     * jMeadows app environment.
     */
    public final String getAppEnvironment() {
        return appProperties.getProperty(APP_ENVIRONMENT);
    }

    /**
     * log4j properties file path.
     */
    public final String getLog4jProperties() {
        return appProperties.getProperty(LOG4J_PROPERTIES);
    }

    /**
     * CHCSWS endpoint.
     */
    public final String getCHCSWSURL() {
        return appProperties.getProperty(CHCS_URL);
    }

    /** BHIE/DoD adapter clinical domain enablement configurations. BHIE use is for DoD data only, not VA.
     *  Please note that if BHIE IS enabled for a given domain CHCS will NOT be queried.
     *  Likewise, if BHIE is NOT enabled, CHCS is queried by default.
     **/

    /**
     * VistaDataService endpoint.
     */
    public final String getVistaWsURL() {
        return appProperties.getProperty(VISTA_URL);
    }

    /**
     * MVI endpoint.
     */
    public final String getMVIURL() {
        return appProperties.getProperty(MVI_URL);
    }

    /**
     * MVI HL7 Message Sender Device ID.
     */
    public final String getMVISndDevId() { return appProperties.getProperty(MVI_SND_DEV_ID); }

    /**
     * MVI search result quantity.
     */
    public final int getMVISearchQty() { return Integer.parseInt(appProperties.getProperty(MVI_SEARCH_QTY)); }

    /**
     * Restrict VA user access to DoD-only patient information.
     */
    public final String getVaUserRestrict() {
        return appProperties.getProperty(VA_USER_RESTRICT);
    }

    /**
     * Allow VA user access to DoD-only patient information (Audited and prompted).
     */
    public final String getVaUserAllowRestrictAccess() {
        return appProperties.getProperty(VA_USER_ALLOW_RESTRICT_ACCESS);
    }

    /**
     * PIX service endpoint.
     */
    public final String getPIXURL() {
        return appProperties.getProperty(PIX_URL);
    }

    /**
     * BHIERelayService endpoint.
     */
    public final String getBHIERelayServiceURL() {
        return appProperties.getProperty(BHIE_RELAY_SERVICE_URL);
    }

    /**
     * PDWSService endpoint.
     */
    public final String getPdwsPsUrl() {
        return appProperties.getProperty(PDWS_PS_URL);
    }
    public final String getPdwsPrUrl() { return appProperties.getProperty(PDWS_PR_URL); }
    public final String getPdwsFsUrl() {
        return appProperties.getProperty(PDWS_FS_URL);
    }
    public final String getPdwsFrUrl() { return appProperties.getProperty(PDWS_FR_URL); }

    /**
     * PDWSService endpoint.
     */
    public final String getPDWSProcCode() {
        return appProperties.getProperty(PDWS_PROC_CODE);
    }

    /**
     * PDWSService endpoint.
     */
    public final String getPDWSSndDevId() {
        return appProperties.getProperty(PDWS_SND_DEV_ID);
    }

    /**
     * PDWSService debug flag.
     */
    public final String getPDWSDebugFlag() {
        return appProperties.getProperty(DEBUG_SOAP_OUTPUT);
    }


    /**
     * CHCSWS username.
     */
    public final String getCHCSWSUsername() {
        return EncryptUtil.decrypt(appProperties.getProperty(CHCSWS_USERNAME));
    }

    /**
     * CHCSWS pwd.
     */
    public final String getCHCSWSPassword() {
        return EncryptUtil.decrypt(appProperties.getProperty(CHCSWS_PWD));
    }

    /**
     * BHIERelayService username.
     */
    public final String getBHIERelayServiceUsername() {
        return EncryptUtil.decrypt(appProperties.getProperty(BHIE_RELAY_SERVICE_USERNAME));
    }

    /**
     * BHIERelayService pwd.
     */
    public final String getBHIERelayServicePassword() {
        return EncryptUtil.decrypt(appProperties.getProperty(BHIE_RELAY_SERVICE_PWD));
    }

    /**
     * true to relax SSL v3 security requirements. (Use this for debugging purposes only!).
     */
    public final boolean isTrustAllSSLHostnameAndCerts() {
        return Boolean.valueOf(appProperties.getProperty(IS_TRUST_ALL_SSL_HOSTNAMES_AND_CERTS));
    }

    /**
     * true to utilize C3PO for as the sql connection pooling api.
     * If false, defaults to utilizing JNDI lookup for connection pool.
     */
    public final boolean useC3PO() {
        return Boolean.valueOf(appProperties.getProperty(USE_C3PO));
    }

    /**
     * SQL server JDBC connection string (for use with C3PO).
     */
    public final String getSQLURL() {
        return appProperties.getProperty(SQL_URL);
    }

    /**
     * SQL server JDBC connection string (for use with C3PO).
     */
    public final String getSQLURLEncrypted() {
        return appProperties.getProperty(SQL_URL_ENCRYPTED);
    }

    /**
     * SQL server driver (for use with C3PO).
     */
    public final String getSQLDriver() {
        return appProperties.getProperty(SQL_DRIVER);
    }

    /**
     * JDNI connection pooling lookup name.
     */
    public final String getJDBC_JNDILookup() {
        return appProperties.getProperty(JNDI_JDBC);
    }

    /**
     * JDNI connection pooling lookup name.
     */
    public final String getJDBC_JNDILookupEncrypted() {
        return appProperties.getProperty(JNDI_JDBC_ENCRYPTED);
    }

    /**
     * General service request timeout setting in milliseconds.
     */
    public final int getRequestTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(REQUEST_TIMEOUT_MS));
    }

    /**
     * General service connection timeout setting in milliseconds.
     */
    public final int getConnectionTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(CONNECTION_TIMEOUT_MS));
    }

    /**
     * General thread timeout in milliseconds.
     */
    public final long getThreadTimeoutMS() {
        return Long.parseLong(appProperties.getProperty(THREAD_TIMEOUT_MS));
    }

    /**
     * VLER service request timeout setting in milliseconds.
     */
    public final int getVLERRequestTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(VLER_REQUEST_TIMEOUT_MS));
    }

    /**
     * VLER service connection timeout setting in milliseconds.
     */
    public final int getVLERConnectionTimeoutMS() {
        return Integer.parseInt(appProperties.getProperty(VLER_CONNECTION_TIMEOUT_MS));
    }

    /**
     * BHIE custodian OIDs. Each OID represents a particular data site that BHIE aggregates from.
     * These custodian OIDs are included as BHIE query included or excluded site parameters.
     */

    /**
     * Default error message string.
     */
    public final String getErrorMsgConnectionUnavailable() {
        return appProperties.getProperty(ERROR_MSG_CONNECTION_UNAVAILABLE);
    }

    /**
     * Default edr service related error message string.
     */
    public final String getErrorMsgEDRDocs() {
        return appProperties.getProperty(ERROR_MSG_EDR_DOCS);
    }

    /**
     * AHLTA custodian OID.
     */
    public final String getBHIECustodianOidAHLTA() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_AHLTA);
    }

    /**
     * VA custodian OID.
     */
    public final String getBHIECustodianOidVA() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_VA);
    }

    /**
     * TMDS custodian OID.
     */
    public final String getBHIECustodianOidTMDS() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_TMDS);
    }

    /**
     * FHIE custodian OID.
     */
    public final String getBHIECustodianOidFHIE() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_FHIE);
    }

    /**
     * ESSENTRIS custodian OID.
     */
    public final String getBHIECustodianOidESS() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_ESS);
    }

    /**
     * CHCS custodian OID.
     */
    public final String getBHIECustodianOidCHCS() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_CHCS);
    }

    /**
     * SHARE custodian OID.
     */
    public final String getBHIECustodianOidSHARE() {
        return appProperties.getProperty(BHIE_CUSTODIAN_OID_SHARE);
    }

    /**
     * Site code to return with BHIE error beans (utilized when BHIE connection is unavailable).
     */
    public final String getBHIEErrorSiteCode() {
        return appProperties.getProperty(BHIE_ERROR_SITE_CODE);
    }

    /**
     * Site name to return with BHIE error bean (utilized when BHIE connection is unavailable).
     */
    public final String getBHIEErrorSiteName() {
        return appProperties.getProperty(BHIE_ERROR_SITE_NAME);
    }

    /**
     * The local VA facility's patient IEN OID - for use with the PIX Service patient identifier map.
     * (2.16.840.1.113883.3.42.10006.100005.8 for North Chicago VA)
     */
    public final String getLocalVAIenOid() {
        return appProperties.getProperty(LOCAL_VA_IEN_OID);
    }

    /**
     * Vista enterprise ICN OID - for use with the PIX Service patient identifier map.
     */
    public final String getVistaIcnOid() {
        return appProperties.getProperty(VISTA_ICN_OID);
    }

    public final String getDODEdipnOid() {
        return appProperties.getProperty(DOD_EDIPN_OID);
    }

    /**
     * VA agency name string Should be configured as VA
     */
    public final String getAgencyVA() {
        return appProperties.getProperty(AGENCY_VA);
    }

    /**
     * DOD agency name string. Should be configured as DOD
     */
    public final String getAgencyDOD() {
        return appProperties.getProperty(AGENCY_DOD);
    }

    //methods that return static constants

    public final String getDODAdapterSourceStatusReportFlag() {
        return CONST_DODADAPTER_SOURCE_STATUS_REPORT_FLAG;
    }

    public final String getSystemTypeCHCS() {
        return CONST_SYSTEM_TYPE_CHCS;
    }

    public final String getSystemTypeVISTA() {
        return CONST_SYSTEM_TYPE_VISTA;
    }

    public final String getConstSystemTypeShare() {
        return CONST_SYSTEM_TYPE_SHARE;
    }

    public final String getSystemTypeDODAdapter() {
        return CONST_SYSTEM_TYPE_DOD_ADAPTER;
    }

    public final String getSourceProtocolCache() {
        return CONST_SOURCE_PROTOCOL_CACHE;
    }

    public final String getSourceProtocolRPC() {
        return CONST_SOURCE_PROTOCOL_RPC;
    }

    public final String getSourceProtocolMedweb() {
        return CONST_SOURCE_PROTOCOL_MEDWEB;
    }

    public final String getSourceProtocolDODAdapter() {
        return CONST_SOURCE_PROTOCOL_DOD_ADAPTER;
    }

    public final String getSourceProtocolShare() {
        return CONST_SOURCE_PROTOCOL_SHARE;
    }

    public final String getIdentifierUnavailableFlag() {
        return IDENTIFIER_UNAVAILABLE_FLAG;
    }

    public final String getEHCacheNameSiteMap() {
        return appProperties.getProperty(EHCACHE_NAME_SITEMAP);
    }

    public final String getRPCEndpointEnvironment() {
        return appProperties.getProperty(RPC_ENDPOINT_ENVIRONMENT);
    }

    public final String getCacheEndpointEnvironment() {
        return appProperties.getProperty(CACHE_ENDPOINT_ENVIRONMENT);
    }

    public final String getShareEndpointEnvironment() {
        return appProperties.getProperty(SHARE_ENDPOINT_ENVIRONMENT);
    }

    public final String getMedwebEndpointEnvironment() {
        return appProperties.getProperty(MEDWEB_ENDPOINT_ENVIRONMENT);
    }

    public final String getAhltaEndpointEnvironment() {
        return appProperties.getProperty(AHLTA_ENDPOINT_ENVIRONMENT);
    }

    public final String getLocalCHCSSiteCode() {
        return appProperties.getProperty(LOCAL_CHCS_SITECODE);
    }

    public final String getLocalVistaSiteCode() {
        return appProperties.getProperty(LOCAL_VISTA_SITECODE);
    }

    /**
     * true to utilize HashMap as cache for terminology mapping.
     * If false, to utilize SQL server for mapping without hashmap cache.
     */
    public final boolean useTermMappingCache() {
        return Boolean.valueOf(appProperties.getProperty(USE_TERM_MAPPING_CACHE));
    }

    /**
     * Load mapping data from SQL Server on startup.
     */
    public final boolean loadVLERSiteCache() {
        return Boolean.valueOf(appProperties.getProperty(LOAD_VLER_SITE_CACHE));
    }
    
    /**
     * true to use snareworks instead of database whitelist.
     */
    public final boolean useSnareworks() {
        return Boolean.valueOf(appProperties.getProperty(USE_SNAREWORKS));
    }

    /**
     * DMIS IDs and facilities used for mapping DMIS IDs to clinic names for patient demographic's PCM clinic
     */
    public final String getDmisFileName() {
        return appProperties.getProperty(DMIS_FILE_NAME);
    }
    public final String getDmisTokenDelim() {
        return appProperties.getProperty(DMIS_TOKEN_DELIM);
    }
    public final int getDmisIdTokenPos() {
        return Integer.parseInt(appProperties.getProperty(DMIS_ID_TOKEN_POS));
    }
    public final int getDmisNameTokenPos() {
        return Integer.parseInt(appProperties.getProperty(DMIS_NAME_TOKEN_POS));
    }

    public final String  getLabTypeCh() {
        return appProperties.getProperty(LAB_TYPE_CH);
    }

    public final String getLabTypeMi() {
        return appProperties.getProperty(LAB_TYPE_MI);
    }

    public final String getLabTypeAp() {
        return appProperties.getProperty(LAB_TYPE_AP);
    }

    public final String getLabTypeEm() {
        return appProperties.getProperty(LAB_TYPE_EM);
    }

    public final String getLabTypeSp() {
        return appProperties.getProperty(LAB_TYPE_SP);
    }

    public final String getLabTypeSt() {
        return appProperties.getProperty(LAB_TYPE_ST);
    }

    public final String getLabTypeCy() {
        return appProperties.getProperty(LAB_TYPE_CY);
    }

    public final String getLabTypeAu() {
        return appProperties.getProperty(LAB_TYPE_AU);
    }

    public final String getLabTypeBb() {
        return appProperties.getProperty(LAB_TYPE_BB);
    }

    public final String getLabTypeHiv() {
        return appProperties.getProperty(LAB_TYPE_HIV);
    }

    public final boolean isVixDodRadiologyRetrieveEnabled() {
        return Boolean.valueOf(appProperties.getProperty(ENABLE_VIX_DOD_RADIOLOGY_RETRIEVE, "false"));
    }

    public final String getVixDodRadExamIdDelim() {
        return appProperties.getProperty(VIX_DOD_RAD_EXAM_ID_DELIM, "#- ");
    }

    /**
     * @return the appProperties
     */
    public Properties getProperties() {
        return appProperties;
    }

    public final String getEnv() {return appProperties.getProperty(ENV);}

    /**
     * true to use any enhanced debugging code
     */
    public final boolean useEnhancedDebug() {
        return Boolean.valueOf(appProperties.getProperty(USE_ENHANCED_DEBUG));
    }

}
