/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.vds.webservice.FreeTextReport;
import gov.va.med.vds.webservice.Site;

import java.util.List;

/**
 * Patient Appointment data access object.
 */
public class PatientApptDao extends DaoBase {

    /**
     * Retrieves a Patient visit note from the provided record location.
     *
     * @param queryBean Query parameters.
     *
     * {@link QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     * recordSiteCode
     * itemId
     *
     * @return A patient visit note.
     * @throws JMeadowsException if there is an error.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public FreeTextReport getVisitNotes(JMeadowsQuery queryBean) throws JMeadowsException {
        
        validateQueryBean(queryBean,
                QueryBeanParams.USER, QueryBeanParams.PATIENT,
                QueryBeanParams.RECORD_SITE_CODE, QueryBeanParams.ITEM_ID);

        auditQuery("VisitNotes", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean);

        FreeTextReport visitNote = null;

        Site recordSite = getSite(queryBean.getRecordSiteCode());

        for (PatientLocation loc : locs) {

            //get vista data
            if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())
                    && LocationHelper.vistaLocationsContainSiteCode(recordSite,
                            queryBean.getPatient().getVistaSites())) {
                
                visitNote = sm_vistaHandler.getVisitNotes(VistaBeanFactory.create(queryBean));
                break;
            }
        }

        return visitNote;
    }


}
