/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBase;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.ProviderMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.dao.util.UserLocation;
import gov.va.med.jmeadows.dao.util.VistaBeanFactory;
import gov.va.med.jmeadows.smartbeans.provider.SmartLabResult;
import gov.va.med.vds.webservice.LabResult;
import gov.va.med.vhahon.common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author vhahonbrunos
 */
public class ProviderLabsDao extends DaoBase {

    /**
     * Gets a list of abnormal lab results which were ordered by the user
     * passed in the queryBean.
     * @param queryBean requires user, startDate and endDate. Start and end
     * dates tell the underlying processing routines when to start and stop
     * searching. The wider the date range the longer it takes to process.
     * @return a list of abnormal lab results for given user and date
     * range. For each abnormal lab result, the abnormal result is included.
     * @throws gov.va.med.jmeadows.common.JMeadowsException when an error occurs in the retrieval process
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<LabResult> getProviderAbnormalLabResults(JMeadowsQuery queryBean) throws JMeadowsException
    {
        List<LabResult> rtc = null;
        validateQueryBean(queryBean, QueryBeanParams.USER);

        auditQuery("ProviderAbnormalLabResults", queryBean);

        if (queryBean.getStartDate() == null || queryBean.getEndDate() == null)
        {
            queryBean.setStartDate(Utils.getCalendar(Utils.pushDateToStartOfDay(Utils.getDate(Utils.getRelativeDate(0, -1, 0))))); //(years,months,days)
            queryBean.setEndDate(Utils.getCalendar(Utils.pushDateToEndOfDay(new java.util.Date())));
        }

        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        List<UserLocation> locations;

        locations = LocationHelper.getUserLocations(queryBean.getUser());

        for (UserLocation location : locations) {
            String systemType = location.getSystemType();
            String method = "getProviderLabAbnormalResults";
            if (SYSTEM_TYPE_CHCS.equalsIgnoreCase(systemType)) {
                rtc = Arrays.asList((LabResult[])
                        new SmartLabResult().generateConnectionUnavailableBean(location, PROVIDER_QUERY_NOT_SUPPORTED_DOD));
                break;
            } else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(systemType)) {
                configs.add(
                    new ProviderMultiLocQueryCfgBuilder()
                            .dataServiceHandler(sm_vistaHandler)
                            .dataServiceMethod(method)
                            .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
                            .location(location)
                            .build()
                );
            }
        }
        if (rtc == null) {
            Object[] labResultObjs = MultiLocationQuery.execute(
                    configs, new SmartLabResult(), queryBean);
    
            List<LabResult> labResults = null;
    
            if (labResultObjs != null) {
                labResults = Arrays.asList((LabResult []) labResultObjs);
            }
    
            rtc = labResults;
        }
        return rtc;
    }
}
