/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

//import gov.va.medora.HuiSvc.ArrayOfTaggedText;
//import gov.va.medora.HuiSvc.PatientTO;
//import gov.va.medora.HuiSvc.TaggedPatientArray;
//import gov.va.medora.HuiSvc.TaggedText;

//import gov.va.med.bhie.PatientDemographicsDataRecordV3;
//import gov.va.med.bhie.PatientDemographicsResultCollectionV3;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.foundations.utilities.Messages;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.ResponsePatientQuery;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dataservicehandler.PDWSBlackList;
import gov.va.med.vds.webservice.*;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

//import org.hui.chcs.DemographicsBean;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static gov.va.med.jmeadows.dao.hdd.VLERSiteCache.getVLERSiteName;

/**
 * @author vhahonbrunos
 */
public class SmartDemographics extends SmartBean implements Comparable {

    public static final int SITE = 1;

    /**
     * SSN OID.
     */
    public static final String SSN_OID = "2.16.840.1.113883.3.42.126.100001.2";

    /**
     * EDIPN OID.
     */
    public static final String EDI_OID = "2.16.840.1.113883.3.42.10001.100001.12";

    /**
     * Repository OID.
     */
    private static final String REP_OID = "2.16.840.1.113883.3.42.126.100001.13";

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartDemographics.class);

    public PatientDemographics m_demographic;


    public SmartDemographics() {
        super();
        m_demographic = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartDemographics(PatientDemographics demographic, String sort) {
        super();
        m_demographic = demographic;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartDemographics) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_demographic);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }


    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartDemographics[] temp = new SmartDemographics[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDemographics newAll = new SmartDemographics((PatientDemographics) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartDemographics newAll = new SmartDemographics((PatientDemographics) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDemographics[] ret = new PatientDemographics[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_demographic;
            }
    
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartDemographics[] temp = new SmartDemographics[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDemographics newAll = new SmartDemographics((PatientDemographics) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDemographics[] ret = new PatientDemographics[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_demographic;
            }
            rtc = ret;
        }
        return rtc;
    }

//    public PatientDemographics[] mapPatientDemographicsFromCHCS(Object[] dodBean) {
//        PatientDemographics[] pats = new PatientDemographics[dodBean.length];
//
//        for (int i = 0; i < dodBean.length; i++) {
//            DemographicsBean chcsPat = (DemographicsBean) dodBean[i];
//            if (chcsPat != null) {
//                PatientDemographics pat = new PatientDemographics();
//                pat.setAddress1(chcsPat.getAddress1());
//                pat.setAddress2(chcsPat.getAddress2());
//                pat.setAge(chcsPat.getAge());
//                pat.setCity(chcsPat.getCity());
//                pat.setClinic(chcsPat.getClinic());
//                pat.setDob(chcsPat.getDob());
//                pat.setGender(chcsPat.getSex());
//                pat.setName(chcsPat.getName());
//                pat.setPrimaryProvider(chcsPat.getPcm());
//                pat.setPhone1(chcsPat.getPhone1());
//                pat.setPhone2(chcsPat.getPhone2());
//                pat.setSSN(chcsPat.getSSN());
//                pat.setState(chcsPat.getState());
//                pat.setZipCode(chcsPat.getZipCode());
//                pat.setCodeGreen("0");
//                addCHCSSiteDataToVistaBean(pat);
//                pats[i] = pat;
//            }
//        }
//
//        return pats;
//    }


    public PatientDemographics[] mapPatientDemographicsFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection patDemoResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDemographics> vistaBeans = new ArrayList<PatientDemographics>();

        //If data record is empty, inform user.
        if (patDemoResultCollection.getDataRecordList() == null || patDemoResultCollection.getDataRecordList().getDemographics() == null) {
            PatientLocation bhieLoc = new PatientLocation();
            bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);

            vistaBeans.addAll(Arrays.asList(
                    (PatientDemographics[]) generateBHIEPatRegErrorBean(bhieLoc)));
        } else {
            try {
                SimpleDateFormat sdf = new SimpleDateFormat("dd MMM yyyy", Locale.getDefault());
                for (DemographicV4 bhiePat : patDemoResultCollection.getDataRecordList().getDemographics()) {

                    //if (dataRecord == null || dataRecord.getPatientDemographics() == null) continue;

                    //gov.va.med.bhie.PatientDemographics bhiePat = dataRecord.getPatientDemographics();
                    PatientDemographics pat = new PatientDemographics();
                    Address patAddress = new Address();
                    if (bhiePat.getAddresses() != null && !bhiePat.getAddresses().isEmpty()) {
                        if (bhiePat.getAddresses().get(0).getStreet() != null && !bhiePat.getAddresses().get(0).getStreet().isEmpty()) {
                            patAddress.setStreet1(bhiePat.getAddresses().get(0).getStreet().get(0));
                            if (bhiePat.getAddresses().get(0).getStreet().size() > 1) {
                                patAddress.setStreet2(bhiePat.getAddresses().get(0).getStreet().get(1));
                            }
                        }

                        patAddress.setCity(bhiePat.getAddresses().get(0).getCity());
                        patAddress.setStateProvince(bhiePat.getAddresses().get(0).getState());
                        patAddress.setPostalCode(bhiePat.getAddresses().get(0).getPostalCode());
                        patAddress.setType(bhiePat.getAddresses().get(0).getType());
                        pat.getAddress().add(patAddress);
                    }
                    //pat.setAge(chcsPat.getAge());
                    //pat.setClinic(bhiePat.get);
                    if (bhiePat.getBirthDate() != null) {
                        //if(bhieToCal(bhiePat.getBirthDate().getStart()) != null)
                        pat.setDob(sdf.format(bhieToXMLCalV4(bhiePat.getBirthDate()).toGregorianCalendar().getTime()));
                    }
                    if (bhiePat.getSex() != null && !bhiePat.getSex().isEmpty() && bhiePat.getSex().get(0) != null) {
                        pat.setGender(bhiePat.getSex().get(0).getCode());
                    }
                    if (bhiePat.getSponsor() != null && bhiePat.getSponsor().getSsn() != null && bhiePat.getSponsor().getSsn().length() > 0) {
                        pat.setFMP(bhiePat.getFmp() + "/" + bhiePat.getSponsor().getSsn());
                    } else {
                        pat.setFMP(bhiePat.getFmp());
                    }
//                pat.getRace().addAll(getCodes(bhiePat.getRace()));

                    pat.getRace().clear();
//                    if (bhiePat.getRace() != null && !bhiePat.getRace().isEmpty()) {
//                        pat.getRace().addAll(Arrays.asList(getCodesV4(bhiePat.getRace())));
//                    }
                    if (bhiePat.getRace() != null && bhiePat.getRace().size() > 0) {
                        List<SpdcBeanV4> raceCodeList = new ArrayList<SpdcBeanV4>();
                        for (SpdcWrapperV4 races : bhiePat.getRace()) {
                            if (getPrimarySpdcV4(races.getSpdcList()) != null) {
                                SpdcBeanV4 primaryRace = getPrimarySpdcV4(races.getSpdcList());
                                if (primaryRace != null && primaryRace.getDisplay() != null) {
                                    raceCodeList.add(primaryRace);
                                }
                            }

                        }
                        if (raceCodeList.size() > 0 && !raceCodeList.isEmpty()) {
                            pat.getRace().addAll(Arrays.asList(getCodesV4(raceCodeList)));
                        }
                    }


                    //pat.setPatientId(chcsPat.getPatientId());
                    if (bhiePat.getName() != null) {
                        String first = "";
                        String last = "";
                        String middle = "";
                        if (bhiePat.getName().getLast() != null) {
                            last = bhiePat.getName().getLast();
                        }
                        if (bhiePat.getName().getFirst() != null) {
                            first = bhiePat.getName().getFirst();
                        }
                        if (bhiePat.getName().getMiddle() != null) {
                            middle = bhiePat.getName().getMiddle();
                        }
                        pat.setName(last + ", " + first + " " + middle);
                    }
                    //pat.setPrimaryProvider(chcsPat.getPcm());
                    if (bhiePat.getTelecoms() != null && !bhiePat.getTelecoms().isEmpty()) {
                        for (TelecomV4 tel : bhiePat.getTelecoms()) {
                            if(tel != null){
                                if ((tel.getUriPrefix() == null)
                                       || (tel.getUriPrefix().contains("mailto"))
                                        || (tel.getValue().contains("@"))) {
                                    continue;
                                } else {
                                    if ("WP".equalsIgnoreCase(tel.getUseType())) {
                                        pat.setPhone2(tel.getValue());
                                    } else {
                                        pat.setPhone1(tel.getValue());
                                    }
                                }
                            }
                        }
                    }
                    //pat.setPhone2(chcsPat.getPhone2());
//                pat.setSSN(bhiePat.getSocialSecurityNumber());
                    String ssn = "";
                    if (bhiePat.getPatientId() != null) {
                        Iterator<RecordIdentifierV4> iter = bhiePat.getPatientId().iterator();
                        while (iter.hasNext()) {
                            RecordIdentifierV4 recordid = iter.next();
                            if (SSN_OID.equals(recordid.getAssigningAuthority())) {
                                ssn = recordid.getId();
                            }
                            if (EDI_OID.equals(recordid.getAssigningAuthority())) {
                                String edi = recordid.getId();
                                if (edi != null && edi.length() > 1 && edi.charAt(0) == 'D') {
                                    edi = edi.substring(1);
                                }
                                if (edi != null && edi.length() > 0 && PDWSBlackList.getInstance().isBlackListed(edi)) {
                                    pat.setSensitive(true);
                                }
                                pat.setEDIPI(edi);
                            }
                        }
                    }
                    pat.setSSN(ssn);
                    pat.setCodeGreen("0");

                    // Set patient demographics rank
                    String rank = "";
                    if (bhiePat.getRank() != null) {
                        for (SpdcBeanV4 rankEl : bhiePat.getRank()) {
                            if (rankEl.isPrimary()) {
                                rank = rankEl.getDisplay();
                                break;
                            }
                        }
                    }
                    pat.setRank(rank);

                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (bhiePat.getRepository() != null && bhiePat.getRepository().getIds() != null && !bhiePat.getRepository().getIds().isEmpty()) {
                        repositoryId = bhiePat.getRepository().getIds().get(0).getId();
                    }
                    if (bhiePat.getRepository() != null) {
                        repositoryName = bhiePat.getRepository().getName();
                    }
                    if (bhiePat.getFacility() != null) {
                        facilityName = bhiePat.getFacility().getName();
                    }
                    if (bhiePat.getRecordId() != null) {
                        eventId = bhiePat.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = REP_OID;
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }

                    if (bhiePat.getNextOfKin() != null && !bhiePat.getNextOfKin().isEmpty()) {
                        for (PersonV4 bhiePerson : bhiePat.getNextOfKin()) {
                            Person nextOfKin = new Person();
                            nextOfKin.setName(getNameFromPerson(bhiePerson));
                            if (bhiePerson.getRelationship() != null && getPrimarySpdcV4(bhiePerson.getRelationship()) != null) {
                                nextOfKin.setRelationship(getPrimarySpdcV4(bhiePerson.getRelationship()).getDisplay());
                            }
                            if (bhiePerson.getAddress() != null && bhiePerson.getAddress().size() > 0) {
                                Address kinAddress = new Address();
                                kinAddress.setCity(bhiePerson.getAddress().get(0).getCity());
                                kinAddress.setStateProvince(bhiePerson.getAddress().get(0).getState());
                                kinAddress.setPostalCode(bhiePerson.getAddress().get(0).getPostalCode());
                                kinAddress.setType(bhiePerson.getAddress().get(0).getType());
                                if (bhiePerson.getAddress().get(0).getStreet() != null && bhiePerson.getAddress().get(0).getStreet().size() > 0) {
                                    if (bhiePerson.getAddress().get(0).getStreet().get(0) != null) {
                                        kinAddress.setStreet1(bhiePerson.getAddress().get(0).getStreet().get(0));
                                    }
                                    if (bhiePerson.getAddress().get(0).getStreet().size() > 1 && bhiePerson.getAddress().get(0).getStreet().get(1) != null) {
                                        kinAddress.setStreet2(bhiePerson.getAddress().get(0).getStreet().get(1));
                                    }
                                    if (bhiePerson.getAddress().get(0).getStreet().size() > 2 && bhiePerson.getAddress().get(0).getStreet().get(2) != null) {
                                        kinAddress.setStreet3(bhiePerson.getAddress().get(0).getStreet().get(2));
                                    }
                                }
                                nextOfKin.getAddress().add(kinAddress);
                            }
                            if (bhiePerson.getPhone() != null) {
                                for (TelecomV4 telcom : bhiePerson.getPhone()) {
                                    Telecom phone = new Telecom();
                                    phone.setUsageType(telcom.getUseType());
                                    phone.setValue(telcom.getValue());
                                    nextOfKin.getTelcoms().add(phone);
                                }
                            }
                            pat.getNextOfKin().add(nextOfKin);
                        }
                    }


                    if (bhiePat.getEmergencyContacts() != null && !bhiePat.getEmergencyContacts().isEmpty()) {
                        for (PersonV4 bhiePerson : bhiePat.getEmergencyContacts()) {
                            Person contact = new Person();
                            contact.setName(getNameFromPerson(bhiePerson));
                            if (bhiePerson.getRelationship() != null && getPrimarySpdcV4(bhiePerson.getRelationship()) != null) {
                                contact.setRelationship(getPrimarySpdcV4(bhiePerson.getRelationship()).getDisplay());
                            }
                            if (bhiePerson.getAddress() != null && bhiePerson.getAddress().size() > 0) {
                                Address contactAddress = new Address();
                                contactAddress.setCity(bhiePerson.getAddress().get(0).getCity());
                                contactAddress.setStateProvince(bhiePerson.getAddress().get(0).getState());
                                contactAddress.setPostalCode(bhiePerson.getAddress().get(0).getPostalCode());
                                contactAddress.setType(bhiePerson.getAddress().get(0).getType());
                                if (bhiePerson.getAddress().get(0).getStreet() != null && bhiePerson.getAddress().get(0).getStreet().size() > 0) {
                                    if (bhiePerson.getAddress().get(0).getStreet().get(0) != null) {
                                        contactAddress.setStreet1(bhiePerson.getAddress().get(0).getStreet().get(0));
                                    }
                                    if (bhiePerson.getAddress().get(0).getStreet().size() > 1 && bhiePerson.getAddress().get(0).getStreet().get(1) != null) {
                                        contactAddress.setStreet2(bhiePerson.getAddress().get(0).getStreet().get(1));
                                    }
                                    if (bhiePerson.getAddress().get(0).getStreet().size() > 2 && bhiePerson.getAddress().get(0).getStreet().get(2) != null) {
                                        contactAddress.setStreet3(bhiePerson.getAddress().get(0).getStreet().get(2));
                                    }
                                }
                                contact.getAddress().add(contactAddress);
                            }
                            if (bhiePerson.getPhone() != null) {
                                for (TelecomV4 telcom : bhiePerson.getPhone()) {
                                    //if (telcom != null && telcom.getUseType() != null && telcom.getValue() != null) {
                                        Telecom phone = new Telecom();
                                        phone.setUsageType(telcom.getUseType());
                                        phone.setValue(telcom.getValue());
                                        contact.getTelcoms().add(phone);
                                   // }
                                }
                            }
                            pat.getEmergencyContact().add(contact);
                        }
                    }

                    if (bhiePat.getMaritalStatus() != null && getPrimarySpdcV4(bhiePat.getMaritalStatus()) != null) {
                        pat.setMaritalStatus(getPrimarySpdcV4(bhiePat.getMaritalStatus()).getDisplay());
                    }

                    if (bhiePat.getLanguages() != null) {
                        for (LanguagesV4 languages : bhiePat.getLanguages()) {
                            for (SpdcBeanV4 language : languages.getLanguageList()) {
                                if (language != null && language.getDisplay() != null && !language.getDisplay().isEmpty()) {
                                    pat.getLanguages().add(language.getDisplay());
                                }
                            }

                        }
                    }

                    if (bhiePat.getReligion() != null) {
                        for (SpdcBeanV4 religion : bhiePat.getReligion()) {
                            if (religion != null && religion.getDisplay() != null && !religion.getDisplay().isEmpty()) {
                                pat.getReligion().add(religion.getDisplay());
                            }
                        }
                    }

                    if (patDemoResultCollection.isVler()) {
                        addBHIESiteDataToVistaBean(repositoryId, "VLER",
                                facilityName, eventId, pat);
                    } else {
                        addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                                facilityName, eventId, pat);
                    }

                    vistaBeans.add(pat);
                }
            } catch (Exception ex) {
                LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
                throw new JMeadowsException(ex);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(patDemoResultCollection.getStatusListv4(),
                patDemoResultCollection.getErrorListv4(), patDemoResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (PatientDemographics[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientDemographics[vistaBeans.size()]);
    }

    public PatientDemographics[] mapPatientDemographicsFromPDWS(Object resultCollection) throws JMeadowsException {
        ResponsePatientQuery response = (ResponsePatientQuery) resultCollection;
        List<PatientDemographics> vistaBeans = new ArrayList<PatientDemographics>();

        if (response == null || response.getDemographics() == null) {
            PatientLocation bhieLoc = new PatientLocation();
            bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);

            vistaBeans.addAll(Arrays.asList(
                    (PatientDemographics[]) generateBHIEPatRegErrorBean(bhieLoc)));
        } else {
            try {
                PatientDemographics demo = response.getDemographics();
//                if (demo.getSponsorSSN() == null || demo.getSponsorSSN().length() == 0) {
//                    demo.setSponsorSSN(demo.getSSN());
//                }
//                if (demo.getSponsorName() == null || demo.getSponsorName().length() == 0) {
//                    demo.setSponsorName(demo.getName());
//                }
                demo.setCodeGreen("0");

                addBHIESiteDataToVistaBean("", "PDWS", "PDWS", "", demo);

                vistaBeans.add(demo);
            } catch (Exception e) {
                LOGGER.error("EXCEPTION: " + e.getLocalizedMessage(), e);
                throw new JMeadowsException(e);
            }
        }

        return vistaBeans.toArray(new PatientDemographics[vistaBeans.size()]);
    }

    public PatientDemographics[] mapDASVLERDemographics(Object resultCollection) throws JMeadowsException {
        DasResultCollection vlerResultCollection = (DasResultCollection) resultCollection;
        List<PatientDemographics> vistaBeans = new ArrayList<PatientDemographics>();

        try {

            if (vlerResultCollection.getAllergies() != null) {
                //SmartAllergy smartAllergy = new SmartAllergy();
                vistaBeans = vlerResultCollection.getDemographics();

                for(PatientDemographics vistaBean : vistaBeans){
                    if(StringUtils.isNotEmpty(vistaBean.getSite().getSiteCode())){
                        String vlerSiteName = getVLERSiteName(vistaBean.getSite().getSiteCode());
                        if(StringUtils.isNotEmpty(vlerSiteName)) {
                            vistaBean.getSite().setName(vlerSiteName);
                        }
                    }
                }
            }

            String siteStatusReport = generateBhieStatusReportV4(mapDASStatusList(vlerResultCollection.getStatusList()),
                    vlerResultCollection.getErrorList(), true);

            vistaBeans.addAll(Arrays.asList(
                    (PatientDemographics[]) generateConnectionUnavailableBean(
                            BHIE_STATUS_REPORT_LOCATION,
                            siteStatusReport)
            ));
        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }

        return vistaBeans.toArray(new PatientDemographics[vistaBeans.size()]);
    }

    public Object[] generateBHIEPatRegErrorBean(Location location) {

        PatientDemographics pat = new PatientDemographics();
        Address patAddress = new Address();
        pat.setName(Messages.get("demographics.dod.results.empty").toUpperCase());
        patAddress.setStreet1(Messages.get("demographics.dod.results.helpdesk").toUpperCase());
        patAddress.setStreet2("");
        pat.setAge("");
        patAddress.setCity("");
        pat.setClinic("");
        pat.setDob("");
        pat.setGender("");
        pat.setPrimaryProvider("");
        pat.setPhone1("");
        pat.setPhone2("");
        pat.setSSN("");
        patAddress.setStateProvince("");
        patAddress.setPostalCode("");
        pat.setCodeGreen("0");
        pat.setSite(getConnectionUnavailableSite(location));
        pat.getAddress().add(patAddress);
        return new PatientDemographics[]{pat};
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientDemographics pat = new PatientDemographics();
        pat.setName(errorMsg);
        pat.setAge("");
        pat.setClinic("");
        pat.setDob("");
        pat.setGender("");
        pat.getRace().clear();
        pat.setPrimaryProvider("");
        pat.setPhone1("");
        pat.setPhone2("");
        pat.setSSN("");
        pat.setCodeGreen("0");
        pat.setSite(getConnectionUnavailableSite(location));
        pat.getAddress().clear();
        return new PatientDemographics[]{pat};
    }

    public static String getNameFromPerson(PersonV4 person) {
        String rtc;
        if (person.getName() == null) {
            rtc = "";
        } else {
            rtc = getNameString(person.getName());
        }
        return rtc;
    }

    public static String getNameString(NameV4 namev4) {
        String rtc;

            if (namev4.getRaw() == null) {
                StringBuilder nameSb = new StringBuilder();
                if (namev4.getLast() != null) {
                    nameSb.append(namev4.getLast());
                }
                nameSb.append(',');
                if (namev4.getPrefix() != null) {
                    nameSb.append(' ');
                    nameSb.append(namev4.getPrefix());
                }
                if (namev4.getFirst() != null) {
                    nameSb.append(' ');
                    nameSb.append(namev4.getFirst());
                }
                if (namev4.getMiddle() != null) {
                    nameSb.append(' ');
                    nameSb.append(namev4.getMiddle());
                }
                if (namev4.getSuffix() != null) {
                    nameSb.append(' ');
                    nameSb.append(namev4.getSuffix());
                }
                rtc = nameSb.toString();
            } else {
                rtc = namev4.getRaw();
            }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartDemographics tmp = (SmartDemographics) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_demographic.getSite() != null && tmp.m_demographic.getSite() != null) {
            retVal = compareString(this.m_demographic.getSite().getSiteCode(), tmp.m_demographic.getSite().getSiteCode());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    //TODO: Why is sort by always SITE?
    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = SITE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }
    }
}
