/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.Form;
import gov.va.med.jmeadows.dao.beans.PatientDocument;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.*;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;


public class SmartDocument extends SmartBean implements Comparable {

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int HCP = 4;
    public static final int LOC = 5;

    public static final String DOCUMENT_PROGRESS_NOTE = "Progress Note";
    public static final String DOCUMENT_ENCOUNTER_NOTE = "Encounter Note";
    public static final String DOCUMENT_OUTPATIENT_ENCOUNTER = "Outpatient Encounter";
    public static final String DOCUMENT_CONSULT_REPORT = "Consult Report";
    public static final String DOCUMENT_DEPLOYMENT_FORM = "Deployment Form";
    public static final String DOCUMENT_INPATIENT_NOTE = "Inpatient Note";
    public static final String DOCUMENT_QUESTIONNAIRE = "Questionnaires";
    public static final String DOCUMENT_RADIOLOGY = "Radiology Exams";
    public static final String DOCUMENT_HAIMS = "HAIMS";

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartDocument.class);

    public PatientDocument m_doc;

    public SmartDocument() {
        super();
        m_doc = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartDocument(PatientDocument doc, String sort) {
        super();
        m_doc = doc;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartDocument) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_doc);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartDocument[] temp = new SmartDocument[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDocument newAll = new SmartDocument((PatientDocument) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartDocument newAll = new SmartDocument((PatientDocument) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDocument[] ret = new PatientDocument[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_doc;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartDocument[] temp = new SmartDocument[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDocument newAll = new SmartDocument((PatientDocument) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDocument[] ret = new PatientDocument[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_doc;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartDocument tmp = (SmartDocument) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_doc.getNoteDate(), tmp.m_doc.getNoteDate());
        }
        if (this.m_sortBy == SITE && this.m_doc.getSite() != null && tmp.m_doc.getSite() != null) {
            retVal = compareString(this.m_doc.getSite().getSiteCode(), tmp.m_doc.getSite().getSiteCode());
        }
        if (this.m_sortBy == TYPE) {
            retVal = compareString(this.m_doc.getNoteType(), tmp.m_doc.getNoteType());
        }
        if (this.m_sortBy == HCP) {
            retVal = compareString(this.m_doc.getProvider().getName(), tmp.m_doc.getProvider().getName());
        }
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_doc.getLocation(), tmp.m_doc.getLocation());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("notedate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("notetype".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("user".equalsIgnoreCase(sort)) {
                m_sortBy = HCP;
            }
            if ("location".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
        }

    }


    public PatientDocument[] mapProgressNotesFromVistA(Object resultCollection) throws JMeadowsException {
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        ProgressNote[] results = (ProgressNote[]) resultCollection;
        for (ProgressNote note : results) {
            PatientDocument pDoc = new PatientDocument();
            pDoc.setDocumentType(DOCUMENT_PROGRESS_NOTE);

            pDoc.setAmended(note.getAmended());
            pDoc.setComplexDataUrl(note.getComplexDataUrl());
            pDoc.setEncounter(note.getEncounter());
            pDoc.setImages(note.getImages());
            pDoc.setLocation(note.getLocation());
            pDoc.setNoteDate(note.getNoteDate());
            pDoc.setNoteId(note.getNoteId());
            pDoc.setNoteText(note.getNoteText());
            pDoc.setNoteTitle(note.getNoteTitle());
            pDoc.setNoteTitleId(note.getNoteTitleId());
            pDoc.setNoteType(note.getNoteType());
            pDoc.setProvider(note.getProvider());
            pDoc.setSensitive(note.getSensitive());
            pDoc.setStatus(note.getStatus());
            pDoc.setUserIen(note.getUserIen());
            pDoc.setVisitDate(note.getVisitDate());
            pDoc.setCdrEventId(note.getCdrEventId());
            pDoc.getCodes().addAll(note.getCodes());
            pDoc.setNarrativeId(note.getNarrativeId());
            pDoc.setPatientId(note.getPatientId());
            pDoc.setPatientName(note.getPatientName());
            pDoc.setSite(note.getSite());
            pDoc.setSourceProtocol(note.getSourceProtocol());

            pDoc.setProgressNote(note);

            vistaBeans.add(pDoc);
        }
        return vistaBeans.toArray(new PatientDocument[0]);
    }

    public PatientDocument[] mapInpatientNotesFromVistA(Object resultCollection) throws JMeadowsException {
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        ProgressNote[] results = (ProgressNote[]) resultCollection;
        for (ProgressNote note : results) {
            PatientDocument pDoc = new PatientDocument();
            pDoc.setDocumentType(DOCUMENT_INPATIENT_NOTE);

            pDoc.setAmended(note.getAmended());
            pDoc.setComplexDataUrl(note.getComplexDataUrl());
            pDoc.setEncounter(note.getEncounter());
            pDoc.setImages(note.getImages());
            pDoc.setLocation(note.getLocation());
            pDoc.setNoteDate(note.getNoteDate());
            pDoc.setNoteId(note.getNoteId());
            pDoc.setNoteText(note.getNoteText());
            pDoc.setNoteTitle(note.getNoteTitle());
            pDoc.setNoteTitleId(note.getNoteTitleId());
            pDoc.setNoteType(note.getNoteType());
            pDoc.setProvider(note.getProvider());
            pDoc.setSensitive(note.getSensitive());
            pDoc.setStatus(note.getStatus());
            pDoc.setUserIen(note.getUserIen());
            pDoc.setVisitDate(note.getVisitDate());
            pDoc.setCdrEventId(note.getCdrEventId());
            pDoc.getCodes().addAll(note.getCodes());
            pDoc.setNarrativeId(note.getNarrativeId());
            pDoc.setPatientId(note.getPatientId());
            pDoc.setPatientName(note.getPatientName());
            pDoc.setSite(note.getSite());
            pDoc.setSourceProtocol(note.getSourceProtocol());

            pDoc.setDischargeSummary(note);

            vistaBeans.add(pDoc);
        }
        return vistaBeans.toArray(new PatientDocument[0]);
    }

    public PatientDocument[] mapEncountersFromVistA(Object resultCollection) throws JMeadowsException {
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        Encounter[] results = (Encounter[]) resultCollection;
        for (Encounter note : results) {
            PatientDocument pDoc = new PatientDocument();
            pDoc.setDocumentType(DOCUMENT_OUTPATIENT_ENCOUNTER);
            pDoc.setOutpatientEncounter(note);

            pDoc.setLocation(note.getClinic());
            pDoc.setNoteDate(note.getApptDate());
            pDoc.setNoteId(note.getEncounterId());
            if (note.getDiagnosis() != null && !note.getDiagnosis().isEmpty()) {
                pDoc.setNoteTitle(note.getDiagnosis().get(0).getDescription());
            }
            pDoc.setProvider(note.getProvider());
            pDoc.setSensitive(note.getSensitive());
            pDoc.setStatus(note.getStatus());
            pDoc.setUserIen(note.getUserIen());
            pDoc.getCodes().addAll(note.getCodes());
            pDoc.setPatientId(note.getPatientId());
            pDoc.setPatientName(note.getPatientName());
            pDoc.setSite(note.getSite());
            pDoc.setSourceProtocol(note.getSourceProtocol());

            pDoc.setOutpatientEncounter(note);

            vistaBeans.add(pDoc);
        }
        return vistaBeans.toArray(new PatientDocument[0]);
    }

    public PatientDocument[] mapConsultsFromVistA(Object resultCollection) throws JMeadowsException {
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        Consult[] results = (Consult[]) resultCollection;
        for (Consult note : results) {
            PatientDocument pDoc = new PatientDocument();
            pDoc.setDocumentType(DOCUMENT_CONSULT_REPORT);

            pDoc.setConsult(note);

            pDoc.setNoteDate(note.getRequestDate());
            pDoc.setNoteId(note.getId());
            pDoc.setNoteTitle(note.getService());
            pDoc.setProvider(note.getProvider());

            pDoc.setStatus(note.getStatus());
            pDoc.getCodes().addAll(note.getCodes());
            pDoc.setPatientId(note.getPatientId());
            pDoc.setPatientName(note.getPatientName());
            pDoc.setSite(note.getSite());
            pDoc.setSourceProtocol(note.getSourceProtocol());

            pDoc.setConsult(note);

            vistaBeans.add(pDoc);
        }
        return vistaBeans.toArray(new PatientDocument[0]);
    }

    public PatientDocument[] mapRadiologyFromVistA(Object resultCollection) throws JMeadowsException {
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        RadiologyReport[] results = (RadiologyReport[]) resultCollection;
        for (RadiologyReport rad : results) {
            PatientDocument pDoc = new PatientDocument();
            pDoc.setDocumentType(DOCUMENT_RADIOLOGY);

            pDoc.setRadiologyReport(rad);

            pDoc.setNoteDate(rad.getExamDate());
            pDoc.setNoteId(rad.getExamId());
            pDoc.setNoteTitle(rad.getStudy());

            pDoc.setStatus(rad.getStatus());
            pDoc.getCodes().addAll(rad.getCodes());
            pDoc.setPatientId(rad.getPatientId());
            pDoc.setPatientName(rad.getPatientName());
            pDoc.setSite(rad.getSite());
            pDoc.setSourceProtocol(rad.getSourceProtocol());

            pDoc.setRadiologyReport(rad);

            vistaBeans.add(pDoc);
        }
        return vistaBeans.toArray(new PatientDocument[0]);
    }

    // Version 4
    public PatientDocument[] mapProgressNotesFromBHIE(Object resultCollection, String docType) throws JMeadowsException {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();
        Boolean hasConsults = false;
        try {

            if (noteResultCollection.getDataRecordList() != null) {
                List<NoteV4> noteList = new ArrayList<NoteV4>();
                if (noteResultCollection.getDataRecordList().getNotes() != null
                        && noteResultCollection.getDataRecordList().getNotes().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getNotes());
                }
                if (noteResultCollection.getDataRecordList().getEncounterNotes() != null
                        && noteResultCollection.getDataRecordList().getEncounterNotes().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getEncounterNotes());
                }
                if (noteResultCollection.getDataRecordList().getInpatientNotes() != null
                        && noteResultCollection.getDataRecordList().getInpatientNotes().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getInpatientNotes());
                }
                if (noteResultCollection.getDataRecordList().getDocuments() != null
                        && noteResultCollection.getDataRecordList().getDocuments().size() > 0) {
                    noteList.addAll(noteResultCollection.getDataRecordList().getDocuments());
                }

                for (NoteV4 bhieNote : noteList) {
                    if (bhieNote == null) {
                        continue;
                    }

                    //filter out consult note
                    String loinc = "";
                    if (bhieNote.getDataDomain() != null && !bhieNote.getDataDomain().isEmpty()) {
                        loinc = bhieNote.getDataDomain().get(0).getCode();
                    }
                    if (LOINC_CODE_NOTES.equals(loinc) && (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && NOTE_TYPE_CODE_CONSULT.contains(bhieNote.getType().get(0).getCode()))) {
                        hasConsults = true;
                        continue;
                    }
                    PatientDocument bean;

                    bean = new PatientDocument();
                    bean.setDocumentType(docType);

                    if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                        bean.setNoteTitle(bhieNote.getTitle());
                    }
                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                            && bhieNote.getType().get(0) != null) {
                        bean.setNoteType(bhieNote.getType().get(0).getDisplay());
                    }
                    bean.setSensitive(bhieNote.getSensitive());
                    if (bhieNote.getContentType() != null && !bhieNote.getContentType().isEmpty()
                            && getPrimarySpdcV4(bhieNote.getContentType()) != null) {
                        bean.setContentType(getPrimarySpdcV4(bhieNote.getContentType()).getCode());
                    } else if(!StringUtils.isEmpty(bhieNote.getComplexDataUrl())){
                        //JLV-1075: temp fix until DES adds contentType : assume that content is of type: RTF
                        bean.setContentType("application/rtf");
                    }

                    if (bhieNote.getStatus() != null && !bhieNote.getStatus().isEmpty()
                            && getPrimarySpdcV4(bhieNote.getStatus()) != null) {
                        bean.setStatus(getPrimarySpdcV4(bhieNote.getStatus()).getDisplay());
                    }
                    bean.setComplexDataUrl(bhieNote.getComplexDataUrl());
                    String custodian = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        custodian = bhieNote.getRepository().getIds().get(0).getId();
                    }

                    if (DOCUMENT_HAIMS.equals(docType)
                            && bhieNote.getEnteredDate() != null
                            && bhieNote.getEnteredDate().getStart() != null) {
                        bean.setNoteDate(bhieToXMLCal(bhieNote.getEnteredDate().getStart()));
                    } else if (bhieNote.getVerifiedDate() != null
                            && bhieNote.getVerifiedDate().getStart() != null) {
                        bean.setNoteDate(bhieToXMLCal(bhieNote.getVerifiedDate().getStart()));
                    }

                    String clinic = "";
                    if (bhieNote.getLocation() != null && bhieNote.getLocation().getName() != null) {
                        clinic = bhieNote.getLocation().getName();
                    } else if (bhieNote.getFacility() != null) {
                        clinic = bhieNote.getFacility().getName();
                    }

                    //dod adapter facility format: location^clinic
                    if (StringUtils.isNotBlank(clinic) && clinic.contains("^")) {
                        clinic = clinic.substring(clinic.indexOf('^') + 1);
                    }

                    bean.setLocation(clinic);

                    //                 ALTHA currrently does not send CDA and uses complexURL
                    //                if ("CDA".equalsIgnoreCase(bhieNote.getNoteFormat())) {
                    //                    //may complete complete note and event id
                    //                    mapCDAToProgressNote(bhieNote, bean);
                    //                } else
                    if (bhieNote.getComplexDataUrl() == null && bhieNote.getContent() != null) {
                        bean.setNoteText(bhieNote.getContent());
                    } else if (bhieNote.getCompleteNote() != null) {
                        bean.setNoteText(bhieNote.getCompleteNote());
                    }

                    if (bhieNote.getVerifiedBy() != null) {
                        bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                    }

                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && StringUtils.isNotBlank(bhieNote.getType().get(0).getCode())) {
                        for( SpdcBeanV4 type : bhieNote.getType()){
                            Code code = new Code();
                            code.setPrimary(String.valueOf(type.isPrimary()));
                            code.setDisplay(type.getDisplay());
                            code.setSystem(type.getSystem());
                            code.setCode(type.getCode());
                            bean.getCodes().add(code);
                        }
                    }

                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getRepository() != null) {
                        repositoryName = bhieNote.getRepository().getName();
                    }
                    if (bhieNote.getFacility() != null) {
                        facilityName = bhieNote.getFacility().getName();
                    }
                    if (bhieNote.getRecordId() != null) {
                        eventId = bhieNote.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                            facilityName, eventId, bean);

                    vistaBeans.add(bean);
                }
            }
        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }
        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(docType, noteResultCollection.getStatusListv4(),
                noteResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        if (hasConsults) {
            vistaBeans.addAll(Arrays.asList((PatientDocument[]) mapConsultsFromBHIE(resultCollection)));
        }

        return vistaBeans.toArray(new PatientDocument[vistaBeans.size()]);
    }

    public PatientDocument[] mapProgressNotesFromBHIE(Object resultCollection) throws JMeadowsException {
        return mapProgressNotesFromBHIE(resultCollection, DOCUMENT_PROGRESS_NOTE);
    }

    public PatientDocument[] mapEncounterNotesFromBHIE(Object resultCollection) throws JMeadowsException {
        return mapProgressNotesFromBHIE(resultCollection, DOCUMENT_ENCOUNTER_NOTE);
    }

    public PatientDocument[] mapPatientDischargeSummariesFromBHIE(Object resultCollection) throws JMeadowsException {
        return mapProgressNotesFromBHIE(resultCollection, DOCUMENT_INPATIENT_NOTE);
    }

    public PatientDocument[] mapDocumentsFromBHIE(Object resultCollection) throws JMeadowsException {
        return mapProgressNotesFromBHIE(resultCollection, DOCUMENT_HAIMS);
    }


    public PatientDocument[] mapConsultsFromBHIE(Object resultCollection) {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();

        if (noteResultCollection.getDataRecordList() != null && noteResultCollection.getDataRecordList().getNotes() != null) {
            for (NoteV4 bhieNote : noteResultCollection.getDataRecordList().getNotes()) {
                //workaround until DoD adaptor returns patient consults
                if ((bhieNote.getDataDomain() != null && !bhieNote.getDataDomain().isEmpty() && !LOINC_CODE_NOTES.equals(bhieNote.getDataDomain().get(0).getCode()))
                        || (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && !NOTE_TYPE_CODE_CONSULT.contains(bhieNote.getType().get(0).getCode()))) {
                    continue;
                }

                PatientDocument consultDoc = new PatientDocument();
                consultDoc.setDocumentType(DOCUMENT_CONSULT_REPORT);

                Consult bean = new Consult();

                if (bhieNote.getEnteredDate() != null && bhieNote.getEnteredDate().getStart() != null) {
                    bean.setRequestDate(bhieToXMLCal(bhieNote.getEnteredDate().getStart()));
                    consultDoc.setNoteDate(bhieToXMLCal(bhieNote.getEnteredDate().getStart()));
                }
                if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                    bean.setService(bhieNote.getTitle());
                    consultDoc.setNoteTitle(bhieNote.getTitle());
                }
                if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                        && bhieNote.getType().get(0) != null) {
                    bean.setProcedureConsult(bhieNote.getType().get(0).getDisplay());
                }
                bean.setReport(bhieNote.getContent());
//                if (bhieNote.getEnteredBy() != null) {
//                    bean.setProvider(mapProvider(bhieNote.getEnteredBy()));
//                }
                if (bhieNote.getVerifiedBy() != null) {
                    bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                    consultDoc.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                }
                if (bhieNote.getStatus() != null && !bhieNote.getStatus().isEmpty()
                        && getPrimarySpdcV4(bhieNote.getStatus()) != null) {
                    bean.setStatus(bhieNote.getStatus().get(0).getDisplay());
                    consultDoc.setStatus(getPrimarySpdcV4(bhieNote.getStatus()).getDisplay());
                }
                bean.setComplexDataUrl(bhieNote.getComplexDataUrl());

                if (bhieNote.getContentType() != null && !bhieNote.getContentType().isEmpty()
                        && getPrimarySpdcV4(bhieNote.getContentType()) != null) {
                    bean.setContentType(getPrimarySpdcV4(bhieNote.getContentType()).getCode());
                    consultDoc.setContentType(getPrimarySpdcV4(bhieNote.getContentType()).getCode());
                } else if(!StringUtils.isEmpty(bhieNote.getComplexDataUrl())){
                    //JLV-1075: temp fix until DES adds contentType : assume that content is of type: RTF
                    bean.setContentType("application/rtf");
                    consultDoc.setContentType("application/rtf");
                }

                consultDoc.setComplexDataUrl(bhieNote.getComplexDataUrl());

//                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
//                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);
//                addBHIESiteDataToVistaBean(bhieNote.getRepository().getId().get(0).getId(),
//                        bhieNote.getRepository().getName(), bhieNote.getFacility().getName(), bhieNote.getRecordId().getId(), bean);

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null
                        && !bhieNote.getRepository().getIds().isEmpty()
                        && bhieNote.getRepository().getIds().get(0) != null) {
                    repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                }
                if (bhieNote.getRepository() != null) {
                    repositoryName = bhieNote.getRepository().getName();
                }
                if (bhieNote.getFacility() != null) {
                    facilityName = bhieNote.getFacility().getName();
                }
                if (bhieNote.getRecordId() != null) {
                    eventId = bhieNote.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, consultDoc);

                consultDoc.setConsult(bean);
                vistaBeans.add(consultDoc);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4("Consult Reports", noteResultCollection.getStatusListv4(),
                noteResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientDocument[vistaBeans.size()]);
    }

    public PatientDocument[] mapEncountersFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection encounterResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDocument> beanResults = new ArrayList<PatientDocument>();
        boolean hasMedcin;
        boolean excludeMedcin;

        if (encounterResultCollection.getDataRecordList() != null
                && encounterResultCollection.getDataRecordList().getEncounters() != null) {

            for (EncounterV4 bhieEncounter : encounterResultCollection.getDataRecordList().getEncounters()) {
                if (bhieEncounter == null) {
                    continue;
                }

                PatientDocument encounterDoc = new PatientDocument();
                encounterDoc.setDocumentType(DOCUMENT_OUTPATIENT_ENCOUNTER);

                Encounter bean = new Encounter();

                encounterDoc.setComplexDataUrl(bhieEncounter.getComplexDataUrl());
                bean.setComplexDataUrl(bhieEncounter.getComplexDataUrl());
                if (bhieEncounter.getContentType() != null && !bhieEncounter.getContentType().isEmpty()
                        && getPrimarySpdcV4(bhieEncounter.getContentType()) != null) {
                    bean.setContentType(getPrimarySpdcV4(bhieEncounter.getContentType()).getCode());
                    encounterDoc.setContentType(getPrimarySpdcV4(bhieEncounter.getContentType()).getCode());
                } else if(!StringUtils.isEmpty(bhieEncounter.getComplexDataUrl())){
                    //JLV-1075: temp fix until DES adds contentType : assume that content is of type: RTF
                    bean.setContentType("application/rtf");
                    encounterDoc.setContentType("application/rtf");
                }

                if (bhieEncounter.getArrivalDate() != null && bhieToXMLCalV4(bhieEncounter.getArrivalDate()) != null && bhieToXMLCalV4(bhieEncounter.getArrivalDate()).isValid()) {
                    bean.setApptDate(bhieToXMLCalV4(bhieEncounter.getArrivalDate()));
                    encounterDoc.setNoteDate(bhieToXMLCalV4(bhieEncounter.getArrivalDate()));
                }

                if (bhieEncounter.getType() != null
                        && bhieEncounter.getType().size() > 0
                        && getPrimarySpdcV4(bhieEncounter.getType()) != null) {
                    bean.setApptType(getPrimarySpdcV4(bhieEncounter.getType()).getDisplay());
                    encounterDoc.setNoteType(getPrimarySpdcV4(bhieEncounter.getType()).getDisplay());
                }

                if (bhieEncounter.getEncounterLocations() != null
                        && !bhieEncounter.getEncounterLocations().isEmpty()) {
                    bean.setClinic(bhieEncounter.getEncounterLocations().get(0).getName());
                    encounterDoc.setLocation(bhieEncounter.getEncounterLocations().get(0).getName());
                }
                if (bhieEncounter.getDisposition() != null && !bhieEncounter.getDisposition().isEmpty()
                        && bhieEncounter.getDisposition().get(0) != null) {
                    bean.setDischargeDisposition(bhieEncounter.getDisposition().get(0).getDisplay());
                }
                if (bhieEncounter.getEncounterNumber() != null) {
                    bean.setEncounterId(bhieEncounter.getEncounterNumber().getId());
                    bean.setEncounterNumber(bhieEncounter.getEncounterNumber().getId());
                }
                if (bhieEncounter.getEnteredBy() != null) {
                    bean.setEnteredBy(bhieEncounter.getEnteredBy().getName());
                }
                if (bhieEncounter.getEnteredDate() != null) {
                    bean.setEnteredByDate(bhieToXMLCalV4(bhieEncounter.getEnteredDate()));
                }
                //not available after update
                //bean.setEvalManagementCode(bhieEncounter.getEvalManagementCode());
                if (bhieEncounter.getProvider() != null) {
                    bean.setProvider(mapProvider(bhieEncounter.getProvider()));
                    encounterDoc.setProvider(mapProvider(bhieEncounter.getProvider()));
                }
                //bean.setReport(bhieEncounter.getReport());
                //bean.setReportFormat(bhieEncounter.getReportFormat());
                if (bhieEncounter.getStatus() != null
                        && !bhieEncounter.getStatus().isEmpty()
                        && getPrimarySpdcV4(bhieEncounter.getStatus()) != null) {
                    bean.setStatus(getPrimarySpdcV4(bhieEncounter.getStatus()).getDisplay());
                    encounterDoc.setStatus(getPrimarySpdcV4(bhieEncounter.getStatus()).getDisplay());
                }
                if (bhieEncounter.getReason() != null && getPrimarySpdcV4(bhieEncounter.getReason()) != null) {
                    bean.setReason(getPrimarySpdcV4(bhieEncounter.getReason()).getDisplay());
                }
                bean.setSensitive(bhieEncounter.getSensitive());
                encounterDoc.setSensitive(bhieEncounter.getSensitive());

                if (bhieEncounter.getDiagnoses() != null) {
                    List<Diagnosis> diagnosisList = new ArrayList<Diagnosis>(bhieEncounter.getDiagnoses().size());

                    boolean firstRun = true;
                    for (EncounterDiagnosisV4 bhieDiagnosis : bhieEncounter.getDiagnoses()) {

                        Diagnosis diagnosis = new Diagnosis();

                        if (bhieDiagnosis.getDiagnosis() != null
                                && !bhieDiagnosis.getDiagnosis().isEmpty()) {
                            hasMedcin = false;
                            excludeMedcin = false;

                            for (SpdcBeanV4 diagnosisBean : bhieDiagnosis.getDiagnosis()) {
                                if (diagnosisBean != null) {
                                    if (CodingSystem.ICD10_CM.getOID().equalsIgnoreCase(diagnosisBean.getSystem())) {
                                        diagnosis.setDescription(diagnosisBean.getDisplay());
                                        diagnosis.setCode(diagnosisBean.getCode());
                                        diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                        excludeMedcin = true;
                                        if(firstRun){
                                            encounterDoc.setNoteTitle(diagnosisBean.getDisplay());
                                        }
                                    }
                                    else if (CodingSystem.ICD9_CM_DISEASES.getOID().equalsIgnoreCase(diagnosisBean.getSystem())
                                            && !StringUtils.isEmpty(diagnosisBean.getCode())) {
                                        diagnosis.setDescription(diagnosisBean.getDisplay());
                                        diagnosis.setCode(diagnosisBean.getCode());
                                        diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                    }

                                    if (CodingSystem.DOD_MEDCIN.getOID().equalsIgnoreCase(diagnosisBean.getSystem())) {
                                        if(!excludeMedcin){
                                            diagnosis.setCode(diagnosisBean.getCode());
                                            diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                            diagnosis.setDescription(diagnosisBean.getDisplay());
                                            if(firstRun){
                                                encounterDoc.setNoteTitle(diagnosisBean.getDisplay());
                                            }
                                        }
                                        hasMedcin = true;
                                    }
                                    if ( firstRun && (CodingSystem.ICD10_SNOMED_CT.getOID().equalsIgnoreCase(diagnosisBean.getSystem()))){
                                        Code code = new Code();
                                        code.setSystem(diagnosisBean.getSystem());
                                        code.setPrimary(String.valueOf(diagnosisBean.isPrimary()));
                                        code.setDisplay(diagnosisBean.getDisplay());
                                        code.setCode(diagnosisBean.getCode());
                                        bean.getCodes().add(code);
                                    }
                                }

                            }

                            // If ICD-10 code exists, exclude/remove MEDCIN from code list, if exists
                            if (hasMedcin && excludeMedcin) {
                                Code code;
                                for (Iterator<Code> codeIterator = bean.getCodes().iterator(); codeIterator.hasNext(); ) {
                                    code = codeIterator.next();
                                    if (code != null && CodingSystem.DOD_MEDCIN.getDisplayName().equalsIgnoreCase(code.getSystem())) {
                                        codeIterator.remove();
                                    }
                                }
                                for (Iterator<Code> codeIterator = encounterDoc.getCodes().iterator(); codeIterator.hasNext(); ) {
                                    code = codeIterator.next();
                                    if (code != null && CodingSystem.DOD_MEDCIN.getDisplayName().equalsIgnoreCase(code.getSystem())) {
                                        codeIterator.remove();
                                    }
                                }
                            }
                        }

                        if (bhieDiagnosis.getStatus() != null
                                && !bhieDiagnosis.getStatus().isEmpty()) {
                            for (SpdcBeanV4 statusBean : bhieDiagnosis.getStatus()) {
                                if (statusBean != null && statusBean.isPrimary()) {
                                    diagnosis.setStatus(statusBean.getDisplay());
                                }
                            }
                        }

                        if (bhieDiagnosis.getEnteredDate() != null) {
                            diagnosis.setDate(bhieToXMLCalV4(bhieDiagnosis.getEnteredDate()));
                        }

                        diagnosisList.add(diagnosis);

                        firstRun = false;
                    }

                    //bean.setDiagnosis(diagnosisList);
                    bean.getDiagnosis().addAll(diagnosisList);
                }

                if (bhieEncounter.getProcedures() != null) {
                    List<Procedure> procedures = new ArrayList<Procedure>(bhieEncounter.getProcedures().size());

                    for (EncounterProcedureV4 bhieProcedure : bhieEncounter.getProcedures()) {
                        Procedure procedure = new Procedure();
                        if (bhieProcedure.getProcedure() != null
                                && !bhieProcedure.getProcedure().isEmpty()) {
                            for (SpdcBeanV4 procedureBean : bhieProcedure.getProcedure()) {
                                if (procedureBean == null) {
                                    continue;
                                }
                                if (procedureBean.isPrimary()) {
                                    procedure.setDescription(procedureBean.getDisplay());
                                }
                                if (CodingSystem.CPT.getOID().equalsIgnoreCase(procedureBean.getSystem())) {
                                    procedure.setCode(procedureBean.getCode());
                                    procedure.setCodeSystem(procedureBean.getSystem());
                                }
                            }

                        }
                        if (bhieProcedure.getDate() != null) {
                            procedure.setDate(bhieToXMLCalV4(bhieProcedure.getDate()));
                        }
                        if (bhieProcedure.getStatus() != null
                                && !bhieProcedure.getStatus().isEmpty()
                                && getPrimarySpdcV4(bhieProcedure.getStatus()) != null) {
                            procedure.setStatus(getPrimarySpdcV4(bhieProcedure.getStatus()).getDisplay());
                        }

                        procedures.add(procedure);
                    }

                    bean.getProcedures().addAll(procedures);
                }

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieEncounter.getRepository() != null
                        && bhieEncounter.getRepository().getIds() != null && !bhieEncounter.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieEncounter.getRepository().getIds().get(0).getId();
                }
                if (bhieEncounter.getRepository() != null) {
                    repositoryName = bhieEncounter.getRepository().getName();
                }
                if (bhieEncounter.getFacility() != null) {
                    facilityName = bhieEncounter.getFacility().getName();
                }
                if (bhieEncounter.getRecordId() != null) {
                    eventId = bhieEncounter.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, encounterDoc);

                encounterDoc.setOutpatientEncounter(bean);
                beanResults.add(encounterDoc);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4(DOCUMENT_OUTPATIENT_ENCOUNTER, encounterResultCollection.getStatusListv4(),
                encounterResultCollection.getErrorListv4());

        beanResults.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return beanResults.toArray(new PatientDocument[beanResults.size()]);
    }

    public PatientDocument[] mapQuestionnairesFromBHIE(Object resultCollection) {
        List<PatientDocument> docBeans = new ArrayList<PatientDocument>();

        V4ResultCollection questionnaireResultCollection = (V4ResultCollection) resultCollection;

        if (questionnaireResultCollection.getDataRecordList() != null && questionnaireResultCollection.getDataRecordList().getQuestionnaires() != null) {

            for (QuestionnaireV4 questionnaire : questionnaireResultCollection.getDataRecordList().getQuestionnaires()) {
                if (questionnaire == null) {
                    continue;
                }

                PatientDocument formDoc = new PatientDocument();
                formDoc.setDocumentType(DOCUMENT_QUESTIONNAIRE);

                Form formBean = new Form();

                QuestionnaireBean bean = new QuestionnaireBean();

                bean.setComments(questionnaire.getComments());
                if (questionnaire.getResponseDate() != null) {
                    bean.setResponseDate(bhieToXMLCalV4(questionnaire.getResponseDate()));
                    formBean.setFormDate(bhieToXMLCalV4(questionnaire.getResponseDate()).toGregorianCalendar());
                    formDoc.setNoteDate(bhieToXMLCalV4(questionnaire.getResponseDate()));
                }

                if (questionnaire.getResponses() != null && !questionnaire.getResponses().isEmpty()) {
                    for (gov.va.med.bhie.QuestionnaireDetailsV4 responses : questionnaire.getResponses()) {
                        gov.va.med.vds.webservice.QuestionnaireDetails det = new gov.va.med.vds.webservice.QuestionnaireDetails();
                        det.setQuestion(responses.getQuestion());
                        det.setSequence(responses.getSequenceNumber());
                        if (responses.getAnswers() != null && !responses.getAnswers().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailAnswersV4 answer : responses.getAnswers()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailAnswers ans = new gov.va.med.vds.webservice.QuestionnaireDetailAnswers();
                                ans.setAnswer(answer.getAnswer());
                                ans.setAnswerType(answer.getAnswerType());
                                ans.setChoice(answer.getChoice());
                                ans.setCorrectAnswer(answer.getCorrectAnswer());
                                det.getAnswers().add(ans);
                            }
                        }
                        if (responses.getChoices() != null && !responses.getChoices().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailChoicesV4 choice : responses.getChoices()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailChoices ichoice = new gov.va.med.vds.webservice.QuestionnaireDetailChoices();
                                ichoice.setAnswerType(choice.getAnswerType());
                                ichoice.setChoice(choice.getChoice());
                                ichoice.setChoiceText(choice.getText());
                                ichoice.setCorrectAnswer(choice.getCorrectAnswer());
                                ichoice.setSelected(choice.isSelected());
                                det.getChoices().add(ichoice);
                            }
                        }
                        if (responses.getComments() != null && !responses.getComments().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailCommentsV4 comment : responses.getComments()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailComments cmt = new gov.va.med.vds.webservice.QuestionnaireDetailComments();
                                cmt.setCommentBy(comment.getEnteredBy().getName());
                                cmt.setDateEntered(comment.getEnteredDate().getStart());
                                cmt.setNoteNumber(comment.getCommentNumber());
                                cmt.setNoteText(comment.getComment());
                                cmt.setSeqNumber(comment.getSequenceNumber());
                                det.getComments().add(cmt);
                            }
                        }
                        bean.getResponses().add(det);
                    }
                }
                formDoc.setProvider(mapProvider(questionnaire.getEnteredBy()));
                bean.setEnteredBy(mapProvider(questionnaire.getEnteredBy()));
                formDoc.setNoteDate(bhieToXMLCalV4(questionnaire.getEnteredDate()));
                bean.setEnteredDate(Utils.formatDisplayDateTime(bhieToXMLCalV4(questionnaire.getEnteredDate())));
                if (questionnaire.getRecordId() != null) {
                    bean.setRecordId(questionnaire.getRecordId().getId());
                }
                bean.setName(questionnaire.getName());
                formDoc.setNoteTitle(questionnaire.getName());
                bean.setType(questionnaire.getType());
                bean.setVersion(questionnaire.getVersion());
                if (questionnaire.getResponseComments() != null && !questionnaire.getResponseComments().isEmpty()) {
                    for (gov.va.med.bhie.QuestionnaireResponseCommentsV4 comment : questionnaire.getResponseComments()) {
                        gov.va.med.vds.webservice.QuestionnaireResponseComments cmt = new gov.va.med.vds.webservice.QuestionnaireResponseComments();
                        cmt.setComment(comment.getComment());
                        cmt.setDateEntered(comment.getEnteredDate().getStart());
                        cmt.setEnteredBy(mapProvider(comment.getEnteredBy()));
                        cmt.setNumber(comment.getNumber());
                        cmt.setSequence(comment.getSequence());
                        bean.getResponseComments().add(cmt);
                    }
                }

                bean.setScore(questionnaire.getScore());
                if (questionnaire.getStatus() != null
                        && !questionnaire.getStatus().isEmpty()
                        && getPrimarySpdcV4(questionnaire.getStatus()) != null) {
                    for(SpdcBeanV4 statusBean : questionnaire.getStatus()){
                        if (statusBean != null && statusBean.isPrimary()) {
                            bean.setStatus(statusBean.getDisplay());
                        }
                    }
                }

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (questionnaire.getRepository() != null && questionnaire.getRepository().getIds() != null && !questionnaire.getRepository().getIds().isEmpty()) {
                    repositoryId = questionnaire.getRepository().getIds().get(0).getId();
                }
                if (questionnaire.getRepository() != null) {
                    repositoryName = questionnaire.getRepository().getName();
                }
                if (questionnaire.getFacility() != null) {
                    facilityName = questionnaire.getFacility().getName();
                }
                if (questionnaire.getRecordId() != null) {
                    eventId = questionnaire.getRecordId().getId();
                    formBean.setFormId(eventId);
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

//                if(questionnaire.getDataDomain()!= null && !questionnaire.getDataDomain().isEmpty()){
//                    for(SpdcBean spdc : questionnaire.getDataDomain()){
//                        Code code = new Code();
//                        code.setSystem("LOINC");
//                        code.setCode(spdc.getCode());
//                        code.setDisplay(spdc.getDisplay());
//
//                        formBean.getCodes().add(code);
//                    }
//                }

                Code code = new Code();
                code.setSystem("2.16.840.1.113883.6.1");
                code.setCode("74465-6");
                code.setDisplay("Questionnaire response section Document");

                formBean.getCodes().add(code);
                formDoc.getCodes().add(code);

                if (questionnaire.getEnteredBy() != null
                        && questionnaire.getEnteredBy().getName() != null) {
                    formBean.setEnteredBy(questionnaire.getEnteredBy().getName());
                }
                formBean.setDocType("Q");
                formBean.setFacilityName(facilityName);
                formBean.setDocumentTitle(questionnaire.getName());
                formBean.setQuestionnaireBean(bean);
                addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, formBean);

                formDoc.setNoteTitle(questionnaire.getName());
                addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, formDoc);

                formDoc.setForm(formBean);
                docBeans.add(formDoc);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4(DOCUMENT_QUESTIONNAIRE, questionnaireResultCollection.getStatusListv4(),
                questionnaireResultCollection.getErrorListv4());

        docBeans.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return docBeans.toArray(new PatientDocument[docBeans.size()]);
    }


    // Version 4
    public PatientDocument[] mapDeploymentFormsFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();

        try {

            if (noteResultCollection != null && noteResultCollection.getDataRecordList() != null && noteResultCollection.getDataRecordList().getNotes() != null) {
                //BDA is currently sending us a null id, so let's create one for JLV cache
                int formId = 0;

                for (NoteV4 bhieNote : noteResultCollection.getDataRecordList().getDeploymentForms()) {
                    if (bhieNote == null) {
                        continue;
                    }

                    PatientDocument formDoc = new PatientDocument();
                    formDoc.setDocumentType(DOCUMENT_DEPLOYMENT_FORM);

                    Form form = new Form();
                    ProgressNote bean = new ProgressNote();

                    if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                        bean.setNoteTitle(bhieNote.getTitle());
                        formDoc.setNoteTitle(bhieNote.getTitle());
                    }
                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                            && bhieNote.getType().get(0) != null) {
                        bean.setNoteType(bhieNote.getType().get(0).getDisplay());
                        formDoc.setNoteType(bhieNote.getType().get(0).getDisplay());
                    }
                    bean.setSensitive(bhieNote.getSensitive());
                    formDoc.setSensitive(bhieNote.getSensitive());

//                    if (bhieNote.getContentType() != null && !bhieNote.getContentType().isEmpty()) {
//                        bean.setStatus(bhieNote.getContentType().get(0).getCode());
//                        formDoc.setStatus(bhieNote.getContentType().get(0).getCode());
//                    }
                    if (bhieNote.getStatus() != null && !bhieNote.getStatus().isEmpty()
                            && getPrimarySpdcV4(bhieNote.getStatus()) != null) {
                        bean.setStatus(getPrimarySpdcV4(bhieNote.getStatus()).getCode());
                        formDoc.setStatus(getPrimarySpdcV4(bhieNote.getStatus()).getCode());
                    }
                    bean.setComplexDataUrl(bhieNote.getComplexDataUrl());
                    String custodian = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        custodian = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getEnteredDate() != null) {
                        bean.setNoteDate(bhieToXMLCalV4(bhieNote.getEnteredDate()));
                        form.setFormDate(bhieToXMLCalV4(bhieNote.getEnteredDate()).toGregorianCalendar());
                        formDoc.setNoteDate(bhieToXMLCalV4(bhieNote.getEnteredDate()));
                    } else if (APP_CONFIG.getBHIECustodianOidSHARE().equalsIgnoreCase(custodian)) {
                        bean.setNoteDate(bhieToXMLCalV4(bhieNote.getVerifiedDate()));
                        form.setFormDate(bhieToXMLCalV4(bhieNote.getVerifiedDate()).toGregorianCalendar());
                        formDoc.setNoteDate(bhieToXMLCalV4(bhieNote.getVerifiedDate()));
                    }
                    String clinic = "";
                    if (bhieNote.getFacility() != null) {
                        clinic = bhieNote.getFacility().getName();
                    }

                    //dod adapter facility format: location^clinic
                    if (StringUtils.isNotBlank(clinic) && clinic.contains("^")) {
                        clinic = clinic.substring(clinic.indexOf('^') + 1);
                    }

                    bean.setLocation(clinic);
                    formDoc.setLocation(clinic);

                    //                 ALTHA currrently does not send CDA and uses complexURL
                    //                if ("CDA".equalsIgnoreCase(bhieNote.getNoteFormat())) {
                    //                    //may complete complete note and event id
                    //                    mapCDAToProgressNote(bhieNote, bean);
                    //                } else

                    bean.setNoteText(bhieNote.getContent());
//                    if(bhieNote.getRepository() != null
//                            && bhieNote.getRepository().getId() != null
//                            && !bhieNote.getRepository().getId().isEmpty()
//                            && bhieNote.getRepository().getId().get(0).getAssigningAuthority() != null
//                            && APP_CONFIG.getBHIECustodianOidSHARE().matches(bhieNote.getRepository().getId().get(0).getAssigningAuthority())
//                            && bhieNote.getContentType() != null
//                            && !bhieNote.getContentType().isEmpty()
//                            && bhieNote.getContentType().get(0).getCode() != null
//                            && "CDA".equalsIgnoreCase(bhieNote.getContentType().get(0).getCode()))
//                        //TODO:populate provider from BHIERelay
//                        bean.setProvider(mapProvider(bhieNote.getEnteredBy()));
                    if (bhieNote.getVerifiedBy() != null) {
                        bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                        formDoc.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                    }

                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && StringUtils.isNotBlank(bhieNote.getType().get(0).getCode())) {
                        for( SpdcBeanV4 type : bhieNote.getType()){
                            Code code = new Code();
                            code.setPrimary(String.valueOf(type.isPrimary()));
                            code.setDisplay(type.getDisplay());
                            code.setSystem(type.getSystem());
                            code.setCode(type.getCode());
                            bean.getCodes().add(code);
                        }
                    }

                    //                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
                    //                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);
                    //                addBHIESiteDataToVistaBean(bhieNote.getRepository().getId().get(0).getId(),
                    //                        bhieNote.getRepository().getName(), bhieNote.getFacility().getName(), bhieNote.getRecordId().getId(), bean);
                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getRepository() != null) {
                        repositoryName = bhieNote.getRepository().getName();
                    }
                    if (bhieNote.getFacility() != null) {
                        facilityName = bhieNote.getFacility().getName();
                    }
                    if (bhieNote.getRecordId() != null) {
                        eventId = bhieNote.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                            facilityName, eventId, bean);

                    Code code = new Code();
                    code.setSystem("2.16.840.1.113883.6.1");
                    code.setCode("64284-3");
                    code.setDisplay("Readiness for duty assessment");

                    form.getCodes().add(code);
                    formDoc.getCodes().add(code);

                    if (bhieNote.getRecordId() != null && bhieNote.getRecordId().getId() != null) {
                        form.setFormId(bhieNote.getRecordId().getId());
                    } else {
                        form.setFormId("deploymentForm_" + formId);
                        formId++;
                    }
                    form.setEnteredBy(form.getEnteredBy());
//                    if(bhieNote.getEnteredDate()!=null)
//                        form.setFormDate(Utils.getCal(bhieNote.getEnteredDate()));
                    form.setDocType("F");
                    form.setFacilityName(facilityName);
                    if (bhieNote.getTitle() != null) {
                        form.setDocumentTitle(bhieNote.getTitle());
                    } else {
                        form.setDocumentTitle("Pre/Post - Deployment Health Assessment Form");
                    }

                    addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, form);
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, formDoc);

                    form.setDeploymentForm(bean);

                    formDoc.setForm(form);
                    vistaBeans.add(formDoc);


                }
            }
        } catch (Exception ex) {
            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }
        //include site status report
        String siteStatusReport = "";
        if (noteResultCollection != null) {
            siteStatusReport = generateBhieStatusReportV4(DOCUMENT_DEPLOYMENT_FORM, noteResultCollection.getStatusListv4(),
                    noteResultCollection.getErrorListv4());
        }

        vistaBeans.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));
        return vistaBeans.toArray(new PatientDocument[vistaBeans.size()]);
    }

    public PatientDocument[] mapRadiologyExamFromBHIE(Object resultCollection) {
        V4ResultCollection radiologyResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDocument> vistaBeans = new ArrayList<PatientDocument>();


        if (radiologyResultCollection.getDataRecordList() != null
                && radiologyResultCollection.getDataRecordList().getRadiologyReports() != null) {
            StringBuilder textSB = new StringBuilder();
            for (RadiologyReportV4 bhieRad : radiologyResultCollection.getDataRecordList().getRadiologyReports()) {
                if (bhieRad == null) {
                    continue;
                }

                PatientDocument radDoc = new PatientDocument();
                radDoc.setDocumentType(DOCUMENT_RADIOLOGY);

                RadiologyReport bean = new RadiologyReport();

                if (bhieRad.getExam() != null && bhieRad.getExam().getType() != null
                        && !bhieRad.getExam().getType().isEmpty()) {

                    if (getPrimarySpdcV4(bhieRad.getExam().getType()) != null) {
                        bean.setStudy(getPrimarySpdcV4(bhieRad.getExam().getType()).getDisplay());
                        radDoc.setNoteTitle(getPrimarySpdcV4(bhieRad.getExam().getType()).getDisplay());
                    }
                    for (SpdcBeanV4 spdc : bhieRad.getExam().getType()) {
                        Code code = new Code();
//                        if (CodingSystem.LOINC.getOID().equalsIgnoreCase(spdc.getSystem())) {
//                            code.setSystem(CodingSystem.LOINC.getDisplayName());
//                        } else
                        code.setPrimary(String.valueOf(spdc.isPrimary()));
                        code.setSystem(spdc.getSystem());
                        code.setDisplay(spdc.getDisplay());
                        code.setCode(spdc.getCode());
                        bean.getCodes().add(code);
                        radDoc.getCodes().add(code);
                    }
                    bean.setExamDate(bhieToXMLCalV4(bhieRad.getExam().getDate()));
                    radDoc.setNoteDate(bhieToXMLCalV4(bhieRad.getExam().getDate()));
                    if (bhieRad.getExam().getExamNumber() != null) {
                        bean.setExamId(bhieRad.getExam().getExamNumber().getId());
                    }
                }
                if (bhieRad.getStatus() != null && !bhieRad.getStatus().isEmpty()
                        && getPrimarySpdcV4(bhieRad.getStatus()) != null) {
                    bean.setStatus(getPrimarySpdcV4(bhieRad.getStatus()).getDisplay());
                    bean.setReportStatus(getPrimarySpdcV4(bhieRad.getStatus()).getDisplay());
                    radDoc.setStatus(getPrimarySpdcV4(bhieRad.getStatus()).getDisplay());
                }

                bean.setReasonForOrder(bhieRad.getReason());

                if (bhieRad.getOrder() != null && bhieRad.getOrder().getAccession() != null) {
                    bean.setAccessionNumber(bhieRad.getOrder().getAccession().getId());

                }

                if (bhieRad.getOrder() != null && bhieRad.getOrder().getOrderNumber() != null) {
                    bean.setOrderNumber(bhieRad.getOrder().getOrderNumber().getId());
                }

                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (bhieRad.getFacility() != null && bhieRad.getFacility().getName() != null) {
                    facilityName = bhieRad.getFacility().getName();
                }

                if (bhieRad.getRepository() != null && bhieRad.getRepository().getName() != null) {
                    repositoryName = bhieRad.getRepository().getName();
                    if (bhieRad.getRepository().getIds() != null && !bhieRad.getRepository().getIds().isEmpty() && bhieRad.getRepository().getIds().get(0) != null) {
                        //repositoryId = bhieRad.getRepository().getIds().get(0).getId();
                        if(!StringUtils.isEmpty(bhieRad.getRepository().getIds().get(0).getId())){
                            repositoryId = bhieRad.getRepository().getIds().get(0).getId();
                        } else {
                            repositoryId = bhieRad.getRepository().getIds().get(0).getAssigningAuthority();
                        }
                    }
                }

                if (bhieRad.getRecordId() != null) {
                    recordId = bhieRad.getRecordId().getId();
                }

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, bean);

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, radDoc);

                //using DMIS ID, include endpoints from Site DB. This is needed for medweb support.
//                String facilityName = bhieRad.getFacility();
                Site site = null;
                if (facilityName != null && facilityName.contains("/")) {
                    int startIdx = facilityName.indexOf('/') + 1;
                    if (startIdx < facilityName.length()) {
                        String dmisId = facilityName.substring(startIdx);
                        site = Sites.getInstance().getSiteByDmisID(dmisId);

                        if (site != null) {
                            bean.getSite().getEndpoints().addAll(site.getEndpoints());
                            bean.getSite().setSiteCode(site.getSiteCode());
                            bean.getSite().setDmisId(site.getDmisId());
                        }
                    }
                }
                if (bhieRad.getApprovedBy() != null) {
                    bean.setApprovedBy(bhieRad.getApprovedBy().getName());
                }

                bean.setImpressionText("");
                if (bhieRad.getInterpretedBy() != null) {
                    bean.setInterpretingHCP(bhieRad.getInterpretedBy().getName());
                }
                if (bhieRad.getPriority() != null) {
                    bean.setPriority(bhieRad.getPriority());
                }

                if (bhieRad.getResult() != null && !bhieRad.getResult().isEmpty()
                        && getPrimarySpdcV4(bhieRad.getResult()) != null) {
                    bean.setResultCode(getPrimarySpdcV4(bhieRad.getResult()).getCode());
                    bean.setAbnormal(getPrimarySpdcV4(bhieRad.getResult()).getDisplay());
                }

                bean.setTranscribeDate(bhieToXMLCalV4(bhieRad.getTranscriptionDate()));
                textSB.setLength(0);
                textSB.append(bhieRad.getContent());

                if (bhieRad.getAmendments() != null && bhieRad.getAmendments().size() > 0) {
                    int lastIndex = bhieRad.getAmendments().size() - 1;
                    AmendmentV4 lastAmendment = bhieRad.getAmendments().get(lastIndex);
                    if (bean.getStatus() != null && !bean.getStatus().toLowerCase().contains("amended")) {
                        StringBuilder statusText = new StringBuilder();
                        statusText.append(bean.getStatus());
                        statusText.append(" (Amended)");
                        bean.setStatus(statusText.toString());
                        radDoc.setStatus(statusText.toString());
                        bean.setReportStatus(statusText.toString());
                    }

                    bean.setAmendedDate(bhieToXMLCalV4(lastAmendment.getVerifiedDate()));
                    if (lastAmendment.getResult() != null && !lastAmendment.getResult().isEmpty()
                            && getPrimarySpdcV4(lastAmendment.getResult()) != null) {
                        bean.setResultCode(getPrimarySpdcV4(lastAmendment.getResult()).getCode());
                        bean.setAbnormal(getPrimarySpdcV4(lastAmendment.getResult()).getDisplay());
                    }
                }
                bean.setReportText(textSB.toString());

                radDoc.setRadiologyReport(bean);
                vistaBeans.add(radDoc);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(DOCUMENT_RADIOLOGY, radiologyResultCollection.getStatusListv4(),
                radiologyResultCollection.getErrorListv4());

        vistaBeans.addAll(Arrays.asList(
                (PatientDocument[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new PatientDocument[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientDocument note = new PatientDocument();
        note.setNoteId("");
        note.setNoteType(errorMsg);
        note.setNoteTitle(errorMsg);
        note.setNoteDate(Utils.backDate());
        note.setSite(getConnectionUnavailableSite(location));

        return new PatientDocument[]{note};
    }
}