package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.*;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Diagnosis;
import gov.va.med.vds.webservice.Encounter;
import gov.va.med.vds.webservice.Procedure;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class SmartEncounter extends SmartBean implements Comparable {
//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartEncounter.class);

    public static final int DATE = 1;
    public static final int SITE = 2;

    public Encounter m_Patient_encounter;

    public SmartEncounter() {
        super();
        m_Patient_encounter = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartEncounter(Encounter patientEncounter, String sort) {
        super();
        m_Patient_encounter = patientEncounter;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartEncounter) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_Patient_encounter);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartEncounter[] temp = new SmartEncounter[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartEncounter newAll = new SmartEncounter((Encounter) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartEncounter newAll = new SmartEncounter((Encounter) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Encounter[] ret = new Encounter[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_encounter;
            }

            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartEncounter[] temp = new SmartEncounter[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartEncounter newAll = new SmartEncounter((Encounter) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Encounter[] ret = new Encounter[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_encounter;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartEncounter tmp = (SmartEncounter) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_Patient_encounter.getApptDate(), tmp.m_Patient_encounter.getApptDate());
        }
        if (this.m_sortBy == SITE && this.m_Patient_encounter.getSite() != null && tmp.m_Patient_encounter.getSite() != null) {
            retVal = compareString(this.m_Patient_encounter.getSite().getSiteCode(), tmp.m_Patient_encounter.getSite().getSiteCode());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("appointmentDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }
    }

    public Encounter[] mapEncountersFromBHIE(Object resultCollection) {
        V4ResultCollection encounterResultCollection = (V4ResultCollection) resultCollection;
        List<Encounter> beanResults = new ArrayList<Encounter>();
        boolean hasMedcin;
        boolean excludeMedcin;

        if (encounterResultCollection.getDataRecordList() != null
                && encounterResultCollection.getDataRecordList().getEncounters() != null) {

            for (EncounterV4 bhieEncounter : encounterResultCollection.getDataRecordList().getEncounters()) {
                if (bhieEncounter == null) {
                    continue;
                }

                Encounter bean = new Encounter();

                bean.setComplexDataUrl(bhieEncounter.getComplexDataUrl());
                if (bhieEncounter.getArrivalDate() != null && bhieToXMLCalV4(bhieEncounter.getArrivalDate()) != null && bhieToXMLCalV4(bhieEncounter.getArrivalDate()).isValid()) {
                    bean.setApptDate(bhieToXMLCalV4(bhieEncounter.getArrivalDate()));
                }

                if (bhieEncounter.getContentType() != null && !bhieEncounter.getContentType().isEmpty()
                        && getPrimarySpdcV4(bhieEncounter.getContentType()) != null) {
                    bean.setContentType(getPrimarySpdcV4(bhieEncounter.getContentType()).getCode());
                } else if(!StringUtils.isEmpty(bhieEncounter.getComplexDataUrl())){
                    //JLV-1075: temp fix until DES adds contentType : assume that content is of type: RTF
                    bean.setContentType("application/rtf");
                }

                if (bhieEncounter.getType() != null
                        && bhieEncounter.getType().size() > 0
                        && getPrimarySpdcV4(bhieEncounter.getType()) != null) {
                    bean.setApptType(getPrimarySpdcV4(bhieEncounter.getType()).getDisplay());
                }

                if (bhieEncounter.getEncounterLocations() != null
                        && !bhieEncounter.getEncounterLocations().isEmpty()) {
                    bean.setClinic(bhieEncounter.getEncounterLocations().get(0).getName());
                }
                if (bhieEncounter.getDisposition() != null && !bhieEncounter.getDisposition().isEmpty()
                        && bhieEncounter.getDisposition().get(0) != null) {
                    bean.setDischargeDisposition(bhieEncounter.getDisposition().get(0).getDisplay());
                }
                if (bhieEncounter.getEncounterNumber() != null) {
                    bean.setEncounterId(bhieEncounter.getEncounterNumber().getId());
                    bean.setEncounterNumber(bhieEncounter.getEncounterNumber().getId());
                }
                if (bhieEncounter.getEnteredBy() != null) {
                    bean.setEnteredBy(bhieEncounter.getEnteredBy().getName());
                }
                if (bhieEncounter.getEnteredDate() != null) {
                    bean.setEnteredByDate(bhieToXMLCalV4(bhieEncounter.getEnteredDate()));
                }
                //not available after update
                //bean.setEvalManagementCode(bhieEncounter.getEvalManagementCode());
                if (bhieEncounter.getProvider() != null) {
                    bean.setProvider(mapProvider(bhieEncounter.getProvider()));
                }
                //bean.setReport(bhieEncounter.getReport());
                //bean.setReportFormat(bhieEncounter.getReportFormat());
                if (bhieEncounter.getStatus() != null
                        && !bhieEncounter.getStatus().isEmpty()
                        && bhieEncounter.getStatus().get(0) != null) {
                    bean.setStatus(bhieEncounter.getStatus().get(0).getDisplay());
                }
                if (bhieEncounter.getReason() != null && getPrimarySpdcV4(bhieEncounter.getReason()) != null) {
                    bean.setReason(getPrimarySpdcV4(bhieEncounter.getReason()).getDisplay());
                }
                bean.setSensitive(bhieEncounter.getSensitive());

                if (bhieEncounter.getDiagnoses() != null) {
                    List<Diagnosis> diagnosisList = new ArrayList<Diagnosis>(bhieEncounter.getDiagnoses().size());
                    boolean firstDiagnosis = true;
                    for (EncounterDiagnosisV4 bhieDiagnosis : bhieEncounter.getDiagnoses()) {

                        Diagnosis diagnosis = new Diagnosis();

                        if (bhieDiagnosis.getDiagnosis() != null
                                && !bhieDiagnosis.getDiagnosis().isEmpty()) {
                            hasMedcin = false;
                            excludeMedcin = false;

                            for (SpdcBeanV4 diagnosisBean : bhieDiagnosis.getDiagnosis()) {
                                if (diagnosisBean != null) {
                                    if (CodingSystem.ICD10_CM.getOID().equalsIgnoreCase(diagnosisBean.getSystem())) {
                                        diagnosis.setDescription(diagnosisBean.getDisplay());
                                        diagnosis.setCode(diagnosisBean.getCode());
                                        diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                        excludeMedcin = true;
                                    } else if (CodingSystem.ICD9_CM_DISEASES.getOID().equalsIgnoreCase(diagnosisBean.getSystem())
                                            && !StringUtils.isEmpty(diagnosisBean.getCode())) {
                                        diagnosis.setDescription(diagnosisBean.getDisplay());
                                        diagnosis.setCode(diagnosisBean.getCode());
                                        diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                    }

                                    if (CodingSystem.DOD_MEDCIN.getOID().equalsIgnoreCase(diagnosisBean.getSystem())) {
                                        if(!excludeMedcin){
                                            diagnosis.setDescription(diagnosisBean.getDisplay());
                                            diagnosis.setCode(diagnosisBean.getCode());
                                            diagnosis.setCodeSystem(diagnosisBean.getSystem());
                                        }
                                        hasMedcin = true;
                                    }
                                    if ( firstDiagnosis && (CodingSystem.ICD10_SNOMED_CT.getOID().equalsIgnoreCase(diagnosisBean.getSystem()))){
                                        Code code = new Code();
                                        code.setSystem(diagnosisBean.getSystem());
                                        code.setPrimary(String.valueOf(diagnosisBean.isPrimary()));
                                        code.setDisplay(diagnosisBean.getDisplay());
                                        code.setCode(diagnosisBean.getCode());
                                        bean.getCodes().add(code);
                                    }
                                }
                            }
                            firstDiagnosis = false;

                            // If ICD-10 code exists, exclude/remove MEDCIN from code list, if exists
                            if (hasMedcin && excludeMedcin) {
                                Code code;
                                for (Iterator<Code> codeIterator = bean.getCodes().iterator(); codeIterator.hasNext(); ) {
                                    code = codeIterator.next();
                                    if (code != null && CodingSystem.DOD_MEDCIN.getDisplayName().equalsIgnoreCase(code.getSystem())) {
                                        codeIterator.remove();
                                    }
                                }
                            }
                        }

                        if (bhieDiagnosis.getStatus() != null
                                && !bhieDiagnosis.getStatus().isEmpty()) {
                            for (SpdcBeanV4 statusBean : bhieDiagnosis.getStatus()) {
                                if (statusBean != null && statusBean.isPrimary()) {
                                    diagnosis.setStatus(statusBean.getDisplay());
                                }
                            }
                        }

                        if (bhieDiagnosis.getEnteredDate() != null) {
                            diagnosis.setDate(bhieToXMLCalV4(bhieDiagnosis.getEnteredDate()));
                        }

                        diagnosisList.add(diagnosis);
                    }

                    //bean.setDiagnosis(diagnosisList);
                    bean.getDiagnosis().addAll(diagnosisList);
                }

//                if (bhieEncounter.getEvaluations() != null)
//                {
//                    List<Evaluation> evaluations = new ArrayList<Evaluation>(bhieEncounter.getEvaluations().size());
//
//                    for(gov.va.med.bhie.Evaluation bhieEvaluation : bhieEncounter.getEvaluations())
//                    {
//                        Evaluation evaluation = new Evaluation();
//                        evaluation.setEvalManagement(bhieEvaluation.getEvalManagement());
//                        evaluation.setEvalManagementCode(bhieEvaluation.getEvalManagementCode());
//
//                        evaluations.add(evaluation);
//                    }
//
//                    bean.setEvaluations(evaluations);
//                }

                if (bhieEncounter.getProcedures() != null) {
                    List<Procedure> procedures = new ArrayList<Procedure>(bhieEncounter.getProcedures().size());

                    for (EncounterProcedureV4 bhieProcedure : bhieEncounter.getProcedures()) {
                        Procedure procedure = new Procedure();
                        if (bhieProcedure.getProcedure() != null
                                && !bhieProcedure.getProcedure().isEmpty()) {
                            for (SpdcBeanV4 procedureBean : bhieProcedure.getProcedure()) {
                                if (procedureBean == null) {
                                    continue;
                                }
                                if (procedureBean.isPrimary()) {
                                    procedure.setDescription(procedureBean.getDisplay());
                                }
                                if (CodingSystem.CPT.getOID().equalsIgnoreCase(procedureBean.getSystem())) {
                                    procedure.setCode(procedureBean.getCode());
                                    procedure.setCodeSystem(procedureBean.getSystem());
                                }
                            }

                        }
                        if (bhieProcedure.getDate() != null) {
                            procedure.setDate(bhieToXMLCalV4(bhieProcedure.getDate()));
                        }
                        if (bhieProcedure.getStatus() != null
                                && !bhieProcedure.getStatus().isEmpty()
                                && bhieProcedure.getStatus().get(0) != null) {
                            procedure.setStatus(bhieProcedure.getStatus().get(0).getDisplay());
                        }

                        procedures.add(procedure);
                    }

                    bean.getProcedures().addAll(procedures);
                    //bean.setProcedures(procedures);
                }


//                addBHIESiteDataToVistaBean(bhieEncounter.getCustodian().getId(),
//                        bhieEncounter.getCustodian().getDisplayName(), bhieEncounter.getEncounterId(), bhieEncounter.getEncounterId(), bean);
//                addBHIESiteDataToVistaBean(bhieEncounter.getRepository().getId().get(0).getId(),
//                        bhieEncounter.getRepository().getName(), bhieEncounter.getFacility().getName(), bhieEncounter.getRecordId().getId(), bean);
                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieEncounter.getRepository() != null
                        && bhieEncounter.getRepository().getIds() != null && !bhieEncounter.getRepository().getIds().isEmpty()) {
//                    repositoryId = bhieEncounter.getRepository().getIds().get(0).getId();
                    if(!StringUtils.isEmpty(bhieEncounter.getRepository().getIds().get(0).getId())){
                        repositoryId = bhieEncounter.getRepository().getIds().get(0).getId();
                    } else {
                        repositoryId = bhieEncounter.getRepository().getIds().get(0).getAssigningAuthority();
                    }
                }
                if (bhieEncounter.getRepository() != null) {
                    repositoryName = bhieEncounter.getRepository().getName();
                }
                if (bhieEncounter.getFacility() != null) {
                    facilityName = bhieEncounter.getFacility().getName();
                }
                if (bhieEncounter.getRecordId() != null) {
                    eventId = bhieEncounter.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

                beanResults.add(bean);
            }
        }

        //Version 2
//        public Encounter[] mapEncountersFromBHIE(Object resultCollection) {
//            EncounterResultCollection encounterResultCollection = (EncounterResultCollection) resultCollection;
//            List<Encounter> beanResults = new ArrayList<Encounter>();
//
//            if (encounterResultCollection.getDataRecordList() != null) {
//
//                for (EncounterDataRecord dataRecord : encounterResultCollection.getDataRecordList())
//                {
//
//                    if (dataRecord == null || dataRecord.getEncounter() == null) continue;
//
//                    Encounter bean = new Encounter();
//                    gov.va.med.bhie.Encounter bhieEncounter = dataRecord.getEncounter();
//
//                    bean.setComplexDataUrl(bhieEncounter.getComplexDataUrl());
//                    if (StringUtils.isNotBlank(bhieEncounter.getApptDateTime()))
//                        bean.setApptDate(bhieToXMLCal(bhieEncounter.getApptDateTime()));
//
//                    bean.setApptType(bhieEncounter.getAppointmentType());
//
//                    if(bhieEncounter.getClinic()!=null && !bhieEncounter.getClinic().isEmpty())
//                        bean.setClinic(bhieEncounter.getClinic().get(0).getDisplay());
//
//                    if(bhieEncounter.getDischargeDisposition()!=null && !bhieEncounter.getDischargeDisposition().isEmpty())
//                        bean.setDischargeDisposition(bhieEncounter.getDischargeDisposition().get(0).getDisplay());
//
//                    bean.setEncounterId(bhieEncounter.getEncounterId());
//                    bean.setEncounterNumber(bhieEncounter.getEncounterNumber());
//
//                    if(bhieEncounter.getEnteredBy()!=null && !bhieEncounter.getEnteredBy().isEmpty()  )
//                        bhieEncounter.getEnteredBy().get(0).getName();
//
//                    bean.setEnteredByDate(Utils.getXMLCal(bhieEncounter.getEnteredByDate()));
//                    //not available after update
//                    //bean.setEvalManagementCode(bhieEncounter.getEvalManagementCode());
//                    if(bhieEncounter.getProvider()!=null && !bhieEncounter.getProvider().isEmpty())
//                        bean.setProvider(mapProvider(bhieEncounter.getProvider().get(0)));
//
//                    //bean.setReport(bhieEncounter.getReport());
//                    //bean.setReportFormat(bhieEncounter.getReportFormat());
//                    if(bhieEncounter.getStatus()!=null && !bhieEncounter.getStatus().isEmpty())
//                        bean.setStatus(bhieEncounter.getStatus().get(0).getDisplay());
//
//                    bean.setReason(bhieEncounter.getReasonForVisit());
//
//                    if (bhieEncounter.getDiagnosis() != null)
//                    {
//                        List<Diagnosis> diagnosisList = new ArrayList<Diagnosis>(bhieEncounter.getDiagnosis().size());
//
//                        for(gov.va.med.bhie.Diagnosis bhieDiagnosis : bhieEncounter.getDiagnosis())
//                        {
//                            Diagnosis diagnosis = new Diagnosis();
//
//                            if(bhieDiagnosis.getDiagnosis()!=null && !bhieDiagnosis.getDiagnosis().isEmpty()){
//                                diagnosis.setDescription(bhieDiagnosis.getDiagnosis().get(0).getDisplay());
//                                diagnosis.setCode(bhieDiagnosis.getDiagnosis().get(0).getCode());
//                            }
//
//                            if(bhieDiagnosis.getMedcinData()!=null && !bhieDiagnosis.getMedcinData().isEmpty()){
//                                Code code = new Code();
//                                code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
//                                code.setCode(bhieDiagnosis.getMedcinData().get(0).getCode());
//                                bean.getCodes().add(code);
//                            }
//
//                            if(bhieDiagnosis.getStatus()!=null && !bhieDiagnosis.getStatus().isEmpty())
//                                diagnosis.setStatus(bhieDiagnosis.getStatus().get(0).getDisplay());
//
//                            diagnosis.setDate(bhieToXMLCal(bhieDiagnosis.getEnteredByDate()));
//
//                            diagnosisList.add(diagnosis);
//                        }
//
//                        //bean.setDiagnosis(diagnosisList);
//                        bean.getDiagnosis().addAll(diagnosisList);
//                    }
//
////                    if (bhieEncounter.getEvaluations() != null)
////                    {
////                        List<Evaluation> evaluations = new ArrayList<Evaluation>(bhieEncounter.getEvaluations().size());
//    //
////                        for(gov.va.med.bhie.Evaluation bhieEvaluation : bhieEncounter.getEvaluations())
////                        {
////                            Evaluation evaluation = new Evaluation();
////                            evaluation.setEvalManagement(bhieEvaluation.getEvalManagement());
////                            evaluation.setEvalManagementCode(bhieEvaluation.getEvalManagementCode());
//    //
////                            evaluations.add(evaluation);
////                        }
//    //
////                        bean.setEvaluations(evaluations);
////                    }
//
//                    if (bhieEncounter.getProcedures() != null)
//                    {
//                        List<Procedure> procedures = new ArrayList<Procedure>(bhieEncounter.getProcedures().size());
//
//                        for(gov.va.med.bhie.Procedure bhieProcedure : bhieEncounter.getProcedures())
//                        {
//                            Procedure procedure = new Procedure();
//                            if(bhieProcedure.getProcedure()!=null && !bhieProcedure.getProcedure().isEmpty()){
//                                procedure.setCode(bhieProcedure.getProcedure().get(0).getCode());
//                                procedure.setDescription(bhieProcedure.getProcedure().get(0).getDisplay());
//                            }
//                            procedure.setDate(bhieToXMLCal(bhieProcedure.getProcedureDate()));
//                            if(bhieProcedure.getStatus()!=null && !bhieProcedure.getStatus().isEmpty())
//                                procedure.setStatus(bhieProcedure.getStatus().get(0).getDisplay());
//
//                            procedures.add(procedure);
//                        }
//
//                        bean.getProcedures().addAll(procedures);
//                        //bean.setProcedures(procedures);
//                    }
//
//
//                    addBHIESiteDataToVistaBean(bhieEncounter.getCustodian().getId(),
//                            bhieEncounter.getCustodian().getDisplayName(), bhieEncounter.getEncounterId(), bhieEncounter.getEncounterId(), bean);
//
//                    beanResults.add(bean);
//                }
//            }
//
//            String siteStatusReport = generateBhieStatusReport(encounterResultCollection.getStatusList(),
//                    encounterResultCollection.getErrorList());
//
//            beanResults.addAll(Arrays.asList(
//                    (Encounter[]) generateConnectionUnavailableBean(
//                            BHIE_STATUS_REPORT_LOCATION,
//                            siteStatusReport)
//            ));
//
//            return beanResults.toArray(new Encounter[0]);
//        }


        String siteStatusReport = generateBhieStatusReportV4(encounterResultCollection.getStatusListv4(),
                encounterResultCollection.getErrorListv4(), encounterResultCollection.isVler());

        beanResults.addAll(Arrays.asList(
                (Encounter[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return beanResults.toArray(new Encounter[beanResults.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Encounter encounter = new Encounter();
        encounter.setClinic(errorMsg);
        encounter.setApptDate(Utils.backDate());
        encounter.setSite(getConnectionUnavailableSite(location));
        return new Encounter[]{encounter};
    }
}
