/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.DemographicV4;
import gov.va.med.bhie.RecordIdentifierV4;
import gov.va.med.bhie.V4DataRecord;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.ResponsePatientSelect;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Alert;
import gov.va.med.vds.webservice.Patient;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

public class SmartPatientSelect extends SmartBean {

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = array1;
        }
        else if (array1 == null) {
            rtc = array2;
        }
        else {
            rtc = ArrayUtils.addAll(array1, array2);
        }

        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        return array1;
    }

    public int compareTo(Object obj) {
        return 0;
    }

    public ResponsePatientSelect[] mapPatientDemographicsFromDes(Object resultCollection) throws JMeadowsException {
        V4ResultCollection demoResultCollection = (V4ResultCollection)resultCollection;
        V4DataRecord demoDataRecord = demoResultCollection.getDataRecordList();
        ResponsePatientSelect response = new ResponsePatientSelect();
        Patient patient = new Patient();
        StringBuilder fmpssn = new StringBuilder();
        String ssn;

        response.setResponseType(ResponsePatientSelect.ResponseType.DEMOGRAPHICS_DES);

        if (demoDataRecord != null && demoDataRecord.getDemographics() != null) {
            for (DemographicV4 demo : demoDataRecord.getDemographics()) {
                if (demo == null) { continue; }

                // AHLTA Unit Number
                if (demo.getPatientId() != null) {
                    for (RecordIdentifierV4 recId : demo.getPatientId()) {
                        if (StringUtils.equals("2.16.840.1.113883.3.42.126.100001.2", recId.getAssigningAuthority())
                                && StringUtils.isNotBlank(recId.getId())) {
                            patient.setAhltaUnitNumber(recId.getId());
                            break;
                        }
                    }
                }

                // Save fmp/ssn to add to PDWS demographics (person retrieve) call
                fmpssn.setLength(0);
                if (StringUtils.isNotBlank(demo.getFmp())) {
                    fmpssn.append(demo.getFmp());
                }
                fmpssn.append("/");
                if (demo.getSponsor() != null) {
                    ssn = demo.getSponsor().getSsn();
                    if(StringUtils.isNotBlank(ssn)) {
                        fmpssn.append(ssn);
                        patient.setSponsorSSN(ssn);
                    }
                    if (demo.getSponsor().getName() != null) {
                        patient.setSponsorName(SmartDemographics.getNameString(demo.getSponsor().getName()));
                    }
                }
                patient.setFMP(fmpssn.toString());

                // HIE Sharing Opt In
                if (demo.getHIESharing() != null) {
                    patient.setHIESharingOptIn(demo.getHIESharing().isOptIn());
                    if (demo.getHIESharing().getLastUpdateDate() != null) {
                        patient.setHIESharingOptInLastUpdateDate(SmartBean.bhieToXMLCalV4(demo.getHIESharing().getLastUpdateDate()));
                    }
                }
            }

            response.setPatient(patient);
            response.setSuccess(true);
        }
        else {
            response.setSuccess(false);
        }

        return new ResponsePatientSelect[]{response};
    }

    public ResponsePatientSelect[] mapSensitivePatientAlertsFromVista(Object resultCollection) throws JMeadowsException {
        List<Alert> alerts = (List<Alert>)resultCollection;
        ResponsePatientSelect response = new ResponsePatientSelect();
        Patient patient = new Patient();
        int oldAlertLevel = -2;
        int newAlertLevel;

        if (alerts != null && alerts.size() > 0) {
            for (Alert alert : alerts) {
                newAlertLevel = Integer.parseInt(alert.getAlertId());
                if (newAlertLevel > oldAlertLevel) {
                    oldAlertLevel = newAlertLevel;
                }
            }
        }

        if (oldAlertLevel > 0) {
            patient.setSensitive(true);
        }
        else {
            if (oldAlertLevel == -2) {
                response.setErrorMsg("Unable to determine if patient has sensitive records");
            }

            patient.setSensitive(false);
        }

        response.setResponseType(ResponsePatientSelect.ResponseType.SENSITIVE_ALERTS);
        response.setSensitivePatientAlerts(alerts);
        response.setPatient(patient);

        return new ResponsePatientSelect[]{response};
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        ResponsePatientSelect patientSelect = new ResponsePatientSelect();
        patientSelect.setErrorMsg(errorMsg);
        patientSelect.setSuccess(false);

        return new ResponsePatientSelect[]{patientSelect};
    }
}
