/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.Consult;
import gov.va.med.vhahon.common.Utils;
import org.apache.commons.lang3.StringUtils;
//import org.hui.chcs.ConsultBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SmartConsult extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartConsult.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int LOC = 3;

    public Consult m_consult;

    public SmartConsult() {
        super();
        m_consult = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartConsult(Consult consult, String sort) {
        super();
        m_consult = consult;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartConsult) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_consult);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartConsult[] temp = new SmartConsult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
    
            for (int i = 0; i < array2.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
    
            Arrays.sort(temp);
            Consult[] ret = new Consult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_consult;
            }
    
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartConsult[] temp = new SmartConsult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
    
            Arrays.sort(temp);
            Consult[] ret = new Consult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_consult;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartConsult tmp = (SmartConsult) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_consult.getRequestDate(), tmp.m_consult.getRequestDate());
        }
        if (this.m_sortBy == SITE && this.m_consult.getSite() != null && tmp.m_consult.getSite() != null) {
            retVal = compareString(this.m_consult.getSite().getSiteCode(), tmp.m_consult.getSite().getSiteCode());
        }
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_consult.getService(), tmp.m_consult.getService());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("requestDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("service".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
        }

    }

//    public Consult[] mapConsultsFromCHCS(Object[] dodBean) {
//        ConsultBean[] dodConsults = (ConsultBean[]) dodBean;
//        Consult[] mappedConsultBeans = new Consult[dodConsults.length];
//        Consult bean = null;
//
//        for (int i = 0; i < dodConsults.length; i++) {
//            bean = new Consult();
//            bean.setId(dodConsults[i].getConsultId());
//            bean.setPatientId(dodConsults[i].getPatientId());
//            bean.setPatientName(dodConsults[i].getPatientName());
//            bean.setRequestDate(dodConsults[i].getConsultDate());
//            bean.setConsultType(dodConsults[i].getConsultType());
//            bean.setService(dodConsults[i].getConsultService());
//            bean.setStatus(dodConsults[i].getConsultStatus());
//            addCHCSSiteDataToVistaBean(bean);
//            mappedConsultBeans[i] = bean;
//        }
//
//        return mappedConsultBeans;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        Consult cons = new Consult();

        if (StringUtils.isBlank(errorMsg)) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        cons.setService(errorMsg);
        cons.setRequestDate(Utils.backDate());
        cons.setSite(getConnectionUnavailableSite(location));

        return new Consult[]{cons};
    }

}