/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.ProblemDao;
import gov.va.med.vds.webservice.Problem;
import gov.va.med.vds.webservice.ProblemDetail;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class ProblemDaoTest extends DaoBaseTest
{
    private ProblemDao problemDao;

    public ProblemDaoTest() throws JMeadowsException
    {
        problemDao = new ProblemDao();
    }

    @Test
    public void testGetPatientProblems()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            qb.setStatus("A");
            qb.setIncludeDASVLER(true);
            List<Problem> problems = problemDao.getPatientProblemList(qb);
            assertTrue(problems.size() > 0);
            testForBeanMeta(problems);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientProblemDetails()
    {
      try {
        JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
        qb.setStatus("A");
        List<Problem> problems = problemDao.getPatientProblemList(qb);
        assertTrue(problems.size() > 0);
        testForBeanMeta(problems);

        Problem prob = problems.get(0);
        JMeadowsQuery qbDetails = setupQuery(user, patient, startDate, endDate);
        qbDetails.setItemId(prob.getId());
        qbDetails.setRecordSiteCode(prob.getSite().getSiteCode());
        ProblemDetail problemDetail = problemDao.getProblemDetail(qbDetails);
        assertTrue(problemDetail != null);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

}
