/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.dao.patient.DaoBaseTest;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.webservice.JMeadowsData;
import gov.va.med.vhahon.common.Utils;
import org.junit.Test;

import java.util.GregorianCalendar;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author vhahonbrunos
 */
public class JanusGUIConfigDaoTest extends DaoBaseTest
{
    public JanusGUIConfigDaoTest() throws JMeadowsException
    {
        super();
    }

    @Test
    public void testGetJanusGUICfg()
    {
        try 
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            
            mockProviderBasedQuery(qb.getUser(), "1522", "258");

            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

//            providerDao = new UserDao();
//            providerDao.setConnection(m_connectionPool.getConnection());
//
//            String cfg = providerDao.getJanusGUIConfig(qb);



            JMeadowsData jData = new JMeadowsData();
            String cfg = jData.getJanusGUIConfig(qb);
            qb.setStartDate(Utils.getCalendar(new GregorianCalendar().getTime()));  // Default to T-1 month
            qb.setEndDate(Utils.getCalendar(new GregorianCalendar().getTime()));
            jData.getPatientProblemList(qb);


//            AuditDao auditDao = new AuditDao();
//            auditDao.setAudit("459","9999","","tester666","0","testing","testing1234","01/01/01","01/01/01");
            assertTrue(cfg.length() > 0);

            // Only uncomment for debugging purpose, code below is picked up as fortify issue
           // logger.debug("CFG: " + cfg);
           // logger.debug("---");
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetProviderFlags()
    {
        try
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            mockProviderBasedQuery(qb.getUser(), qb.getUser().getUserIen(), null);

            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

            JMeadowsData jData = new JMeadowsData();
            String flags = jData.getUserFlags(qb);

            assertTrue(flags.indexOf( "allergyWriteBack" ) != -1);

            // Only uncomment for debugging purpose, code below is picked up as fortify issue
           // logger.debug("Flags: " + flags);
           // logger.debug("---");
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testSetProviderFlags()
    {
        try
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            mockProviderBasedQuery(qb.getUser(), qb.getUser().getUserIen(), null);

            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

            String flags = "testFlag01^allergyWriteBack^testFlag02";

            JMeadowsData jData = new JMeadowsData();
            boolean success = jData.setUserFlags(qb, flags);

            assertTrue(success);

            logger.debug("Flags: " + flags);
            logger.debug("---");
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
