/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBaseTest;
import gov.va.med.jmeadows.dao.provider.ProviderAppointmentsDao;
import gov.va.med.vds.webservice.PatientAppointments;
import org.junit.Test;

import java.text.SimpleDateFormat;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ProviderAppointmentsDaoTest extends DaoBaseTest
{
    private ProviderAppointmentsDao appointmentsDao;
    
    public ProviderAppointmentsDaoTest() throws JMeadowsException
    {
        super();
        appointmentsDao = new ProviderAppointmentsDao();
    }

    @Test
    public void testGetProviderAppointments()
    {
        try
        {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            mockProviderBasedQuery(qb.getUser(), null, "258");
            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

            List<PatientAppointments> appointments = appointmentsDao.getProviderAppointments(qb);
            assertTrue(appointments.size() > 0);
            logger.debug("# of User Appointments: " + appointments.size());
            testForProviderBeanMeta(appointments);
            for(PatientAppointments appt : appointments)
            {
                SimpleDateFormat sdf = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
                logger.debug("ApptDateTime: " + sdf.format(appt.getApptDate().toGregorianCalendar().getTime()));
            }
        } 
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
